/*
 * Range.java  -  This file is part of KEA.
 *
 * Copyright (C) 2002/03 PG 419, Department of Computer Science, 
 * University of Dortmund.          eMail: <peter.senft@udo.edu>
 */

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the KEA License Agreement which is based on 
 * the GNU General Public License as published by the Free Software 
 * Foundation; either version 2 of the License, or (at your option) 
 * any later version.  For details see License.txt.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

package kea;

import java.io.Serializable;

/**
 * Range.java represents the range of values for a Value-object.
 * To represent boundless values, we recommend the usage of
 * the constants MIN_VALUE and MAX_VALUE of the given datatype.
 * 
 * @author Stefan Kusper, Peter Senft, Norman Welp
 * @version 0.1.0
 */
public class Range implements Cloneable,Serializable {
	
    /**
     * Minimum-value in the range
     */
    private Object minVal;
    /**
     * Maximum-value in the range
     */
    private Object maxVal;

    /**
     * Constructor for a range-object with a given minVal and a given maxVal.
     * No type-check oder logical correctness is checked here
     * TODO:
     * - Implement typecheck
     * - Implement logical correctnes (minVal <= maxVal)
     *
     * @param minVal - minimum Object of the range-object
     *        maxVal - maximum Object of the range-object
     */
    public Range(Object minVal, Object maxVal){
	this.minVal = minVal;
	this.maxVal = maxVal;
    }

    /**
     * Returns the minVal of this range-object
     *
     * @return minVal as an Object
     */
    public Object getMinVal() {
	return minVal;
    }

    /**
     * Sets the minVal of this range-object
     *
     * @param minVal - new minimum Object of the range-object
     */
    public void setMinVal (Object minVal) {
	this.minVal = minVal;
    }

    /**
     * Returns the maxVal of this range-object
     *
     * @return maxVal as an Object
     */
    public Object getMaxVal() {
	return maxVal;
    }

    /**
     * Sets the maxVal of this range-object
     *
     * @param minVal - new maximum Object of the range-object
     */
    public void setMaxVal (Object maxVal) {
	this.maxVal = maxVal;
    }

    public Object clone() {
        Range retval = null;
        try {
            retval = (Range)super.clone();
            retval.setMaxVal(maxVal);
            retval.setMinVal(minVal);
        }
        catch (CloneNotSupportedException e) {
            System.out.println(e);
        }
        return retval;
    }
}
