/*
 * Value.java  -  This file is part of KEA.
 *
 * Copyright (C) 2002/03 PG 419, Department of Computer Science, 
 * University of Dortmund.          eMail: <peter.senft@udo.edu>
 */

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the KEA License Agreement which is based on 
 * the GNU General Public License as published by the Free Software 
 * Foundation; either version 2 of the License, or (at your option) 
 * any later version.  For details see License.txt.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

package kea;
import java.io.Serializable;

/**
 * @author Stefan Kusper, Peter Senft, Norman Welp
 * @version $Revision: 712 $
 */
public class Value implements Serializable, Cloneable {

    public Value() {
        this.range = null;
        this.rangeCount = 0;
	this.type = new Type();
        this.data = null;
    }

    public Value(Object myobj) {
        // first try a type
        if (myobj != null) {
        }
        if (myobj instanceof Type) {
            this.type = (Type)myobj;
            this.range = null;
            this.rangeCount = 0;
            this.data = null;
        } // end type
        else  { // Now, all other Objects...
	    // test for valid object
	    if (myobj.getClass().getName().compareTo("java.lang.Boolean") == 0) {
		this.setType(Type.BOOLEAN);
	    }
	    else if (myobj.getClass().getName().compareTo("java.lang.Integer") == 0) {
		this.setType(Type.INTEGER);
	    }
	    else if (myobj.getClass().getName().compareTo("java.lang.Short") == 0) {
		this.setType(Type.SHORT);
	    }
	    else if (myobj.getClass().getName().compareTo("java.lang.Long") == 0) {
		this.setType(Type.LONG);
	    }
	    else if (myobj.getClass().getName().compareTo("java.lang.Byte") == 0) {
		this.setType(Type.BYTE);
	    }
	    else if (myobj.getClass().getName().compareTo("java.lang.Character") == 0) {
		this.setType(Type.CHAR);
	    }
	    else if (myobj.getClass().getName().compareTo("java.lang.Double") == 0) {
		this.setType(Type.DOUBLE);
	    }
	    else if (myobj.getClass().getName().compareTo("java.lang.Float") == 0) {
		this.setType(Type.FLOAT);
	    }
	    else if (myobj.getClass().getName().compareTo("java.lang.String") == 0) {
		this.setType(Type.STRING);
	    } else if (myobj.getClass().getName().compareTo("kea.Value") == 0) {
		this.type = ((Value)myobj).getType();
		this.range = ((Value)myobj).getRange();
		myobj = ((Value)myobj).getData();
		if (this.range != null) {
		    this.rangeCount = this.range.length;
		}
		else {
		    this.rangeCount = 0;
		}
	    } else {
    //            if (Kea.logger.debugOn()) {
    //              Kea.logger.logDebug("Cannot construct Value from this Object (" + myobj.getClass().getName() + ").", this);
    //            }
		this.type = new Type();
		this.type.setTypeInfo(Type.UNDEFINED);
	    }
	    // now set the object
	    if (type.getTypeInfo() != Type.UNDEFINED) {
		this.data = myobj;
	    }
	    else {
		this.data = null;
	    }
            this.range = null;
            this.rangeCount = 0;
        }
    }

    public Value(Type type) {
        this.type = type;
        this.range = null;
        this.rangeCount = 0;
        this.data = null;
    }

    public Value(boolean data) {
        Boolean newData = new Boolean(data);
        this.type = new Type(Type.BOOLEAN);
        this.data = (Object)newData;
        this.range = null;
        this.rangeCount = 0;
    }

    public Value(int data) {
        Integer newData = new Integer(data);
        this.type = new Type(Type.INTEGER);
        this.data = (Object)newData;
        this.range = null;
        this.rangeCount = 0;
    }

    public Value(short data) {
        Short newData = new Short(data);
        this.type = new Type(Type.SHORT);
        this.data = (Object)newData;
        this.range = null;
        this.rangeCount = 0;
    }

    public Value(long data) {
        Long newData = new Long(data);
        this.type = new Type(Type.LONG);
        this.data = (Object)newData;
        this.range = null;
        this.rangeCount = 0;
    }

    public Value(byte data) {
        Byte newData = new Byte(data);
        this.type = new Type(Type.BYTE);
        this.data = (Object)newData;
        this.range = null;
        this.rangeCount = 0;
    }

    public Value(char data) {
        Character newData = new Character(data);
        this.type = new Type(Type.CHAR);
        this.data = (Object)newData;
        this.range = null;
        this.rangeCount = 0;
    }

    public Value(double data) {
        Double newData = new Double(data);
        this.type = new Type(Type.DOUBLE);
        this.data = (Object)newData;
        this.range = null;
        this.rangeCount = 0;
    }

    public Value(float data) {
        Float newData = new Float(data);
        this.type = new Type(Type.FLOAT);
        this.data = (Object)newData;
        this.range = null;
        this.rangeCount = 0;
    }

    public Value(String data) {
        this.type = new Type(Type.STRING);
        this.data = (Object)data;
        this.range = null;
        this.rangeCount = 0;
    }

    public Value(int type, Range range) {
        this.type = new Type(type);
        this.range = new Range[1];
        this.range[0] = range;
        this.rangeCount = 1;
    }

    public Value(int type, Range[] range) {
        this.type = new Type(type);
        this.range = range;
        if (this.range != null) {
          this.rangeCount = this.range.length;
        }
        else {
          this.rangeCount = 0;
        }
    }

    public Range[] getRange(){
	return this.range;
    }

    public boolean setRange(Range[] range) {
        boolean retval = true;
        // testing all given Ranges...
        for (int i=0; i < range.length; ++i) {
            retval = (retval &&
              ( (range[i].getMinVal().getClass().getName().compareTo(
                          this.data.getClass().getName() ) == 0) &&
                (range[i].getMaxVal().getClass().getName().compareTo(
                          this.data.getClass().getName() ) == 0) )
            );
        }
        // setting the range[] only if all Types match...
        if (retval) {
	    this.range = range;
            this.rangeCount = range.length;
        }
        return retval;
    }

    public boolean setRange(Range range) {
        boolean retval = false;
        if ( (range.getMinVal().getClass().getName().compareTo(
                    this.data.getClass().getName() ) == 0) &&
             (range.getMaxVal().getClass().getName().compareTo(
                    this.data.getClass().getName() ) == 0 ) ) {
            this.range = new Range[1];
	    this.range[0] = range;
            this.rangeCount = 1;
            retval = true;
        }
        else {
            retval = false;
        }
        return retval;
    }

    public boolean setRange(Object minmax) {
        return this.setRange(minmax, minmax);
    }

    public boolean setRange(boolean minmax) {
        return this.setRange(minmax, minmax);
    }

    public boolean setRange(int minmax) {
        return this.setRange(minmax, minmax);
    }

    public boolean setRange(short minmax) {
        return this.setRange(minmax, minmax);
    }

    public boolean setRange(long minmax) {
        return this.setRange(minmax, minmax);
    }

    public boolean setRange(byte minmax) {
        return this.setRange(minmax, minmax);
    }

    public boolean setRange(char minmax) {
        return this.setRange(minmax, minmax);
    }

    public boolean setRange(double minmax) {
        return this.setRange(minmax, minmax);
    }

    public boolean setRange(float minmax) {
        return this.setRange(minmax, minmax);
    }

    public boolean setRange(String minmax) {
        return this.setRange(minmax, minmax);
    }

    public boolean setRange(Object min, Object max) {
        boolean retval = true;
        if (this.typeCheck(min) && this.typeCheck(max)) {
            Range newRange = new Range(min, max);
            this.setRange(newRange);
            retval = true;
        }
        else {
            retval = false;
        }
        return retval;
    }

    public boolean setRange(boolean min, boolean max) {
        Boolean minVal = new Boolean(min);
        Boolean maxVal = new Boolean(max);
        return this.setRange((Object)minVal, (Object)maxVal);
    }

    public boolean setRange(int min, int max) {
        Integer minVal = new Integer(min);
        Integer maxVal = new Integer(max);
        return this.setRange((Object)minVal, (Object)maxVal);
    }

    public boolean setRange(short min, short max) {
        Short minVal = new Short(min);
        Short maxVal = new Short(max);
        return this.setRange((Object)minVal, (Object)maxVal);
    }

    public boolean setRange(long min, long max) {
        Long minVal = new Long(min);
        Long maxVal = new Long(max);
        return this.setRange((Object)minVal, (Object)maxVal);
    }

    public boolean setRange(byte min, byte max) {
        Byte minVal = new Byte(min);
        Byte maxVal = new Byte(max);
        return this.setRange((Object)minVal, (Object)maxVal);
    }

    public boolean setRange(char min, char max) {
        Character minVal = new Character(min);
        Character maxVal = new Character(max);
        return this.setRange((Object)minVal, (Object)maxVal);
    }

    public boolean setRange(double min, double max) {
        Double minVal = new Double(min);
        Double maxVal = new Double(max);
        return this.setRange((Object)minVal, (Object)maxVal);
    }

    public boolean setRange(float min, float max) {
        Float minVal = new Float(min);
        Float maxVal = new Float(max);
        return this.setRange((Object)minVal, (Object)maxVal);
    }

    public boolean setRange(String min, String max) {
        return this.setRange((Object)min, (Object)max);
    }

    public boolean addRange(Object minmax) {
        return this.addRange(minmax, minmax);
    }

    public boolean addRange(boolean minmax) {
        return this.addRange(minmax, minmax);
    }

    public boolean addRange(int minmax) {
        return this.addRange(minmax, minmax);
    }

    public boolean addRange(short minmax) {
        return this.addRange(minmax, minmax);
    }

    public boolean addRange(long minmax) {
        return this.addRange(minmax, minmax);
    }

    public boolean addRange(byte minmax) {
        return this.addRange(minmax, minmax);
    }

    public boolean addRange(char minmax) {
        return this.addRange(minmax, minmax);
    }

    public boolean addRange(double minmax) {
        return this.addRange(minmax, minmax);
    }

    public boolean addRange(float minmax) {
        return this.addRange(minmax, minmax);
    }

    public boolean addRange(String minmax) {
        return this.addRange(minmax, minmax);
    }

    public boolean addRange(Object min, Object max) {
        boolean retval = false;

        if (this.typeCheck(min) && this.typeCheck(max)) {
            /*
             * not quite elegant, but it works :)
             */
            Range range = new Range(min, max);
            Range newRange[] = new Range[this.rangeCount + 1];
	    if (rangeCount != 0) {
		System.arraycopy(this.range, 0, newRange, 0, this.rangeCount);
	    }
            newRange[this.rangeCount] = range;
            ++rangeCount;
            this.range = newRange;
            retval = true;
        }
        else {
            retval = false;
        }

        return retval;
    }

    public boolean addRange(boolean min, boolean max) {
        Boolean minVal = new Boolean(min);
        Boolean maxVal = new Boolean(max);

        return this.addRange((Object)minVal, (Object)maxVal);
    }

    public boolean addRange(int min, int max) {
        Integer minVal = new Integer(min);
        Integer maxVal = new Integer(max);

        return this.addRange((Object)minVal, (Object)maxVal);
    }

    public boolean addRange(short min, short max) {
        Short minVal = new Short(min);
        Short maxVal = new Short(max);

        return this.addRange((Object)minVal, (Object)maxVal);
    }

    public boolean addRange(long min, long max) {
        Long minVal = new Long(min);
        Long maxVal = new Long(max);

        return this.addRange((Object)minVal, (Object)maxVal);
    }

    public boolean addRange(byte min, byte max) {
        Byte minVal = new Byte(min);
        Byte maxVal = new Byte(max);

        return this.addRange((Object)minVal, (Object)maxVal);
    }

    public boolean addRange(char min, char max) {
        Character minVal = new Character(min);
        Character maxVal = new Character(max);

        return this.addRange((Object)minVal, (Object)maxVal);
    }

    public boolean addRange(double min, double max) {
        Double minVal = new Double(min);
        Double maxVal = new Double(max);

        return this.addRange((Object)minVal, (Object)maxVal);
    }

    public boolean addRange(float min, float max) {
        Float minVal = new Float(min);
        Float maxVal = new Float(max);

        return this.addRange((Object)minVal, (Object)maxVal);
    }

    public boolean addRange(String min, String max) {
        return this.addRange((Object)min, (Object)max);
    }

    public Type getType(){
	return this.type;
    }

    public int getTypeInfo() {
        return this.type.getTypeInfo();
    }

    public String getTypeName() {
        return this.type.getTypeName();
    }

    public void setType(Type type){
        this.type = type;
        this.range = null;
        this.rangeCount = 0;
        this.initData();
        // this.data = null;
    }

    public void setType(int type){
	this.type = new Type(type);
        this.range = null;
        this.rangeCount = 0;
        this.initData();
        // this.data = null;
    }

    /**
     * Sets the value of data to the given Object. The type of the
     * value is tested and if it does not match the new value is rejected.
     *
     * @param data the value for data, as Object.
     * @return Success of setting the internal data to the given value.
     */

    public boolean setData(Object data) {
        boolean retval = true;

        if (this.typeCheck(data)) {
            this.data = data;
            retval = true;
        }
        else {
            retval = false;
        }
        return retval;
    }

    /**
     * Sets the value of data to the given boolean. The type of the
     * value is tested and if it does not match the new value is rejected.
     *
     * @param data the value for data, as boolean.
     * @return Success of setting the internal data to the given value.
     */

    public boolean setData(boolean data) {
        Boolean newData = new Boolean(data);

        return (this.setData((Object)newData));
    }

    /**
     * Sets the value of data to the given int. The type of the
     * value is tested and if it does not match the new value is rejected.
     *
     * @param data the value for data, as int.
     * @return Success of setting the internal data to the given value.
     */

    public boolean setData(int data) {
        Integer newData = new Integer(data);

        return (this.setData((Object)newData));
    }

    /**
     * Sets the value of data to the given short. The type of the
     * value is tested and if it does not match the new value is rejected.
     *
     * @param data the value for data, as short.
     * @return Success of setting the internal data to the given value.
     */

    public boolean setData(short data) {
        Short newData = new Short(data);

        return (this.setData((Object)newData));
    }

    /**
     * Sets the value of data to the given long. The type of the
     * value is tested and if it does not match the new value is rejected.
     *
     * @param data the value for data, as long.
     * @return Success of setting the internal data to the given value.
     */

    public boolean setData(long data) {
        Long newData = new Long(data);

        return (this.setData((Object)newData));
    }

    /**
     * Sets the value of data to the given byte. The type of the
     * value is tested and if it does not match the new value is rejected.
     *
     * @param data the value for data, as byte.
     * @return Success of setting the internal data to the given value.
     */

    public boolean setData(byte data) {
        Byte newData = new Byte(data);

        return (this.setData((Object)newData));
    }

    /**
     * Sets the value of data to the given char. The type of the
     * value is tested and if it does not match the new value is rejected.
     *
     * @param data the value for data, as char.
     * @return Success of setting the internal data to the given value.
     */

    public boolean setData(char data) {
        Character newData = new Character(data);

        return (this.setData((Object)newData));
    }

    /**
     * Sets the value of data to the given double. The type of the
     * value is tested and if it does not match the new value is rejected.
     *
     * @param data the value for data, as double.
     * @return Success of setting the internal data to the given value.
     */

    public boolean setData(double data) {
        Double newData = new Double(data);

        return (this.setData((Object)newData));
    }

    /**
     * Sets the value of data to the given float. The type of the
     * value is tested and if it does not match the new value is rejected.
     *
     * @param data the value for data, as float.
     * @return Success of setting the internal data to the given value.
     */

    public boolean setData(float data) {
        Float newData = new Float(data);

        return (this.setData((Object)newData));
    }

    /**
     * Sets the value of data to the given String. The type of the
     * value is tested and if it does not match the new value is rejected.
     *
     * @param data the value for data, as String.
     * @return Success of setting the internal data to the given value.
     */

    public boolean setData(String data) {
        String newData = new String(data);

        return (this.setData((Object)newData));
    }

    /**
     * Sets the value of data to the given Object. The type and range of the
     * value is tested and if it does not match the new value is rejected.
     *
     * @param data the value for data, as Object.
     * @return Success of setting the internal data to the given value.
     */

    public boolean setDataChecked(Object data) {
        boolean retval = true;

        if (this.typeCheck(data) && this.inRange(data)) {
            this.data = data;
            retval = true;
        }
        else {
            retval = false;
        }
        return retval;
    }

    /**
     * Sets the value of data to the given boolean. The type and range of the
     * value is tested and if it does not match the new value is rejected.
     *
     * @param data the value for data, as boolean.
     * @return Success of setting the internal data to the given value.
     */

    public boolean setDataChecked(boolean data) {
        Boolean newData = new Boolean(data);

        return this.setDataChecked((Object)newData);
    }

    /**
     * Sets the value of data to the given int. The type and range of the
     * value is tested and if it does not match the new value is rejected.
     *
     * @param data the value for data, as int
     * @return Success of setting the internal data to the given value.
     */

    public boolean setDataChecked(int data) {
        Integer newData = new Integer(data);

        return this.setDataChecked((Object)newData);
    }

    /**
     * Sets the value of data to the given short. The type and range of the
     * value is tested and if it does not match the new value is rejected.
     *
     * @param data the value for data, as short
     * @return Success of setting the internal data to the given value.
     */

    public boolean setDataChecked(short data) {
        Short newData = new Short(data);

        return this.setDataChecked((Object)newData);
    }

    /**
     * Sets the value of data to the given long. The type and range of the
     * value is tested and if it does not match the new value is rejected.
     *
     * @param data the value for data, as long
     * @return Success of setting the internal data to the given value.
     */

    public boolean setDataChecked(long data) {
        Long newData = new Long(data);

        return this.setDataChecked((Object)newData);
    }

    /**
     * Sets the value of data to the given byte. The type and range of the
     * value is tested and if it does not match the new value is rejected.
     *
     * @param data the value for data, as byte
     * @return Success of setting the internal data to the given value.
     */

    public boolean setDataChecked(byte data) {
        Byte newData = new Byte(data);

        return this.setDataChecked((Object)newData);
    }

    /**
     * Sets the value of data to the given char. The type and range of the
     * value is tested and if it does not match the new value is rejected.
     *
     * @param data the value for data, as char
     * @return Success of setting the internal data to the given value.
     */

    public boolean setDataChecked(char data) {
        Character newData = new Character(data);

        return this.setDataChecked((Object)newData);
    }

    /**
     * Sets the value of data to the given double. The type and range of the
     * value is tested and if it does not match the new value is rejected.
     *
     * @param data the value for data, as double
     * @return Success of setting the internal data to the given value.
     */

    public boolean setDataChecked(double data) {
        Double newData = new Double(data);

        return this.setDataChecked((Object)newData);
    }

    /**
     * Sets the value of data to the given float. The type and range of the
     * value is tested and if it does not match the new value is rejected.
     *
     * @param data the value for data, as float
     * @return Success of setting the internal data to the given value.
     */

    public boolean setDataChecked(float data) {
        Float newData = new Float(data);

        return this.setDataChecked((Object)newData);
    }

    /**
     * Sets the value of data to the given String. The type and range of the
     * value is tested and if it does not match the new value is rejected.
     *
     * @param data the value for data, as float
     * @return Success of setting the internal data to the given value.
     */

    public boolean setDataChecked(String data) {
        return this.setDataChecked((Object)data);
    }

    /**
     * Returns the member data as an Object.
     *
     * @return The member data is returned as an Object.
     */

    public Object getData(){
	return data;
    }

    /**
     * Tests, if the given Object is valid regarding the ranges of this Value.
     *
     * @param myobj the Object to test.
     * @return Success of this test.
     */

    public boolean inRange(Object myobj) {
        boolean retval = false;

        // test for valid object
        if (!this.typeCheck(myobj)) {
            retval = false;
        }
        else {
            if (range != null) {
                retval = false;
                // test the ranges
                if (myobj instanceof Boolean) {
                    // special test for Boolean
                    for (int i = 0; i < rangeCount; ++i) {
                        if ( (Boolean)range[i].getMinVal()
                             == (Boolean)range[i].getMaxVal() ) {
                            retval = ( retval ||
                                (Boolean)range[i].getMinVal() == (Boolean)myobj );
                        }
                        else {
                            // If the min and max are different they must be
                            // true and false. Therefore every value is
                            // in this range :)
                            retval = true;
                        }
                    } // end for
                }
                else {
                    // standard test for all other objects
                    for (int i = 0; i < rangeCount; ++i) {
                        if (myobj instanceof Integer) {
                            retval = (retval ||
                                ( (((Integer)myobj).compareTo((Integer)range[i].getMinVal()) >= 0)
                                && (((Integer)myobj).compareTo((Integer)range[i].getMaxVal()) <= 0)
                                ) );
                        } // end if Integer
                        else if (myobj instanceof Short) {
                            retval = (retval ||
                                ( (((Short)myobj).compareTo((Short)range[i].getMinVal()) >= 0)
                                && (((Short)myobj).compareTo((Short)range[i].getMaxVal()) <= 0)
                                ) );
                        } // end if Short
                        else if (myobj instanceof Long) {
                            retval = (retval ||
                                ( (((Long)myobj).compareTo((Long)range[i].getMinVal()) >= 0)
                                && (((Long)myobj).compareTo((Long)range[i].getMaxVal()) <= 0)
                                ) );
                        } // end if Long
                        else if (myobj instanceof Byte) {
                            retval = (retval ||
                                ( (((Byte)myobj).compareTo((Byte)range[i].getMinVal()) >= 0)
                                && (((Byte)myobj).compareTo((Byte)range[i].getMaxVal()) <= 0)
                                ) );
                        } // end if Byte
                        else if (myobj instanceof Character) {
                            retval = (retval ||
                                ( (((Character)myobj).compareTo((Character)range[i].getMinVal()) >= 0)
                                && (((Character)myobj).compareTo((Character)range[i].getMaxVal()) <= 0)
                                ) );
                        } // end if Character
                        else if (myobj instanceof Double) {
                            retval = (retval ||
                                ( (((Double)myobj).compareTo((Double)range[i].getMinVal()) >= 0)
                                && (((Double)myobj).compareTo((Double)range[i].getMaxVal()) <= 0)
                                ) );
                        } // end if Double
                        else if (myobj instanceof Float) {
                            retval = (retval ||
                                ( (((Float)myobj).compareTo((Float)range[i].getMinVal()) >= 0)
                                && (((Float)myobj).compareTo((Float)range[i].getMaxVal()) <= 0)
                                ) );
                        } // end if Float
                        else if (myobj instanceof String) {
                            retval = (retval ||
                                ( (((String)myobj).compareTo((String)range[i].getMinVal()) >= 0)
                                && (((String)myobj).compareTo((String)range[i].getMaxVal()) <= 0)
                                ) );
                        } // end if String
                    } // end for
                } // end if (test for boolean)
            }
            // no ranges to test
            else {
               retval = true;
            } // end if (test the ranges)
        } // end if (test for valid object)
        return retval;
    }

    /**
     * Tests, if the given boolean is valid regarding the ranges of this Value.
     *
     * @param testdata the boolean to test.
     * @return Success of this test.
     */

    public boolean inRange(boolean testdata) {
        Boolean testData = new Boolean(testdata);

        return this.inRange((Object)testData);
    }

    /**
     * Tests, if the given int is valid regarding the ranges of this Value.
     *
     * @param testdata the int to test.
     * @return Success of this test.
     */

    public boolean inRange(int testdata) {
        Integer testData = new Integer(testdata);

        return this.inRange((Object)testData);
    }

    /**
     * Tests, if the given short is valid regarding the ranges of this Value.
     *
     * @param testdata the short to test.
     * @return Success of this test.
     */

    public boolean inRange(short testdata) {
        Short testData = new Short(testdata);

        return this.inRange((Object)testData);
    }

    /**
     * Tests, if the given long is valid regarding the ranges of this Value.
     *
     * @param testdata the long to test.
     * @return Success of this test.
     */

    public boolean inRange(long testdata) {
        Long testData = new Long(testdata);

        return this.inRange((Object)testData);
    }

    /**
     * Tests, if the given byte is valid regarding the ranges of this Value.
     *
     * @param testdata the byte to test.
     * @return Success of this test.
     */

    public boolean inRange(byte testdata) {
        Byte testData = new Byte(testdata);

        return this.inRange((Object)testData);
    }

    /**
     * Tests, if the given char is valid regarding the ranges of this Value.
     *
     * @param testdata the char to test.
     * @return Success of this test.
     */

    public boolean inRange(char testdata) {
        Character testData = new Character(testdata);

        return this.inRange((Object)testData);
    }

    /**
     * Tests, if the given double is valid regarding the ranges of this Value.
     *
     * @param testdata the double to test.
     * @return Success of this test.
     */

    public boolean inRange(double testdata) {
        Double testData = new Double(testdata);

        return this.inRange((Object)testData);
    }

    /**
     * Tests, if the given float is valid regarding the ranges of this Value.
     *
     * @param testdata the float to test.
     * @return Success of this test.
     */

    public boolean inRange(float testdata) {
        Float testData = new Float(testdata);

        return this.inRange((Object)testData);
    }

    /**
     * Tests, if the given String is valid regarding the ranges of this Value.
     *
     * @param testdata the String to test.
     * @return Success of this test.
     */

    public boolean inRange(String testdata) {
        return this.inRange((Object)testdata);
    }

    /**
     * Tests, if the given Object is valid regarding the type of this Value.
     *
     * @param myobj the Object to test.
     * @return Success of this test.
     */

    public boolean typeCheck(Object myobj) {
        boolean retval = false;
        retval = (
             (myobj instanceof Boolean) && (this.getTypeInfo() == Type.BOOLEAN)
          || (myobj instanceof Integer) && (this.getTypeInfo() == Type.INTEGER)
          || (myobj instanceof Short) && (this.getTypeInfo() == Type.SHORT)
          || (myobj instanceof Long) && (this.getTypeInfo() == Type.LONG)
          || (myobj instanceof Byte) && (this.getTypeInfo() == Type.BYTE)
          || (myobj instanceof Character) && (this.getTypeInfo() == Type.CHAR)
          || (myobj instanceof Double) && (this.getTypeInfo() == Type.DOUBLE)
          || (myobj instanceof Float) && (this.getTypeInfo() == Type.FLOAT)
          || (myobj instanceof String) && (this.getTypeInfo() == Type.STRING)
        ); // end of retval

        return retval;
    }

    /**
     * Tests, if the given boolean is valid regarding the type of this Value.
     *
     * @param testdata the boolean to test.
     * @return Success of this test.
     */

    public boolean typeCheck(boolean testdata) {
        return (this.type.getTypeInfo() == Type.BOOLEAN);
    }

    /**
     * Tests, if the given int is valid regarding the type of this Value.
     *
     * @param testdata the int to test.
     * @return Success of this test.
     */

    public boolean typeCheck(int testdata) {
        return (this.type.getTypeInfo() == Type.INTEGER);
    }

    /**
     * Tests, if the given short is valid regarding the type of this Value.
     *
     * @param testdata the short to test.
     * @return Success of this test.
     */

    public boolean typeCheck(short testdata) {
        return (this.type.getTypeInfo() == Type.SHORT);
    }

    /**
     * Tests, if the given long is valid regarding the type of this Value.
     *
     * @param testdata the long to test.
     * @return Success of this test.
     */

    public boolean typeCheck(long testdata) {
        return (this.type.getTypeInfo() == Type.LONG);
    }

    /**
     * Tests, if the given byte is valid regarding the type of this Value.
     *
     * @param testdata the byte to test.
     * @return Success of this test.
     */

    public boolean typeCheck(byte testdata) {
        return (this.type.getTypeInfo() == Type.BYTE);
    }

    /**
     * Tests, if the given char is valid regarding the type of this Value.
     *
     * @param testdata the char to test.
     * @return Success of this test.
     */

    public boolean typeCheck(char testdata) {
        return (this.type.getTypeInfo() == Type.CHAR);
    }

    /**
     * Tests, if the given double is valid regarding the type of this Value.
     *
     * @param testdata the double to test.
     * @return Success of this test.
     */

    public boolean typeCheck(double testdata) {
        return (this.type.getTypeInfo() == Type.DOUBLE);
    }

    /**
     * Tests, if the given float is valid regarding the type of this Value.
     *
     * @param testdata the float to test.
     * @return Success of this test.
     */

    public boolean typeCheck(float testdata) {
        return (this.type.getTypeInfo() == Type.FLOAT);
    }

    /**
     * Tests, if the given String is valid regarding the type of this Value.
     *
     * @param testdata the String to test.
     * @return Success of this test.
     */

    public boolean typeCheck(String testdata) {
        return (this.type.getTypeInfo() == Type.STRING);
    }

    /**
     * Initialises this.data with a standard value of the correct type, or
     * null if not defined.
     */

    private void initData() {
      Object setval = null;
      switch (this.getTypeInfo()) {
        case Type.UNDEFINED : setval = null;
                              break;
        case Type.BOOLEAN   : setval = new Boolean("false");
                              break;
        case Type.INTEGER   : setval = new Integer(0);
                              break;
        case Type.SHORT     : setval = new Short((short)0);
                              break;
        case Type.LONG      : setval = new Long(0);
                              break;
        case Type.BYTE      : setval = new Byte((byte)0);
                              break;
        case Type.CHAR      : setval = new Character('\u0000');
                              break;
        case Type.DOUBLE    : setval = new Double(0.0);
                              break;
        case Type.FLOAT     : setval = new Float(0.0);
                              break;
        case Type.STRING    : setval = new String("");
                              break;
      }
      this.data = setval;
    }

    /**
     * Returns a String representation of this Value.
     * The type in paranthesis and the value are always returned by
     * this function. Existing ranges are listed afterwards.
     *
     *  @return String representation of this Value.
     */

    public String toString() {
        String retval = new String();
	// if u want the type, take the following line :)
	// retval = "(" + this.type.getTypeName() + "): " + this.data;
        if (data != null){
	    retval = this.data.toString();
        }
        if (rangeCount != 0) {
            retval = retval + " |";
            for (int i = 0; i < rangeCount; ++i) {
                retval = retval + " " + range[i].getMinVal() + "/" +
                         range[i].getMaxVal();
            }
        }

        return retval;
    }

    public Object clone() {
        Value retval = null;
        try {
            retval = (Value)super.clone();
            retval.setType((Type)this.type.clone());
            retval.setData(this.data);
            Range[] n_range = new Range[0];
            if (this.range!=null)
                n_range = new Range[this.range.length];
            for (int i = 0; i < n_range.length; i++) {
                n_range[i] = (Range)(this.range[i].clone());
            }
            retval.setRange(n_range);
        }
        catch (CloneNotSupportedException e) {
            System.out.println(e);
        }
        return retval;
    }

    /**
     * Array of ranges for this value.
     */

    private Range[] range;
    private int rangeCount;
    private Type type;
    private Object data;

    public static void main(String[] argv) {
        Value v1, v2;
        v1 = new Value(5.0);
        v2 = (Value)v1.clone();
        System.out.println("v1:\n" + v1 + "\n" + "v2:\n" + v2);
    }
}
