/*
 * DataSetSorter.java  -  This file is part of KEA.
 *
 * Copyright (C) 2002/03 PG 419, Department of Computer Science, 
 * University of Dortmund.          eMail: <peter.senft@udo.edu>
 */

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the KEA License Agreement which is based on 
 * the GNU General Public License as published by the Free Software 
 * Foundation; either version 2 of the License, or (at your option) 
 * any later version.  For details see License.txt.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

package kea.algorithms.common;

import java.util.Comparator;

import kea.Data;
import kea.Type;

public class DataSetSorter implements Comparator, Cloneable {
	private int mode;
    private int fctNo;
	public static final int OBJ_FCT = 1;

/*
    public DataSetSorter(int m, int f) {
        this.mode = m;
        this.fctNo = f;
    }
*/
    /**
     * The one and only constructor for this class.
     * @param f the number of the value in the objective vector. This value
     * will be used for sorting.
     */
    public DataSetSorter(int f) {
        this.fctNo = f;
    }

    /**
     * This method should be used to compare two Data objects.
     * @param data1 a Data object
     * @param data2 the Data object to compare with data1
     * @return the result of comparing the primitive types (like double)
     * encapsulated in the Value objects.
     */
	public int compare(Object data1, Object data2) {
        return compareObjFct(data1, data2);
    }

    /**
     * @see DataSetSorter#compare
     */
	public int compareObjFct(Object data1, Object data2) {
        if ((((Data)data1).getObjVector().length==((Data)data2).getObjVector().length) &&
        fctNo < ((Data)data1).getObjVector().length)
            switch(((Data)data1).getObjVector()[fctNo].getType().getTypeInfo()) {
                case Type.BOOLEAN :
                    if (((Boolean)(((Data)data1).getObjVector()[fctNo].getData())).booleanValue() && ((Boolean)(((Data)data1).getObjVector()[fctNo].getData())).booleanValue())
                        return 0;
                    if (!((Boolean)(((Data)data1).getObjVector()[fctNo].getData())).booleanValue() && !((Boolean)(((Data)data1).getObjVector()[fctNo].getData())).booleanValue())
                        return 0;
                    if (!((Boolean)(((Data)data1).getObjVector()[fctNo].getData())).booleanValue() && ((Boolean)(((Data)data1).getObjVector()[fctNo].getData())).booleanValue())
                        return -1;
                    if (((Boolean)(((Data)data1).getObjVector()[fctNo].getData())).booleanValue() && !((Boolean)(((Data)data1).getObjVector()[fctNo].getData())).booleanValue())
                        return 1;
                //break;
                case Type.BYTE :
                    return ((Byte)(((Data)data1).getObjVector()[fctNo].getData())).compareTo(((Byte)(((Data)data2).getObjVector()[fctNo].getData())));
                //break;
                case Type.CHAR :
                    return ((Character)(((Data)data1).getObjVector()[fctNo].getData())).compareTo(((Character)(((Data)data2).getObjVector()[fctNo].getData())));
                //break;
                case Type.DOUBLE :
                    return ((Double)(((Data)data1).getObjVector()[fctNo].getData())).compareTo(((Double)(((Data)data2).getObjVector()[fctNo].getData())));
                //break;
                case Type.FLOAT :
                    return ((Float)(((Data)data1).getObjVector()[fctNo].getData())).compareTo(((Float)(((Data)data2).getObjVector()[fctNo].getData())));
                //break;
                case Type.INTEGER :
                    return ((Integer)(((Data)data1).getObjVector()[fctNo].getData())).compareTo(((Integer)(((Data)data2).getObjVector()[fctNo].getData())));
                //break;
                case Type.LONG :
                    return ((Long)(((Data)data1).getObjVector()[fctNo].getData())).compareTo(((Long)(((Data)data2).getObjVector()[fctNo].getData())));
                //break;
                case Type.SHORT :
                    return ((Short)(((Data)data1).getObjVector()[fctNo].getData())).compareTo(((Short)(((Data)data2).getObjVector()[fctNo].getData())));
                //break;
                case Type.STRING :
                    return ((String)(((Data)data1).getObjVector()[fctNo].getData())).compareTo(((String)(((Data)data2).getObjVector()[fctNo].getData())));
                //break;
                case Type.UNDEFINED :
                break;
                default : throw new AlgoException("Objective function values are not comparable!");
                //break;
            }
        return 0; // should never be reached!
    }

    /**
     * @param data some object
     * @return true if this object equals the given object else false
     */
    public boolean equals(Object data) {
        return this.equals(data);
    }

    /**
     * @return a copy of this object
     */
    public Object clone() {
        DataSetSorter retval = null;
        try {
            retval = (DataSetSorter)super.clone();
        }
        catch(CloneNotSupportedException e) {
            System.out.println(e);
        }
        return retval;
    }
}
