/*
 * OptDataDomination.java  -  This file is part of KEA.
 *
 * Copyright (C) 2002/03 PG 419, Department of Computer Science, 
 * University of Dortmund.          eMail: <peter.senft@udo.edu>
 */

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the KEA License Agreement which is based on 
 * the GNU General Public License as published by the Free Software 
 * Foundation; either version 2 of the License, or (at your option) 
 * any later version.  For details see License.txt.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

package kea.algorithms.common;

import java.util.Comparator;

import kea.Data;
import kea.Type;
import kea.Value;

/**
 * This class implements the Comparator interface and can be used for comparing
 * Data objects (only). The class also implements the Cloneable interface.
 */
public class OptDataDomination implements Comparator, Cloneable {
	public OptDataDomination(){
    };

    /**
     * This method compares two Data objects. If the arrays in Data do not have
     * the same length or the entries do not match in type, a
     * AlgoComparatorException will be thrown.
     * @param o1 The interface requieres Object but only Data should be used here!
     * @param o2 The interface requieres Object but only Data should be used here!
     * @return -1 if all entries in (Data)o1 smaller than in (Data)o2, returns +1
     * if entries in o1 are all bigger than in o2 and 0 if some are bigger and
     * some are smaller or all are equal.
     */
    public int compare(Object o1, Object o2) {
        int r1 = 0;
        int r2 = 0;

        if ( ((Data)o1).getObjVector().length == ((Data)o1).getObjVector().length ) {
		for (int i=0; i<((Data)o1).getObjVector().length; ++i) {
	        if (((((Data)o1).getObjVector()[i]).getType()).getTypeInfo()==((((Data)o2).getObjVector()[i]).getType()).getTypeInfo())
				switch (((((Data)o1).getObjVector()[i]).getType()).getTypeInfo()) {
			    case Type.BOOLEAN 	:
        	    	if ( !((Boolean)(((Data)o1).getObjVector()[i]).getData()).booleanValue() && ((Boolean)(((Data)o2).getObjVector()[i]).getData()).booleanValue() )
	        	    	++r1;
					if ( ((Boolean)(((Data)o1).getObjVector()[i]).getData()).booleanValue() && !((Boolean)(((Data)o2).getObjVector()[i]).getData()).booleanValue() )
		            	++r2;
	    	        break;
		        case Type.BYTE 		:
    	        	if ( ((Byte)(((Data)o1).getObjVector()[i]).getData()).byteValue() < ((Byte)(((Data)o2).getObjVector()[i]).getData()).byteValue() )
	    	        	++r1;
					if ( ((Byte)(((Data)o1).getObjVector()[i]).getData()).byteValue() > ((Byte)(((Data)o2).getObjVector()[i]).getData()).byteValue() )
		            	++r2;
	    	        break;
		        case Type.CHAR 		:
    	        	if ( ((Character)(((Data)o1).getObjVector()[i]).getData()).charValue() < ((Character)(((Data)o2).getObjVector()[i]).getData()).charValue() )
	    	        	++r1;
					if ( ((Character)(((Data)o1).getObjVector()[i]).getData()).charValue() > ((Character)(((Data)o2).getObjVector()[i]).getData()).charValue() )
	            		++r2;
		            break;
		        case Type.DOUBLE 	:
        	    	if ( ((Double)(((Data)o1).getObjVector()[i]).getData()).doubleValue() < ((Double)(((Data)o2).getObjVector()[i]).getData()).doubleValue() )
	        	    	++r1;
					if ( ((Double)(((Data)o1).getObjVector()[i]).getData()).doubleValue() > ((Double)(((Data)o2).getObjVector()[i]).getData()).doubleValue() )
		            	++r2;
		            break;
	    	    case Type.FLOAT 	:
            		if ( ((Float)(((Data)o1).getObjVector()[i]).getData()).floatValue() < ((Float)(((Data)o2).getObjVector()[i]).getData()).floatValue() )
	            		++r1;
					if ( ((Float)(((Data)o1).getObjVector()[i]).getData()).floatValue() > ((Float)(((Data)o2).getObjVector()[i]).getData()).floatValue() )
		            	++r2;
	    	        break;
	        	case Type.INTEGER 	:
            		if ( ((Integer)(((Data)o1).getObjVector()[i]).getData()).intValue() < ((Integer)(((Data)o2).getObjVector()[i]).getData()).intValue() )
		            	++r1;
					if ( ((Integer)(((Data)o1).getObjVector()[i]).getData()).intValue() > ((Integer)(((Data)o2).getObjVector()[i]).getData()).intValue() )
	    	        	++r2;
	        	    break;
		        case Type.LONG 		:
    	        	if ( ((Long)(((Data)o1).getObjVector()[i]).getData()).longValue() < ((Long)(((Data)o2).getObjVector()[i]).getData()).longValue() )
	    	        	++r1;
					if ( ((Long)(((Data)o1).getObjVector()[i]).getData()).longValue() > ((Long)(((Data)o2).getObjVector()[i]).getData()).longValue() )
	            		++r2;
		            break;
		        case Type.SHORT 	:
        	    	if ( ((Short)(((Data)o1).getObjVector()[i]).getData()).shortValue() < ((Short)(((Data)o2).getObjVector()[i]).getData()).shortValue() )
	        	    	++r1;
					if ( ((Short)(((Data)o1).getObjVector()[i]).getData()).shortValue() > ((Short)(((Data)o2).getObjVector()[i]).getData()).shortValue() )
		            	++r2;
	    	        break;
	        	case Type.STRING 	: // TODO: Testen!! Sind < und > richtig gesetzt? -> Ist der Vergleich korrekt?
	            	if ( ((String)(((Data)o1).getObjVector()[i]).getData()).compareTo( ((String)(((Data)o2).getObjVector()[i]).getData()) ) < 0 )
		            	++r1;
					if ( ((String)(((Data)o1).getObjVector()[i]).getData()).compareTo( ((String)(((Data)o2).getObjVector()[i]).getData()) ) > 0 )
	        	    	++r2;
	            	break;
				//case Type.UNDEFINED	: break;
    			} // end of switch
                else throw new AlgoException("Cannot compare "+ ((((Data)o1).getObjVector()[i]).getType()).getTypeName()
                + " with " + ((((Data)o2).getObjVector()[i]).getType()).getTypeName() );
        	} //end of for
            }//end of if
            else throw new AlgoException("OptDataDomination: Objective function vectors don't  have same length!\n");

        if (r1==0 && r2!=0)
	        return -1; // o1 < o2 : o1 dominated by o2
        if (r1!=0 && r2==0)
            return 1; // o1 > o2 : o1 dominates o2
        else return 0; // non dominated
    }//end of compare

    /**
     * This method returns true if two comparators impose the same ordering on
     * a set.
	 * So this method compares comparators.
     * @param o another Comparator. The interface requires this parameter to be
     * of type Object.
     * @return true if Objects match, false if not.
     */
    public boolean equals(Object o) {
        return this.equals(o);
    }

    /**
     * @return a copy of this Object.
     */
    public Object clone() {
        OptDataDomination retval = null;
        try {
            retval = (OptDataDomination)super.clone();
        }
        catch (CloneNotSupportedException e) {
            System.out.println(e);
        }

        return retval;
    }


    /**
     * Used for testing purposes only!
     */
    public static void main(String[] argv) {
		System.out.println("Starte Test!");
		OptDataDomination odd = new OptDataDomination();
        System.out.println("OptDataDomination-Objekt angelegt");
		Value[][] matrix = new Value[4][5];
		for (int i=0; i<4; ++i) {
			for (int j=0; j<5; ++j) {
                matrix[i][j] = new Value(Math.random());
                matrix[2][j] = new Value(1.25);
            }
        }
		System.out.println("Value-Objekte angelegt");
        Data opData1, opData2, opData3, opData4;
        opData1 = new Data(null,matrix[0]);
        opData2 = new Data(null,matrix[1]);
        opData3 = new Data(null,matrix[2]);
        opData4 = new Data(null,matrix[3]);
        System.out.println("Data-Objekte angelegt");

		for (int i = 0; i<5; ++i) {
	        System.out.print(opData1.getObjVector()[i].getData() + "\t");
			System.out.print(opData2.getObjVector()[i].getData() + "\t");
            System.out.print(opData3.getObjVector()[i].getData() + "\t");
            System.out.print(opData4.getObjVector()[i].getData() + "\n");
        }

        System.out.print("compare(1,2):\t" + odd.compare(opData1,opData2) + "\n");
        System.out.print("compare(2,3):\t" + odd.compare(opData2,opData3) + "\n");
        System.out.print("compare(3,4):\t" + odd.compare(opData3,opData4) + "\n");

		for (int i=0; i<4; ++i) {
			for (int j=0; j<5; ++j) {
                matrix[i][j] = new Value("aa");
                matrix[2][j] = new Value("bb");
            }
        }
        System.out.println("Data-Objekte angelegt");

		for (int i = 0; i<5; ++i) {
	        System.out.print(opData1.getObjVector()[i].getData() + "\t");
			System.out.print(opData2.getObjVector()[i].getData() + "\t");
            System.out.print(opData3.getObjVector()[i].getData() + "\t");
            System.out.print(opData4.getObjVector()[i].getData() + "\n");
        }

        System.out.print("compare(1,2):\t" + odd.compare(opData1,opData2) + "\n");
        System.out.print("compare(2,3):\t" + odd.compare(opData2,opData3) + "\n");
        System.out.print("compare(3,4):\t" + odd.compare(opData3,opData4) + "\n");

		System.out.println("Fertig!");
    } // end of main
}; // end of class
