///*
// * Simplex.java  -  This file is part of KEA.
// *
// * Copyright (C) 2002/03 PG 419, Department of Computer Science, 
// * University of Dortmund.          eMail: <peter.senft@udo.edu>
// */
//
///*
// * This program is free software; you can redistribute it and/or modify
// * it under the terms of the KEA License Agreement which is based on 
// * the GNU General Public License as published by the Free Software 
// * Foundation; either version 2 of the License, or (at your option) 
// * any later version.  For details see License.txt.
// *
// * This program is distributed in the hope that it will be useful,
// * but WITHOUT ANY WARRANTY; without even the implied warranty of
// * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// * GNU General Public License for more details.
// *
// * You should have received a copy of the GNU General Public License
// * along with this program; if not, write to the Free Software
// * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
// * 
// */
//
//package kea.algorithms.deterministic;
//
//import kea.*;
//import kea.algorithms.*;
//import kea.problems.*;
//import java.lang.Math.*;
//import java.util.StringTokenizer;
//import java.util.Vector;
//import kea.algorithms.common.AlgoException;
//
///**
// * Simplex.java is based on the downhill simplex method of Nelder and Mead.
// * Actually, this class is a redesign of the original amoeba/amotry code to make
// * it respect the KEA abstract algorithm interface.
// * Also the author manipulated the code to his liking to achieve his specific goals.
// *
// * @author Norman Welp
// * @version $Revision: 723 $
// */
//public class Simplex extends AbstractAlgorithm {
//	
//	private AbstractProblem problem; // the problem to be solved by simplex
//	private Parameters parameters; // the algo params
//	private Value[] decResults; 
//	private Value[] objResults;
//	private int iterationCount; // max iterations
//	private boolean firstCall = true;
//	private Vector resultData;
//    
//    // internal algo data
//	private int i, ihi, ilo, inhi, j, ndim, nmax;
//	private int mpts, nfunk;
//	private double y[], p[][], rtol, sum,swap, ysave,ytry,psum[], ftol;
//	private double weights[];
//	
//	/**
//     * Constructor that initializes and sets all internal data and algorithm default parameters.
//     */ 
//	public Simplex () {
//		nmax 					= 5000;  // default param for max iterations
//		weights 				= new double[0];
//		y 						= new double[0];
//		p						= new double[0][0];
//		ftol 					= 0.000000000000000000000001; 
//		resultData = new Vector();
//		
//        // setting up the algos default parameters 
//        
//		parameters = new Parameters();
//		Parameter p = new Parameter();
//		p.setName("maxIterations");
//		p.setToolTip("number of maximum iterations");
//		p.setType(new Type(Type.INTEGER));
//		p.setData(new Integer(nmax));
//		p.setRange(1,Integer.MAX_VALUE);
//
//		parameters.addParameter(p);
//
//		p = new Parameter();
//		p.setName("functionWeights");
//		p.setToolTip("for each objective function define a weight here (doubles seperated by comma)");
//		p.setType(new Type(Type.STRING));
//		p.setData("");
//
//		parameters.addParameter(p);
//
//		p = new Parameter();
//		p.setName("startingVertex");
//		p.setToolTip("optional components of a starting vertex (doubles) - remains empty in most cases");
//		p.setType(new Type(Type.STRING));
//		p.setData("");
//		parameters.addParameter(p);
//
//		p = new Parameter();
//		p.setName("fracTol");
//		p.setType(new Type(Type.DOUBLE));
//		p.setData( new Double(ftol));
//		p.setToolTip(" fractional convergence tolerance to be achieved in the function value (double)");
//		parameters.addParameter(p);
//
//		parameters.setMembership(Simplex.class.getName());
//
//		nfunk = 0;
//
//	}
//				
//	/**
//	 * Performs a single iteration of the downhill simplex method.
//	 */
//	public void iterate() {
//		iterationCount++;
//
//		if(firstCall){firstCall=false;computeY();}
//		
//		ilo=0;
//
//		if (y[0]>y[1]) {
//			inhi=1;
//			ihi =0;
//		}
//		else {
//			inhi=0;
//			ihi=1;
//		}
//
//		for(int i=0;i<mpts;i++){
//			if(y[i] <= y[ilo]) ilo=i;
//
//			if(y[i] > y[ihi]){
//				inhi=ihi;
//				ihi=i;
//			}
//			else if (y[i] > y[inhi] && i !=ihi) inhi = i;
//		}
//
//		rtol= (double) (2.0*Math.abs(y[ihi]-y[ilo])/(Math.abs(y[ihi])+Math.abs(y[ilo])));
//
//		if(rtol < ftol){
//			swap 	= y[0];
//			y[0] 	= y[ilo];
//			y[ilo]	= swap;
//			
//			for(int i=0;i<ndim;i++){
//				swap 		= p[0][i];
//				p[0][i] 	= p[ilo][i];
//				p[ilo][i]	= swap;
//				
//			}
//			
//			iterationCount = nmax; // stop further iterations if rtol<ftol
//			return;
//		}
//
//
//		nfunk+=2;
//
//		ytry=amotry(-1.0);
//		if(ytry <= y[ilo]) ytry = amotry(2.0);
//		else
//		if(ytry >= y[inhi]){
//			ysave = y[ihi];
//			ytry=amotry(0.5);
//			if(ytry>=ysave){
//				
//				if(Kea.logger.debugOn()) Kea.logger.logDebug("-------------------------------------------------------",this);
//				resultData = new Vector();
//				for(int i=0;i<mpts;i++){
//					
//					if(i!=ilo){
//						
//						for(int j=0;j<ndim;j++) p[i][j] = psum[j]= (double) 0.5*(p[i][j]+p[ilo][j]);
//
//						Value[] values = new Value[psum.length];
//						for(int v=0;v<psum.length;v++){
//							values[v] = new Value();
//							values[v].setRange(0,Double.MAX_VALUE);
//
//							values[v].setType(new Type(Type.DOUBLE));
//
//							if(psum[v]>((Number)problem.getRanges()[v].getMaxVal()).doubleValue()) {
//								values[v].setData(((Number)problem.getRanges()[v].getMaxVal()).doubleValue());
//								if(Kea.logger.debugOn()) Kea.logger.logDebug("Value out of range: Too HIGH! Set to max range" + values[v]);
//							}
//							else
//								if(psum[v]<((Number)problem.getRanges()[v].getMinVal()).doubleValue()){
//									values[v].setData(((Number)problem.getRanges()[v].getMinVal()).doubleValue());
//									if(Kea.logger.debugOn()) Kea.logger.logDebug("Value out of range: Too LOW! Set to min range" + values[v]);
//							}
//							else {
//									values[v].setData(psum[v]);
//							}
//
//						}
//						decResults = new Value[values.length];
//						for (int q=0; q<values.length;q++){
//							if(Kea.logger.debugOn()) Kea.logger.logDebug("Dec("+q+")" + values[q].getData(), this);
//							decResults[q] = values[q]; 
//						}
//						Value[] results = problem.score(values);
//						
//						objResults = new Value[results.length];
//						for(int r=0;r<results.length;r++){
//							objResults[r] = results[r];
//							if(Kea.logger.debugOn()) Kea.logger.logDebug("Obj("+r+") " + results[r].getData(),this);
//						}
//						
//						resultData.add(new Data(decResults, objResults));
//						
//						double sum = 0;
//
//						for(int r=0;r<results.length;r++){
//							sum = sum + (((Number)results[r].getData()).doubleValue()  * weights[r] );  // weighted sums							
//						}
//
//						y[i]= sum;
//
//					}
//				
//				} // i
//						if(Kea.logger.debugOn()) Kea.logger.logDebug("-------------------------------------------------------",this);
//				
//				
//				nfunk += ndim;
//				calcPSUM();
//			}
//		}
//		else --nfunk;
//
//		Value[] values = new Value[psum.length];
//		for(int v=0;v<psum.length;v++){
//			values[v]= new Value();;
//			values[v].setRange(0,Double.MAX_VALUE);
//			values[v].setType(new Type(Type.DOUBLE));
//
//			if(psum[v]>((Number)problem.getRanges()[v].getMaxVal()).doubleValue()) {
//				values[v].setData(((Number)problem.getRanges()[v].getMaxVal()).doubleValue());
//				if(Kea.logger.debugOn()) Kea.logger.logDebug("Value out of range: Too HIGH! Set to max range" + values[v]);
//			}
//			else
//				if(psum[v]<((Number)problem.getRanges()[v].getMinVal()).doubleValue()){
//					values[v].setData(((Number)problem.getRanges()[v].getMinVal()).doubleValue());
//					if(Kea.logger.debugOn()) Kea.logger.logDebug("Value out of range: Too LOW! Set to min range" + values[v]);
//				}
//			else {
//
//				values[v].setData(psum[v]);
//				
//			}
//		}
//		
//		//Value[] results = problem.repair(values);	
//	
//	}
//	
//	/**
//	 * Extrapolates by a factor fac through the face of the simplex across from high point.
//	 * Replaces the old high point if the new one is better.
//     * @param fac The factor to extrapolate by.
//	 */
//	private double amotry(double fac){
//		
//		double fac1, fac2,ytry,ptry[];
//        ptry=new double[ndim];
//		fac1=(double)(1.0-fac)/ndim;
//       	fac2=(double) (fac1-fac);
//
//		for(int j=0;j<ndim;j++) ptry[j] = psum[j]*fac1-p[ihi][j]*fac2;
//
//			Value[] values = new Value[psum.length];
//			for(int v=0;v<psum.length;v++){
//				values[v]= new Value();;
//				values[v].setRange(0,Double.MAX_VALUE);
//				values[v].setType(new Type(Type.DOUBLE));
//
//				if(psum[v]>((Number)problem.getRanges()[v].getMaxVal()).doubleValue()) {
//					values[v].setData(((Number)problem.getRanges()[v].getMaxVal()).doubleValue());
//					if(Kea.logger.debugOn()) Kea.logger.logDebug("Value out of range: Too HIGH! Set to max range" + values[v]);
//				}
//				else
//					if(psum[v]<((Number)problem.getRanges()[v].getMinVal()).doubleValue()){
//						values[v].setData(((Number)problem.getRanges()[v].getMinVal()).doubleValue());
//						if(Kea.logger.debugOn()) Kea.logger.logDebug("Value out of range: Too LOW! Set to min range" + values[v]);
//					}
//				else {
//
//					values[v].setData(psum[v]);
//				
//				}
//			}
//		   // this.problem.ObjectiveName();
//			Value[] results = problem.score(values);	
//			double sum = 0;
//		
//			for(int r=0;r<results.length;r++){
//				sum = sum + (((Number)results[r].getData()).doubleValue() * weights[r] ); // weighted sums
//			}
//
//			ytry=sum;
//		
//           if(ytry<y[ihi]){
//               y[ihi]=ytry;
//               for(int j=0;j<ndim;j++){
//                   psum[j]+= ptry[j] -p[ihi][j];
//                   p[ihi][j] = ptry[j];
//               }
//           }
//			
//		return (double)ytry;
//	}
//	/**
//	 * Checks whether or not the stop condition is reached.
//	 * @return boolean <b>true</b> if the stop condition is true, <b>false</b> otherwise
//	 */
//    public boolean checkStopCondition() {
//		if(iterationCount<nmax) return false;
//		return true;
//    }
//	/**
//	 * Returns the current internal Paramters of the algorithm
//	 * @see kea.Parameters
//	 * @return  Current Parameters
//	 */
//    public Parameters getParameter() {
//		return parameters;
//     }
//
//
//	/**
//	 * This method sets all the things the algorithm needs to be ready to work.
//	 * @return  <b>true</b> if the setup was successful, <b> false</b> if a problem occured
//	 */
//	public void setup(Parameters params, AbstractProblem problem){
//
//			try{
//				if(params!=null) parameters = params;
//					// MAX ITERATIONS
//					Parameter ps = parameters.getParameterByName("maxIterations");
//					nmax = ((Number)ps.getData()).intValue();
//					
//					// PROBLEM
//					this.problem=problem;
//					ndim = problem.getDimDecisionSpace();
//					//objDim = problem.getDimObjectiveSpace();
//					mpts= ndim+1;
//						
//						
//					// FTOL
//					ps = parameters.getParameterByName("fracTol");
//					ftol = ((Number)ps.getData()).doubleValue();
//					
//					// WEIGHTS
//					weights = new double[problem.getDimObjectiveSpace()];
//					ps = parameters.getParameterByName("functionWeights");
//					
//                    String valueStr = "";
//					if(ps.getData().equals("")){
//						int funcs = weights.length;
//						for(int i=0;i<funcs;i++){
//							weights[i] =  (1/(double)funcs);
//							valueStr = valueStr + weights[i] + ",";
//						}
//
//						ps.setData(valueStr.substring(0,valueStr.length()-1));
//					}
//					else{
//						StringTokenizer splitter  = new StringTokenizer((String)ps.getData(),",");
//						int funcs = weights.length;
//						for(int i=0;i<funcs;i++){
//							String wStr = splitter.nextToken();
//
//							weights[i] = new Double(wStr).doubleValue();
//							
//						}
//						
//					}
//
//					// STARTING VERTEX P0 AND MATRIX Pi
//					p = new double[ndim+1][ndim];
//
//					Range[] rs = problem.getRanges();
//					ps = parameters.getParameterByName("startingVertex");
//					
//
//					if(!ps.getData().equals("")){
//						StringTokenizer splitter  = new StringTokenizer((String)ps.getData(),",");
//						int funcs = weights.length;
//						for(int i=0;i<ndim;i++){
//							String wStr = splitter.nextToken();
//							p[0][i] = new Double(wStr).doubleValue();
//							
//						}
//
//					}
//					else{
//						for(int q=0;q<ndim;q++){
//							p[0][q]=((((Number)rs[q].getMaxVal()).doubleValue()-((Number)rs[q].getMinVal()).doubleValue())/(double)2)+((Number)rs[q].getMinVal()).doubleValue() ;   //p0[q];
//						}
//					}
//
//
//					for(int reihe=1;reihe<ndim+1;reihe++){
//						String str ="";
//						for(int spalte=0;spalte<ndim;spalte++){
//							p[reihe][spalte] = 0;//p0[spalte];
//							if(spalte==reihe-1) p[reihe][spalte] = /*p0[spalte] + */((Number)rs[spalte].getMaxVal()).doubleValue() ;
//						
//							str = str + p[reihe][spalte] + " ";
//						}
//						if(Kea.logger.debugOn()) Kea.logger.logDebug("p[" + reihe + "]:  " + str, this);
//					}
//					
//				parameters.setMembership(Simplex.class.getName());
//				nfunk=0;
//				psum=new double [ndim];
//				calcPSUM();
//				//computeY();
//				
//
//			}
//
//			catch(Exception e){
//				throw new AlgoException("While setting up Simplex");
//                          
//			}
//            
//	}
//	/**
//	 * Computes the an y vector at the beginning of the algorithm. 
//     * Y contains the function values evaluated at the ndim+1 vertices of the starting matrix 
//	 */
//	private void computeY(){
//		y = new double[ndim+1];
//		resultData = new Vector();	
//		for(int o=0;o<ndim+1;o++){
//		Value[] values = new Value[ndim];
//			for(int v=0;v<ndim;v++){
//				values[v] = new Value();
//				values[v].setRange(0,Double.MAX_VALUE);
//				values[v].setType(new Type(Type.DOUBLE));
//				if(p[o][v]>((Number)problem.getRanges()[v].getMaxVal()).doubleValue()) {
//					values[v].setData(((Number)problem.getRanges()[v].getMaxVal()).doubleValue());
//					if(Kea.logger.debugOn()) Kea.logger.logDebug("Value out of range: Too HIGH! Set to max range" + values[v]);
//				}
//				else
//					if(p[o][v]<((Number)problem.getRanges()[v].getMinVal()).doubleValue()){
//						values[v].setData(((Number)problem.getRanges()[v].getMinVal()).doubleValue());
//						if(Kea.logger.debugOn()) Kea.logger.logDebug("Value out of range: Too LOW! Set to min range" + values[v]);
//				}
//				else {
//
//				values[v].setData(p[o][v]);
//				}
//				
//			}
//			decResults = new Value[values.length];
//			for (int q=0; q<values.length;q++){
//				
//				decResults[q] = values[q]; 
//			}
//			Value[] results = new Value[values.length];
//						
//						for (int q=0; q<values.length;q++){
//				
//							results[q] = values[q]; 
//						}
//			
//			try{
//			
//			if(problem.score(values).length > 0){
//			
//			results = problem.score(values);
//			Kea.logger.logError("the length of results is :"+results.length);
//			objResults = new Value[results.length];
//			
//						for(int r=0;r<results.length;r++){
//							objResults[r] = results[r];
//			
//			}
//			}
//			else
//			{
//			
//			objResults = new Value[results.length];
//			
//			for(int r=0;r<results.length;r++){
//				objResults[r] = results[r];
//				
//			}
//			}
//			}
//			catch(Exception e) {	Kea.logger.logError(
//								" simplex SCORE------>problem with the read double value "+ e.toString());
//			}
//			
//						
//			resultData.add(new Data(decResults, objResults));
//			
//			double sum = 0;
//			for(int r=0;r<results.length;r++){
//				sum = sum + (((Number)results[r].getData()).doubleValue() * weights[r] ); // weighted sums
//			}
//			y[o] = sum;
//			
//		}
//		
//		
//	}
//	/**
//	 * NOT IMPLEMENTED IN THIS CLASS.
//     * Method to set the result data of the state of a formerly paused optimization.
//	 * Used to continue the optimization from the last step of the former run. 
//     * @param data The data to continue from
//	 */
//	public void setResults(Data[] data){
//		
//		// NOT IMPLEMENTED FOR SIMPLEX
//		Kea.logger.logError("Method void setResults(Data[] data) not implemented.");
//        throw new AlgoException();
//	}
//
//	/**
//	 * Provides the caller with the current result data produced by the algorithm.
//	 * @return The results stored in an array of kea.Data
//	 * @see kea.Data
//	 */
//
//	public Data[] getResults(){
//		Data[] result = new Data[resultData.size()];
//		
//		for(int i=0;i<resultData.size();i++){
//			result[i] = (Data)resultData.get(i);
//		}			
//				
//		return result;
//	}
//
//
//	/**
//	 * Resturns the current problem to be solved by the simplex algorithm
//	 * @return The current problem to be solved
//	 * @see kea.problems.AbstractProblem
//	 */
//	public AbstractProblem getProblem(){
//		return problem;
//	}
//	/**
//	 * Sets the problem to be solved by the simplex algorithm
//	 * @param problem The problem to be solved
//	 * @see kea.problems.AbstractProblem
//	 */
//
//    public void setProblem(AbstractProblem problem){
//		this.problem=problem;
//  		ndim = problem.getDimObjectiveSpace();
//		mpts= ndim+1;
//		p = new double[ndim+1][ndim];
//
//	}
//
//
// 	/**
//	 * Calculates a sum over the simplex vector matrix
//	 */
//	private void calcPSUM(){
//		for (j=0;j<ndim;j++) {
//					for (sum=0,i=0;i<mpts;i++) {sum +=  p[i][j];}
//					psum[j]=sum;
//					}
//	}
//
//}
/*
 * Simplex.java  -  This file is part of KEA.
 *
 * Copyright (C) 2002/03 PG 419, Department of Computer Science, 
 * University of Dortmund.          eMail: <peter.senft@udo.edu>
 */

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the KEA License Agreement which is based on 
 * the GNU General Public License as published by the Free Software 
 * Foundation; either version 2 of the License, or (at your option) 
 * any later version.  For details see License.txt.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

package kea.algorithms.deterministic;

import java.util.StringTokenizer;
import java.util.Vector;

import kea.Data;
import kea.Kea;
import kea.Parameter;
import kea.Parameters;
import kea.Range;
import kea.Type;
import kea.Value;
import kea.algorithms.AbstractAlgorithm;
import kea.algorithms.common.AlgoException;
import kea.problems.AbstractProblem;

/**
 * Simplex.java is based on the downhill simplex method of Nelder and Mead.
 * Actually, this class is a redesign of the original amoeba/amotry code to make
 * it respect the KEA abstract algorithm interface.
 * Also the author manipulated the code to his liking to achieve his specific goals.
 *
 * @author Norman Welp
 * @version $Revision: 723 $
 */
public class Simplex extends AbstractAlgorithm {
	
	private AbstractProblem problem; // the problem to be solved by simplex
	private Parameters parameters; // the algo params
	private Value[] decResults; 
	private Value[] objResults;
	private int iterationCount; // max iterations
	private boolean firstCall = true;
	private Vector resultData;
    
	// internal algo data
	private int i, ihi, ilo, inhi, j, ndim, nmax;
	private int mpts, nfunk;
	private double y[], p[][], rtol, sum,swap, ysave,ytry,psum[], ftol;
	private double weights[];
	
	/**
	 * Constructor that initializes and sets all internal data and algorithm default parameters.
	 */ 
	public Simplex () {
		nmax 					= 5000;  // default param for max iterations
		weights 				= new double[0];
		y 						= new double[0];
		p						= new double[0][0];
		ftol 					= 0.000000000000000000000001; 
		resultData = new Vector();
		
		// setting up the algos default parameters 
        
		parameters = new Parameters();
		Parameter p = new Parameter();
		p.setName("maxIterations");
		p.setToolTip("number of maximum iterations");
		p.setType(new Type(Type.INTEGER));
		p.setData(new Integer(nmax));
		p.setRange(1,Integer.MAX_VALUE);

		parameters.addParameter(p);

		p = new Parameter();
		p.setName("functionWeights");
		p.setToolTip("for each objective function define a weight here (doubles seperated by comma)");
		p.setType(new Type(Type.STRING));
		p.setData("");

		parameters.addParameter(p);

		p = new Parameter();
		p.setName("startingVertex");
		p.setToolTip("optional components of a starting vertex (doubles) - remains empty in most cases");
		p.setType(new Type(Type.STRING));
		p.setData("");
		parameters.addParameter(p);

		p = new Parameter();
		p.setName("fracTol");
		p.setType(new Type(Type.DOUBLE));
		p.setData( new Double(ftol));
		p.setToolTip(" fractional convergence tolerance to be achieved in the function value (double)");
		parameters.addParameter(p);

		parameters.setMembership(Simplex.class.getName());

		nfunk = 0;

	}
				
	/**
	 * Performs a single iteration of the downhill simplex method.
	 */
	public void iterate() {
		iterationCount++;

		if(firstCall){firstCall=false;computeY();
			this.problem.StartIndividuals(this.decResults);}
		
		ilo=0;

		if (y[0]>y[1]) {
			inhi=1;
			ihi =0;
		}
		else {
			inhi=0;
			ihi=1;
		}

		for(int i=0;i<mpts;i++){
			if(y[i] <= y[ilo]) ilo=i;

			if(y[i] > y[ihi]){
				inhi=ihi;
				ihi=i;
			}
			else if (y[i] > y[inhi] && i !=ihi) inhi = i;
		}

		rtol= (double) (2.0*Math.abs(y[ihi]-y[ilo])/(Math.abs(y[ihi])+Math.abs(y[ilo])));

		if(rtol < ftol){
			swap 	= y[0];
			y[0] 	= y[ilo];
			y[ilo]	= swap;
			
			for(int i=0;i<ndim;i++){
				swap 		= p[0][i];
				p[0][i] 	= p[ilo][i];
				p[ilo][i]	= swap;
				
			}
			
			iterationCount = nmax; // stop further iterations if rtol<ftol
			return;
		}


		nfunk+=2;

		ytry=amotry(-1.0);
		if(ytry <= y[ilo]) ytry = amotry(2.0);
		else
		if(ytry >= y[inhi]){
			ysave = y[ihi];
			ytry=amotry(0.5);
			if(ytry>=ysave){
				
				if(Kea.logger.debugOn()) Kea.logger.logDebug("-------------------------------------------------------",this);
				resultData = new Vector();
				for(int i=0;i<mpts;i++){
					
					if(i!=ilo){
						
						for(int j=0;j<ndim;j++) p[i][j] = psum[j]= (double) 0.5*(p[i][j]+p[ilo][j]);

						Value[] values = new Value[psum.length];
						for(int v=0;v<psum.length;v++){
							values[v] = new Value();
							values[v].setRange(0,Double.MAX_VALUE);

							values[v].setType(new Type(Type.DOUBLE));

							if(psum[v]>((Number)problem.getRanges()[v].getMaxVal()).doubleValue()) {
								values[v].setData(((Number)problem.getRanges()[v].getMaxVal()).doubleValue());
								if(Kea.logger.debugOn()) Kea.logger.logDebug("Value out of range: Too HIGH! Set to max range" + values[v]);
							}
							else
								if(psum[v]<((Number)problem.getRanges()[v].getMinVal()).doubleValue()){
									values[v].setData(((Number)problem.getRanges()[v].getMinVal()).doubleValue());
									if(Kea.logger.debugOn()) Kea.logger.logDebug("Value out of range: Too LOW! Set to min range" + values[v]);
							}
							else {
									values[v].setData(psum[v]);
							}

						}
						decResults = new Value[values.length];
						for (int q=0; q<values.length;q++){
							if(Kea.logger.debugOn()) Kea.logger.logDebug("Dec("+q+")" + values[q].getData(), this);
							decResults[q] = values[q]; 
						}
						Value[] results = problem.repair(values);
						
						objResults = new Value[results.length];
						for(int r=0;r<results.length;r++){
							objResults[r] = results[r];
							if(Kea.logger.debugOn()) Kea.logger.logDebug("Obj("+r+") " + results[r].getData(),this);
						}
						
						resultData.add(new Data(decResults, objResults));
						
						double sum = 0;

						for(int r=0;r<results.length;r++){
							sum = sum + (((Number)results[r].getData()).doubleValue()  * weights[r] );  // weighted sums							
						}

						y[i]= sum;

					}
				
				} // i
						if(Kea.logger.debugOn()) Kea.logger.logDebug("-------------------------------------------------------",this);
				
				
				nfunk += ndim;
				calcPSUM();
			}
		}
		else --nfunk;

		Value[] values = new Value[psum.length];
		for(int v=0;v<psum.length;v++){
			values[v]= new Value();;
			values[v].setRange(0,Double.MAX_VALUE);
			values[v].setType(new Type(Type.DOUBLE));

			if(psum[v]>((Number)problem.getRanges()[v].getMaxVal()).doubleValue()) {
				values[v].setData(((Number)problem.getRanges()[v].getMaxVal()).doubleValue());
				if(Kea.logger.debugOn()) Kea.logger.logDebug("Value out of range: Too HIGH! Set to max range" + values[v]);
			}
			else
				if(psum[v]<((Number)problem.getRanges()[v].getMinVal()).doubleValue()){
					values[v].setData(((Number)problem.getRanges()[v].getMinVal()).doubleValue());
					if(Kea.logger.debugOn()) Kea.logger.logDebug("Value out of range: Too LOW! Set to min range" + values[v]);
				}
			else {

				values[v].setData(psum[v]);
				
			}
		}
		
		//Value[] results = problem.score(values);	
	
	}
	
	/**
	 * Extrapolates by a factor fac through the face of the simplex across from high point.
	 * Replaces the old high point if the new one is better.
	 * @param fac The factor to extrapolate by.
	 */
	private double amotry(double fac){
		
		double fac1, fac2,ytry,ptry[];
		ptry=new double[ndim];
		fac1=(double)(1.0-fac)/ndim;
		fac2=(double) (fac1-fac);

		for(int j=0;j<ndim;j++) ptry[j] = psum[j]*fac1-p[ihi][j]*fac2;

			Value[] values = new Value[psum.length];
			for(int v=0;v<psum.length;v++){
				values[v]= new Value();;
				values[v].setRange(0,Double.MAX_VALUE);
				values[v].setType(new Type(Type.DOUBLE));

				if(psum[v]>((Number)problem.getRanges()[v].getMaxVal()).doubleValue()) {
					values[v].setData(((Number)problem.getRanges()[v].getMaxVal()).doubleValue());
					if(Kea.logger.debugOn()) Kea.logger.logDebug("Value out of range: Too HIGH! Set to max range" + values[v]);
				}
				else
					if(psum[v]<((Number)problem.getRanges()[v].getMinVal()).doubleValue()){
						values[v].setData(((Number)problem.getRanges()[v].getMinVal()).doubleValue());
						if(Kea.logger.debugOn()) Kea.logger.logDebug("Value out of range: Too LOW! Set to min range" + values[v]);
					}
				else {

					values[v].setData(psum[v]);
				
				}
			}
		   // this.problem.ObjectiveName();
			Value[] results = problem.repair(values);	
			double sum = 0;
		
			for(int r=0;r<results.length;r++){
				sum = sum + (((Number)results[r].getData()).doubleValue() * weights[r] ); // weighted sums
			}

			ytry=sum;
		
		   if(ytry<y[ihi]){
			   y[ihi]=ytry;
			   for(int j=0;j<ndim;j++){
				   psum[j]+= ptry[j] -p[ihi][j];
				   p[ihi][j] = ptry[j];
			   }
		   }
			
		return (double)ytry;
	}
	/**
	 * Checks whether or not the stop condition is reached.
	 * @return boolean <b>true</b> if the stop condition is true, <b>false</b> otherwise
	 */
	public boolean checkStopCondition() {
		if(iterationCount<nmax) return false;
		return true;
	}
	/**
	 * Returns the current internal Paramters of the algorithm
	 * @see kea.Parameters
	 * @return  Current Parameters
	 */
	public Parameters getParameter() {
		return parameters;
	 }


	/**
	 * This method sets all the things the algorithm needs to be ready to work.
	 * @return  <b>true</b> if the setup was successful, <b> false</b> if a problem occured
	 */
	public void setup(Parameters params, AbstractProblem problem){

			try{
				if(params!=null) parameters = params;
					// MAX ITERATIONS
					Parameter ps = parameters.getParameterByName("maxIterations");
					nmax = ((Number)ps.getData()).intValue();
					
					// PROBLEM
					this.problem=problem;
					ndim = problem.getDimDecisionSpace();
					//objDim = problem.getDimObjectiveSpace();
					mpts= ndim+1;
						
						
					// FTOL
					ps = parameters.getParameterByName("fracTol");
					ftol = ((Number)ps.getData()).doubleValue();
					
					// WEIGHTS
					weights = new double[problem.getDimDecisionSpace()];
					ps = parameters.getParameterByName("functionWeights");
					
					String valueStr = "";
					if(ps.getData().equals("")){
						int funcs = weights.length;
						for(int i=0;i<funcs;i++){
							weights[i] =  (1/(double)funcs);
							valueStr = valueStr + weights[i] + ",";
						}

						ps.setData(valueStr.substring(0,valueStr.length()-1));
					}
					else{
						StringTokenizer splitter  = new StringTokenizer((String)ps.getData(),",");
						int funcs = weights.length;
						for(int i=0;i<funcs;i++){
							String wStr = splitter.nextToken();

							weights[i] = new Double(wStr).doubleValue();
							
						}
						
					}

					// STARTING VERTEX P0 AND MATRIX Pi
					p = new double[ndim+1][ndim];

					Range[] rs = problem.getRanges();
					ps = parameters.getParameterByName("startingVertex");
					

					if(!ps.getData().equals("")){
						StringTokenizer splitter  = new StringTokenizer((String)ps.getData(),",");
						int funcs = weights.length;
						for(int i=0;i<ndim;i++){
							String wStr = splitter.nextToken();
							p[0][i] = new Double(wStr).doubleValue();
							
						}

					}
					else{
						for(int q=0;q<ndim;q++){
							p[0][q]=((((Number)rs[q].getMaxVal()).doubleValue()-((Number)rs[q].getMinVal()).doubleValue())/(double)2)+((Number)rs[q].getMinVal()).doubleValue() ;   //p0[q];
						}
					}


					for(int reihe=1;reihe<ndim+1;reihe++){
						String str ="";
						for(int spalte=0;spalte<ndim;spalte++){
							p[reihe][spalte] = 0;//p0[spalte];
							if(spalte==reihe-1) p[reihe][spalte] = /*p0[spalte] + */((Number)rs[spalte].getMaxVal()).doubleValue() ;
						
							str = str + p[reihe][spalte] + " ";
						}
						if(Kea.logger.debugOn()) Kea.logger.logDebug("p[" + reihe + "]:  " + str, this);
					}
					
				parameters.setMembership(Simplex.class.getName());
				nfunk=0;
				psum=new double [ndim];
				calcPSUM();
				//computeY();
				

			}

			catch(Exception e){
				throw new AlgoException("While setting up Simplex");
                          
			}
            
	}
	/**
	 * Computes the an y vector at the beginning of the algorithm. 
	 * Y contains the function values evaluated at the ndim+1 vertices of the starting matrix 
	 */
	private void computeY(){
		y = new double[ndim+1];
		resultData = new Vector();	
		for(int o=0;o<ndim+1;o++){
		Value[] values = new Value[ndim];
			for(int v=0;v<ndim;v++){
				values[v] = new Value();
				values[v].setRange(0,Double.MAX_VALUE);
				values[v].setType(new Type(Type.DOUBLE));
				if(p[o][v]>((Number)problem.getRanges()[v].getMaxVal()).doubleValue()) {
					values[v].setData(((Number)problem.getRanges()[v].getMaxVal()).doubleValue());
					if(Kea.logger.debugOn()) Kea.logger.logDebug("Value out of range: Too HIGH! Set to max range" + values[v]);
				}
				else
					if(p[o][v]<((Number)problem.getRanges()[v].getMinVal()).doubleValue()){
						values[v].setData(((Number)problem.getRanges()[v].getMinVal()).doubleValue());
						if(Kea.logger.debugOn()) Kea.logger.logDebug("Value out of range: Too LOW! Set to min range" + values[v]);
				}
				else {

				values[v].setData(p[o][v]);
				}
				
			}
			decResults = new Value[values.length];
			for (int q=0; q<values.length;q++){
				
				decResults[q] = values[q]; 
			}
			Value[] results = new Value[values.length];
						
						for (int q=0; q<values.length;q++){
				
							results[q] = values[q]; 
						}
			
			try{
			
			if(problem.repair(values).length > 0){
			
			results = problem.repair(values);
			Kea.logger.logInfo("the length of results is :"+results.length);
			objResults = new Value[results.length];
			
						for(int r=0;r<results.length;r++){
							objResults[r] = results[r];
			
			}
			}
			else
			{
			
			objResults = new Value[results.length];
			
			for(int r=0;r<results.length;r++){
				objResults[r] = results[r];
				
			}
			}
			}
			catch(Exception e) {	Kea.logger.logError(
								" simplex SCORE------>problem with the read double value "+ e.toString());
			}
			
						
			resultData.add(new Data(decResults, objResults));
			
			double sum = 0;
			
			for(int r=0;r<results.length;r++){
				sum = sum + (((Number)results[r].getData()).doubleValue() * weights[r] ); // weighted sums
			}
			y[o] = sum;
			
		}
		
		
	}
	/**
	 * NOT IMPLEMENTED IN THIS CLASS.
	 * Method to set the result data of the state of a formerly paused optimization.
	 * Used to continue the optimization from the last step of the former run. 
	 * @param data The data to continue from
	 */
	public void setResults(Data[] data){
		
		// NOT IMPLEMENTED FOR SIMPLEX
		Kea.logger.logError("Method void setResults(Data[] data) not implemented.");
		throw new AlgoException();
	}

	/**
	 * Provides the caller with the current result data produced by the algorithm.
	 * @return The results stored in an array of kea.Data
	 * @see kea.Data
	 */

	public Data[] getResults(){
		Data[] result = new Data[resultData.size()];
		
		for(int i=0;i<resultData.size();i++){
			result[i] = (Data)resultData.get(i);
		}			
				
		return result;
	}


	/**
	 * Resturns the current problem to be solved by the simplex algorithm
	 * @return The current problem to be solved
	 * @see kea.problems.AbstractProblem
	 */
	public AbstractProblem getProblem(){
		return problem;
	}
	/**
	 * Sets the problem to be solved by the simplex algorithm
	 * @param problem The problem to be solved
	 * @see kea.problems.AbstractProblem
	 */

	public void setProblem(AbstractProblem problem){
		this.problem=problem;
		ndim = problem.getDimObjectiveSpace();
		mpts= ndim+1;
		p = new double[ndim+1][ndim];

	}


	/**
	 * Calculates a sum over the simplex vector matrix
	 */
	private void calcPSUM(){
		for (j=0;j<ndim;j++) {
					for (sum=0,i=0;i<mpts;i++) {sum +=  p[i][j];}
					psum[j]=sum;
					}
	}

}
