/*
 * Particle.java  -  This file is part of KEA.
 *
 * Copyright (C) 2002/03 PG 419, Department of Computer Science, 
 * University of Dortmund.          eMail: <peter.senft@udo.edu>
 */

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the KEA License Agreement which is based on 
 * the GNU General Public License as published by the Free Software 
 * Foundation; either version 2 of the License, or (at your option) 
 * any later version.  For details see License.txt.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

package kea.algorithms.populationBased.dops;

import java.io.Serializable;

import kea.Data;
import kea.Kea;
import kea.Range;
import kea.Value;
import kea.problems.AbstractProblem;

/**
 * The MOPSO algorithm as proposed in ... <br>
 * This class is the data structure for a particle.
 * 
 * @author Philipp Limbourg
 * @version 0.1.0
 * @see kea.algorithms.AbstractAlgorithm
 */
public class Particle implements Serializable {

	public Particle(
		int dimx,
		int dimy,
		AbstractProblem problem,
		double w,
		double c1,
		double c2,
		boolean oSpace,
		int repsize,
		double[] edgelength) {
		this.problem = problem;
		positionx = new double[dimx];
		positiony = new double[dimy];
		speed = new double[dimx];
		pbestx = new double[dimx];
		pbesty = new double[dimy];
		pbestx = new double[dimx];
		pbesty = new double[dimy];
		denseX = 1;
		denseY = 1;
		this.w = w;
		this.c1 = c1;
		this.c2 = c2;
		this.oSpace = oSpace;
		this.repsize = repsize;
		role = this.keeper;
		Value[] genotyp = vectorify();
		if (oSpace) {
			rep = new NewRep(true, dimy, repsize, edgelength, problem);
		} else {
			rep = new NewRep(false, dimx, repsize, edgelength, problem);
		}
		initialize();
	}
	public double[] fly(double[] speed) {
		if (speed.length != positionx.length) {
			return positionx;
		}
		for (int i = 0; i < speed.length; i++) {
			if (max[i] < positionx[i] + speed[i]
				|| min[i] > positionx[i] + speed[i]) {
				return positionx;
			}
		}
		for (int i = 0; i < speed.length; i++) {
			positionx[i] += speed[i];
		}
		evaluate();
		/*
		if (isDominant (positiony, pbesty)) {
		  for (int i = 0; i < positionx.length; i++) {
		    pbestx[i]=positionx[i];
		  }
		    for (int i = 0; i < positiony.length; i++) {
		      pbesty[i]=positiony[i];
		      }
		
		}
		else if (isDominant (positiony, pbesty)) {
		}
		else {
		    if (Math.random()<0.5) {
		  for (int i = 0; i < positionx.length; i++) {
		    pbestx[i]=positionx[i];
		  }
		    for (int i = 0; i < positiony.length; i++) {
		      pbesty[i]=positiony[i];
		      }
		  }
		}*/
		Particle[] temp = { this };
		rep.updateRep(temp);
		Particle newMax = rep.selectFromRep();
		for (int i = 0; i < positionx.length; i++) {
			pbestx[i] = newMax.positionx[i];
		}
		for (int i = 0; i < positiony.length; i++) {
			pbesty[i] = newMax.positiony[i];
		}

		return positionx;
	}

	public static boolean isDominant(double[] dominant, double[] dominated) {
		boolean ret = false;
		for (int i = 0; i < dominant.length; i++) {
			if (dominant[i] != dominated[i]) {
				ret = true;
			}
		}
		for (int i = 0; i < dominant.length; i++) {
			if (dominant[i] > dominated[i]) {
				return false;
			}

		}
		return ret;
	}
	private void arrify(Value[] phaenotyp) {
		if (phaenotyp == null)
			Kea.logger.logDebug("--> phaenotyp = null");
		else
			for (int i = 0; i < phaenotyp.length; i++) {
				positiony[i] = ((Number) phaenotyp[i].getData()).doubleValue();
			}
	}
	public boolean equals(Object o) {
		Particle p = (Particle) o;
		for (int i = 0; i < positionx.length; i++) {
			if (positionx[i] != p.positionx[i]) {
				return false;
			}
		}
		return true;
	}
	public boolean phequals(Object o) {
		Particle p = (Particle) o;
		for (int i = 0; i < positiony.length; i++) {
			if (positiony[i] != p.positiony[i]) {
				return false;
			}
		}
		return true;
	}
	Value[] vectorify() {
		Value[] genotyp = new Value[positionx.length];
		for (int i = 0; i < positionx.length; i++) {
			genotyp[i] = new Value(positionx[i]);
		}
		return genotyp;
	}
	private void initialize() {
		Value[] genotyp = vectorify();
		Range[] ranges = problem.getRanges();
		max = new double[positionx.length];
		min = new double[positionx.length];
		for (int i = 0; i < positionx.length; i++) {

			max[i] = ((Number) ranges[i].getMaxVal()).doubleValue();
			min[i] = ((Number) ranges[i].getMinVal()).doubleValue();
			positionx[i] = Math.random() * (max[i] - min[i]) + min[i];
			pbestx[i] = positionx[i];

			//this.problem.StartIndividuals(vectorify());
		}
		//evaluate();

	}
	public Particle klone() {
		Particle klon =
			new Particle(
				positionx.length,
				positiony.length,
				problem,
				w,
				c1,
				c2,
				oSpace,
				repsize,
				rep.getEdgelength());
		klon.denseX = denseX;
		klon.denseY = denseY;
		for (int i = 0; i < positionx.length; i++) {
			klon.positionx[i] = positionx[i];
			klon.pbestx[i] = pbestx[i];
			klon.speed[i] = speed[i];
		}
		for (int i = 0; i < positiony.length; i++) {
			klon.positiony[i] = positiony[i];
			klon.pbesty[i] = pbesty[i];

		}

		return klon;
	}
	public String toString() {
		String erg = "x: ";
		for (int i = 0; i < positionx.length; i++) {
			erg += positionx[i] + " ";
		}
		erg += "y: ";
		for (int i = 0; i < positiony.length; i++) {
			erg += positiony[i] + " ";
		}

		return erg;
	}

	public void evaluate() {
		Value[] genotyp = vectorify();
		//         problem.ObjectiveName();
		arrify(problem.score(genotyp));
		//  if (problem.verify(genotyp)) {
		//  arrify(problem.score(genotyp));
		//  }
		//  else {
		for (int i = 0; i < positiony.length; i++) {
			positiony[i] = Double.MAX_VALUE;
		}
		//}
	}
	public Data toData() {
		Value[] x = new Value[positionx.length];
		Value[] y = new Value[positiony.length];
		for (int i = 0; i < x.length; i++) {
			x[i] = new Value(positionx[i]);
		}
		for (int i = 0; i < y.length; i++) {
			y[i] = new Value(positiony[i]);
		}
		Data erg = new Data(x, y);
		return erg;

	}
	public AbstractProblem problem;
	public double[] positionx;
	public double[] positiony;
	public double[] speed;
	public double[] pbestx;
	public double[] pbesty;
	public Repository rep;
	public boolean oSpace;
	public int repsize;
	public double[] max;
	public double[] min;
	public int denseX;
	public int denseY;
	public double e;
	public int role;
	public double w;
	public double c1;
	public double c2;
	public static final int keeper = 0;
	public static final int seeker = 1;
}
