/*
 * Repository.java  -  This file is part of KEA.
 *
 * Copyright (C) 2002/03 PG 419, Department of Computer Science, 
 * University of Dortmund.          eMail: <peter.senft@udo.edu>
 */

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the KEA License Agreement which is based on 
 * the GNU General Public License as published by the Free Software 
 * Foundation; either version 2 of the License, or (at your option) 
 * any later version.  For details see License.txt.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

package kea.algorithms.populationBased.dops;

import java.io.Serializable;
import java.util.Hashtable;
import java.util.Random;
import java.util.Vector;

import kea.problems.AbstractProblem;

public class Repository implements Serializable{
public Repository() {
}
  public Repository(boolean oSpace, int dim, int repsize, double[] edgelength, AbstractProblem problem) {
    this.dim=dim;
    this.edgelength=new double[dim];
    for (int i = 0; i < edgelength.length; i++) {
    this.edgelength[i]=edgelength[i];
    }
    this.oSpace=oSpace;
    this.problem=problem;
    this.r=new Random();
    this.rep=new Vector();
    this.repsize=repsize;
    this.Space=new Hashtable();
  }

  /**
   * Updates the repository after a new iteration step. Tests if a particle is pareto dominant and removes all dominated particles from the repository.
   */
  public boolean updateRep(Particle[] particles) {
    boolean ret=false;
    for (int j=0;j<particles.length;j++) {
    boolean add =true;
    for (int i = 0; i < rep.size(); i++) {
    Particle p=(Particle)rep.get(i);
    if (Particle.isDominant(p.positiony, particles[j].positiony)) {
      add=false;
      i=rep.size();
      break;
      }
      if (Particle.isDominant(particles[j].positiony, p.positiony)) {
      ret=true;
      Particle toKill=(Particle)rep.remove(i);
      Hypercube cube;
      if (oSpace) {
      cube=new Hypercube(edgelength, toKill.positiony);
      }
      else {
      cube=new Hypercube(edgelength, toKill.positionx);
      }
      Vector v=(Vector)Space.remove(cube);
      v.remove(toKill);
      for (i = 0; i < v.size(); i++) {
	if (oSpace) {
        ((Particle)v.get(i)).denseY=v.size();
	}
	else {
        ((Particle)v.get(i)).denseX=v.size();
	}
      }
      Space.put(cube, v);
      i--;
      }
      }
      if (add==true) {
      addToRep(particles[j].klone(), particles);
      }
    }
    return ret;
  }

    /**
     * Selects a random particle from the repository used in the calculation of the new particle positions. Selected by roulette wheel selection. Fitness is 10/Particles in the same hypercube. This is for a well-distributed pareto front.
     */
    public Particle selectFromRep() {
      double p=r.nextDouble();
      int sumfit=0;
      for (int i = 0; i < rep.size(); i++) {
	if (oSpace) {
	sumfit+= 10/((Particle)(rep.get(i))).denseY;
	}
	else {
	sumfit+= 10/((Particle)(rep.get(i))).denseX;
	}
      }
      p=p*sumfit;
      sumfit=0;
      for (int i = 0; i < rep.size(); i++) {
	if (oSpace) {
	sumfit+=10/((Particle)(rep.get(i))).denseY;
	}
	else {
	sumfit+=10/((Particle)(rep.get(i))).denseX;
	}
	if (sumfit>=p) {
        return (Particle)rep.get(i);
	}
      }
      Particle part=new Particle(dim, problem.getDimObjectiveSpace(), problem, 0.4, 1, 1, oSpace, repsize, edgelength);
      part.evaluate();
      return part;
    }
    /**
     * Selects a particle to remove from the repository. Selected by roulette wheel selection. Fitness is Sum of Particles in the same hypercube. This is for a well-distributed pareto front.
     */
    public void removeFromRep(Particle[] particles) {
      double p=r.nextDouble();
      double sumfit=0;
      for (int i = 0; i < rep.size(); i++) {
      if (oSpace) {
      double basefit=((Particle)(rep.get(i))).denseY;
	sumfit+=Math.pow(2,basefit);
      }
      else {
      double basefit=((Particle)(rep.get(i))).denseX;
	sumfit+=Math.pow(2,basefit);
      }
      }
      p=p*sumfit;
      sumfit=0;
      for (int i = 0; i < rep.size(); i++) {
      if (oSpace) {
      double basefit=((Particle)(rep.get(i))).denseY;
	sumfit+=Math.pow(2,basefit);
      }
      else {
      double basefit=((Particle)(rep.get(i))).denseX;
	sumfit+=Math.pow(2,basefit);
      }
	if (sumfit>=p) {
          Particle toKill=(Particle)rep.remove(i);
Hypercube cube;
      if (oSpace) {
cube=new Hypercube(edgelength, toKill.positiony);
      }
      else {
cube=new Hypercube(edgelength, toKill.positionx);
      }
	  Vector v=(Vector)Space.remove(cube);
	  v.remove(toKill);
	  for (i = 0; i < v.size(); i++) {
	          if (oSpace) {
	    ((Particle)v.get(i)).denseY=v.size();
	          }
		  else {
	    ((Particle)v.get(i)).denseX=v.size();
		  }
	  }
	  Space.put(cube, v);
	  return;
	}
      }
    }
    /**
     * Adds a new dominant particle to the repository. If the size surpasses its maximum, a random particle is removed.
     */
    public void addToRep(Particle p, Particle[] particles) {
	rep.add(p);
	Hypercube cube;
	if (oSpace) {
	cube=new Hypercube(edgelength, p.positiony);
	}
	else {
	cube=new Hypercube(edgelength, p.positionx);
	}
	Vector v=(Vector)Space.remove(cube);
	if (v==null) {
	  v=new Vector();
	}
	v.add(p);
	for (int i = 0; i < v.size(); i++) {
	  if (oSpace) {
	  ((Particle)v.get(i)).denseY=v.size();
	  }
	  else {
	  ((Particle)v.get(i)).denseX=v.size();
	  }
	}
	Space.put(cube, v);
	if (rep.size()>repsize) {
	  removeFromRep(particles);
	}
    }
    public void setEdgelength (double[] edgelength) {
    this.edgelength=edgelength;
    updateDense();
    }
    public double[] getEdgelength() {
      return edgelength;
    }

    public void updateDense() {
    this.Space=new Hashtable();
    for (int i = 0; i < rep.size();i++) {
        Particle p=(Particle)rep.get(i);
	   	Hypercube cube;
	if (oSpace) {
   	cube=new Hypercube(edgelength, p.positiony);
	}
	else {
   	cube=new Hypercube(edgelength, p.positionx);
	}
	Vector v=(Vector)Space.remove(cube);
	if (v==null) {
	  v=new Vector();
	}
	v.add(p);
	for (int j = 0; j < v.size(); j++) {
	if (oSpace) {
	  ((Particle)v.get(j)).denseY=v.size();
	}
	else {
	   ((Particle)v.get(j)).denseX=v.size();
	}
	}
	Space.put(cube, v);
 }

}



    boolean oSpace;
    private Hashtable Space;
    private int dim;
    private int repsize;
    public Vector rep;
    private double[] edgelength;
    private Random r;
    private AbstractProblem problem;

}
