/*
 * NSGA2Individual.java  -  This file is part of KEA.
 *
 * Copyright (C) 2002/03 PG 419, Department of Computer Science, 
 * University of Dortmund.          eMail: <peter.senft@udo.edu>
 */

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the KEA License Agreement which is based on 
 * the GNU General Public License as published by the Free Software 
 * Foundation; either version 2 of the License, or (at your option) 
 * any later version.  For details see License.txt.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

package kea.algorithms.populationBased.nsga2;

import java.io.Serializable;
import java.util.Vector;

import kea.Data;
import kea.Value;

public class NSGA2Individual extends Data implements Serializable {
  private double error;
  public Vector dominationList;
  public int rank;
  public int domCount;
  public double cuboidLength = 0.0;
//*****************************************************************************/

  public NSGA2Individual(int gLength, int fLength) {
   // data = new Data();
    setDecVector(new Value[gLength]);
    setObjVector(new Value[fLength]);
    dominationList = new Vector();
  }

  public NSGA2Individual(Value[] nGene, Value[] nFitness) {
   // data = new Data();
    setDecVector(nGene);
    setObjVector(nFitness);
    dominationList = new Vector();
  }

  public boolean checkDomination(NSGA2Individual individual) {
    int x = 0;
    int g = 0;
    boolean b = false;
    Value[] ownFitness = getFitness();
    Value[] testFitness = individual.getFitness();

    for (int i=0;i<ownFitness.length;i++) {
      if (((Number)ownFitness[i].getData()).doubleValue() <= ((Number)testFitness[i].getData()).doubleValue()) {
        x++;
        if ((((Number)ownFitness[i].getData()).doubleValue() - ((Number)testFitness[i].getData()).doubleValue()) < 1e-7) {
          g++;
        }
      }
      if (x == ownFitness.length) {
        if (g == ownFitness.length) {
          b = false;
        }
        else {
          b = true;
        }
      }
      else {
        b = false;
      }
    }//for i
    return b;
  }

  public void setFitness(Value[] newFitness){
    setObjVector(newFitness);
  }

  public void setFitness(int nr, Value newFitness){
    objVector[nr].setData(newFitness);
  }

  public Value getFitness(int nr) {
    return objVector[nr];
  }

  public Value[] getFitness() {
    return getObjVector();
  }

   public Value getGene(int nr) {
    return getDecVector()[nr];
  }

  public Value[] getGene() {
    return getDecVector();
  }
}
