/*
 * NSGA2Population.java  -  This file is part of KEA.
 *
 * Copyright (C) 2002/03 PG 419, Department of Computer Science, 
 * University of Dortmund.          eMail: <peter.senft@udo.edu>
 */

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the KEA License Agreement which is based on 
 * the GNU General Public License as published by the Free Software 
 * Foundation; either version 2 of the License, or (at your option) 
 * any later version.  For details see License.txt.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

package kea.algorithms.populationBased.nsga2;

import java.io.Serializable;
import java.util.Arrays;
import java.util.Vector;

import kea.Kea;
import kea.Value;
import kea.algorithms.common.DataSet;
import kea.algorithms.common.DataSetSorter;
import kea.algorithms.common.OptDataDomination;

/**
 * @author Torsten Kohlen
 * @version 0.1.0
 */
public class NSGA2Population extends DataSet implements Serializable {
  public int maxrank;
  public Vector fronts;
  //DataSet[] front;

//*****************************************************************************/

  public NSGA2Population() {
    super();
    fronts = new Vector();
  }

  public NSGA2Population(int size) {
    super(size);
    fronts = new Vector();
  }

//*****************************************************************************/

  private int checkDomination(NSGA2Individual indA, NSGA2Individual indB){
    int equal = 0;
    int count = 0;
    int fNumber = indA.getFitness().length;
    int retVal = 0;

    // System.out.print(fNumber);
    for (int i=0;i < fNumber;i++){
      double a = ((Number)((Value)indA.getFitness(i)).getData()).doubleValue();
      double b = ((Number)((Value)indB.getFitness(i)).getData()).doubleValue();
    //  System.out.println(i+"a:"+a);
   //   System.out.println(i+"b:"+b);

      if (a <= b) {
      //System.out.println("diff:"+Math.abs(a-b));
      //System.out.println("-------");
        if (Math.abs(a - b) < 1e-7) {
          equal++;
        }
        count++;
      }
    }
  //  System.out.println(equal);
   // System.out.println(count);
  //  System.out.println("===========");

    if (count == fNumber) {//indA war immer kleiner
      if (equal == fNumber) {//alle waren "gleich"
        retVal = 0; //beide gleich!
      }
      else {
        retVal = 1;//a dominiert
      }
    }
    else {
      equal = 0;
      count = 0;

      for (int i=0;i < fNumber;i++){
        double a = ((Number)((Value)indA.getFitness(count)).getData()).doubleValue();
        double b = ((Number)((Value)indB.getFitness(count)).getData()).doubleValue();
       // System.out.println(i+"a:"+a);
       // System.out.println(i+"b:"+b);
        if (a >= b) {
          if (Math.abs(a - b) < 1e-7) {
            equal++;
          }
          count++;
        }
      }

   // System.out.println(equal);
   // System.out.println(count);
   // System.out.println("+++++++++++++++++++++++");
      if (count == fNumber) {
        if (equal == fNumber) {
          retVal = 0;
        }
        else {
          retVal = -1;
        }
      }
    }
    //System.out.println(retVal);
    return retVal;
  }

  /**
   *
   */
  public void rank() {
    NSGA2Individual indi,indj, indh;
    int frontCount;
    OptDataDomination comp = new OptDataDomination();

    fronts = new Vector();
    for (int i=0;i<data.length;i++){
      ((NSGA2Individual)data[i]).rank = 0;
      ((NSGA2Individual)data[i]).domCount = 0;
      ((NSGA2Individual)data[i]).dominationList = new Vector();
      ((NSGA2Individual)data[i]).cuboidLength = 0;
    }
    int x =0;

    //searching Individuals for the first front, one must exist at least
    Vector firstFront = new Vector();
    for (int i=0;i<data.length;i++){
      indi = (NSGA2Individual)data[i];

      //test i with all other Individuals for domination
      for (int j=0;j<data.length;j++){
        if (i != j) {
           indj = (NSGA2Individual)data[j];
           //test domination
           switch (/*checkDomination(indi,indj)){//*/comp.compare(indi,indj)) {
              case -1: //i is domiated by j
                indi.domCount++;
                break;
              case 0://neither domination
                break;
              case 1: //i domiates j
                indi.dominationList.add(new Integer(j));
                break;
           }
        }
      }//for j

      // test if i is nondominated, if so the it is put in first front
      if (indi.domCount == 0) {
        indi.rank = 1;
        firstFront.add(indi);
        x++;
      }
    }//for i

    fronts.add(firstFront);
    //System.out.println("1. hat: " + firstFront.size());

    frontCount = 0;
    while (!((Vector)fronts.get(frontCount)).isEmpty()) {
      //build next front
      Vector newFront = new Vector();
      Vector upperFront = (Vector)fronts.get(frontCount);

      for (int i=0;i<upperFront.size();i++){
        indi = (NSGA2Individual)upperFront.get(i);
        for (int j=0;j<indi.dominationList.size();j++){
          indj = (NSGA2Individual)data[(((Integer)indi.dominationList.get(j)).intValue())];
          indj.domCount--;
          //test if j belongs to next front
          if (indj.domCount == 0) {
            indj.rank = frontCount + 2;
            newFront.add(indj);
            x++;
          }
        }//j
      }// i
      fronts.add(newFront);
      //System.out.println((frontCount+2) + ". hat: " +newFront.size());
      frontCount++;
    }//while
   // System.out.println("AddCount: " + x);
    //remove empty last front
    fronts.remove(fronts.lastElement());
    maxrank = frontCount;

 /**   x = 0;
    for (int k=0;k<this.fronts.size();k++){
      System.out.print((k+1) + ". Front hat: ");
      x =  x + ((Vector)this.fronts.get(k)).size();
      System.out.println(((Vector)this.fronts.get(k)).size());
    }
    System.out.println("Summe: " + x);*/
  }//rank


  public void assignCrodwingDistance(){
    Vector cFront;
    NSGA2Individual ind1, ind2, ind3;

    for (int r=0;r<maxrank;r++){ //gehe ueber alle Fronten/Raenge
      cFront = (Vector)fronts.get(r);//hole die r-te Front
      int IndInFront = cFront.size(); //#Individuen in der r-ten Front
      for (int j=0;j<data[0].getObjVector().length;j++){//gehe ueber alle Zielfunktionen
        //sortieren des Array nach den j-ten fitnesswerten Absteigend
        cFront = frontSort(cFront,j);
        for (int i=0;i<IndInFront;i++){
          ind1 = (NSGA2Individual)cFront.get(i);
          if ((i == 0) || (i == (IndInFront-1))){//Randwerte besonders behandeln
            ind1.cuboidLength = Double.MAX_VALUE;
          }
          else{
            ind2 = (NSGA2Individual)cFront.get(i-1);
            ind3 = (NSGA2Individual)cFront.get(i+1);
            ind1.cuboidLength = ind1.cuboidLength + Math.abs(((Double)ind3.getFitness(j).getData()).doubleValue() - ((Double)ind2.getFitness(j).getData()).doubleValue());
          }
        }
      }
    }
    for (int j=0;j<data.length;j++){
    //  System.out.print(j+", " + ((NSGA2Individual)data[j]).cuboidLength + "\n");
    }
  }

  private Vector frontSort(Vector sfront, int fitIndex){
    NSGA2Individual[] sortFront = new NSGA2Individual[sfront.size()];

 /*   for (int i=0;i<sfront.size();i++){
      double x = ((Number)((NSGA2Individual)sfront.get(i)).getFitness(fitIndex).getData()).doubleValue();
      System.out.println("org :" +x);
    }*/

    for (int i=0;i<sfront.size();i++){
      sortFront[i] = (NSGA2Individual)sfront.get(i);
    }

    Arrays.sort(sortFront, new DataSetSorter(fitIndex));

    /*for (int i=0;i<sfront.size();i++){
      System.out.println(((Number)sortFront[i].getFitness(fitIndex).getData()).doubleValue());
    }
    System.out.println("------------");*/

    sfront.clear();
    for (int i=0;i<sortFront.length;i++){
      sfront.add(sortFront[i]);
    }
    return sfront;
  }


//  public void crowdingSort(int frontIndex){
//    NSGA2Individual[] sortFront = new NSGA2Individual[((Vector)fronts.get(frontIndex)).size()];
//    Vector sfront = (Vector)fronts.get(frontIndex);
//    int x = sfront.size();
//
//   // System.out.println(x);
//    if (x > 2) {
//      for (int j=0;j<x;j++){
//        for (int i=1;i<(x-1);i++){
//            NSGA2Individual member1 = (NSGA2Individual)sfront.get(i);
//            NSGA2Individual member2 = (NSGA2Individual)sfront.get(i+1);
//            if (member1.cuboidLength < member2.cuboidLength) {
//              sfront.set(i,member2);
//              sfront.set(i+1,member1);
//            }
//        }
//      }
//    }
//    fronts.set(frontIndex, sfront);
//  }
//

public void crowdingSort(int frontIndex){
  //NSGA2Individual[] sortFront = new NSGA2Individual[((Vector)fronts.get(frontIndex)).size()];
  Vector sfront = (Vector)fronts.get(frontIndex);
  int n = sfront.size();

 // System.out.println(x);
  if (n > 1) {
	for (int j=1;j<n;j++){
	  for (int i=0;i<(n-1);i++){
		  NSGA2Individual member1 = (NSGA2Individual)sfront.get(i);
		  NSGA2Individual member2 = (NSGA2Individual)sfront.get(i+1);
		  if (member1.cuboidLength < member2.cuboidLength) {
			sfront.set(i,member2);
			sfront.set(i+1,member1);
		  }
	  }
	}
  }
  fronts.set(frontIndex, sfront);
}




  public void addIndividual(Value[] geneVal, Value[] fitVal){
 //   NSGA2Individual newInd = new NSGA2Individual(geneVal, fitVal);
 //   individuals.add(newInd);
  }

  public NSGA2Individual getIndividual(int nr) {
    if (nr < data.length){
      return (NSGA2Individual)data[nr];
    }
    else {
      if (Kea.logger.debugOn()){
        Kea.logger.logError("Zugriffsfehler, gewuenschtes Individuum nicht vorhanden", this);
      }
      return null;
    }
  }

  public void setIndividual(int nr, NSGA2Individual newInd){
   // individuals.set(nr,newInd);
  }

  public void printInfo(){
    for (int i=0;i<data.length;i++){
      System.out.print(i+"\t" );
      for (int j=0;j<data[i].getDecVector().length;j++){
        System.out.print(data[i].getDecVector()[j].toString() + "\t");
      }
      System.out.print("||\t" );
      for (int j=0;j<data[i].getObjVector().length;j++){
        System.out.print(data[i].getObjVector()[j].toString() + "\t");
      }
      System.out.print(((NSGA2Individual)data[i]).rank + "\t");
      System.out.print(((NSGA2Individual)data[i]).cuboidLength + "\t");
      System.out.print("\n" );
    }
  }

  public void printRanking(){
    int c = 1;

    for (int r=0;r<fronts.size();r++){
      System.out.println("--------------------------------------------------------------------------------");
      System.out.println("Individuals in Rank Nr. " + (r+1));
      System.out.println("--------------------------------------------------------------------------------");
      Vector currentFront = (Vector)fronts.get(r);

      for (int j=0;j<currentFront.size();j++){
        NSGA2Individual indi = (NSGA2Individual)currentFront.get(j);
        System.out.print(c+"\t" );
        c++;
        //System.out.print(i+"\t" );
        for (int d=0;d<indi.getDecVector().length;d++){
          System.out.print(indi.getDecVector()[d].toString() + "\t");
        }
        System.out.print("||\t" );
        for (int d=0;d<indi.getObjVector().length;d++){
          System.out.print(indi.getObjVector()[d].toString() + "\t");
        }
        System.out.print(indi.rank + "\t");
        System.out.print(indi.cuboidLength + "\t");
        System.out.print("\n" );
      }
    }
    System.out.print("\n" );
  }

  public int getSize(){
    return data.length;
  }

}
