/*
 * SPEA2.java  -  This file is part of KEA.
 *
 * Copyright (C) 2002/03 PG 419, Department of Computer Science, 
 * University of Dortmund.          eMail: <peter.senft@udo.edu>
 */

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the KEA License Agreement which is based on 
 * the GNU General Public License as published by the Free Software 
 * Foundation; either version 2 of the License, or (at your option) 
 * any later version.  For details see License.txt.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

package kea.algorithms.populationBased.spea2;

import java.io.Serializable;

import kea.Data;
import kea.Parameter;
import kea.Parameters;
import kea.Range;
import kea.Type;
import kea.algorithms.AbstractAlgorithm;
import kea.algorithms.common.AlgoException;
import kea.problems.AbstractProblem;


/**
 * Implementation of the algorithm SPEA2. <br>
 *
 * For further information see:
 * SPEA2: Improving the Strength Pareto Evolutionary Algorithm for
 *        Multiobjective Optimization.
 * E. Zitzler, M. Laumanns, and L. Thiele, September 2001
 * 
 * @author Peter Senft
 * @version $Revision: 723 $
 */
//public class SPEA2 extends AbstractAlgorithm  implements Serializable {
//
//    /**
//     * Constructor for SPEA2-object
//     */
//    public SPEA2() {
//      this.setDefaultParameter();
//    }
//
//    /**
//     * Gives the set of parameters needed to run a problem properly.
//     *
//     * @see Parameters
//     * @return The set of Prameters, combined in the Parameters-class.
//     */
//    public Parameters getParameter() {
//      Parameters retval = new Parameters();
//
//      Parameter myparam;
//      // Set Membership
//      retval.setMembership(this.getClass().getName());
//      // Create all Parameters
//      myparam = new Parameter("Generations", new Integer(this.generations), "Number of generations to run (minimum 1)");
//      myparam.addRange(1, Integer.MAX_VALUE);
//      retval.addParameter(myparam);
//      myparam = new Parameter("ArchiveSize", new Integer(this.archiveSize), "Size of the archive (minimum 2)");
//      myparam.addRange(2, Integer.MAX_VALUE);
//      retval.addParameter(myparam);
//      myparam = new Parameter("PopulationSize", new Integer(this.populationSize), "Size of the population (minimum 2)");
//      myparam.addRange(2, Integer.MAX_VALUE);
//      retval.addParameter(myparam);
//      myparam = new Parameter("OptimizationType", new Integer(this.optimizationType), "Type of optimization (0 - minimium, 1 - maximum)");
//      myparam.addRange(0, 1);
//      retval.addParameter(myparam);
//      retval.addParameter(new Parameter("RandomSeed", new Long(this.randomSeed), "Randomseed for random number generator"));
//      myparam = new Parameter("RandomSeedType", new Integer(this.randomSeedType), "0 - RandomSeed set to systemtime, 1 - RandomSeed set by parameter");
//      myparam.addRange(0, 1);
//      retval.addParameter(myparam);
//      retval.addParameter(new Parameter("k", new Integer(this.theK), "k for environmental selection (minimum: 1, maximium: sqrt(ArchiveSize + PopulationSize)"));
//      myparam = new Parameter("RecombinationType", new Integer(this.recombinationType), "Type of recombination (0 - ES, 1 - GA)");
//      myparam.addRange(0, 1);
//      retval.addParameter(myparam);
//      myparam = new Parameter("pRecombination", new Double(this.pRecombination), "Probabilty of recombination for type 1 (0.0 - 1.0)");
//      myparam.addRange(0.0, 1.0);
//      retval.addParameter(myparam);
//      myparam = new Parameter("MutationType", new Integer(this.mutationType), "Type of mutation (0 - ES (Don't use!), 1 - GA, 2 - gaussian, 3 - random, 4 - random addition)");
//      myparam.addRange(0, 4);
//      retval.addParameter(myparam);
//      myparam = new Parameter("pMutation", new Double(this.pMutation), "Probabilty of mutation (0.0 - 1.0)");
//      myparam.addRange(0.0, 1.0);
//      retval.addParameter(myparam);
//      retval.addParameter(new Parameter("UpperBound", new Double(this.upperBound), "Upper bound for mutation type 0, 2, 4 (must be greater than LowerBound)"));
//      retval.addParameter(new Parameter("LowerBound", new Double(this.lowerBound), "Lower bound for mutation type 0, 2, 4 (must be less than GreaterBound)"));
//      myparam = new Parameter("MutationScale", new Integer(this.mutationScale), "Scale for mutation type 2 (5 - 200)");
//      myparam.addRange(5, 200);
//      retval.addParameter(myparam);
//      retval.addParameter(new Parameter("SigmaAdaption", new Boolean(this.sigmaAdaption), "Use sigma adaption (true/false)"));
//      retval.addParameter(new Parameter("OneSigma", new Boolean(this.oneSigma), "Use 1 sigma per individual (true/false)"));
//      myparam = new Parameter("SigmaInit", new Double(this.sigmaInit), "Initial value for sigma (0.0 - 1.0)");
//      myparam.addRange(0.0, 1.0);
//      retval.addParameter(myparam);
//      myparam = new Parameter("SigmaMin", new Double(this.sigmaMin), "Minimal value for sigma during computation (0.0 - 1.0)");
//      myparam.addRange(0.0, 1.0);
//      retval.addParameter(myparam);
//
//      return retval;
//    }
//
//    /**
//     * Sets the Parameter-object to default values
//     */
//    private void setDefaultParameter() {
//      this.theK = 1;
//      this.iterationCount = 1;
//      this.generations = 1000;
//      this.archiveSize = 100;
//      this.populationSize = 100;
//      this.pMutation = 0.2;
//      this.mutationType = 1;
//      this.mutationScale = 20;
//      this.upperBound = 1000.0;
//      this.lowerBound = -1000.0;
//      this.sigmaAdaption = true;
//      this.oneSigma = false;
//      this.sigmaInit = 1.0;
//      this.sigmaMin = 0.00001;
//      this.recombinationType = 1;
//      this.pRecombination = 0.8;
//      this.randomSeed = System.currentTimeMillis();
//      this.randomSeedType = 0;
//      this.speaRandom = new SPEA2Random(this.randomSeed);
//    }
//
//    /**
//     * This method prepares the algorithm to run properly.
//     *
//     * @param para The Parameters a user set to run this algorithm. If para is
//     *             <b>null</b> the algorithm must use its default Parameters.
//     *        prob The Problem this algorithm has to solve.
//     */
//    public void setup(Parameters para, AbstractProblem prob) {
//      boolean retval = true;
//      String errorMessage = "";
//
//      if (para == null) {
//        this.setDefaultParameter();
//      }
//      else {
//        this.iterationCount = 1;
//        this.generations = ((Integer)para.getParameterByName("Generations").getData()).intValue();
//        this.archiveSize = ((Integer)para.getParameterByName("ArchiveSize").getData()).intValue();
//        this.populationSize = ((Integer)para.getParameterByName("PopulationSize").getData()).intValue();
//        this.optimizationType = ((Integer)para.getParameterByName("OptimizationType").getData()).intValue();
//        this.randomSeedType = ((Integer)para.getParameterByName("RandomSeedType").getData()).intValue();
//        if (this.randomSeedType == 0) {
//          this.randomSeed = System.currentTimeMillis();
//        }
//        else {
//          this.randomSeed = ((Long)para.getParameterByName("RandomSeed").getData()).longValue();
//        }
//        this.speaRandom = new SPEA2Random(this.randomSeed);
//	this.theK = ((Integer)para.getParameterByName("k").getData()).intValue();
//        this.recombinationType = ((Integer)para.getParameterByName("RecombinationType").getData()).intValue();
//        this.pRecombination = ((Double)para.getParameterByName("pRecombination").getData()).doubleValue();
//        this.mutationType= ((Integer)para.getParameterByName("MutationType").getData()).intValue();
//        this.pMutation = ((Double)para.getParameterByName("pMutation").getData()).doubleValue();
//        this.mutationScale = ((Integer)para.getParameterByName("MutationScale").getData()).intValue();
//        this.upperBound = ((Double)para.getParameterByName("UpperBound").getData()).doubleValue();
//        this.lowerBound = ((Double)para.getParameterByName("LowerBound").getData()).doubleValue();
//        this.sigmaAdaption = ((Boolean)para.getParameterByName("SigmaAdaption").getData()).booleanValue();
//        this.oneSigma = ((Boolean)para.getParameterByName("OneSigma").getData()).booleanValue();
//        this.sigmaInit = ((Double)para.getParameterByName("SigmaInit").getData()).doubleValue();
//        this.sigmaMin = ((Double)para.getParameterByName("SigmaMin").getData()).doubleValue();
//      }
//
//      // Check for correct parameters
//
//      // Check if problem is null
//      this.problem = prob;
//      if (this.problem == null) {
//        errorMessage += "SPEA2.setup : given problem is null.\n";
//        retval = false;
//      }
//      // Population size to archive size.
//      if (this.archiveSize > this.populationSize) {
//        errorMessage += "SPEA2.setup : archive size is greater than population.\n";
//        retval = false;
//      }
//      // Check size of population
//      if (this.populationSize < 2) {
//        errorMessage += "SPEA2.setup : population size minimum is 2.\n";
//        retval = false;
//      }
//      // Check size of archive
//      if (this.archiveSize < 2) {
//        errorMessage += "SPEA2.setup : archive size minimum is 2.\n";
//        retval = false;
//      }
//      // Number of generations
//      if (this.generations < 1) {
//        errorMessage += "SPEA2.setup : number of genereation minimum is 1.\n";
//        retval = false;
//      }
//      // Optimization type in range
//      if ((this.optimizationType < 0) || (this.optimizationType > 1)) {
//        errorMessage += "SPEA2.setup : optimization type is 0 or 1.\n";
//        retval = false;
//      }
//      // Randomseed type in range
//      if ((this.randomSeedType < 0) || (this.randomSeedType > 1)) {
//        errorMessage += "SPEA2.setup : type of the randomseed is 0 or 1.\n";
//        retval = false;
//      }
//      // Recombination type in range
//      if ((this.recombinationType < 0) || (this.recombinationType > 1)) {
//        errorMessage += "SPEA2.setup : type of recombination is 0 or 1.\n";
//        retval = false;
//      }
//      // pRecombination in range
//      if ((this.pRecombination < 0.0) || (this.pRecombination > 1.0)) {
//        errorMessage += "SPEA2.setup : probability of recombination is between 0.0 and 1.0.\n";
//        retval = false;
//      }
//      // Mutation type in range
//      if ((this.mutationType < 0) || (this.mutationType > 4)) {
//        errorMessage += "SPEA2.setup : type of mutation is between 0 and 4.\n";
//        retval = false;
//      }
//      // pMutation in range
//      if ((this.pMutation < 0.0) || (this.pMutation > 1.0)) {
//        errorMessage += "SPEA2.setup : probability of mutation is between 0.0 and 1.0.\n";
//        retval = false;
//      }
//      // Mutation scale in range
//      if ((this.mutationScale < 20) || (this.mutationScale > 500)) {
//        errorMessage += "SPEA2.setup : mutation scale is between 20 and 500.\n";
//        retval = false;
//      }
//      // Bounds don't cross
//      if (this.upperBound <= this.lowerBound) {
//        errorMessage += "SPEA2.setup : upper bound is less or equal to lower bound.\n";
//        retval = false;
//      }
//      // SigmaInit in range
//      if ((this.sigmaInit < 0.0) || (this.sigmaInit > 1.0)) {
//        errorMessage += "SPEA2.setup : initial sigma value is between 0.0 and 1.0.\n";
//        retval = false;
//      }
//      // SigmaMin in range
//      if ((this.sigmaMin < 0.0) || (this.sigmaMin > 1.0)) {
//        errorMessage += "SPEA2.setup : minimum sigma value is between 0.0 and 1.0.\n";
//        retval = false;
//      }
//      // Check k
//      if (this.theK < 1) {
//        errorMessage += "SPEA2.setup : k has minimum 1.\n";
//        retval = false;
//      }
//      if (this.theK > Math.sqrt(this.archiveSize + this.populationSize)) {
//        errorMessage += "SPEA2.setup : k has maximum sqrt(ArchiveSize + PopulationSize).\n";
//        retval = false;
//      }
//      // if nothing fails initialize all needed objects
//      if (retval) {
//        this.initialize();
//        // Check for correct types in problem
//        for (int i = 0; i < this.decTypes.length; ++i) {
//          if (!this.decTypes[i].isValid()) {
//            errorMessage += "SPEA2.setup : Wrong type in decision space(Dim: " + i + " / Type:" + this.decTypes[i].getTypeName() + ").\n";
//            retval = false;
//          }
//        }
//      }
//      if (!retval) {
//        errorMessage = "=======================================================\n" + errorMessage;
//        errorMessage = "\nSPEA2.setup : Setup failed due to the following errors:\n" + errorMessage;
//        errorMessage = errorMessage + "Please correct these errors first.\n";
//        throw new AlgoException(errorMessage);
//      }
//    }
//
//    /**
//     * Test for stop the run of SPEA2.
//     */
//    public boolean checkStopCondition() {
//      return this.iterationCount > this.generations;
//    }
//
//    /**
//     * Performs one iteration (i.e. one generation) of SPEA2
//     */
//    public void iterate() {
//      this.population.score();
//      this.archive.score();
//      this.calcFitness();
//      this.environmentalSelection();
//      this.matingSelection();
//      this.variation();
//      this.iterationCount++;
//    }
//
//    /**
//     * Gibt zu speichernde Werte des Algos zur\uFFFDck, die nicht durch getParameter()
//     * zu erhalten sind. (Archiv, aktuelle Population + Fitnesswerte ...)
//     */
//    public Data[] getResults() {
//      return this.archive.getResults();
//    }
//
//    /**
//     * Setzt Werte des Algos, die nicht durch setParameter() zu setzen sind.
//     * (Archiv, aktuelle Population + Fitnesswerte... ). Wichtig z.B. zum Fortsetzen
//     * eines unterbrochenen (geladenen) Laufes.
//     */
//    public void setResults(Data[] myDataSet) {
//      this.population.setResults(myDataSet);
//    }
//
//    /**
//     * Returns the given problem or NULL if not initialized
//     */
//    public AbstractProblem getProblem() {
//      return this.problem;
//    }
//
//    /**
//     * Mating
//     */
//    private void matingSelection() {
//      // Mating
//      int firstIndex = 0;
//      int secondIndex = 0;
//      int thirdIndex = 0;
//      int fourthIndex = 0;
//      int mother = 0;
//      int father = 0;
//      SPEA2Individual[] pool = new SPEA2Individual[this.populationSize];
//
//      if (this.recombinationType == 0) {
//        // ES-version
//        // get 4 individuals and mate the best 2 of them
//        for (int i = 0; i < this.populationSize; ++i) {
//          // make a binary tournament between 2 of them and produce 2 parents
//          firstIndex = this.speaRandom.uniform0Max(this.archiveSize);
//          secondIndex = this.speaRandom.uniform0Max(this.archiveSize);
//          while (firstIndex == secondIndex) {
//            secondIndex = (secondIndex + 1) % pool.length;
//          }
//          thirdIndex = this.speaRandom.uniform0Max(this.archiveSize);
//          while ((firstIndex == thirdIndex) || (secondIndex == thirdIndex)) {
//            thirdIndex = (thirdIndex + 1) % pool.length;
//          }
//          fourthIndex = this.speaRandom.uniform0Max(this.archiveSize);
//          while (firstIndex == fourthIndex || (secondIndex == fourthIndex) || (thirdIndex == fourthIndex)) {
//            fourthIndex = (fourthIndex + 1) % pool.length;
//          }
//          if (this.population.getAt(firstIndex).getFitness() < this.population.getAt(secondIndex).getFitness()) {
//            mother = firstIndex;
//          }
//          else {
//            mother = secondIndex;
//          }
//          if (this.population.getAt(thirdIndex).getFitness() < this.population.getAt(fourthIndex).getFitness()) {
//            father = thirdIndex;
//          }
//          else {
//            father = fourthIndex;
//          }
//          if (father == mother) {
//            System.out.println("Mama = Papa");
//          }
//          // mate 2 of them and produce one child
//          pool[i] = this.population.getAt(father).mateWith(this.population.getAt(mother));
//        }
//      }
//      else {
//        // GA-version
//        // get 4 individuals and mate the best 2 of them
//        for (int i = 0; i < this.populationSize / 2; ++i) {
//          // make a binary tournament between 2 of them and produce 2 parents
//          SPEA2Individual[] children = new SPEA2Individual[2];
//          children[0] = null;
//          children[1] = null;
//
//          firstIndex = this.speaRandom.uniform0Max(this.archiveSize);
//          secondIndex = this.speaRandom.uniform0Max(this.archiveSize);
//          while ((firstIndex == secondIndex)) {
//            secondIndex = (secondIndex + 1) % pool.length;
//          }
//          thirdIndex = this.speaRandom.uniform0Max(this.archiveSize);
//          while ((firstIndex == thirdIndex) || (secondIndex == thirdIndex)) {
//            thirdIndex = (thirdIndex + 1) % pool.length;
//          }
//          fourthIndex = this.speaRandom.uniform0Max(this.archiveSize);
//          while (firstIndex == fourthIndex || (secondIndex == fourthIndex) || (thirdIndex == fourthIndex)) {
//            fourthIndex = (fourthIndex + 1) % pool.length;
//          }
//          if (this.population.getAt(firstIndex).getFitness() < this.population.getAt(secondIndex).getFitness()) {
//            mother = firstIndex;
//          }
//          else {
//            mother = secondIndex;
//          }
//          if (this.population.getAt(thirdIndex).getFitness() < this.population.getAt(fourthIndex).getFitness()) {
//            father = thirdIndex;
//          }
//          else {
//            father = fourthIndex;
//          }
//          if (mother == father) {
//            System.out.println("Mama = Papa");
//          }
//          // mate 2 of them and produce one child
//          children = this.population.getAt(father).mateWith(this.pRecombination, this.population.getAt(mother));
//          pool[(i * 2)] = children[0];
//          pool[((i * 2) + 1)] = children[1];
//        }
//      } // end of recombinations
//
//      // copy pool to population
//      for (int i = 0; i < this.populationSize; ++i) {
//        this.population.setAt(i, pool[i]);
//      }
//    }
//
//    /**
//     * Variation
//     * - Mutation
//     */
//    private void variation() {
//      this.population.mutate(this.pMutation, this.mutationType, this.upperBound, this.lowerBound, this.mutationScale);
//    }
//
//    /**
//     * Sort the given array by fitness
//     */
//    private SPEA2Individual[] sort(SPEA2Individual[] toSort) {
//      SPEA2Individual swapper = null;
//
//      for (int i =0; i < toSort.length - 1; ++i) {
//        for (int j = i + 1; j < toSort.length; ++j) {
//          if (toSort[i].getFitness() > toSort[j].getFitness()) {
//            swapper = toSort[i];
//            toSort[i] = toSort[j];
//            toSort[j] = swapper;
//          }
//        }
//      }
//
//      return toSort;
//    }
//
//    /**
//     * Environmental selection
//     */
//    private void environmentalSelection() {
//      SPEA2Population interimArchive = new SPEA2Population(this.archiveSize + this.populationSize, this.speaRandom, this.decTypes, this.objTypes, this.decRanges, this.problem);
//      SPEA2Individual[] allind = new SPEA2Individual[(this.population.getSize() + this.archive.getSize())];
//      int nondominated = 0;
//      int count = 0;
//      int swapInt = 0;
//      double swapDouble = 0.0;
//      boolean[] selected = new boolean[this.archiveSize + this.populationSize];
//      int[] k_nearest = new int[this.archiveSize + this.populationSize];
//      int[] sorted = new int[this.archiveSize + this.populationSize];
//      double[] distance = new double[this.archiveSize + this.populationSize];
//      double[] distance_tmp = new double[this.archiveSize + this.populationSize];
//
//      // calculate k
//      int k = this.theK;
//
//      // get all individuals
//      // get population
//      for (int i = 0; i < this.population.getSize(); ++i) {
//        allind[i] = (SPEA2Individual)this.population.getAt(i).clone();
//      }
//      // get archive
//      for (int i = 0; i < this.archive.getSize(); ++i) {
//        // get real copies of individuals...
//        allind[(this.population.getSize() + i)] = (SPEA2Individual)this.archive.getAt(i).clone();
//      }
//      // sort allind
//      allind = this.sort(allind);
//      // copy alle nondominated individual to the archive (F(i) < 1)
//      nondominated = 0;
//      for (int i = 0; i < allind.length; ++i) {
//        if (allind[nondominated].getFitness() < 1) {
//          interimArchive.setAt(nondominated, allind[i]);
//          ++nondominated;
//        }
//      }
//      if (nondominated != this.archiveSize) {
//        if (nondominated < this.archiveSize) {
//          // fill up archive with best individuals
//          count = 0;
//          while ((nondominated < this.archiveSize) && (count < allind.length)) {
//            if (allind[count].getFitness() >= 1) {
//              interimArchive.setAt(nondominated, allind[count]);
//              ++nondominated;
//            }
//            ++count;
//          }
//        }
//        else {
//          // truncate archive to given size;
//          // get all individuals from interim archive to allind
//          for (int i = 0; i < nondominated; ++i) {
//            allind[i] = interimArchive.getAt(i);
//          }
//          // initialize selected, distance, k_nearest array
//          for (int i = 0; i < selected.length; ++i) {
//            if (i < nondominated) {
//              selected[i] = true;
//            }
//            else {
//              selected[i] = false;
//            }
//            distance[i] = Double.MAX_VALUE;
//            k_nearest[i] = i;
//          }
//          while (nondominated > this.archiveSize) {
//            // calculate distances and index of k-nearest for every individual
//            for (int i = 0; i < allind.length; ++i) {
//              // initialize temporary distances
//              for (int j = 0; j < allind.length; ++j) {
//                distance_tmp[j] = Double.MAX_VALUE;
//                sorted[j] = j;
//              }
//              // calculate distances from i to all other individuals
//              if (selected[i]) {
//                for (int j = 0; j < allind.length; ++j) {
//                  if (selected[j]) {
//                    distance_tmp[j] = allind[i].distanceTo(allind[j]);
//                  }
//                }
//                // find out, which is the k-nearest and set the entire value
//                for (int j = 0; j < allind.length - 1; ++j) {
//                  for (int l = j + 1; l < allind.length; ++l) {
//                    if (distance_tmp[j] > distance_tmp[l]) {
//                      // swap distance
//                      swapDouble = distance_tmp[j];
//                      distance_tmp[j] = distance_tmp[l];
//                      distance_tmp[l] = swapDouble;
//                      // swap positions
//                      swapInt = sorted[j];
//                      sorted[j] = sorted[l];
//                      sorted[l] = swapInt;
//                    }
//                  }
//                }
//              }
//              // add 1 to k because of the distance 0 to myself
//              k_nearest[i] = sorted[k + 1];
//              distance[i] = distance_tmp[k + 1];
//            } // end of calculating all distances and indeces of k-nearest
//            // sort the k-nearest according to the distance
//            for (int j = 0; j < allind.length; ++j) {
//              sorted[j] = j;
//            }
//            for (int j = 0; j < allind.length - 1; ++j) {
//              for (int l = j + 1; l < allind.length; ++l) {
//                if (distance[j] > distance[l]) {
//                  // swap distance
//                  swapDouble = distance[j];
//                  distance[j] = distance[l];
//                  distance[l] = swapDouble;
//                  // swap positions
//                  swapInt = sorted[j];
//                  sorted[j] = sorted[l];
//                  sorted[l] = swapInt;
//                }
//              }
//            }
//            // select individual and remove it from selected list
//            selected[k_nearest[sorted[0]]] = false;
//            --nondominated;
//          }
//          // get all individuals from allind to interim archive
//          count = 0;
//          for (int i = 0; i < allind.length; ++i) {
//            if (selected[i]) {
//              interimArchive.setAt(count, allind[i]);
//              ++count;
//            }
//          }
//        }
//      }
//      // copy interim archive to archive
//      for (int i = 0; i < this.archiveSize; ++i) {
//        this.archive.setAt(i, interimArchive.getAt(i));
//      }
//    }
//
//    /**
//     * Fitness calculation in the population an the archive
//     */
//    private void calcFitness() {
//      // 0 - minimum / 1 - maximum
//      // nullzeiger des Values abfangen!!
//
//      SPEA2Individual[] allind = new SPEA2Individual[(this.population.getSize() + this.archive.getSize())];
//      // get all individuals
//      // get population
//      for (int i = 0; i < this.population.getSize(); ++i) {
//        allind[i] = this.population.getAt(i);
//      }
//      // get archive
//      for (int i = 0; i < this.archive.getSize(); ++i) {
//        allind[(this.population.getSize() + i)] = this.archive.getAt(i);
//      }
//
//      // initialize all values (strength, raw, density and fitness)
//      for (int i = 0; i < allind.length; ++i) {
//        allind[i].setStrength(0);
//        allind[i].setRaw(0);
//        allind[i].setDensity(0.0);
//        allind[i].setFitness(0.0);
//      }
//
//      // calculate strength
//      for (int i = 0; i < allind.length; ++i) {
//        // compare with all other
//        for (int j = 0; j < allind.length; ++j) {
//          if ((allind[i].getResults().objVector[0].getData() != null) && (allind[j].getResults().objVector[0].getData() != null)) {
//            if (allind[i].dominates(this.optimizationType, allind[j])) {
//              allind[i].setStrength(allind[i].getStrength() + 1);
//            }
//          }
//          if ((allind[j].getResults().objVector[0].getData() == null) && (allind[i].getResults().objVector[0].getData() != null)) {
//              allind[i].setStrength(allind[i].getStrength() + 1);
//          }
//        }
//      }
//
//      // calculate raw fitness
//      for (int i = 0; i < allind.length; ++i ) {
//        for (int j = 0; j < allind.length; ++j ) {
//          if ((allind[i].getResults().objVector[0].getData() != null) && (allind[j].getResults().objVector[0].getData() != null)) {
//            if (allind[i].dominates(this.optimizationType, allind[j])) {
//              allind[j].setRaw(allind[j].getRaw() + allind[i].getStrength());
//            }
//          }
//          if (allind[j].getResults().objVector[0].getData() == null && (allind[i].getResults().objVector[0].getData() != null)) {
//            allind[j].setRaw(allind[j].getRaw() + allind[i].getStrength());
//          }
//        }
//      }
//
//      // calculate density
//      // calculate k
//      // int k = new Double(Math.sqrt(allind.length)).intValue();
//      int k = this.theK;
//      double[] distances = new double[allind.length];
//      // calculate density for each individual
//      for (int i = 0; i < allind.length; ++i) {
//        // calculate distances to individual i
//        for (int j = 0; j < allind.length; ++j) {
//          distances[j] = allind[i].distanceTo(allind[j]);
//        }
//        // sort distances
//        Arrays.sort(distances);
//        allind[i].setDensity( (1 / (distances[k] + 2.0)) );
//      }
//
//      // calculate fitness
//      for (int i = 0; i < allind.length; ++i) {
//        allind[i].setFitness(allind[i].getRaw() + allind[i].getDensity());
//      }
//
//      // return individuals to population and archive
//      // set population
//      for (int i = 0; i < this.population.getSize(); ++i) {
//        this.population.setAt(i, allind[i]);
//      }
//      // set archive
//      for (int i = 0; i < this.archive.getSize(); ++i) {
//        this.archive.setAt(i, allind[(this.population.getSize() + i)]);
//      }
//    }
//
//    /**
//     * Initializes the algorithm...
//     */
//    private void initialize() {
//      // get type from problem and set it in the population
//      this.decTypes = this.problem.getDecTypes();
//      this.objTypes = this.problem.getObjTypes();
//      this.decRanges = this.problem.getRanges();
//      // generate startpopulation (random initialized)
//      this.population = new SPEA2Population(this.populationSize, this.speaRandom, this.decTypes, this.objTypes, this.decRanges, this.problem);
//      this.population.initRandom();
//      // generate empty archive
//      this.archive = new SPEA2Population(this.archiveSize, this.speaRandom, this.decTypes, this.objTypes, this.decRanges, this.problem);
//    }
//
//    /*
//     * Member-variables
//     */
//
//    /**
//     * locally stored random seed
//     */
//    private long randomSeed;
//
//    /**
//     * The problem we want to evolve.
//     */
//    private AbstractProblem problem;
//
//    /*
//     * Counts the nunber of calls of AbstractAlgorithm.iterate().
//     */
//    private int iterationCount;
//
//    /*
//     * SPEA2 Parameters
//     */
//    private int randomSeedType;
//    private int theK;
//    private double pRecombination;
//    private int recombinationType;
//    private double pMutation;
//    private int mutationType;
//    private int mutationScale;
//    private double upperBound;
//    private double lowerBound;
//    private boolean sigmaAdaption;
//    private boolean oneSigma;
//    private double sigmaInit;
//    private double sigmaMin;
//    private int generations;
//    private int archiveSize;
//    private int populationSize;
//    private int optimizationType;
//    private SPEA2Population population;
//    private SPEA2Population archive;
//    private SPEA2Population matingPool;
//    private SPEA2Random speaRandom;
//    private Type[] decTypes;
//    private Type[] objTypes;
//    private Range[] decRanges;
//}




public class SPEA2 extends AbstractAlgorithm  implements Serializable {

	/**
	 * Constructor for SPEA2-object
	 */
	public SPEA2() {
	  this.setDefaultParameter();
	}

	/**
	 * Gives the set of parameters needed to run a problem properly.
	 *
	 * @see Parameters
	 * @return The set of Prameters, combined in the Parameters-class.
	 */
	public Parameters getParameter() {
	  Parameters retval = new Parameters();

	  Parameter myparam;
	  // Set Membership
	  retval.setMembership(this.getClass().getName());
	  // Create all Parameters
	  myparam = new Parameter("Generations", new Integer(this.generations), "Number of generations to run (minimum 1)");
	  myparam.addRange(1, Integer.MAX_VALUE);
	  retval.addParameter(myparam);
	  myparam = new Parameter("ArchiveSize", new Integer(this.archiveSize), "Size of the archive (minimum 2)");
	  myparam.addRange(2, Integer.MAX_VALUE);
	  retval.addParameter(myparam);
	  myparam = new Parameter("PopulationSize", new Integer(this.populationSize), "Size of the population (minimum 2)");
	  myparam.addRange(2, Integer.MAX_VALUE);
	  retval.addParameter(myparam);
	  myparam = new Parameter("OptimizationType", new Integer(this.optimizationType), "Type of optimization (0 - minimium, 1 - maximum)");
	  myparam.addRange(0, 1);
	  retval.addParameter(myparam);
	  retval.addParameter(new Parameter("RandomSeed", new Long(this.randomSeed), "Randomseed for random number generator"));
	  myparam = new Parameter("RandomSeedType", new Integer(this.randomSeedType), "0 - RandomSeed set to systemtime, 1 - RandomSeed set by parameter");
	  myparam.addRange(0, 1);
	  retval.addParameter(myparam);
	  retval.addParameter(new Parameter("k", new Integer(this.theK), "k for environmental selection (minimum: 1, maximium: sqrt(ArchiveSize + PopulationSize)"));
	  myparam = new Parameter("RecombinationType", new Integer(this.recombinationType), "Type of recombination (0 - ES, 1 - GA)");
	  myparam.addRange(0, 1);
	  retval.addParameter(myparam);
	  myparam = new Parameter("pRecombination", new Double(this.pRecombination), "Probabilty of recombination for type 1 (0.0 - 1.0)");
	  myparam.addRange(0.0, 1.0);
	  retval.addParameter(myparam);
	  myparam = new Parameter("MutationType", new Integer(this.mutationType), "Type of mutation (0 - ES (Don't use!), 1 - GA, 2 - gaussian, 3 - random, 4 - random addition)");
	  myparam.addRange(0, 4);
	  retval.addParameter(myparam);
	  myparam = new Parameter("pMutation", new Double(this.pMutation), "Probabilty of mutation (0.0 - 1.0)");
	  myparam.addRange(0.0, 1.0);
	  retval.addParameter(myparam);
	  retval.addParameter(new Parameter("UpperBound", new Double(this.upperBound), "Upper bound for mutation type 0, 2, 4 (must be greater than LowerBound)"));
	  retval.addParameter(new Parameter("LowerBound", new Double(this.lowerBound), "Lower bound for mutation type 0, 2, 4 (must be less than GreaterBound)"));
	  myparam = new Parameter("MutationScale", new Integer(this.mutationScale), "Scale for mutation type 2 (5 - 200)");
	  myparam.addRange(5, 200);
	  retval.addParameter(myparam);
	  retval.addParameter(new Parameter("SigmaAdaption", new Boolean(this.sigmaAdaption), "Use sigma adaption (true/false)"));
	  retval.addParameter(new Parameter("OneSigma", new Boolean(this.oneSigma), "Use 1 sigma per individual (true/false)"));
	  myparam = new Parameter("SigmaInit", new Double(this.sigmaInit), "Initial value for sigma (0.0 - 1.0)");
	  myparam.addRange(0.0, 1.0);
	  retval.addParameter(myparam);
	  myparam = new Parameter("SigmaMin", new Double(this.sigmaMin), "Minimal value for sigma during computation (0.0 - 1.0)");
	  myparam.addRange(0.0, 1.0);
	  retval.addParameter(myparam);

	  return retval;
	}

	/**
	 * Sets the Parameter-object to default values
	 */
	private void setDefaultParameter() {
	  this.theK = 1;
	  this.iterationCount = 1;
	  this.generations = 1000;
	  this.archiveSize = 100;
	  this.populationSize = 100;
	  this.pMutation = 0.2;
	  this.mutationType = 1;
	  this.mutationScale = 20;
	  this.upperBound = 1000.0;
	  this.lowerBound = -1000.0;
	  this.sigmaAdaption = true;
	  this.oneSigma = false;
	  this.sigmaInit = 1.0;
	  this.sigmaMin = 0.00001;
	  this.recombinationType = 1;
	  this.pRecombination = 0.8;
	  this.randomSeed = System.currentTimeMillis();
	  this.randomSeedType = 0;
	  this.speaRandom = new SPEA2Random(this.randomSeed);
	}

	/**
	 * This method prepares the algorithm to run properly.
	 *
	 * @param para The Parameters a user set to run this algorithm. If para is
	 *             <b>null</b> the algorithm must use its default Parameters.
	 *        prob The Problem this algorithm has to solve.
	 */
	public void setup(Parameters para, AbstractProblem prob) {
	  boolean retval = true;
	  String errorMessage = "";

	  if (para == null) {
		this.setDefaultParameter();
	  }
	  else {
		this.iterationCount = 1;
		this.generations = ((Integer)para.getParameterByName("Generations").getData()).intValue();
		this.archiveSize = ((Integer)para.getParameterByName("ArchiveSize").getData()).intValue();
		this.populationSize = ((Integer)para.getParameterByName("PopulationSize").getData()).intValue();
		this.optimizationType = ((Integer)para.getParameterByName("OptimizationType").getData()).intValue();
		this.randomSeedType = ((Integer)para.getParameterByName("RandomSeedType").getData()).intValue();
		if (this.randomSeedType == 0) {
		  this.randomSeed = System.currentTimeMillis();
		}
		else {
		  this.randomSeed = ((Long)para.getParameterByName("RandomSeed").getData()).longValue();
		}
		this.speaRandom = new SPEA2Random(this.randomSeed);
	this.theK = ((Integer)para.getParameterByName("k").getData()).intValue();
		this.recombinationType = ((Integer)para.getParameterByName("RecombinationType").getData()).intValue();
		this.pRecombination = ((Double)para.getParameterByName("pRecombination").getData()).doubleValue();
		this.mutationType= ((Integer)para.getParameterByName("MutationType").getData()).intValue();
		this.pMutation = ((Double)para.getParameterByName("pMutation").getData()).doubleValue();
		this.mutationScale = ((Integer)para.getParameterByName("MutationScale").getData()).intValue();
		this.upperBound = ((Double)para.getParameterByName("UpperBound").getData()).doubleValue();
		this.lowerBound = ((Double)para.getParameterByName("LowerBound").getData()).doubleValue();
		this.sigmaAdaption = ((Boolean)para.getParameterByName("SigmaAdaption").getData()).booleanValue();
		this.oneSigma = ((Boolean)para.getParameterByName("OneSigma").getData()).booleanValue();
		this.sigmaInit = ((Double)para.getParameterByName("SigmaInit").getData()).doubleValue();
		this.sigmaMin = ((Double)para.getParameterByName("SigmaMin").getData()).doubleValue();
	  }

	  // Check for correct parameters

	  // Check if problem is null
	  this.problem = prob;
	  if (this.problem == null) {
		errorMessage += "SPEA2.setup : given problem is null.\n";
		retval = false;
	  }
	  // Population size to archive size.
	  if (this.archiveSize > this.populationSize) {
		errorMessage += "SPEA2.setup : archive size is greater than population.\n";
		retval = false;
	  }
	  // Check size of population
	  if (this.populationSize < 2) {
		errorMessage += "SPEA2.setup : population size minimum is 2.\n";
		retval = false;
	  }
	  // Check size of archive
	  if (this.archiveSize < 2) {
		errorMessage += "SPEA2.setup : archive size minimum is 2.\n";
		retval = false;
	  }
	  // Number of generations
	  if (this.generations < 1) {
		errorMessage += "SPEA2.setup : number of genereation minimum is 1.\n";
		retval = false;
	  }
	  // Optimization type in range
	  if ((this.optimizationType < 0) || (this.optimizationType > 1)) {
		errorMessage += "SPEA2.setup : optimization type is 0 or 1.\n";
		retval = false;
	  }
	  // Randomseed type in range
	  if ((this.randomSeedType < 0) || (this.randomSeedType > 1)) {
		errorMessage += "SPEA2.setup : type of the randomseed is 0 or 1.\n";
		retval = false;
	  }
	  // Recombination type in range
	  if ((this.recombinationType < 0) || (this.recombinationType > 1)) {
		errorMessage += "SPEA2.setup : type of recombination is 0 or 1.\n";
		retval = false;
	  }
	  // pRecombination in range
	  if ((this.pRecombination < 0.0) || (this.pRecombination > 1.0)) {
		errorMessage += "SPEA2.setup : probability of recombination is between 0.0 and 1.0.\n";
		retval = false;
	  }
	  // Mutation type in range
	  if ((this.mutationType < 0) || (this.mutationType > 4)) {
		errorMessage += "SPEA2.setup : type of mutation is between 0 and 4.\n";
		retval = false;
	  }
	  // pMutation in range
	  if ((this.pMutation < 0.0) || (this.pMutation > 1.0)) {
		errorMessage += "SPEA2.setup : probability of mutation is between 0.0 and 1.0.\n";
		retval = false;
	  }
	  // Mutation scale in range
	  if ((this.mutationScale < 20) || (this.mutationScale > 500)) {
		errorMessage += "SPEA2.setup : mutation scale is between 20 and 500.\n";
		retval = false;
	  }
	  // Bounds don't cross
	  if (this.upperBound <= this.lowerBound) {
		errorMessage += "SPEA2.setup : upper bound is less or equal to lower bound.\n";
		retval = false;
	  }
	  // SigmaInit in range
	  if ((this.sigmaInit < 0.0) || (this.sigmaInit > 1.0)) {
		errorMessage += "SPEA2.setup : initial sigma value is between 0.0 and 1.0.\n";
		retval = false;
	  }
	  // SigmaMin in range
	  if ((this.sigmaMin < 0.0) || (this.sigmaMin > 1.0)) {
		errorMessage += "SPEA2.setup : minimum sigma value is between 0.0 and 1.0.\n";
		retval = false;
	  }
	  // Check k
	  if (this.theK < 1) {
		errorMessage += "SPEA2.setup : k has minimum 1.\n";
		retval = false;
	  }
	  if (this.theK > Math.sqrt(this.archiveSize + this.populationSize)) {
		errorMessage += "SPEA2.setup : k has maximum sqrt(ArchiveSize + PopulationSize).\n";
		retval = false;
	  }
	  // if nothing fails initialize all needed objects
	  if (retval) {
		this.initialize();
		// Check for correct types in problem
		for (int i = 0; i < this.decTypes.length; ++i) {
		  if (!this.decTypes[i].isValid()) {
			errorMessage += "SPEA2.setup : Wrong type in decision space(Dim: " + i + " / Type:" + this.decTypes[i].getTypeName() + ").\n";
			retval = false;
		  }
		}
	  }
	  if (!retval) {
		errorMessage = "=======================================================\n" + errorMessage;
		errorMessage = "\nSPEA2.setup : Setup failed due to the following errors:\n" + errorMessage;
		errorMessage = errorMessage + "Please correct these errors first.\n";
		throw new AlgoException(errorMessage);
	  }
	}

	/**
	 * Test for stop the run of SPEA2.
	 */
	public boolean checkStopCondition() {
	  return this.iterationCount > this.generations;
	}

	/**
	 * Performs one iteration (i.e. one generation) of SPEA2
	 */
	public void iterate() {
	   // System.out.println("popscore");
	  this.population.score();
		 //     System.out.println("archscore");
	  this.archive.score();
              
		   // System.out.println("calcFit"+System.currentTimeMillis());
	  this.calcFitness();

			 // System.out.println("envSel"+System.currentTimeMillis());
	  this.environmentalSelection();
			  //System.out.println("matSel"+System.currentTimeMillis());
	  this.matingSelection();
			  //System.out.println("variation");
	  this.variation();
	  this.iterationCount++;
	}

	/**
	 * Gibt zu speichernde Werte des Algos zur\uFFFDck, die nicht durch getParameter()
	 * zu erhalten sind. (Archiv, aktuelle Population + Fitnesswerte ...)
	 */
	public Data[] getResults() {
	  return this.archive.getResults();
	}

	/**
	 * Setzt Werte des Algos, die nicht durch setParameter() zu setzen sind.
	 * (Archiv, aktuelle Population + Fitnesswerte... ). Wichtig z.B. zum Fortsetzen
	 * eines unterbrochenen (geladenen) Laufes.
	 */
	public void setResults(Data[] myDataSet) {
	  this.population.setResults(myDataSet);
	}

	/**
	 * Returns the given problem or NULL if not initialized
	 */
	public AbstractProblem getProblem() {
	  return this.problem;
	}

	/**
	 * Mating
	 */
	private void matingSelection() {
	  // Mating
	  int firstIndex = 0;
	  int secondIndex = 0;
	  int thirdIndex = 0;
	  int fourthIndex = 0;
	  int mother = 0;
	  int father = 0;
	  SPEA2Individual[] pool = new SPEA2Individual[this.populationSize];

	  if (this.recombinationType == 0) {
		// ES-version
		// get 4 individuals and mate the best 2 of them
		for (int i = 0; i < this.populationSize; ++i) {
		  // make a binary tournament between 2 of them and produce 2 parents
		  firstIndex = this.speaRandom.uniform0Max(this.archiveSize);
		  secondIndex = this.speaRandom.uniform0Max(this.archiveSize);
		  while (firstIndex == secondIndex) {
			secondIndex = (secondIndex + 1) % pool.length;
		  }
		  thirdIndex = this.speaRandom.uniform0Max(this.archiveSize);
		  while ((firstIndex == thirdIndex) || (secondIndex == thirdIndex)) {
			thirdIndex = (thirdIndex + 1) % pool.length;
		  }
		  fourthIndex = this.speaRandom.uniform0Max(this.archiveSize);
		  while (firstIndex == fourthIndex || (secondIndex == fourthIndex) || (thirdIndex == fourthIndex)) {
			fourthIndex = (fourthIndex + 1) % pool.length;
		  }
		  if (this.population.getAt(firstIndex).getFitness() < this.population.getAt(secondIndex).getFitness()) {
			mother = firstIndex;
		  }
		  else {
			mother = secondIndex;
		  }
		  if (this.population.getAt(thirdIndex).getFitness() < this.population.getAt(fourthIndex).getFitness()) {
			father = thirdIndex;
		  }
		  else {
			father = fourthIndex;
		  }
		  if (father == mother) {
			System.out.println("Mama = Papa");
		  }
		  // mate 2 of them and produce one child
		  pool[i] = this.population.getAt(father).mateWith(this.population.getAt(mother));
		}
	  }
	  else {
		// GA-version
		// get 4 individuals and mate the best 2 of them
		for (int i = 0; i < this.populationSize / 2 + this.populationSize%2; ++i) {
		  // make a binary tournament between 2 of them and produce 2 parents
		  SPEA2Individual[] children = new SPEA2Individual[2];
		  children[0] = null;
		  children[1] = null;

		  firstIndex = this.speaRandom.uniform0Max(this.archiveSize);
		  secondIndex = this.speaRandom.uniform0Max(this.archiveSize);
		  while ((firstIndex == secondIndex)) {
			secondIndex = (secondIndex + 1) % pool.length;
		  }
		  thirdIndex = this.speaRandom.uniform0Max(this.archiveSize);
		  while ((firstIndex == thirdIndex) || (secondIndex == thirdIndex)) {
			thirdIndex = (thirdIndex + 1) % pool.length;
		  }
		  fourthIndex = this.speaRandom.uniform0Max(this.archiveSize);
		  while (firstIndex == fourthIndex || (secondIndex == fourthIndex) || (thirdIndex == fourthIndex)) {
			fourthIndex = (fourthIndex + 1) % pool.length;
		  }
		  if (this.population.getAt(firstIndex).getFitness() < this.population.getAt(secondIndex).getFitness()) {
			mother = firstIndex;
		  }
		  else {
			mother = secondIndex;
		  }
		  if (this.population.getAt(thirdIndex).getFitness() < this.population.getAt(fourthIndex).getFitness()) {
			father = thirdIndex;
		  }
		  else {
			father = fourthIndex;
		  }
		  if (mother == father) {
			System.out.println("Mama = Papa");
		  }
		  // mate 2 of them and produce one child
		  children = this.population.getAt(father).mateWith(this.pRecombination, this.population.getAt(mother));
		  pool[(i * 2)] = children[0];
		  if (i < this.populationSize / 2) {
			pool[((i * 2) + 1)] = children[1];
		  }
		}
	  } // end of recombinations

	  // copy pool to population
	  for (int i = 0; i < this.populationSize; ++i) {
		this.population.setAt(i, pool[i]);
	  }
	}

	/**
	 * Variation
	 * - Mutation
	 */
	private void variation() {
	  this.population.mutate(this.pMutation, this.mutationType, this.upperBound, this.lowerBound, this.mutationScale);
	}

	/**
	 * Sort the given array by fitness
	 */
	private SPEA2Individual[] sort(SPEA2Individual[] toSort) {
	   /* 
	  SPEA2Individual swapper = null;

	  for (int i =0; i < toSort.length - 1; ++i) {
		for (int j = i + 1; j < toSort.length; ++j) {
		  if (toSort[i].getFitness() > toSort[j].getFitness()) {
			swapper = toSort[i];
			toSort[i] = toSort[j];
			toSort[j] = swapper;
		  }
		}
	  }
      
	  return toSort;
	  */
	  a = toSort;
	  quicksort(0,a.length-1);
	  return a;
       
	}
    
	private void quicksort(int lo, int hi) {
		int i = lo, j = hi;
		double x = a[(hi + lo)/2].getFitness();
		SPEA2Individual swapper = null;
        
		while (i<=j) {
			while (a[i].getFitness() < x) i++;
			while (a[j].getFitness() > x) j--;
			if (i <= j) {
				swapper = a[i];
				a[i] = a[j];
				a[j] = swapper;
				i++;
				j--;
			}
		}
        
		if (lo < j) quicksort(lo, j);
		if (i < hi) quicksort(i, hi);
            
	}
    
	private void quickselect(int lo, int hi) {
        
	}
  
  
	/*
	private void environmentalSelection() {
	  SPEA2Population interimArchive = new SPEA2Population(this.archiveSize + this.populationSize, this.speaRandom, this.decTypes, this.objTypes, this.decRanges, this.problem);
	  SPEA2Individual[] allind = new SPEA2Individual[(this.population.getSize() + this.archive.getSize())];
	  int nondominated = 0;
	  int count = 0;
	  int swapInt = 0;
	  double swapDouble = 0.0;
	  boolean[] selected = new boolean[this.archiveSize + this.populationSize];
	  int[] k_nearest = new int[this.archiveSize + this.populationSize];
	  int[] sorted = new int[this.archiveSize + this.populationSize];
	  double[] distance = new double[this.archiveSize + this.populationSize];
	  double[] distance_tmp = new double[this.archiveSize + this.populationSize];

	  // calculate k
	  int k = this.theK;

	  // get all individuals
	  // get population
	  for (int i = 0; i < this.population.getSize(); ++i) {
		allind[i] = (SPEA2Individual)this.population.getAt(i).clone();
	  }
	  // get archive
	  for (int i = 0; i < this.archive.getSize(); ++i) {
		// get real copies of individuals...
		allind[(this.population.getSize() + i)] = (SPEA2Individual)this.archive.getAt(i).clone();
	  }
	  // sort allind
	  allind = this.sort(allind);
	  // copy alle nondominated individual to the archive (F(i) < 1)
	  nondominated = 0;
	  for (int i = 0; i < allind.length; ++i) {
		if (allind[nondominated].getFitness() < 1) {
		  interimArchive.setAt(nondominated, allind[i]);
		  ++nondominated;
		}
	  }
	  if (nondominated != this.archiveSize) {
		if (nondominated < this.archiveSize) {
		  // fill up archive with best individuals
		  count = 0;
		  while ((nondominated < this.archiveSize) && (count < allind.length)) {
			if (allind[count].getFitness() >= 1) {
			  interimArchive.setAt(nondominated, allind[count]);
			  ++nondominated;
			}
			++count;
		  }
		}
		else {
		  // truncate archive to given size;
		  // get all individuals from interim archive to allind
		  for (int i = 0; i < nondominated; ++i) {
			allind[i] = interimArchive.getAt(i);
		  }
		  // initialize selected, distance, k_nearest array
		  for (int i = 0; i < selected.length; ++i) {
			if (i < nondominated) {
			  selected[i] = true;
			}
			else {
			  selected[i] = false;
			}
			distance[i] = Double.MAX_VALUE;
			k_nearest[i] = i;
		  }
		  System.out.println("start");
		  while (nondominated > this.archiveSize) {
			// calculate distances and index of k-nearest for every individual
			for (int i = 0; i < allind.length; ++i) {
			  // initialize temporary distances
			  for (int j = 0; j < allind.length; ++j) {
				distance_tmp[j] = Double.MAX_VALUE;
				sorted[j] = j;
			  }
			  // calculate distances from i to all other individuals
			  if (selected[i]) {
				for (int j = 0; j < allind.length; ++j) {
				  if (selected[j]) {
					distance_tmp[j] = allind[i].distanceTo(allind[j]);
				  }
				}
				// find out, which is the k-nearest and set the entire value
				for (int j = 0; j < allind.length - 1; ++j) {
				  for (int l = j + 1; l < allind.length; ++l) {
					if (distance_tmp[j] > distance_tmp[l]) {
					  // swap distance
					  swapDouble = distance_tmp[j];
					  distance_tmp[j] = distance_tmp[l];
					  distance_tmp[l] = swapDouble;
					  // swap positions
					  swapInt = sorted[j];
					  sorted[j] = sorted[l];
					  sorted[l] = swapInt;
					}
				  }
				}
			  }
			  // add 1 to k because of the distance 0 to myself
			  k_nearest[i] = sorted[k];
			  distance[i] = distance_tmp[k];
			} // end of calculating all distances and indeces of k-nearest
			// sort the k-nearest according to the distance
			for (int j = 0; j < allind.length; ++j) {
			  sorted[j] = j;
			}
			for (int j = 0; j < allind.length - 1; ++j) {
			  for (int l = j + 1; l < allind.length; ++l) {
				if (distance[j] > distance[l]) {
				  // swap distance
				  swapDouble = distance[j];
				  distance[j] = distance[l];
				  distance[l] = swapDouble;
				  // swap positions
				  swapInt = sorted[j];
				  sorted[j] = sorted[l];
				  sorted[l] = swapInt;
				}
			  }
			}
			// select individual and remove it from selected list
			selected[k_nearest[sorted[0]]] = false;
			--nondominated;
		  }
		  System.out.println("stop");
		  // get all individuals from allind to interim archive
		  count = 0;
		  for (int i = 0; i < allind.length; ++i) {
			if (selected[i]) {
			  interimArchive.setAt(count, allind[i]);
			  ++count;
			}
		  }
		}
	  }
	  // copy interim archive to archive
	  for (int i = 0; i < this.archiveSize; ++i) {
		this.archive.setAt(i, interimArchive.getAt(i));
	  }
	}

	/**
	 * Environmental selection
	 */
    
	private void environmentalSelection() {
	  SPEA2Population interimArchive = new SPEA2Population(this.archiveSize + this.populationSize, this.speaRandom, this.decTypes, this.objTypes, this.decRanges, this.problem);
	  SPEA2Individual[] allind = new SPEA2Individual[(this.population.getSize() + this.archive.getSize())];
	  int nondominated = 0;
	  int minimum = 0;
	  int deselected = 0;
	  int count = 0;
	  int swapInt = 0;
	  double swapDouble = 0.0;
	  int temp = 0;
	  double disToTemp = 0.0;
	  boolean[] selected = new boolean[this.archiveSize + this.populationSize];
	  int[] k_nearest = new int[this.archiveSize + this.populationSize];
	  int[] sorted = new int[this.archiveSize + this.populationSize];
	  double[] distance = new double[this.archiveSize + this.populationSize];
	  double[] distance_tmp = new double[this.archiveSize + this.populationSize];

	  // calculate k
	  int k = this.theK;

	  // get all individuals
	  // get population
	  for (int i = 0; i < this.population.getSize(); ++i) {
		allind[i] = (SPEA2Individual)this.population.getAt(i).clone();
	  }
	  // get archive
	  for (int i = 0; i < this.archive.getSize(); ++i) {
		// get real copies of individuals...
		allind[(this.population.getSize() + i)] = (SPEA2Individual)this.archive.getAt(i).clone();
	  }
	  // sort allind
	  allind = this.sort(allind);
	  // copy alle nondominated individual to the archive (F(i) < 1)
	  nondominated = 0;
	  for (int i = 0; i < allind.length; ++i) {
		if (allind[nondominated].getFitness() < 1) {
		  interimArchive.setAt(nondominated, allind[i]);
		  ++nondominated;
		}
	  }
	  if (nondominated != this.archiveSize) {
		if (nondominated < this.archiveSize) {
		  // fill up archive with best individuals
		  count = 0;
		  while ((nondominated < this.archiveSize) && (count < allind.length)) {
			if (allind[count].getFitness() >= 1) {
			  interimArchive.setAt(nondominated, allind[count]);
			  ++nondominated;
			}
			++count;
		  }
		}
		else {
		  // truncate archive to given size;
		  // get all individuals from interim archive to allind
		  for (int i = 0; i < nondominated; ++i) {
			allind[i] = interimArchive.getAt(i);
		  }
		  // initialize selected, distance, k_nearest array
		  for (int i = 0; i < selected.length; ++i) {
			if (i < nondominated) {
			  selected[i] = true;
			}
			else {
			  selected[i] = false;
			}
			distance[i] = Double.MAX_VALUE;
			k_nearest[i] = i;
		  }
          

	//      System.out.println("multi" + (nondominated-archiveSize) + " " + System.currentTimeMillis()); 
		  int [] multiple = new int[archiveSize+populationSize];
		  //int [] multiple = new int[2*(nondominated-archiveSize)];
		  count = 0;
		  int m = 0;
		  while (m < allind.length && count < multiple.length) {
			 // System.out.println(allind[m].getDensity());
			if (selected[m] && allind[m].getDensity() == 0.5) {
			  multiple[count] = m;
			 // System.out.println(m);
			  count++;
			}
			m++;
		  }
		  int newCount = 0;
    
          
		  while (nondominated > this.archiveSize && newCount < count-1) {
			  boolean ka = false;
			if (selected[multiple[newCount]]) {
			int kcount = 0;
			for (int i = newCount + 1; i < count; ++i) {
			  if (allind[multiple[newCount]].distanceTo(allind[multiple[i]]) == 0) {
				kcount ++;
				if (kcount >= k) {
				  selected[multiple[i]] = false;
			   // System.out.println(multiple[i] + " weg wegen " + multiple[newCount] + " Distanz war " + 
				//                   allind[multiple[newCount]].distanceTo(allind[multiple[i]]));
				  nondominated--;
				}
			   // ka = true;
				if (nondominated == this.archiveSize) 
				  break;
			  }
			}
			//if (!ka) {
			//selected[multiple[newCount]] = false;
		   // System.out.println(multiple[newCount] + " weg wegen ka");
		   // nondominated--;
			//}
			}
			newCount++;
		  }
      
         
          
		  int r = 1;
	  //    System.out.println("start " + (nondominated-archiveSize) + " " + System.currentTimeMillis());
		  if (nondominated > this.archiveSize) {
			// calculate distances and index of k-nearest for every individual
			for (int i = 0; i < allind.length; ++i) {
			  // initialize temporary distances
			  // calculate distances from i to all other individuals 
			  if (selected[i]) {
                  
				  /*
				for (int j = 0; j < allind.length; ++j) {
				  sorted[j] = j;
				  if (selected[j]) {
					distance_tmp[j] = allind[i].distanceTo(allind[j]);
				  }
				  else {
					distance_tmp[j] = Double.MAX_VALUE;
				  }
				}
				*/
    
				count = 0;
				for (int j=0; j < allind.length; ++j) {
				  if (selected[j]) {
					sorted[count] = j;
					distance_tmp[count] = allind[i].distanceTo(allind[j]);
					count++;                    
				  }
				}
				// find out, which is the k-nearest and set the entire value
		   //        System.out.println("Distanz zu 0 ist " + distance_tmp[0]);
				for (int j = 0; j <= k; ++j) {
				  //for (int l = j + 1; l < allind.length; ++l) {
					for (int l = j + 1; l < count; ++l) {
					 // r++;
					if (distance_tmp[j] > distance_tmp[l]) {
					  // swap distance
					  swapDouble = distance_tmp[j];
					  distance_tmp[j] = distance_tmp[l];
					  distance_tmp[l] = swapDouble;
					  // swap positions
					  swapInt = sorted[j];
					  sorted[j] = sorted[l];
					  sorted[l] = swapInt;
					}
				  }
				}
                
              
              
				// add 1 to k because of the distance 0 to myself ??? schwachsinn
				k_nearest[i] = sorted[k];
				distance[i] = distance_tmp[k];
			  }

                
              
		 //     System.out.println(i + " hat Entf. zu " + k_nearest[i] + " und zwar " + distance[i]);
           
              
			} // end of calculating all distances and indeces of k-nearest
			// sort the k-nearest according to the distance
			//for (int j = 0; j < allind.length; ++j) {
			//  sorted[j] = j;
			//}
			//for (int j = 0; j < allind.length - 1; ++j) {
  /*            if (distance[i] == 0) {
							  // select individual and remove it from selected list
				deselected = k_nearest[minimum];
				selected[k_nearest[minimum]] = false;
				distance[deselected] = Double.MAX_VALUE;
	   //    System.out.println("minimum ist " + minimum + " und " + k_nearest[minimum]+" ist weg. Distanz war " + distance[minimum]);
				--nondominated;
				continue;
			  }
	*/
    
         
            
            
			minimum = 0;
			count = 0;
			double minDistance = distance[0];
			  for (int j = 1; j < allind.length; ++j) {
				if (minDistance > distance[j]) {
				  minDistance = distance[j];
				  minimum = j;
				  //count = 1;
				}
			   // if (minDistance == distance[j]) {
				 //  count ++;
				//}
				}     
		   /*   int [] presorted = new int [count];
			  k = 0;
			  for (int j = 1; j < allind.length; ++j) {                  
				  if (minDistance == distance[j]) {
					  presorted[k] = j;
					  k++;
				  }
			  }
			  int [] next_nearest = new int [presorted.length];
			  for (int i = 0; i < presorted.length; ++i) {
				  next_nearest[i] = k_nearest[presorted[i]];
			  }
			  for (int i = 0; i < presorted.length; ++i) {
			  // initialize temporary distances    
				double disToK = allind[i].distanceTo(allind[next_nearest[i]]);

				double disToJ;
				disToTemp = Double.MAX_VALUE;
				for (int j = 0; j < allind.length; ++j) {
				  if (selected[j]) {
					disToJ = allind[i].distanceTo(allind[j]);
					if (disToJ > disToK && disToJ < disToTemp){
					  temp = j;
					  disToTemp = disToJ;
					}
					else if (disToJ == disToK && j > k_nearest[i]) {
					  temp = j;
					  disToTemp = disToJ;
					  break;
					}
				  }
				}
          
             
			  k_nearest[i] = temp;
			  distance[i] = disToTemp;
              
		 //     System.out.println(i + " hat Entf. zu " + k_nearest[i] + " und zwar " + distance[i]);
           
			  }  
			 */ 
              
			//}
			// select individual and remove it from selected list
			deselected = k_nearest[minimum];
			selected[k_nearest[minimum]] = false;
			distance[deselected] = Double.MAX_VALUE;
		//    if (distance[minimum] != 0) 
			  //System.out.println("minimum ist " + minimum + " und " + k_nearest[minimum]+" ist weg. Distanz war " + distance[minimum]);
			--nondominated;
		  }
		//  System.out.println(System.currentTimeMillis());
		  while (nondominated > this.archiveSize) {
			  int u = 0;
			// calculate distances and index of k-nearest for every individual
			for (int i = 0; i < allind.length; ++i) {
			  // initialize temporary distances
			  double disToK = allind[i].distanceTo(allind[k_nearest[i]]);
			  double disToDes = allind[i].distanceTo(allind[deselected]);
			  if (selected[i] && (disToK > disToDes || (disToK == disToDes && k_nearest[i] >= deselected))) {
				u ++ ;
				double disToJ;
				disToTemp = Double.MAX_VALUE;
				for (int j = 0; j < allind.length; ++j) {
				  if (selected[j]) {
					disToJ = allind[i].distanceTo(allind[j]);
					if (disToJ > disToK && disToJ < disToTemp){
					  temp = j;
					  disToTemp = disToJ;
					}
					else if (disToJ == disToK && j > k_nearest[i]) {
					  temp = j;
					  disToTemp = disToJ;
					  break;
					}
				  }
				}
             
			  k_nearest[i] = temp;
			  distance[i] = disToTemp;
              
		 //     System.out.println(i + " hat Entf. zu " + k_nearest[i] + " und zwar " + distance[i]);
           
			  }  
			} // end of calculating all distances and indeces of k-nearest
	  //  System.out.println(u + " mal");
			// sort the k-nearest according to the distance              
			minimum = 0;
			double minDistance = distance[0];
			  for (int j = 1; j < allind.length; ++j) {
				if (minDistance > distance[j]) {
				  minDistance = distance[j];
				  minimum = j;
				}
			  }
			//}
			// select individual and remove it from selected list
			deselected = k_nearest[minimum];
			selected[k_nearest[minimum]] = false;
			distance[deselected] = Double.MAX_VALUE;
		  //  if (distance[minimum] != 0)
			  //System.out.println("minimum ist " + minimum + " und " + k_nearest[minimum]+" ist weg. Distanz war " + distance[minimum]);
			--nondominated;
		  }

	 //     System.out.println("end " + System.currentTimeMillis());
		  // get all individuals from allind to interim archive
		  count = 0;
		  for (int i = 0; i < allind.length; ++i) {
			if (selected[i]) {
			  interimArchive.setAt(count, allind[i]);
			  ++count;
			}
		  }
		}
	  }
	  // copy interim archive to archive
	  for (int i = 0; i < this.archiveSize; ++i) {
		this.archive.setAt(i, interimArchive.getAt(i));
	  }
	}

    
    
     
	/**
	 * Fitness calculation in the population an the archive
	 */
	private void calcFitness() {
	  // 0 - minimum / 1 - maximum
	  // nullzeiger des Values abfangen!!
      
	  double swapDouble;
	  SPEA2Individual[] allind = new SPEA2Individual[(this.population.getSize() + this.archive.getSize())];
	  // get all individuals
	  // get population
	  for (int i = 0; i < this.population.getSize(); ++i) {
		allind[i] = this.population.getAt(i);
	  }
	  // get archive
	  for (int i = 0; i < this.archive.getSize(); ++i) {
		allind[(this.population.getSize() + i)] = this.archive.getAt(i);
	  }

	  // initialize all values (strength, raw, density and fitness)
	  for (int i = 0; i < allind.length; ++i) {
		allind[i].setStrength(0);
		allind[i].setRaw(0);
		allind[i].setDensity(0.0);
		allind[i].setFitness(0.0);
	  }

	  // calculate strength
 //     System.out.println("strength" + System.currentTimeMillis());
	  for (int i = 0; i < allind.length-1; ++i) {
		// compare with all other
		for (int j = i+1; j < allind.length; ++j) {
		  if ((allind[i].getResults().objVector[0].getData() != null) && (allind[j].getResults().objVector[0].getData() != null)) {
			if (allind[i].dominates(this.optimizationType, allind[j])) {
			  allind[i].setStrength(allind[i].getStrength() + 1);
			}
			if (allind[j].dominates(this.optimizationType, allind[i])) {
			  allind[j].setStrength(allind[j].getStrength() + 1);
			}
		  }
		  if ((allind[j].getResults().objVector[0].getData() == null) && (allind[i].getResults().objVector[0].getData() != null)) {
			  allind[i].setStrength(allind[i].getStrength() + 1);
		  }
		  if ((allind[j].getResults().objVector[0].getData() != null) && (allind[i].getResults().objVector[0].getData() == null)) {
			  allind[j].setStrength(allind[j].getStrength() + 1);
		  }
		}
	  }

	  // calculate raw fitness
  //    System.out.println("raw" + System.currentTimeMillis());
	  for (int i = 0; i < allind.length-1; ++i ) {
		for (int j = i+1; j < allind.length; ++j ) {
		  if ((allind[i].getResults().objVector[0].getData() != null) && (allind[j].getResults().objVector[0].getData() != null)) {
			if (allind[i].dominates(this.optimizationType, allind[j])) {
			  allind[j].setRaw(allind[j].getRaw() + allind[i].getStrength());
			}
			if (allind[j].dominates(this.optimizationType, allind[i])) {
			  allind[i].setRaw(allind[i].getRaw() + allind[j].getStrength());
			}
		  }
		  if (allind[j].getResults().objVector[0].getData() == null && (allind[i].getResults().objVector[0].getData() != null)) {
			allind[j].setRaw(allind[j].getRaw() + allind[i].getStrength());
		  }
		  if (allind[j].getResults().objVector[0].getData() != null && (allind[i].getResults().objVector[0].getData() == null)) {
			allind[i].setRaw(allind[i].getRaw() + allind[j].getStrength());
		  }
		}
	  }

	  // calculate density
  //    System.out.println("density" + System.currentTimeMillis());
	  // calculate k
	  // int k = new Double(Math.sqrt(allind.length)).intValue();
	  int k = this.theK;
	  double[] distances = new double[allind.length];
	  // calculate density for each individual
	  for (int i = 0; i < allind.length; ++i) {
		// calculate distances to individual i
		for (int j = 0; j < i; ++j) {
		  distances[j] = allind[i].distanceTo(allind[j]);
		}
		for (int j = i+1; j < allind.length; ++ j) {
		  distances[j-1] = allind[i].distanceTo(allind[j]);
		}
		// sort distances
		//Arrays.sort(distances);
        
		for (int j = 0; j < k; ++j) {
		  for (int l = 0; l < allind.length-1; ++l) {
			if (distances[j] > distances[l]) {
			  swapDouble = distances[j];
			  distances[j] = distances[l];
			  distances[l] = swapDouble;
			}
		  }
		}
         
		allind[i].setDensity( (1 / (distances[k-1] + 2.0)) );
        
	  }

	  // calculate fitness
	  //System.out.println("fitness" + System.currentTimeMillis());
	  for (int i = 0; i < allind.length; ++i) {
		allind[i].setFitness(allind[i].getRaw() + allind[i].getDensity());
	  }

	  // return individuals to population and archive
	  // set population
	  for (int i = 0; i < this.population.getSize(); ++i) {
		this.population.setAt(i, allind[i]);
	  }
	  // set archive
	  for (int i = 0; i < this.archive.getSize(); ++i) {
		this.archive.setAt(i, allind[(this.population.getSize() + i)]);
	  }
	}

	/**
	 * Initializes the algorithm...
	 */
	private void initialize() {
	  // get type from problem and set it in the population
	  this.decTypes = this.problem.getDecTypes();
	  this.objTypes = this.problem.getObjTypes();
	  this.decRanges = this.problem.getRanges();
	  // generate startpopulation (random initialized)
	  this.population = new SPEA2Population(this.populationSize, this.speaRandom, this.decTypes, this.objTypes, this.decRanges, this.problem);
	  this.population.initRandom();
	  // generate empty archive
	  this.archive = new SPEA2Population(this.archiveSize, this.speaRandom, this.decTypes, this.objTypes, this.decRanges, this.problem);
	}

	/*
	 * Member-variables
	 */

	/**
	 * locally stored random seed
	 */
	private long randomSeed;

	/**
	 * The problem we want to evolve.
	 */
	private AbstractProblem problem;

	/*
	 * Counts the nunber of calls of AbstractAlgorithm.iterate().
	 */
	private int iterationCount;

	/*
	 * SPEA2 Parameters
	 */
	private int randomSeedType;
	private int theK;
	private double pRecombination;
	private int recombinationType;
	private double pMutation;
	private int mutationType;
	private int mutationScale;
	private double upperBound;
	private double lowerBound;
	private boolean sigmaAdaption;
	private boolean oneSigma;
	private double sigmaInit;
	private double sigmaMin;
	private int generations;
	private int archiveSize;
	private int populationSize;
	private int optimizationType;
	private SPEA2Population population;
	private SPEA2Population archive;
	private SPEA2Population matingPool;
	private SPEA2Random speaRandom;
	private Type[] decTypes;
	private Type[] objTypes;
	private Range[] decRanges;
	private SPEA2Individual[] a;
}
