/*
 * AbstractComparison.java  -  This file is part of KEA.
 *
 * Copyright (C) 2002/03 PG 419, Department of Computer Science, 
 * University of Dortmund.          eMail: <peter.senft@udo.edu>
 */

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the KEA License Agreement which is based on 
 * the GNU General Public License as published by the Free Software 
 * Foundation; either version 2 of the License, or (at your option) 
 * any later version.  For details see License.txt.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

package kea.comparisons;

import kea.AlgoState;
import kea.CompResult;
import kea.EvalResult;
import kea.Parameters;

/**
 * By inheriting from this abstract Class, a new kind of comparison can be
 * implemented. The constructor must create a new comparison with the needed
 * parameters to run.
 *
 * @author Torsten Kohlen
 * @version 0.5.0
 */
public abstract class AbstractComparison {
    /**
     * Holds the Parameter for the problem.<p>
     * <b>Attention:</b> Since KEA stores its Data in AlgoState or EvalResult
     * Files, the comparison-Parameters need at least two different Inputfiles
     * to do the comparison on. For recordnizing these special Perameters e.g.
     * with the GUI, we advice to begin the names of these (File-) Paramters
     * with "file" (e.g. fileA or fileForComp1)
     *
     * @see AlgoState
     * @see EvalResult
     * @see Parameters
     */
    protected Parameters param;


    /**
     *
     */
    private String fileName = "";


    /**
     * Implementing this method means doing the actually comparion. The user
     * who wants to do a comparison gives with the *.param the files which hold
     * the Data to be compared. The result of this method must be a CompResult,
     * in which the compared Data will be saved.
     *
     * @see CompResult
     * @return The result of a comparion.
     */
    public abstract CompResult compare();


    /**
     *
     */
    public abstract Parameters getParameter();


    /**
     *
     */
    public abstract void setParameter(Parameters parameter);


    /**
     *
     */
    public void setFileName(String newName) {
      fileName = newName;
    }


    /**
     *
     */
    public String getFileName() {
      return fileName;
    }
}
