/*
 * FilesToFile.java  -  This file is part of KEA.
 *
 * Copyright (C) 2002/03 PG 419, Department of Computer Science, 
 * University of Dortmund.          eMail: <peter.senft@udo.edu>
 */

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the KEA License Agreement which is based on 
 * the GNU General Public License as published by the Free Software 
 * Foundation; either version 2 of the License, or (at your option) 
 * any later version.  For details see License.txt.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

package kea.comparisons.common;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;

import kea.Datacontainer;
import kea.Kea;
import kea.evaluations.common.GrepHeader;

/**
 * Describe class <code>FilesToFile</code> here.
 *
 * @author PG 419, University of Dortmund (dk)
 * @version $Revision: 723 $
 */
public class FilesToFile {
  
  private FilesToFile(){};

  /*  public static void main(String[] argv){
      groupResultsInSubDirsOf("toGroup");
      groupDir("toGroup");
      }
  */

  /**
   * <code>groupResultsInSubDirsOf</code> collects all result-data
   * from each direct subdirectory and places it in a file in its
   * parent-directory.  The files are called < subdirname >_EWG.ers
   * or < subdirname >_DSG.ers, respective.  Here, EWG means "error
   * while grouping" and DSG "directory successfully grouped.  The
   * first case should only occur, if a direct subdirectory contains
   * no proper data-files.
   *
   * <b>Attention</b> see <code>groupDir</code>.
   * @param dirName This <code>String</code> should be the name of
   * the directory, which direct subdirectories you want to group.
   */
  public static void groupResultsInSubDirsOf(String dirName){

    /* shall contain the names of the files
     * in directory "dirName" 
     */
    String[] fileNames;
    File[] filePaths;
    File dirPath = new File(dirName);

    if(!dirPath.isDirectory())
      {
	Kea.logger.logError
	  (dirName + " isn't a directory name, though expected."
	   , new FilesToFile());
	System.exit(1);
      }

    filePaths = new File[]{dirPath};
    filePaths = filePaths[0].listFiles();
    fileNames = new String[filePaths.length];
    for (int i=0; i < filePaths.length ;i++){
      fileNames[i] = filePaths[i].getPath();
    }
    
    for(int i=0; i<fileNames.length; i++){
      if(new File(fileNames[i]).isDirectory())
	groupDir(fileNames[i]);
    }
  }

  /**
   * <code>groupDir</code> groups all result-files of
   * the given directory in a file.
   * <p>
   * <b>Attention</b> The header will be output only once
   * at the beginning of the output-file.  Therefore it's
   * important that all files are from the same type;
   * i.e. same algorithm, problem and also evaluations,
   * if dimensions were chosen.  Furthermore, Results from different
   * files will be separated by a line starting with "# ***".
   *
   * @param dirName <code>String</code> names the directory
   * which results shall be grouped. 
   */
  public static void groupDir(String dirName){
    boolean firstFileFound = false;
    File dirPath = new File(dirName);
    String[] fileNames;
    File[] filePaths;
      
    if (!dirPath.isDirectory()){
      Kea.logger.logError
	(dirName + " isn't a directory name, though expected."
	 , new FilesToFile());
      System.exit(-1);
    }
     
    filePaths = new File[]{dirPath};
    filePaths = filePaths[0].listFiles();
    fileNames = new String[filePaths.length];
    for (int i=0; i < filePaths.length ;i++){
      fileNames[i] = filePaths[i].getPath();
    }
  
    
    try
      {
	File startFile =
	  new File(dirName + "_EWG.ers");
	PrintWriter outPrint =
	  new PrintWriter
	  ( new BufferedWriter
	    ( new FileWriter(startFile)));
	BufferedReader inRead;
	
	String lineRead;
	for(int i=0; i<fileNames.length; i++)
	  {
	    if(new File(fileNames[i]).isDirectory())
	      {
		Kea.logger.logWarn
		  (fileNames[i]+" is a directory and will be ignored.");
		continue;
	      }

	    if (GrepHeader.makeParams(fileNames[i])==null)
	      {
		Kea.logger.logWarn("File \""+fileNames[i]+"\" ignored."
				   + "The header is missing.");
		continue;
	      }


	    if(!firstFileFound)
	      {
		outPrint.close();
		startFile.delete();

		startFile = new File(fileNames[i]);

		outPrint =
		  new PrintWriter
		  ( new BufferedWriter
		    ( new FileWriter
		      ( startFile.getParent()+"_DSG.ers")));
		
		outPrint.println("# ***");
		outPrint.println("# *** Grouped results of directory " + startFile.getParent());
		outPrint.println("# *** Results from file " + fileNames[i]);
		outPrint.println("# ***");
		

		inRead = new BufferedReader
		  (new FileReader(fileNames[i]));
		
		lineRead = inRead.readLine();
		while(lineRead!=null)
		  {
		    outPrint.println(lineRead);
		    lineRead = inRead.readLine();
		  }
		inRead.close();

		firstFileFound = true;
		continue;
	      }

	    /* now the results of the other files must be appended */
	    inRead = new BufferedReader
	      (new FileReader(fileNames[i]));
	    
	    lineRead = inRead.readLine();
	    
	    /* since GrepHeader has found a header,
	     * one line starts with HEADER_END_IDENTIFIER and we needn't check for null
	     */
	    while(!lineRead.startsWith(Datacontainer.HEADER_END_IDENTIFIER))
	      lineRead = inRead.readLine();
	    ;

	    outPrint.println("# *** Results from file " + fileNames[i]);
	    lineRead = inRead.readLine();

	    while(lineRead!=null)
		  {
		    outPrint.println(lineRead);
		    lineRead = inRead.readLine();
		  }
	    inRead.close();
	    
	  } 
	outPrint.close();

      }
    catch(FileNotFoundException e)
      {
	Kea.logger.logError( "\nOnly the name of a directory, which contains files "
			     + "of the same type (subdirectories will be ignored), "
			     + "is recommended as input\n",e,new FilesToFile());
      }
    catch(IOException e)
      {
	Kea.logger.logError("IOError occured while working with ..."
			    , e, new FilesToFile());
      }
  }
}
