/*
 * ShiftResults.java  -  This file is part of KEA.
 *
 * Copyright (C) 2002/03 PG 419, Department of Computer Science, 
 * University of Dortmund.          eMail: <peter.senft@udo.edu>
 */

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the KEA License Agreement which is based on 
 * the GNU General Public License as published by the Free Software 
 * Foundation; either version 2 of the License, or (at your option) 
 * any later version.  For details see License.txt.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

package kea.comparisons.common;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.PrintWriter;
import java.util.Vector;

import kea.Kea;

/**
 * @author Tim Hohm
 * @version $Revision: 723 $
 */
public class ShiftResults {

  /**
   * Name of the file wich objective values get shifted.
   */
  private String run;
  private File runFile;

  public ShiftResults(String run) {
    this.run = run;
    runFile = new File(run);
  }

  /**
   * It shifts the objective values by adding the given <code>double<\code>s to
   * the corresponding objective.
   *
   * @param shifti An Array of <code>double<\cose>s that contains the values wich should get added to the corresponding
   * objective.
   * @return Name of the ASCII file with shifted objectives.
   */
  public String shift(double[] shifti){
    String curLine = "";
    String returnString = "";
    try {
      BufferedReader in = new BufferedReader(new FileReader(run));
      returnString = (runFile.getPath()).substring(0, (runFile.getPath()).indexOf(runFile.getName()))+"shifted&"+ runFile.getName();
      PrintWriter out = new PrintWriter(new BufferedWriter(new FileWriter(new File(returnString))));
      // Skip Header.
      while(in.ready()) {
        curLine = in.readLine();
        out.println(curLine);
        if (curLine != null){
          // Search for end of header.
          if (curLine.indexOf("###") != -1){
            if (Kea.logger.debugOn()) {
              Kea.logger.logDebug("Ende Header!", this);
            }
            break;
          }
        }
      }
      while (in.ready()){
      curLine = in.readLine();
      if (curLine == null)break;
      // If the current line is the evaluationCount line: print it.
      if (curLine.indexOf("evaluationCount") != -1){
        out.println(curLine);
      }
      // If the current line is the iterationCount line: print it.
      if (curLine.indexOf("iterationCount") != -1){
        out.println(curLine);
      }
      // If the current line is the runtime line: print it.
      if (curLine.indexOf("runtime") != -1){
        out.println(curLine);
      }
      if ((curLine.indexOf("iterationCount") == -1) && (curLine.indexOf("evaluationCount") == -1) && (curLine.indexOf("runtime") == -1)) {
        // If current line is empty print an empty line ...
        if (curLine.equals("")){
          out.println(curLine);
        }else{
          // ... otherwise print the design variables ...
          out.print(curLine.substring(0, curLine.indexOf("/")+1));
          // ... and shift the objective values.
          curLine = curLine.substring(curLine.indexOf("/")+1);
          Vector obs = new Vector();
          int counter = 0;
          while (!curLine.equals("")){
            curLine = curLine.trim();
            if (curLine.indexOf(" ") != -1){
              String it = curLine.substring(0, curLine.indexOf(" "));
              curLine = curLine.substring(curLine.indexOf(" "));
              curLine.trim();
              // Shift value.
              obs.addElement(Double.toString(Double.parseDouble(it) + shifti[counter]));
              counter++;
            }else{
              obs.addElement(Double.toString(Double.parseDouble(curLine) + shifti[counter]));
              break;
            }
          }
          // Print objectives.
          for (int i = 0; i < obs.size(); i++){
            out.print((String)obs.elementAt(i));
            out.print(" ");
          }
          out.println();
        }
      }
      }
      out.close();//*/
    }
    catch(Exception exc){
      throw new CompException("File "+run+" not found. No operation on this file possible");
    }

  return (returnString+", ");
  }

  // Tests wheather the used Problem was Kursawe Testfunktion.
  public boolean test(){
    String curLine = "";
    boolean isKursawe = false;
    try {
      BufferedReader in = new BufferedReader(new FileReader(run));
      // Check Header.
      while(in.ready()) {
        curLine = in.readLine();
        if (curLine != null){
          // Search for end of header.
          if (curLine.indexOf("Kursawe") != -1){
            isKursawe = true;
            if (Kea.logger.debugOn()) {
              Kea.logger.logDebug("Its Kursawe", this);
            }
            break;
          }
          if (curLine.indexOf("###") != -1){
            if (Kea.logger.debugOn()) {
              Kea.logger.logDebug("Ende Header!", this);
            }
            break;
          }
        }
      }
    }
    catch(Exception exc){
      throw new CompException("File "+run+" not found. No operation on this file possible");
    }
    return isKursawe;
  }

}
