/*
 * Collector.java  -  This file is part of KEA.
 *
 * Copyright (C) 2002/03 PG 419, Department of Computer Science, 
 * University of Dortmund.          eMail: <peter.senft@udo.edu>
 */

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the KEA License Agreement which is based on 
 * the GNU General Public License as published by the Free Software 
 * Foundation; either version 2 of the License, or (at your option) 
 * any later version.  For details see License.txt.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

package kea.comparisons.hypervolume;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.util.Vector;

import kea.CompResult;
import kea.Kea;
import kea.Parameter;
import kea.Parameters;
import kea.comparisons.AbstractComparison;
import kea.evaluations.common.GrepHeader;

/**
  * This class collects the different results from  k optimization runs on one problem and one paramtersetting
  * of one algorithm.
  * It was implemented for the Hypervolume-metric to improve the automatic comparison of results.
  * This class computes additionally the average volume of the k optimization runs.
  */
  
public class Collector extends AbstractComparison {

  String directory;
  double average;
  String currentLine;
  Parameters param;
  
  File[] contents = {};
  String[] stringArray =  {};
  Vector vec = new Vector();
  CompResult compResult = new CompResult();

  public Collector() {
    param = new Parameters();
    param.setMembership(this.getClass().getName());

    param.addParameter(new Parameter("Directory", "", "The directory where the files are you want to collect in one file."));
    param.addParameter(new Parameter("resulting file name", this.getClass().getName(), "The name of the file containing the result(s) ending with .crs."));
  }



  private int countFiles(String directory){
    Kea.logger.logDebug("is counting the number of files in the directory ...", this);
    File dir = new File(directory);

    try{
    if (dir.isDirectory()){
      contents = dir.listFiles();
      Kea.logger.logDebug("Contents-Lnge: "+contents.length, this);
    }
    else throw new Exception("Directory is not a directory name!");
    }
    catch (Exception e)
    {
      e.printStackTrace();
    }
    Kea.logger.logDebug("Contents-Lnge: "+contents.length, this);
    return contents.length;
  }

  private void readData(File filename, int i){

    try{
    BufferedReader in = new BufferedReader(new FileReader(filename));
    // Skip Header.
    while(in.ready()) {
      currentLine = in.readLine();
      if (currentLine != null){
        // Search for end of header.
        if (currentLine.indexOf("###") != -1){
          if (Kea.logger.debugOn()) {
            Kea.logger.logDebug("end of header!", this);
          }
          break;
        }
      }
    }

    //Kea.logger.logInfo("Line = "+currentLine, this);
    while (in.ready()){
      currentLine = in.readLine();
      if (currentLine != null) {
        // Skip empty lines.
        if (!currentLine.equals("")) {
	  break;
        }
      }
    }

    // All empty lines up to the first individuum or end of file skipped.
    if (Kea.logger.debugOn()) {
      Kea.logger.logDebug("First not empty line = " + currentLine, this);
    }
    }
    catch (Exception e){}

    Kea.logger.logDebug("stringArray-Platz: "+i, this);

    stringArray[i]=currentLine;
    Kea.logger.logDebug("Datum in "+i+":"+stringArray[i], this);
  }

  public CompResult compare(){

  Kea.logger.logDebug("is comparing ...", this);

  compResult.setExportFileName((String)(param.getParameterByName("resulting file name")).getData());

  int numberOfFiles = countFiles((String)(param.getParameterByName("Directory")).getData());
  Kea.logger.logDebug("Contentslength: "+contents.length, this);

  stringArray= new String[(contents.length)+1];
  Kea.logger.logDebug("ArrayLnge: "+stringArray.length, this);

  for (int i=0; i<numberOfFiles; i++) {
  Kea.logger.logDebug("reads Data ...", this);
    readData(contents[i],i);
  }

  compResult.setSetOfParameters(GrepHeader.makeParams(contents[0].toString()));

  //compute the average volume
  for (int j=0; j < stringArray.length-1; j++){
  Kea.logger.logDebug("StringArray["+j+"]: "+stringArray[j], this);
    average += (new Double(stringArray[j])).doubleValue();
  }

  average = average / stringArray.length;
  stringArray[contents.length]= (new Double(average)).toString()+" Durchschnitt der obigen Volumina";

  compResult.setCompResult(stringArray);

  return compResult;
  }

  public void setParameter(Parameters Parameter){

    param = Parameter;
  }

  public Parameters getParameter(){

    return param;
  }

}
