/*
 * AbstractDynamicDisplay.java  -  This file is part of KEA.
 *
 * Copyright (C) 2002/03 PG 419, Department of Computer Science, 
 * University of Dortmund.          eMail: <peter.senft@udo.edu>
 */

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the KEA License Agreement which is based on 
 * the GNU General Public License as published by the Free Software 
 * Foundation; either version 2 of the License, or (at your option) 
 * any later version.  For details see License.txt.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

package kea.displays;
 
/**
 * This is the abstract class to extend when making your own display classes.
 * It only contains 2 methods to prepare/ update your input (most of the times a file)
 * for the choosen visualization tool (like GNUplot, R or others) and one to start the display.
 * 
 * @version 0.1.0
 */
public abstract class AbstractDynamicDisplay {

	/**
	 * The task of this method is to prepare/update and show the input data which you will need
	 * for the usage of a visualization tool.
	 * Most of the times you will create a file that contains understandable information for 
	 * any visualization tool you may choose.
	 * The dynamic displays need to communicate with your program.
	 * Therefore you will need to open a piped stream to your program
	 * and tell it about any updated data (from a file).
	 * You won't need a pipe if your program automatically detect changes of files.
	 *
	 * @param obj An object that contains some raw data you'll need to prepare
	 * @return True if the preparation was done successfully, else false
	 */
	public abstract boolean prepareAndShow(Object obj);

	public abstract void close();
	/**
	 * This method executes your visualization program.
	 * By executing it you get a Process object which you can set globally for
	 * later use in the updateAndShow method. You may need this Process to get a
	 * a stream from it.
	 * @return true if execution was successful, else false
	 */

	public abstract boolean execProgram();

}


