/*
 * AbstractStaticDisplay.java  -  This file is part of KEA.
 *
 * Copyright (C) 2002/03 PG 419, Department of Computer Science, 
 * University of Dortmund.          eMail: <peter.senft@udo.edu>
 */

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the KEA License Agreement which is based on 
 * the GNU General Public License as published by the Free Software 
 * Foundation; either version 2 of the License, or (at your option) 
 * any later version.  For details see License.txt.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

package kea.displays;

/**
 * This is the abstract class to extend when making your own static display classes.
 * It only contains a  method to prepare your input (most of the times a file)
 * for the choosen visualization tool (like GNUplot, R or others).
 * 
 * @version 0.1.0
 */
public abstract class AbstractStaticDisplay {

	/**
	 * The task of this method is to prepare data and then execute a program to display the data.
	 * @param obj An object that contains some raw data you'll need to prepare and show
	 * @return True if the preparation was done successfully, else false
	 */
	public abstract boolean prepareAndExec(Object obj);


}
