/*
 * EvaluationDisplay.java  -  This file is part of KEA.
 *
 * Copyright (C) 2002/03 PG 419, Department of Computer Science, 
 * University of Dortmund.          eMail: <peter.senft@udo.edu>
 */

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the KEA License Agreement which is based on 
 * the GNU General Public License as published by the Free Software 
 * Foundation; either version 2 of the License, or (at your option) 
 * any later version.  For details see License.txt.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

package kea.displays;

import java.io.BufferedWriter;
import java.io.FileWriter;

import kea.Data;
import kea.EvalResult;
import kea.Kea;
import kea.Type;
import kea.Value;

public class EvaluationDisplay extends AbstractStaticDisplay{

  /**
   * The task of this method is to prepare data and then execute a program to display the data.
   * @param obj An object that contains some raw data you'll need to prepare and show
   * @return True if the preparation was done successfully, else false
   */
  public boolean prepareAndExec(Object obj) {
    EvalResult[] results = ((EvalResult[])obj);
    String name="",datFileName = "";

    try{
      // create and save the raw data files for each evaluation result
      for(int n=0; n< results.length;n++){
      name="";
	datFileName = ((EvalResult)results[n]).getExportFileName();
	datFileName=datFileName.substring(0,datFileName.length()-4);
	name = name + datFileName + "_";
	BufferedWriter writer = new BufferedWriter(new FileWriter(datFileName+".dat"));

	writer.write("###################################################");
	writer.newLine();
	writer.write("# This file is for visualization purpose and contains raw data");
	writer.newLine();
	writer.write("# This file was automatically generated by Kea");
	writer.newLine();
	writer.write("###################################################");
	writer.newLine();
	Data[] result = ((EvalResult)results[n]).getEvalResult();
	String newLine ="";
	for(int i=0;i<result.length;i++){
	  Data data = result[i];
	  Value[] values = data.getObjVector();
	  int length = values.length;

	  for(int v=0;v<length;v++){
	    if(values[v].getTypeInfo()==Type.STRING) break;
	    newLine = newLine + " " +  values[v].getData();
	  }

	  writer.write(newLine);
	  writer.newLine();
	  newLine="";
	}
	writer.close();

      }

      // now create and save the gnuplot script
      name  = name.substring(0,name.length()-1);
//      name = "ParetoSets_"+name+".vis";
      name = name+".vis";
      BufferedWriter writer = new BufferedWriter(new FileWriter(name));
      writer.write("set title \" Evaluation : "
		   + ((EvalResult)results[0]).getEvalParameter().getMembership()
		   //                        .getEvalParameter().getMembership()
		   + " \"" );
      writer.newLine();
      String plotString ="plot ";
      for(int i=0;i<results.length;i++){
	String s = ((EvalResult)results[i]).getExportFileName() ;
	s = (s.substring(0,s.length()-4)) + ".dat";
	plotString = plotString + "\""+s+ "\""+ ",";
      }
      plotString = plotString.substring(0,plotString.length()-1);
      writer.write(plotString);
      writer.newLine();
      writer.write("pause -1");
      writer.newLine();
      writer.close();
      // exec gnuplot

      if(System.getProperty("os.name").indexOf("indows")!=-1){
	Runtime.getRuntime().exec("pgnuplot " + name);
      }
      else {
	Runtime.getRuntime().exec("gnuplot -persist " + name);
      }

      return true;
    }
    catch (Exception e){
      Kea.logger.logError("While trying to create file " + datFileName, e, this);
      return false;
    }

  }

}
