/*
 * OnTheFlyDisplay.java  -  This file is part of KEA.
 *
 * Copyright (C) 2002/03 PG 419, Department of Computer Science, 
 * University of Dortmund.          eMail: <peter.senft@udo.edu>
 */

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the KEA License Agreement which is based on 
 * the GNU General Public License as published by the Free Software 
 * Foundation; either version 2 of the License, or (at your option) 
 * any later version.  For details see License.txt.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

package kea.displays;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.PrintStream;

import kea.AlgoState;
import kea.Data;
import kea.Kea;
import kea.Parameters;
import kea.Value;

public class OnTheFlyDisplay extends AbstractDynamicDisplay {

	PrintStream gnuStream ;
	private Process visualizationProcess;
	String algoName,problemName;
	public boolean prepareAndShow(Object ast){

		boolean stop = ((AlgoState)ast).getAlgorithm().checkStopCondition();
		if(ast instanceof AlgoState){
			// first set up the plot script
			if( ((AlgoState)ast).getIterationCount()==1 || stop ){ //at 1st and last iteration we need to create/modify the script
				Parameters algoParams = ((AlgoState)ast).getAlgorithm().getParameter();
				algoName = algoParams.getMembership();

				Parameters probParams = ((AlgoState)ast).getAlgorithm().getProblem().getParameter();
				problemName = probParams.getMembership();

				int index = algoName.lastIndexOf(".");
				if(index !=-1) algoName = algoName.substring(index+1,algoName.length());
				index = problemName.lastIndexOf(".");
				if(index !=-1) problemName = problemName.substring(index+1,problemName.length());

				try{
					BufferedWriter writer = new BufferedWriter(new FileWriter("plotScript"));
					writer.write("set title \""+"Algorithm " +algoName.toUpperCase() + " solving Problem " + problemName.toUpperCase() +"\"");
					writer.newLine();
					writer.write("set xlabel \"" + "Current iteration "+ ((AlgoState)ast).getIterationCount()+"\"");
					writer.newLine();
					writer.write("plot \"plotData.dat\"");
					if(stop){ //modify to keep the last windows open
						Kea.logger.logDebug("Last generation displayed.");
						writer.newLine();
						writer.write("pause -1");
					}
					writer.newLine();
					writer.close();
					Kea.logger.logInfo("New plot script created.", this);
				}
				catch(Exception e){
					Kea.logger.logError("While trying to create file plotData", e, this);
					return false;
				}

			}
			// from now on we create the data file and the piped communication
			Data[] result = ((AlgoState)ast).getAlgorithm().getResults();
			try{
				BufferedWriter writer = new BufferedWriter(new FileWriter("plotData.dat"));

				writer.write("###################################################");
				writer.newLine();
				writer.write("# This file is for visualization purpose and contains raw data");
				writer.newLine();
				writer.write("# This file was automatically generated by Kea");
				writer.newLine();
				writer.write("###################################################");
				writer.newLine();

				String newLine ="";
				for(int i=0;i<result.length;i++){
					Data data = result[i];
					Value[] values = data.getObjVector();
					int length = values.length;
					for(int v=0;v<length;v++){
						newLine = newLine + " " +  values[v].getData();
					}
					writer.write(newLine);
					writer.newLine();
					newLine="";
				}
				writer.close();
				Kea.logger.logDebug("Updated plotData.dat.................OK ", this);

				if(gnuStream==null) {
					Kea.logger.logError("No stream found for communication with gnuplot !");
				}
				else {
					String str = "set title \""+"Algorithm " +algoName.toUpperCase() + " solving Problem " + problemName.toUpperCase() +"\"" + ";"+ "set xlabel \"Current iteration " + ((AlgoState)ast).getIterationCount() + "\"" +";";
					if( ((AlgoState)ast).getIterationCount()==1 || stop ){
						str = str +"load \"plotScript\""; // special call for first iteration and last
					}
					else{
						str = str +"replot"; // normal iteration replot call
					}

					gnuStream.println(str);
					Kea.logger.logDebug("Sent : " +str , this );
					gnuStream.flush();
				}
				return true;
		}
		catch (Exception e){
			Kea.logger.logError("While trying to create file plotData", e, this);
			return false;
		}
    	}

		return true;
	}

	public void close(){
		gnuStream.flush();
		gnuStream.close();
	}

	public boolean execProgram() {

		try{

			if(System.getProperty("os.name").indexOf("indows")!=-1){
				visualizationProcess = Runtime.getRuntime().exec("pgnuplot");
			}
			else {
				visualizationProcess = Runtime.getRuntime().exec("gnuplot -persist");
			}
			Kea.logger.logInfo("Executing gnuplot ", this);
			gnuStream = new PrintStream(visualizationProcess.getOutputStream());

			return true;
		}
		catch(Exception e){
			Kea.logger.logError("Could not execute gnuplot ", e, this);
			return false;
		}
	}

}

