/*
 * AbstractEvaluation.java  -  This file is part of KEA.
 *
 * Copyright (C) 2002/03 PG 419, Department of Computer Science, 
 * University of Dortmund.          eMail: <peter.senft@udo.edu>
 */

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the KEA License Agreement which is based on 
 * the GNU General Public License as published by the Free Software 
 * Foundation; either version 2 of the License, or (at your option) 
 * any later version.  For details see License.txt.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

package kea.evaluations;

import kea.EvalResult;
import kea.Parameters;

/**
 * Diese Klasse soll "einfache" Auswertungen "ubernehmen. Dazu geh"oren die Auswahl der Dimensionen, die dargestellt werden sollen, und die Bestimmung der Pareto-Front.
 * Mehrere Auswertungen sollen mittels Hintereinanderausf"uhrung kombiniert werden k"onnen.
 *
 * @version 1.0.0
 */
public abstract class AbstractEvaluation  {
    /**
     *
     */
    private String fileName = "";

    /**
     * Holds the Parameter for the Evaluation.
     */
    protected Parameters param;

    /**
     * Fuehrt die Auswertung auf einem als Parameter uebergebenen AlgoState aus und gibt einen entsprechenden Vector von EvalResult zurueck.
     */
    public abstract EvalResult[] evaluate();

    /**
     *
     */
    public abstract Parameters getParameter();

    /**
     *
     */
    public abstract void setParameter(Parameters parameter);

    /**
     *
     */
    public void setFileName(String newName) {
      fileName = newName;
    }

    /**
     *
     */
    public String getFileName() {
      return fileName;
    }
}
