/*
 * CheckParams.java  -  This file is part of KEA.
 *
 * Copyright (C) 2002/03 PG 419, Department of Computer Science, 
 * University of Dortmund.          eMail: <peter.senft@udo.edu>
 */

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the KEA License Agreement which is based on 
 * the GNU General Public License as published by the Free Software 
 * Foundation; either version 2 of the License, or (at your option) 
 * any later version.  For details see License.txt.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

package kea.evaluations.common;

import java.util.Vector;

import kea.Kea;

/**
 * @author Tim Hohm
 * @version $Revision: 723 $
 */
public class CheckParams {

  public CheckParams() {
  }

  public static Vector checkGens(Vector it, DataChooser chooser){
    Vector toCheck = it;
    int lastValidIndex = chooser.getNumberOfLastGeneration();
    // If there is an not int entry (besides "last") it gets deleted ...
    for (int i = 0; i < toCheck.size(); i++){
      if (((String)toCheck.elementAt(i)).indexOf("last") != -1){
	toCheck = new Vector();
	toCheck.addElement("last");
	i = 1;
      }
      else{
        try{
          int tmp = Integer.parseInt((String)toCheck.elementAt(i));
          // ... and if the generation index is greater than the total number of generations
          // it gets deleted too.
          if (tmp > lastValidIndex){
            toCheck.removeElementAt(i);
            i = i-1;
            Kea.logger.logWarn("An int greater than the number of generations in generation list! Parmeters changed automatically!", new CheckParams());
          }
        }
        catch(NumberFormatException e){
          Kea.logger.logWarn("A non int in generation list! Parmeters changed automatically!", new CheckParams());
          toCheck.removeElementAt(i);
          i = i-1;
        }
      }
    }
    // If the vector is empty take all generations in the .ast file.
    if (toCheck.size() == 0){
      if (lastValidIndex > 0){
        String gens = "";
        for (int i = 1; i < lastValidIndex; i++){
          gens = gens + Integer.toString(i) + " ";
        }
        gens = gens + Integer.toString(lastValidIndex);
        toCheck = chooser.resolveString(gens);
      }
    }
    else{
      // If one entry is 0, then remove it. If the o entry is the only entry
	// a vector with size 0 is the result.
      for (int i = 0; i< toCheck.size(); i++){
        if (((String)toCheck.elementAt(i)).equals("0")){
          toCheck.removeElementAt(i);
        }
      }
	try{
        // Check for right order of the generationindecies.
        if (toCheck.size()>1){
          int last = Integer.parseInt((String)toCheck.firstElement());
          for (int i = 1; i< toCheck.size(); i++){
            if (Integer.parseInt((String)toCheck.elementAt(i)) <= last){
              String temp = (String)toCheck.elementAt(i);
              toCheck.removeElementAt(i);
              // If an element is sorted in wrong it gets sorted in right.
              for (int j = 0; j < i; j++){
                if(Integer.parseInt((String)toCheck.elementAt(j)) > Integer.parseInt(temp)){
                  toCheck.insertElementAt(temp, j);
                  j = i;
                }
                // If an element is in the vector twice it gets sorted out.
                else if (Integer.parseInt((String)toCheck.elementAt(j)) == Integer.parseInt(temp)){
                  j = i;
                }
              }
            }
            if (toCheck.size() > i){
              last = Integer.parseInt((String)toCheck.elementAt(i));
            }
          }
        }
      }
	catch(NumberFormatException theException){
	  // If an exception is thrown it can be ignored because it result from the
	  // invalid cast of the String "last" to an int.
	}
    }
    return toCheck;
  }

  public static Vector checkVars(Vector it, DataChooser chooser, int dim){
    Vector toCheck = it;
    // If there is an not int entry it gets deleted ...
    for (int i = 0; i < toCheck.size(); i++){
      try{
        int tmp = Integer.parseInt((String)toCheck.elementAt(i));
        // ... and if the index is greater than the dim it gets deleted too.
        if (tmp > dim){
          toCheck.removeElementAt(i);
          i = i-1;
          Kea.logger.logWarn("An int greater than the number of variables in variable list! Parmeters changed automatically!", new CheckParams());
        }
      }
      catch(NumberFormatException e){
        Kea.logger.logWarn("A non int in variable list! Parmeters changed automatically!", new CheckParams());
        toCheck.removeElementAt(i);
        i = i-1;
      }
    }
    // If the vector is empty take all vars.
    if (toCheck.size() == 0){
      if (dim>0){
        String vars = "";
        for (int i = 1; i < dim; i++){
          vars = vars + Integer.toString(i) + " ";
        }
        vars = vars + Integer.toString(dim);
        toCheck = chooser.resolveString(vars);
      }
    }
    else{
      // If one entry is 0, then remove it.
      for (int i = 0; i< toCheck.size(); i++){
        if (((String)toCheck.elementAt(i)).equals("0")){
          toCheck.removeElementAt(i);
        }
      }
      // Check for right order of the generationindecies.
      if (toCheck.size()>1){
        int last = Integer.parseInt((String)toCheck.firstElement());
        for (int i = 1; i< toCheck.size(); i++){
          if (Integer.parseInt((String)toCheck.elementAt(i)) <= last){
            String temp = (String)toCheck.elementAt(i);
            toCheck.removeElementAt(i);
            // If an element is sorted in wrong it gets sorted in right.
            for (int j = 0; j < i; j++){
              if(Integer.parseInt((String)toCheck.elementAt(j)) > Integer.parseInt(temp)){
                toCheck.insertElementAt(temp, j);
                j = i;
              }
              // If an element is in the vector twice it gets sorted out.
              else if (Integer.parseInt((String)toCheck.elementAt(j)) == Integer.parseInt(temp)){
                j = i;
              }
            }
          }
          if (toCheck.size() > i){
            last = Integer.parseInt((String)toCheck.elementAt(i));
          }
        }
      }
    }
    return toCheck;
  }
}
