/*
 * GenAfterEvalCount.java  -  This file is part of KEA.
 *
 * Copyright (C) 2002/03 PG 419, Department of Computer Science, 
 * University of Dortmund.          eMail: <peter.senft@udo.edu>
 */

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the KEA License Agreement which is based on 
 * the GNU General Public License as published by the Free Software 
 * Foundation; either version 2 of the License, or (at your option) 
 * any later version.  For details see License.txt.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

package kea.evaluations.common;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;

import kea.Kea;

/**
 * @author Tim Hohm
 * @version $Revision: 723 $
 */
public class GenAfterEvalCount {

  private String amountOfEvals;
  private String run;

  public GenAfterEvalCount(String it) {
    run = it;
  }

  /**
   * Sorts out the generation with the right evaluationCount.
   */
  public String makeOutGen(String it){
    it = it.trim();
    int whichGen = Integer.parseInt(it);
    String curLine;
    int genCount = 0;
    boolean stop = false;
    try {
      BufferedReader in = new BufferedReader(new FileReader(run));
      // Skip Header.
      while(in.ready()) {
        curLine = in.readLine();
        if (curLine != null){
          // Search for end of header.
          if (curLine.indexOf("###") != -1){
            if (Kea.logger.debugOn()) {
              Kea.logger.logDebug("Ende Header!", this);
            }
            break;
          }
        }
      }

      while (in.ready()){
        if (stop){
          break;
        }
        curLine = in.readLine();
        if (curLine != null) {
          // Skip empty lines.
          while (curLine.equals("")) {
            curLine = in.readLine();
            if (curLine == null)break;
          }
          // All empty lines up to the first individuum or end of file skipped.
          if (Kea.logger.debugOn()) {
            Kea.logger.logDebug("First not empty line = " + curLine, this);
          }
          // Increase generationcounter.
          genCount ++;

          while (!curLine.equals("")) {
            // If the current line is the evaluationCount line ...
            if (curLine.indexOf("evaluationCount") != -1){
              curLine = curLine.substring(curLine.indexOf("=") +2);
              int f = Integer.parseInt(curLine);
              if (f >= whichGen){
                stop = true;
              }
            }
            curLine = in.readLine();
            if (curLine == null)break;
          }
        }
      }
    }
    catch (FileNotFoundException exc) {
      throw new EvalException("File "+run+" not found. No operation on this file possible.");
    }
    catch (IOException k) {
      throw new EvalException("An IO-Error occured while reading run data. Please try again.");
    }
    if (stop){
      if (Kea.logger.debugOn()) {
        Kea.logger.logDebug("Which generation is right? " + Integer.toString(genCount), this);
      }
      return Integer.toString(genCount);
    }else{
      Kea.logger.logWarn("There is no generation with number of evaluations >= "+it+".", this);
      return "0";
    }
  }

}
