/*
 * GrepHeader.java  -  This file is part of KEA.
 *
 * Copyright (C) 2002/03 PG 419, Department of Computer Science, 
 * University of Dortmund.          eMail: <peter.senft@udo.edu>
 */

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the KEA License Agreement which is based on 
 * the GNU General Public License as published by the Free Software 
 * Foundation; either version 2 of the License, or (at your option) 
 * any later version.  For details see License.txt.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

package kea.evaluations.common;

import java.io.BufferedReader;
import java.io.FileReader;
import java.util.HashMap;

import kea.Datacontainer;
import kea.Kea;
import kea.Parameter;
import kea.Parameters;

/**
 * <code>GrepHeader</code> allows it to read the header of
 * .ast- and .ers-files.  Since it's only important
 * method is <code>makeParams()</code> it's useless.
 *
 * @author Tim Hohm
 * @version $Revision: 723 $
 */
public class GrepHeader {

  private static int counter = 0;
  private static String curLine;
  private static Parameters[] ast;
  private static final String GZEI = Datacontainer.GZIP_END_IDENTIFIER; //"# A A A"
  private static final String KCI = Datacontainer.KEA_CLASS_IDENTIFIER; //">>>"
  private static final String HEI = Datacontainer.HEADER_END_IDENTIFIER;//"###"
  /*
   * Creates a new <code>GrepHeader</code> instance.
   * Its only use is to replace "this" in Kea.logger-messages,
   * because they're called from static methods which doesn't
   * allow "this".  If there are problems make it public again!
   */
  private GrepHeader(){}

  /*
   * copyParmas()<em>without any use for the moment</em>,
   * maybe it will be removed again ...
   * idea was to avoid casting from ascii to parameters
   * and then back while exporting ...
   */
  /* public static void copyParams(String astName, String appendTo){
     try {
     BufferedReader in = new BufferedReader(new FileReader(astName));
     PrintWriter outPrint =
     new PrintWriter
     ( new BufferedWriter
     ( new FileWriter(appendTo, true)));

     // Skip Header.
     while(in.ready()) {
     curLine = in.readLine();
     if (!curLine.startsWith(GZEI)) continue;
     else break;
     }
     while(in.ready()) {
     if (!curLine.startsWith(HEI))
     outPrint.println(curLine);
     }
     }
     catch (Exception exc){
     Kea.logger.logError("Failure while reconstructing " + astName
     + " parameters.", new GrepHeader() );
     }

     }
  */

  /**
   * <code>makeParams</code> reads the header of a .ast- or .ers-file
   * and tries to reconstruct the parameters from it.
   *
   * @param astName <code>String</code> names the file to work with.
   * @return <code>Parameters[]</code> the parameters found.
   */
  public static Parameters[] makeParams(String astName){

    /* // If you're using the GZIP-Parameters-header, the following
     * // lines could accelerate makeParams 
     *
     * ast = Datacontainer.readSetOfParametersGZipFrom(astName);
     * if(ast != null) return ast;
     *
     * // else try to find out the parameters manually
     *
     * Kea.logger.logWarn("\nTrying to find out parameters without GZip-Header now ...\n",new GrepHeader());
     */

    HashMap astParamsInfo = countLines(astName);
    int nrParameters = ((Integer)astParamsInfo.get("counter")).intValue();

    if (nrParameters != 0){
      ast = new Parameters[nrParameters];
      Parameters tmpParameters = new Parameters();

      /* initialize ast-Array
       */
      for(int i=0; i < nrParameters; i++)
	ast[i] = new Parameters();//tmpParameters;

      try {
	BufferedReader in = new BufferedReader(new FileReader(astName));

	// Skip Header.
	while(in.ready()) {
	  curLine = in.readLine();
	  if (!curLine.startsWith(GZEI)){
	    if(!curLine.startsWith(KCI))
	      continue;
	    else break;
	  }
	  else{
	    curLine = in.readLine();
	    /* if gZipEndIdentifier was found, we look at the next line
	     * if we find the keaClassIdentifier which we want to keep
	     */
	    break;
	  }
	}

	for (int i=0; i<nrParameters; i++){
	  int subNrOfParam =
	    ((Integer)astParamsInfo.get(new Integer(i+1))).intValue();
	  Parameter[] pa = new Parameter[subNrOfParam];

	  if (!in.ready()) throw new Exception();

	  int j = 0;
	  while(j<subNrOfParam){
	    //if (curLine == null) break;
	    if (curLine.startsWith(HEI)){
	      break;
	    }
	    else if (curLine.startsWith(KCI)){
	      //System.out.println(i+curLine);
	      ast[i].setMembership
		(curLine.substring(KCI.length(),
				   curLine.length())
		 .trim());
	    }

	    else if (!curLine.startsWith("#")
		     && !curLine.trim().equals("")){
	      //because those lines were ignored during countLines()

	      int equalIdx = curLine.indexOf("=");
	      int rangeIdx = curLine.indexOf("|");
	      int hashIdx = curLine.indexOf("#");

	      if (hashIdx != -1){
		if (rangeIdx == -1) rangeIdx = hashIdx;

		pa[j] = new Parameter
		  ( curLine.substring(0, equalIdx-1).trim()
		    , curLine.substring(equalIdx+1,rangeIdx-1).trim()
		    , curLine.substring(hashIdx+1,curLine.length()).trim());

		while (rangeIdx < hashIdx){
		  Kea.logger.logWarn("Sorry can't set ranges in this version of GrepHeader", new GrepHeader());
		  /* just to come out of the loop for the moment */
		  rangeIdx = hashIdx;
		  //pa[j].setRange(...)
		}
	      }
	      else pa[j] = new Parameter
		     ( curLine.substring(0, equalIdx-1).trim()
		       , curLine.substring(equalIdx+1, curLine.length()).trim()
		       , "no parameter, but run specific data evaluated by Kea");

	      j++;
	    }
	    curLine = in.readLine();
	  }
	  ast[i].setParameterArray(pa);
	}
      } catch (Exception exc){
	Kea.logger.logWarn
	  ( "Failure while reconstructing "
	    + astName + " parameters.", new GrepHeader()
	    );
      }

    }
    return ast;
  }

  private static HashMap countLines(String astName){

    counter = 0;
    int nrOfParam = 0;
    HashMap hashParametersInfo = new HashMap();
    hashParametersInfo.put("counter", new Integer(counter));


    try {
      BufferedReader in = new BufferedReader(new FileReader(astName));
      while(in.ready()) {
        curLine = in.readLine();
	if (!curLine.startsWith(GZEI)){
	  if (!curLine.startsWith(KCI)) continue;
	  else {
	    counter++;
	    break;
	  }
	}
	else break;
      }

      if (!in.ready()) throw new Exception();

      while(in.ready()){
	curLine = in.readLine();
        if (curLine.startsWith(KCI)){

	  // case:  begin of first set of param found
	  if (counter == 0){
	    //System.out.println(curLine);
	    counter++;
	  }

	  // case:  some sets of param were already found
	  else {
	    hashParametersInfo.put
	      (new String("counter"), new Integer(counter));
	    hashParametersInfo.put
	      (new Integer(counter), new Integer(nrOfParam));
	    Kea.logger.logInfo
	      ( "counter"+counter+"nrOfParam"+nrOfParam, new GrepHeader() );
	    //System.out.println(curLine);
	    counter++;
	    nrOfParam = 0;
	  }
	}
	else if(curLine.startsWith(HEI)){
	  hashParametersInfo.put
	    (new String("counter"), new Integer(counter));
	  hashParametersInfo.put
	    (new Integer(counter), new Integer(nrOfParam));
	  Kea.logger.logInfo
	    ( "counter"+counter+"nrOfParam"+nrOfParam, new GrepHeader() );
	  break;
        }
	else if(!curLine.trim().equals("")
		//&& !curLine.trim().equals("Parameters:")
		&& !curLine.startsWith("#")){
	  nrOfParam++;
	}
      }
    }
    catch(Exception exc){
      Kea.logger.logWarn
	( "Failure while reading "
	  + astName +" parameters.", new GrepHeader());
    }
    Kea.logger.logInfo("Left countLines", new GrepHeader());
    return hashParametersInfo;
  }
}
