/*
 * Pareto.java  -  This file is part of KEA.
 *
 * Copyright (C) 2002/03 PG 419, Department of Computer Science, 
 * University of Dortmund.          eMail: <peter.senft@udo.edu>
 */

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the KEA License Agreement which is based on 
 * the GNU General Public License as published by the Free Software 
 * Foundation; either version 2 of the License, or (at your option) 
 * any later version.  For details see License.txt.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

package kea.evaluations.common;

import java.util.Vector;

import kea.Data;
import kea.Kea;
import kea.Value;
import kea.algorithms.common.OptSet;

/**
 * @author Tim Hohm
 * @version $Revision: 723 $
 */
public class Pareto {

  private String run;
  private Vector gens;
  private Vector dec;
  private Vector obj;
  private Data[] results;
  private Vector individuumOld;
  private Vector individuum;
  private Vector set;
  private Vector remove;
  private Data[] returnValue;
  private Vector temp;
  private Data empty;
  private Vector result;
  private int newDimObj;
  private int newDimDec;
  private OptSet comp;

  public Pareto(String run) {
    this.run = run;
    individuum = new Vector();
    result = new Vector();
    // Build an empty data entry (empty line in result file).
    String it = "empty";
    Value[] nada = new Value[1];
    nada[0] = new Value(it);
    empty = new Data(nada, nada);
  }

  public Data[] evaluate(String generations, String decVar, String objVar){
    gens = DataChooser.resolveString(generations);
    gens = CheckParams.checkGens(gens, new DataChooser(run));
    dec = DataChooser.resolveString(decVar);
    obj = DataChooser.resolveString(objVar);
    newDimDec = dec.size();
    newDimObj = obj.size();
    buildSets(decVar, objVar);
    // Put remaining set into returnValue.
    returnValue = new Data[result.size()];
    for (int i = 0; i < result.size(); i++){
      returnValue[i] = (Data)result.elementAt(i);
    }
    return returnValue;
  }

  private void buildSets(String decVar, String objVar){
    // Run through every single generation.
    for (int i=0; i < gens.size(); i++){
      DataChooser chooser = new DataChooser(run);
      set = new Vector();
      temp = new Vector();
      results = chooser.choose((String)gens.elementAt(i), decVar, objVar);
      individuumOld = chooser.getIndividuum();
      // Run through generations and build to vectors: one containing all individums called set,
      // one containing all the status lines called temp
      for (int j=0; j < results.length; j++){
        if (((Integer)individuumOld.elementAt(j)).intValue() == 0){
          set.addElement(results[j]);
        }
        if (((Integer)individuumOld.elementAt(j)).intValue() == 1){
          temp.addElement(results[j]);
        }
      }
      // Create Data[] for OptSet.
      Data[] gen = new Data[set.size()];
      for (int j = 0; j < set.size(); j++){
        gen[j] = (Data)set.elementAt(j);
      }
      // OptSet computes the paretoset of the individums given in gen.
      comp = new OptSet(set.size());
      comp.setData(gen);
      Data[] tempData = comp.nonDomSet();
      // Append individuums, status lines and empty lines to vector result.
      for (int j = 0; j < tempData.length; j++){
        if (Kea.logger.debugOn()) {
          Kea.logger.logDebug("Add to result = " + tempData[j].toString(), this);
        }
        result.addElement(tempData[j]);
        individuum.addElement(new Integer(0));
      }
      for (int j = 0; j < temp.size(); j++){
        result.addElement((Data)temp.elementAt(j));
        individuum.addElement(new Integer(1));
      }
      result.addElement((Data)empty);
      individuum.addElement(new Integer(2));
    }
  }

  public Vector getIndividuum(){
    return individuum;
  }

  public String getNewDimObj(){
    String dim = new Integer(newDimObj).toString();
    return (dim);
  }

  public String getNewDimDec(){
    String dim = new Integer(newDimDec).toString();
    return (dim);
  }
}
