/*
 * Created on 23.10.2003
 *
 * To change the template for this generated file go to
 * Window>Preferences>Java>Code Generation>Code and Comments
 */
//package kea.problems;

/**
 * @author suher
 *
 * To change the template for this generated type comment go to
 * Window>Preferences>Java>Code Generation>Code and Comments
 */
package kea.problems.drilling;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.net.Socket;
import java.net.UnknownHostException;
import java.text.DecimalFormat;
import java.util.Vector;

import kea.Kea;
import kea.Parameters;
import kea.Range;
import kea.Type;
import kea.Value;
import kea.problems.AbstractProblem;

/**
 * This class serves as an interface between <code>KEA</code>
 * and the Evolver-Software written by Thomas Michelitsch. <br>
 * 
 * Since this class uses a <code>BufferedReader</code> 
 * as a class attribute, it is not serializable. This means, 
 * optimization runs involving this class cannot be stopped 
 * and resumed afterwards. <br>
 * 
 * Stopping a run involving this class by pressing CTRL-C
 * can cause the Evolver software to hang. :( 
 * 
 * @author PG 419, University of Dortmund (hp)
 * @version $Revision: 414 $
 */
public class Drilling extends AbstractProblem {

	/**
	 * Socket for communication with Evolver.
	 */
	private Socket socket;
	

	/**
	 * Writer for sending data to Evolver.
	 */
	private PrintWriter out;

	/**
	 * Reader for reading data from Evolver.
	 */
	private BufferedReader in;

	/**
	 * Number formating for Evolver communication.
	 */
	DecimalFormat fmt = new DecimalFormat("0.000000000000");

	/**
	 * Contains the parameters for this problem.
	 */
	private Parameters param;

	/**
	 * IP address of computer running the Evolver.
	 * Default is localhost (127.0.0.1),
	 */
	private final String IP_ADDR = "127.0.0.1";

	/**
	 * Port where Evolver is listening.
	 * Default is 6201.
	 */
	private final int IP_PORT = 6201;

	/**
	 * Selected objective function.
	 * Default is 1. (Weighted sum of objectives and restrictions.)
	 */
	private int objective = 1;

	/**
	 * Number of points in frequency polygon.
	 * Default is 2. (Smallest possible value.)
	 */
	private int dim3D = 2;

	/**
	 * Creates a socket connection to the Evolver software.
	 * Sets all adjustable parameters to their defaults.
	 */
	public Drilling() {
		try {
			// Create socket connection.
			socket = new Socket(IP_ADDR, IP_PORT);
			out = new PrintWriter(socket.getOutputStream());
			in =
				new BufferedReader(
					new InputStreamReader(socket.getInputStream()));
		} catch (UnknownHostException e) {
			Kea.logger.logError(
				"Problem while connecting to Evolver: ",
				e,
				this);
		} catch (IOException e) {
			Kea.logger.logError(
				"Problem while connecting to Evolver: ",
				e,
				this);
		}

		param = new Parameters();
		param.setMembership(this.getClass().getName());
		param.addParameter(
			"Dimension3D",
			5,
			"number of points in frequency polygon");
		param.addParameter(
			"objective function",
			1,
			"1 Weighted Sum; "
				+ "2 Cooling vs Variance; "
				+ "3 Cooling vs DLength; "
				+ "4 Cooling vs CCLength; "
				+ "5 Cooling vs DLength vs CCLength");
		if (Kea.logger.infoOn()) {
			Kea.logger.logInfo(
				"Since Drilling ist _not_ serializable, "
					+ "optimizations can not be resumed after they were stopped."
					+ " See documentation for details.",
				this);
		}
	}
	
	/**
		 * Takes an array of concretely variable values and repair him by means of ???
		 * @param genotype an array of decision variable values which should be scored
		 * @return the corresponding array of objective values
		 */
	public Value[] repair(Value[]genotype)
			{
				return genotype;
			}
	
	

	/* (non-Javadoc)
	 * @see kea.problems.AbstractProblem#score(kea.Value[])
	 */
	/* (non-Javadoc)
	 * @see kea.problems.AbstractProblem#score(kea.Value[])
	 */
	/**
	 * Takes an array of decision variable values
	 * and calculates the corresponding array of objective values
	 * (depending on the selected objective function).
	 *
	 * @param genotype an array of decision variable values which should be scored
	 * @return the corresponding array of objective values
	 */
	public Value[] score(Value[] genotype) {
		Vector received = new Vector();
		Value[] results = new Value[this.getDimObjectiveSpace()];
		// Send ID 301 (INTERCOM_ID_EXTERNAL_GETINDIVIDUAL).
		sendInt(301);
		// Send Dimension3D ...
		sendInt(dim3D);
		// Send coordinates ...
		for (int i = 0; i < dim3D; i++) {
			// Send x coordinate.
			double x = ((Number) genotype[3 * i].getData()).doubleValue();
			sendDouble(x);
			// Send y coordinate.
			double y = ((Number) genotype[3 * i + 1].getData()).doubleValue();
			sendDouble(y);
			// Send z coordinate.
			double z = ((Number) genotype[3 * i + 2].getData()).doubleValue();
			sendDouble(z);
			// Send drilling direction.
			// Not yet considered.
			int drilling = 0;
			sendInt(drilling);
			// Debugging ...
			if (Kea.logger.debugOn())
				Kea.logger.logDebug(
					"Send to Evolver: ("
						+ x
						+ ", "
						+ y
						+ ", "
						+ z
						+ ") / Drilling direction: "
						+ drilling,
					this);
		}
		// Send data now.
		out.flush();
		// Receive results ...
		// Check ID, must be 302 (INTERCOM_ID_EXTERNAL_PUTEVALUATION). 
		if (readInt() != 302)
			Kea.logger.logError("Received wrong ID.", this);
		// Collect received values.
		for (int i = 0; i < 12; i++) {
			received.addElement(new Double(readDouble()));
		}
		// Debugging ...
		if (Kea.logger.debugOn())
			Kea.logger.logDebug("Received from Evolver: " + received, this);
		// Calculate result from received values ...
		switch (this.objective) {
			case 1 : // Weighted Sum
				// CoolingPowerWorstValue + 0.05 CoolingPowerVariance 
				// + 1000000 RestrictionBoundingBoxCollision + 1000000 RestrictionObjectCollision
				results[0] =
					new Value(
						((Number) received.get(0)).doubleValue()
							+ 0.05 * ((Number) received.get(1)).doubleValue()
							+ 1000000 * ((Number) received.get(5)).doubleValue()
							+ 1000000 * ((Number) received.get(6)).doubleValue());
				break;
			case 2 : // Cooling vs Variance
				// CoolingPowerWorstValue 
				// + 1000000 RestrictionBoundingBoxCollision + 1000000 RestrictionObjectCollision
				results[0] =
					new Value(
						((Number) received.get(0)).doubleValue()
							+ 1000000 * ((Number) received.get(5)).doubleValue()
							+ 1000000 * ((Number) received.get(6)).doubleValue());
				// CoolingPowerVariance 
				// + 1000000 RestrictionBoundingBoxCollision + 1000000 RestrictionObjectCollision
				results[1] =
					new Value(
						((Number) received.get(1)).doubleValue()
							+ 1000000 * ((Number) received.get(5)).doubleValue()
							+ 1000000 * ((Number) received.get(6)).doubleValue());
				break;
			case 3 : // Cooling vs DrillingLength
				// CoolingPowerWorstValue + 0.05 CoolingPowerVariance
				// + 1000000 RestrictionBoundingBoxCollision + 1000000 RestrictionObjectCollision
				results[0] =
					new Value(
						((Number) received.get(0)).doubleValue()
							+ 0.05 * ((Number) received.get(1)).doubleValue()
							+ 1000000 * ((Number) received.get(5)).doubleValue()
							+ 1000000 * ((Number) received.get(6)).doubleValue());
				// DrillingLength
				// + 1000000 RestrictionBoundingBoxCollision + 1000000 RestrictionObjectCollision
				results[1] =
					new Value(
						((Number) received.get(10)).doubleValue()
							+ 1000000 * ((Number) received.get(5)).doubleValue()
							+ 1000000 * ((Number) received.get(6)).doubleValue());
				break;
			case 4 : // Cooling vs CoolingCircuitLength  
				// CoolingPowerWorstValue + 0.05 CoolingPowerVariance
				// + 1000000 RestrictionBoundingBoxCollision + 1000000 RestrictionObjectCollision
				results[0] =
					new Value(
						((Number) received.get(0)).doubleValue()
							+ 0.05 * ((Number) received.get(1)).doubleValue()
							+ 1000000 * ((Number) received.get(5)).doubleValue()
							+ 1000000 * ((Number) received.get(6)).doubleValue());
				// CoolingCircuitLength  
				// + 1000000 RestrictionBoundingBoxCollision + 1000000 RestrictionObjectCollision
				results[1] =
					new Value(
						((Number) received.get(11)).doubleValue()
							+ 1000000 * ((Number) received.get(5)).doubleValue()
							+ 1000000 * ((Number) received.get(6)).doubleValue());
				break;
			case 5 : // Cooling vs CoolingCircuitLength vs DrillingLength
				// CoolingPowerWorstValue 
				// + 1000000 RestrictionBoundingBoxCollision + 1000000 RestrictionObjectCollision
				results[0] =
					new Value(
						((Number) received.get(0)).doubleValue()
							+ 0.05 * ((Number) received.get(1)).doubleValue()
							+ 1000000 * ((Number) received.get(5)).doubleValue()
							+ 1000000 * ((Number) received.get(6)).doubleValue());
				// DrillingLength + CoolingCircuitLength  
				// + 1000000 RestrictionBoundingBoxCollision + 1000000 RestrictionObjectCollision
				results[1] =
					new Value(
						((Number) received.get(10)).doubleValue()
							+ 1000000 * ((Number) received.get(5)).doubleValue()
							+ 1000000 * ((Number) received.get(6)).doubleValue());
				// DrillingLength + CoolingCircuitLength  
				// + 1000000 RestrictionBoundingBoxCollision + 1000000 RestrictionObjectCollision
				results[2] =
					new Value(
						((Number) received.get(11)).doubleValue()
							+ 1000000 * ((Number) received.get(5)).doubleValue()
							+ 1000000 * ((Number) received.get(6)).doubleValue());
				break;
			default : // weighted sum
				// CoolingPowerWorstValue + 0.05 CoolingPowerVariance 
				// + 1000000 RestrictionBoundingBoxCollision + 1000000 RestrictionObjectCollision
				results[0] =
					new Value(
						((Number) received.get(0)).doubleValue()
							+ 0.05 * ((Number) received.get(1)).doubleValue()
							+ 1000000 * ((Number) received.get(5)).doubleValue()
							+ 1000000 * ((Number) received.get(6)).doubleValue());
				break;
		}
		// Increase evaluation counter ...
		this.setEvalCount(this.getEvalCount() + 1);
		// Debugging ...
		if (Kea.logger.debugOn()) {
			// EvalCount
			Kea.logger.logDebug("EvalCount = " + getEvalCount(), this);
			// Objective function value ...
			Vector resvec = new Vector();
			for (int i = 0; i < results.length; i++) {
				resvec.addElement((Number) results[i].getData());
			}
			Kea.logger.logDebug(
				"Calculated objective function value: " + resvec,
				this);
		}
		return results;
	}

	/**
	 * Calculates the dimension of decision space
	 * (depending on the dim3D-Parameter).
	 *
	 * @return dimension of decision space
	 */
	public int getDimDecisionSpace() {
		// Each point needs three coordinates.
		// (Drilling direction not yet considered.)
		return 3 * dim3D;
	}

	/**
	 * Returns an int representing the dimension of objective space
	 * (depending on the selected objective function).
	 *
	 * @return dimension of objective space
	 */
	public int getDimObjectiveSpace() {
		switch (objective) {
			case 1 : // Weighted Sum
				return 1;
			case 2 : // Cooling vs Variance
				return 2;
			case 3 : // Cooling vs DLength
				return 2;
			case 4 : // Cooling vs CCLength
				return 2;
			case 5 : // Cooling vs DLength vs CCLength
				return 3;
			default : // Weighted Sum
				return 1;
		}
	}

	/**
	 * Returns an array of <code>Range</code> objects describing the allowed ranges 
	 * for each descision variable.
	 *
	 * @return an array of <code>Range</code> objects for each decision variable
	 */
	public Range[] getRanges() {
		// Each point needs three coordinates.
		// (Drilling direction not yet considered.)
		Range[] ranges = new Range[3 * dim3D];
		for (int i = 0; i < dim3D; i++) {
			// x \in [-2, 2]
			ranges[3 * i] = new Range(new Double(-2), new Double(2));
			// y \in [-2, 2]
			ranges[3 * i + 1] = new Range(new Double(-2), new Double(2));
			// z in [0, 2]
			ranges[3 * i + 2] = new Range(new Double(0), new Double(2));
		}
		return ranges;
	}

	/**
	 * Returns an array of <code>Type</code> objects describing the decision variable types.
	 *
	 * @return a Type[] containing exactly one <code>Type</code> object for each decision variable
	 */
	public Type[] getDecTypes() {
		Type[] types = new Type[3 * dim3D];
		// Each point needs three coordinates.
		// (Drilling direction not yet considered.)
		for (int i = 0; i < 3 * dim3D; i++) {
			// Coordinates as double values.
			types[i] = new Type(Type.DOUBLE);
		}
		return types;
	}

	/**
	 * Returns an array of <code>Type</code> objects describing the objective variable types.
	 *
	 * @return a <code>Type[]</code> containing exactly one <code>Type</code> objects for each objective variable
	 */
	public Type[] getObjTypes() {
		// Every objective has type double.
		Type[] types = new Type[this.getDimObjectiveSpace()];
		for (int i = 0; i < types.length; i++) {
			types[i] = new Type(Type.DOUBLE);
		}
		return types;
	}

	/**
	 * Returns false, if the given array of decision variable values
	 * violates any of the problem's constraints.
	 *
	 * @param toVerify a Value[] that should be verified
	 * @return <code>true</code> or <code>false</code>, depending on the violation of constraints
	 */
	public boolean verify(Value[] toVerify) {
		// Has toVerify the correct length?
		// For each dimension 3 coordinates are needed.
		// (4, if drilling direction is consdired.) 
		if (toVerify.length != dim3D * 3)
			return false;
		//Check if coordinates have the correct range.
		Range[] ranges = this.getRanges();
		for (int i = 0; i < toVerify.length; i++) {
			if (((Number) ranges[i].getMinVal()).doubleValue()
				>= ((Number) toVerify[i].getData()).doubleValue()
				|| ((Number) ranges[i].getMaxVal()).doubleValue()
					<= ((Number) toVerify[i].getData()).doubleValue())
				return false;
		}
		return true;
	}

	/**
	 * Returns the <code>Parameters</code> object of this class.
	 * @return the <code>Parameters</code> object of this class
	 */
	public Parameters getParameter() {
		return this.param;
	}

	/**
	 * Sets a new <code>Parameters</code> object for this problem.
	 * @param parameter the <code>Parameters</code> object that will be set
	 */
	public void setParameter(Parameters parameter) {
		this.param = parameter;
		this.objective =
			((Number) param.getParameterByName("objective function").getData())
				.intValue();
		this.dim3D =
			((Number) param.getParameterByName("Dimension3D").getData())
				.intValue();
	}

	/**
	 * Sends disconnect signal to Evolver.
	 */
	public void cleanUp() {
		if (Kea.logger.debugOn())
			Kea.logger.logDebug("Starting cleanUp() ...", this);
		try {
			// Send ID 1001 (INTERCOM_CMD_DISCONNECT).
			sendInt(1001);
			// Close connection ...
			out.close();
			in.close();
			if (Kea.logger.debugOn())
				Kea.logger.logDebug("cleanUp() done.", this);
		} catch (Exception e) {
			Kea.logger.logError("Problem while cleaning up: ", e, this);
		}
	}

	/**
	 * Sends a <code>double</code> value to the Evolver.
	 * @param d the <code>double</code> to send
	 */
	private void sendDouble(double d) {
		String strD = fmt.format(d);
		strD = strD.replace(',', '.');
		sendInt(strD.length());
		out.write(strD);
	}

	/**
	 * Reads a <code>double</code> value from the Evolver,
	 * @return the read <code>double</code> 
	 */
	private double readDouble() {
		double d = 0;
		try {
			int length = readInt();
			byte[] chars = new byte[length];
			for (int i = 0; i < length; i++) {
				chars[i] = (new Integer(in.read())).byteValue();
			}
			String str = new String(chars);
			Double dou = new Double(str);
			d = dou.doubleValue();
		} catch (Exception e) {
			System.out.println("Could not read double from evolver.");
		}
		return d;
	}

	/**
	 * Send an <code>int</code> value to the Evolver.
	 * @param i the <code>int</code> value to send
	 */
	private void sendInt(int i) {
		try {
			out.write(i & 0xff);
			out.write((i >>> 8) & 0xff);
			out.write((i >>> 16) & 0xff);
			out.write(i >>> 24);
		} catch (Exception e) {
			Kea.logger.logError("Could not send int to evolver.", e, this);
		}
	}

	/**
	 * Reads an <code>int</code> value from evolver.
	 * @return the read <code>int</code>
	 */
	private int readInt() {
		int i = 0;
		try {
			i += (in.read() & 0xff);
			i += ((in.read() & 0xff) << 8);
			i += ((in.read() & 0xff) << 16);
			i += ((in.read() & 0xff) << 24);
		} catch (Exception e) {
			Kea.logger.logError("Could not read int from evolver,", e, this);
		}
		return i;
	}

	/* (non-Javadoc)
	 * @see kea.problems.AbstractProblem#IndividuumRepair(java.util.Vector)
	 */
	public Vector IndividuumRepair(Vector individuum) {
		return individuum;
	}

	/* (non-Javadoc)
	 * @see kea.problems.AbstractProblem#StartIndividuals(kea.Value[])
	 */
	public Value[] StartIndividuals(Value[] genotype) {
		return null;
	}

	/* (non-Javadoc)
	 * @see kea.problems.AbstractProblem#generatNewInd(kea.Value[])
	 */
	public Value[] generatNewInd(Value[] genotype) {
		return null;
	}

	/* (non-Javadoc)
	 * @see kea.problems.AbstractProblem#ObjectiveName()
	 */
	public void ObjectiveName() {
		
	}

} // Drilling