package kea.problems.external;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.io.Serializable;
import java.net.InetAddress;
import java.net.Socket;
import java.net.UnknownHostException;
import java.text.DecimalFormat;
import java.util.LinkedList;
import java.util.Vector;
import java.util.logging.Level;
import java.util.logging.Logger;

import kea.Parameter;
import kea.Parameters;
import kea.Range;
import kea.Type;
import kea.Value;
import kea.problems.AbstractProblem;

/*
 * Created on 14.12.2003
 *
 * To change the template for this generated file go to
 * Window>Preferences>Java>Code Generation>Code and Comments
 */
/**
 * @author suher
 * external ist eine eine allgemeine schnittstelle
 * zu externe  programme wie ZB der EVOLVER
 * sie ist t?tig fur allgemeine reparierung von idividuen,
 * erstellung von populationen und korrection der fitness einzelne individuen
 * zuletzt eingecheckt unter pg447 
 * $Author: noot $
 * pg447-version $Revision: 726 $
 * pg447-Date: $Date: 2005-02-02 00:50:20 +0100 (Mi, 02 Feb 2005) $
 *  */
public class External extends AbstractProblem implements Serializable{
	/**
	 * Number of points in frequency polygon.
	 * Default is 9. (Smallest possible value.)
	 */
	private int dimension = 9;
	/**
	 * Number of boolean points in frequency polygon.
	 * Default is 2. (Smallest possible value.)
	 */
	private int booleandimension = 2;
	/**
	 * Number of integer points in frequency polygon.
	 * not supported yet, must be 0
	 */
	private int intdimension = 0;
	/**
	 * Number of  points in frequency polygon.
	 *
	 */
	private int doubledimension = dimension;
	/**
	 * create new parameter
	 */
	private Parameters param = new Parameters();
	
	/**
	 * Selected objective function.
	 * Default is 1. (Weighted sum of objectives and restrictions.)
	 */
	//private int objective = 1;
	
	/**
	 * Socket for communication with Evolver.
	 */
	private Socket socket;
	/**
	 * question of the charaktere list
	 */
	public static final int OBJECTIVENAMES=1006;
	/**
	 * the Extern. Programm. request with the value: OBJECTIVENAMES_REPLY.
	 */
	public static final int OBJECTIVENAMES_REPLY=2006;
	/**
	 * Writer for sending data to Evolver.
	 */
	private PrintWriter out;
	
	/**
	 * Reader for reading data from Evolver.
	 */
	private BufferedReader in;
	
	/**
	 * Number formating for Evolver communication.
	 */
	DecimalFormat fmt = new DecimalFormat("0.000000000000");
	/**
	 * Message:   DISCONNECT
	 * Sent by:   KEA
	 * Comment:   Externes Programm beendet Verbindung. Falls 'Shutdown'==true
	 * beendet sich das externe Programm vollst?ndig.
	 
	 * int	1001 (DISCONNECT)					ID
	 * boolean	Shutdown
	 */
	public static final int DISCONNECT = 1001;
	/**
	 * Message:   PING
	 * Sent by:   KEA
	 * Comment:   Zum Testen der Verbindung. Das Ext. Prg. antwortet mit PING_REPLY.
	 * int	1002 (PING)						ID
	 */
	public static final int PING = 1002;
	/** EVALUATE
	 * KEA
	 * Individuumsdaten werden zur Bewertung gesendet. Das Ext. Prg.
	 * antwortet mit EVALUATE_REPLY.
	 */
	public static final int EVALUATE = 1003;
	/** GENERATE_START_INDIVIDUAL
	 *  KEA
	 *  Anfrage nach Startwerten des Individuums. Das Ext. Prg.
	 *  antwortet mit GENERATE_START_INDIVIDUAL_REPLY. Letzteres kann die
	 *  unterschiedlichen Individuen der Startpopulation anhand der
	 *  Variable 'Individual_Number' unterscheiden.
	 *
	 * */
	public static final int GENERATE_START_INDIVIDUAL = 1004;
	/**
	 * REPAIR_INDIVIDUAL
	 * KEA
	 * Anfrage nach Reparatur des Individuums. Diese erfolgt stets
	 * vor der Bewertung mit 'EVALUATE'. Das Ext. Prg. antwortet mit
	 * REPAIR_INDIVIDUAL_REPLY.
	 */
	public static final int REPAIR_INDIVIDUAL = 1005;
	/**
	 * Message:   PING_REPLY
	 * Sent by:   Externes Programm
	 * Comment:   Antwort auf PING.
	 * int	2002 (PING_REPLY)					ID
	 */
	public static final int PING_reply = 2002;
	/**
	 * EVALUATE_REPLY
	 * Externes Programm
	 * Antwort auf mit EVALUATE. Bewertung eines Individuums.
	 */
	public static final int EVALUATE_reply = 2003;
	/**
	 * GENERATE_START_INDIVIDUAL_REPLY
	 * Externes Programm
	 * Antwort auf GENERATE_START_INDIVIDUAL.
	 * Das ext. Prg. kann auf eine Initiierung der Startindividuen verzichten, KEA verwendet
	 * dann die Standardwerte.
	 */
	public static final int GENERATE_START_INDIVIDUAL_reply = 2004;
	/**
	 * REPAIR_INDIVIDUAL_REPLY
	 * Externes Programm
	 * Antwort auf REPAIR_INDIVIDUAL.
	 * Durch Angabe von 'Reject'==true, kann das ext. Prg. auf
	 * eine Reparatur verzichten.
	 */
	public static final int REPAIR_INDIVIDUAL_reply = 2005;
	/**
	 * IP PORT
	 */
	public static int IP_PORT = 6201;
	/**
	 * IP ADDR
	 */
	public static String IP_ADDR = "127.0.0.1";
	
	/**
	 * zwischen variable um verktoren zu speichern
	 */
	Vector received = null;
	/**
	 * zwischen variable um booleache  werte zu empfangen und speichern
	 */
	boolean Reject = false;
	/**
	 * Array um ergebnisse zu speichern
	 */
	Value[] results;
	
	/**
	 * erstellung eine socket connection zu dem evolver software.
	 * setzung von alle parameters.
	 */
	
	public static Logger log;
	
	public External(String server,int port) {
		IP_ADDR=server;
		IP_PORT=port;
		log = Logger.getLogger(this.getClass().getName());
		log.setLevel(Level.WARNING);
		log.log(Level.INFO,"*** creating External object . ****");
		try {
			
			// Create socket connection.
			socket = new Socket(InetAddress.getByName(IP_ADDR),IP_PORT);
			//socket.setSoTimeout(300);
			while (!socket.isConnected()){
				try {
					Thread.sleep(10);
				} catch (InterruptedException e1) {
					e1.printStackTrace();
				}
				socket = new Socket(InetAddress.getByName(IP_ADDR),IP_PORT);
				
			}
			if (socket.isConnected()){
				log.log(Level.INFO,"Verbingung hergestellt: "+socket.toString());
			}
			out = new PrintWriter(socket.getOutputStream());
			in = new BufferedReader(
					new InputStreamReader(socket.getInputStream()));
		} catch (UnknownHostException e) {
			log.log(Level.ALL,e.toString());
		} catch (IOException e) {
			log.log(Level.ALL,e.toString());
		} 
		
		param = new Parameters();
		param.setMembership(this.getClass().getName());
		
		param.addParameter(
				new Parameter(
						"dimension",
						dimension,
				"dimension of decision space"));
		param.addParameter(
				"minRange",
				-2147483640,
		"Lower border of decision space (for all dimensions).");
		param.addParameter(
				"maxRange",
				2147483640,
		"Upper border of decision space (for all dimensions).");
		log.log(Level.INFO,"*** object of External is ready. ****");
		
		
	}
	/** GENERATE_START_INDIVIDUAL
	 *  KEA
	 *  Anfrage nach Startwerten des Individuums. Das Ext. Prg.
	 *  antwortet mit GENERATE_START_INDIVIDUAL_REPLY. Letzteres kann die
	 *  unterschiedlichen Individuen der Startpopulation anhand der
	 *  Variable 'Individual_Number' unterscheiden.
	 *
	 * */
	public Value[] StartIndividuals(Value[] genotype) {
		int num = 0;
		log.log(Level.INFO,"**** Starting method  StartIndividuals()****");
		results = new Value[genotype.length];
		sendInt(GENERATE_START_INDIVIDUAL);
		sendInt(num++);
		sendInt(doubledimension);
		sendInt(intdimension);
		sendInt(booleandimension);
		out.flush();
		received = new Vector();
		if (readInt() == GENERATE_START_INDIVIDUAL_reply) {
			if (!readboolean()) {
				for (int r = 0; r < doubledimension; r++) {
					
					results[r] = new Value(new Double(readDouble()));
				}
				for (int i = 0; i < intdimension; i++) {
				}
				for (int i = 0; i < booleandimension; i++) {
					results[doubledimension+i] = new Value(readboolean());
					
				}
			}
			else {
				return genotype;
			}
		}
		return results;
	}
	/**
	 * Takes an array of concretely variable values and repair him by means of ???
	 * @param genotype an array of decision variable values which should be scored
	 * @return the corresponding array of objective values
	 */
	public Value[] repair(Value[] geno) {
		received = new Vector();
		results = new Value[geno.length];
		log.log(Level.INFO,"**** Starting method Repair() ****");
		
		sendInt(External.REPAIR_INDIVIDUAL);
		sendInt(doubledimension);
		for (int j = 0; j < doubledimension; j++) {
			try {
				double y =
					Double.parseDouble(((Number) geno[j].getData()).toString());
				sendDouble(y);
			} catch (Exception e) {
				log.log(Level.ALL,e.toString());
			}
		}
		sendInt(this.intdimension);//Ist immer 0
		this.sendInt(this.booleandimension);
		for (int counter=0;counter<booleandimension;counter++){
			this.sendboolean(0);
		}
		out.flush();
		
		if (readInt() == REPAIR_INDIVIDUAL_reply) {
			if (!readboolean()) {
				for (int r = 0; r < doubledimension; r++) {
					results[r] = new Value(new Double(readDouble()));
				}
				for (int i = 0; i < intdimension; i++) {
				}
				for (int i = 0; i < booleandimension; i++) {
					results[doubledimension+i] = new Value(readboolean());
				}
			}
			else {
				return geno;
			}
		}
		
		return results;
		
		
	}
	
	Value[] Objresults;
	int dataresult[][] ;
	String datacreterien[];
	int creterienCount =0;
	int creteriennamecount=0;
	LinkedList objCretList= new LinkedList();
	
	
	/**
	 * Takes an array of decision variable values
	 * and calculates the corresponding array of objective values
	 * (depending on the selected objective function).
	 *
	 * @param genotype an array of decision variable values which should be scored
	 * @return the corresponding array of objective values
	 */
	
	public Value[] score(Value[] geno) {
		log.log(Level.INFO,"+++++++++++++++++++++++++score aufruf");
		received = new Vector();
		
		sendInt(External.EVALUATE);
		sendInt(doubledimension);
		for (int j = 0; j < doubledimension; j++) {
			try {
				double y =
					Double.parseDouble(((Number) geno[j].getData()).toString());
				sendDouble(y);
			} catch (Exception e) {
				log.log(Level.ALL,e.toString());
			}
		}
		sendInt(this.intdimension);
		this.sendInt(this.booleandimension);
		for (int counter=0;counter<booleandimension;counter++){
			this.sendboolean((((Boolean)geno[counter+doubledimension].getData()).booleanValue())?1:0);
		}
		out.flush();
		
		if (EVALUATE_reply==readInt()) {
			int objectivcount = readInt();
			
			Objresults = new Value[objectivcount];
			
			{
				
				for (int r = 0; r < objectivcount; r++)
				{
					Objresults[r]=new Value(new Double(readDouble()));
					//received.add(new Double(readDouble()));
				}
			}
			
		}
		else{
			log.log(Level.ALL,"Evaluate_reply is incorrect");
		}
		return Objresults;
		
	}
	
	String objectiveString = "";	
	
	public  void  ObjectiveName()  {
		objectiveString="";
		log.log(Level.INFO,"objectiveString -------->"  + objectiveString);
		sendInt(OBJECTIVENAMES);
		out.flush();
		int ObjectiveName_replay=0;
		try	{
			
			ObjectiveName_replay = readInt();
			out.flush();
		}
		catch(Exception e){
			e.printStackTrace();
		}
		if (ObjectiveName_replay == External.OBJECTIVENAMES_REPLY) {
			int NumberOfObjective = readInt();
			
			for (int i = 0; i < NumberOfObjective; i++) {
				int NumberOfChar_Objective = readInt();
				for (int j = 0; j < NumberOfChar_Objective; j++) {
					readstring();					
				}
				this.Objnamelist.add(objectiveString);
				objectiveString ="";
			}
			log.log(Level.INFO,"---->at the end of objectivName() <----");
		}
		try {
			int a=in.read();
			while (a > -1) {
				a=in.read();
			}
			log.log(Level.INFO,"----> end of objectivName<---------");
			
		}
		catch (IOException ex) {
			log.log(Level.ALL,"can not read a value ---------"+ ex.toString());
		}
	}
	/**
	 * liste der Namen der Kriterien
	 */
	public  LinkedList Objnamelist = new LinkedList();
	
	
	public String readstring() {
		
		try {
			log.log(Level.INFO,"**** Starting method readString() ****");
			char x = readchar();
			out.flush();
			char c = 0;
			try{		
				c=x;
			}
			catch (Exception e) {
				log.log(Level.ALL,"Could not read string from evolver "+ e.toString());
			}
			objectiveString = objectiveString + c;
			
		} catch (Exception e) {
			log.log(Level.ALL,"Could not read String from evolver,"+ e.toString());
		}
		return objectiveString;
	}
	
	
	/**
	 * Returns an int representing the dimension of objective space
	 * (depending on the selected objective function).
	 *
	 * @return dimension of objective space
	 */
	
	public int getDimObjectiveSpace() {
		//Fest auf den Evolver mit 12 Kriterien eingestellt.
		return 12;
	}
	
	/**
	 * Returns an array of <code>Range</code> objects describing the allowed ranges
	 * for each descision variable.
	 *
	 * @return an array of <code>Range</code> objects for each decision variable
	 */
	public Range[] getRanges() {
		// Each point needs three coordinates.
		// (Drilling direction not yet considered.)
		//		Range[] ranges = new Range[3 * dimension];
		//				return ranges;
		int n = getDimDecisionSpace();
		double minRange =
			((Number) param.getParameterByName("minRange").getData())
			.doubleValue();
		double maxRange =
			((Number) param.getParameterByName("maxRange").getData())
			.doubleValue();
		Range[] ranges = new Range[n];
		for (int i = 0; i <= n - 1; i++) {
			ranges[i] = new Range(new Double(minRange), new Double(maxRange));
		}
		return ranges;
	}
	
	/**
	 * Returns an array of <code>Type</code> objects describing the decision variable types.
	 *
	 * @return a <code>Type[]</code> containing exactly one <code>Type</code> object for each decision variable
	 */
	public Type[] getDecTypes() {
		int n = getDimDecisionSpace();
		Type[] types = new Type[n];
		for (int i = 0; i <= n - 1; i++) {
			types[i] = new Type(Type.DOUBLE);
		}
		return types;
	}
	
	/**
	 * Returns an array of <code>Type</code> objects describing the decision variable types.
	 *
	 * @return a <code>Type[]</code> containing exactly two <code>Type</code> objects, one for each objective
	 */
	public Type[] getObjTypes() {
		int n = getDimObjectiveSpace();
		Type[] types = new Type[n];
		for (int i = 0; i <= n - 1; i++) {
			types[i] = new Type(Type.DOUBLE);
		}
		return types;
	}
	
	/**
	 * Returns the <code>Parameters</code> object of this class.
	 * @return the <code>Parameters</code> object of this class
	 */
	public Parameters getParameter() {
		return this.param;
	}
	
	/**
	 * Sets a new <code>Parameters</code> object for this problem.
	 * @param parameter the <code>Parameters</code> object that will be set
	 */
	public void setParameter(Parameters parameter) {
		this.param = parameter;
		Parameter p = param.getParameterByName("dimension");
		dimension = ((Integer) p.getData()).intValue();
		doubledimension = dimension;
		booleandimension = dimension/3-1;
		
	}
	/* (non-Javadoc)
	 * @see kea.problems.AbstractProblem#IndividuumRepair(java.util.Vector)
	 */
	public Vector IndividuumRepair(Vector Individiuum) {
		
		return null;
	}
	/* (non-Javadoc)
	 * @see kea.problems.AbstractProblem#verify(kea.Value[])
	 */
	public boolean verify(Value[] toVerify) {
		
		return false;
	}
	
	/* (non-Javadoc)
	 * @see kea.problems.AbstractProblem#generatNewInd(kea.Value[])
	 */
	public Value[] generatNewInd(Value[] genotype) {
		sendInt(External.PING);
		out.flush();
		int w= readInt();
		out.flush();
		if(w==2002)
			log.log(Level.INFO,"ping_replay is true");
		
		return genotype;
	}
	
	/* (non-Javadoc)
	 * @see kea.problems.AbstractProblem#getDimDecisionSpace()
	 */
	public int getDimDecisionSpace() {
		
		return dimension;
	}
	
	/**
	 * Sends a <code>double</code> value to the Evolver.
	 * @param d the <code>double</code> to send
	 */
	private void sendDouble(double d) {

		// Evolver mag keine E-Schreibweise, Rest dazu siehe in Temp.java
		String strD = org.nobeltje.tools.io.Number.getDecimal(d, 3);
		// Strings abfangen, die den Wert "-0" haben
		if (Double.parseDouble(strD) == 0) {
			strD = "0";
		}
		
		strD = strD.replace(',', '.');
		sendInt(strD.length());
		out.write(strD);
	}
	/**
	 * Sends a <code>int</code> value to the Evolver.
	 * @param i
	 */
	private void sendInt(int i) {
		try {
			int a=i  & 0xff;
			out.write(i  & 0xff);
			out.write((i >>> 8) & 0xff);
			a=(i >>> 8) & 0xff;
			out.write((i >>> 16) & 0xff);
			a=(i >>> 16) & 0xff;
			out.write(i >>> 24 & 0xff);
			a=i >>> 24;
		} catch (Exception e) {
			log.log(Level.ALL,"Could not send int value  to evolver."+
					e.toString());
		}
		
		
	}
	private char readchar() {
		byte bytevalue = 0;
		char charvalue =0;
		try {
			bytevalue = (new Integer(in.read())).byteValue();
		}
		catch (Exception e) {
			log.log(Level.ALL,"Could not read char from evolver,"+ e.toString());
		}
		charvalue=(char)bytevalue;
		return charvalue;
	}
	
	/**
	 * Reads an <code>int</code> value from evolver.
	 * @return the read <code>int</code>
	 */
	private int readInt() {
		int i = 0;
		
		try {
			i += (in.read() & 0xff);
			i += ((in.read() & 0xff) << 8);
			i += ((in.read() & 0xff) << 16);
			i += ((in.read() & 0xff) << 24);
		} catch (Exception e) {
			log.log(Level.ALL,"Could not read int from evolver,"+ e.toString());
		}
		return i;
	}
	/**
	 * Read a <code>boolean</code> value from the Evolver.
	 * @return
	 */
	private boolean readboolean() {
		int i = 0;
		try {
			i += ((in.read() & 0xff) << 8);
		} catch (Exception e) {
			log.log(Level.ALL,"Could not read int from evolver,"+ e.toString());
		}
		if (i == 0)
			return false;
		return true;
		
	}
	/**
	 * Reads a <code>double</code> value from the Evolver,
	 * @return the read <code>double</code>
	 */
	private double readDouble() {
		double d = 0;
		try {
			int length = readInt();
			byte[] chars = new byte[length];
			for (int i = 0; i < length; i++) {
				chars[i] = (new Integer(in.read())).byteValue();
			}
			String str = new String(chars);
			Double dou = new Double(str);
			d = dou.doubleValue();
		} catch (Exception e) {
			log.log(Level.ALL,"Could not read double from evolver."+e.toString());
		}
		return d;
	}
	/**
	 * Sends a <code>boolean</code> value to the Evolver.
	 * @param i
	 * @return true wenn can read
	 */
	private boolean sendboolean(int i) {
		boolean b = false;
		try {
			out.write((i >>> 8) & 0xff);
			
			if (i > 0) {
				b = true;
			}
		} catch (Exception e1) {
			log.log(Level.ALL,"Could not send boolean to evolver."+e1.toString());
		}
		return b;
	}
	
	/**
	 * Sends disconnect signal to Evolver.
	 */
	public void cleanUp() {
		log.log(Level.INFO,"Starting cleanUp() ...");
		try {
			sendInt(DISCONNECT);
			// Close connection ...
			out.close();
			in.close();
			log.log(Level.INFO,"cleanUp() done.");
		} catch (Exception e) {
			log.log(Level.ALL,"Problem while cleaning up: "+ e.toString());
		}
	}
	
	public boolean ping() {
		boolean result =false;
		log.log(Level.INFO,"Starting ping() ...");
		try {
			sendInt(PING);
			out.flush();
			int pingReply=0;
			pingReply = readInt();
			out.flush();
			log.log(Level.INFO,"pingReply  :" + pingReply);
			if (pingReply==PING_reply)
				result=true;
		} catch (Exception e) {
			log.log(Level.ALL,"Problem while pinging: "+ e.toString());
		}
		return result;
	}
	
}
