/*
 * Copyright (C) 2005  PG447 - University Dortmund
 * 
 * This file is part of NObELTJE.
 * 
 * NObELTJE is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as published by
 * the Free Software Foundation.
 * 
 * NObELTJE is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with NObELTJE; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

 /*
 * Created on June 16, 2004
 *
 */
package org.nobeltje.algo;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.logging.Logger;

import org.nobeltje.tools.DoubleIEEE754;
import org.nobeltje.tools.Individual;
import org.nobeltje.tools.io.AlgoOutput;
import org.nobeltje.tools.io.ArgsInputBeautifier;
import org.nobeltje.exceptions.BinaryStringFormatException;
import org.nobeltje.exceptions.InvalidBoundException;

/**
 * Diese Klasse implementiert einen um Mehrkriterialit&auml;t erweiterten Basis-GA mit weiteren Erg&auml;nzungen.
 * Momentan implementiert:
 * <ul>
 * <li>Mehrkriterialit&auml;t durch rudiment&auml;re Ermittlung der "Gesamtfitness" mittels Berechnung des Arithmetischen Mittels</li>
 * <li>Rekombination durch 1-Punkt-Crossover</li>
 * <li>Ermittlung der L&ouml;sungs-Individuen durch verwendung eines unbegrentzten Pareto-Archivs</li>
 * <li>Grundlegend funktionierende fitnessproportionale Selektion</li>
 * </ul>
 * @author $Author: noot $
 * @version $Revision: 726 $<br>
 * $Date: 2005-02-02 00:50:20 +0100 (Mi, 02 Feb 2005) $
 */
/* 
 *Anzahl der evaluationen: gsize * g
 */
public class BasisGAPG extends AbstractAlgo
{
	
	/* Variablen */
		private String classnameTestfkt;// Name des zu nutzenden Simulators/Testfunktion
		private static int objectiveNumber;//Anzahl der Kriterien
		private int dimension; //Anzahl der Variablen fuer Testfunktion
		private ArrayList archive; // Archiv mit Individuen mit Bit-Objektvariablen
		private ArrayList archiveDouble; // Archiv mit Individuen mit Double-Objektvariablen
		private ArrayList population;
		private int populationSize; //Groesse der Population
		private double recoProbability; //Rekombinationswahrscheinlichkeit
		private double mutaProbability; //Mutationswahrscheinlichkeit
		private int crossoverPoints; //Anzahl der Crossover-Punkte, wird benoetigt bei Umstellung auf variables N-Point-Crossover
		private int actualGeneration; //In der wievielten Generation befinden wir uns gerade?
		private int chromLength; //Die gesamtlaenge des Chromosoms in (in bit)
		private static Logger logger;
		
	
	
	   /**
	    * Vom Algo nicht verwendeter Konstruktor, wird fuer die Klasse Help benoetigt und beinhaltet keinerlei Funktionalitaet.
	    * @see tools.io.Help
	    */
		public BasisGAPG()
		{
		}
		
		/**
		 * Vom Algo verwendeter Konstruktor.
		 * In diesem Konstruktor werden von der Super-Klasse die Kommandozeilen-Parameter geparsed
		 * und diese Werte der Klasse global verfuegbar gemacht. Alle benoetigten globalen Variablen werden initialisiert.
		 * @see algo.AbstractAlgo
		 * @param args String[]: Bekommt die Kommandozeilenparameter uebergeben
		 */
		public BasisGAPG (String[] args)
		{
			super(args);
			
			test.init(specificTestfktParameters);
			
		    populationSize = ArgsInputBeautifier.getIntProperty(specificParameters, "psize", 200, false);
		    recoProbability = ArgsInputBeautifier.getDoubleProperty(specificParameters, "rprob", 0.6, false);
		    mutaProbability = ArgsInputBeautifier.getDoubleProperty(specificParameters, "mprob", 0.01, false);
			dimension=test.getDimension();
			chromLength=dimension*DoubleIEEE754.VARSIZE;
			this.archive=new ArrayList();
			this.archiveDouble=new ArrayList();
		}
		
		/**
		 * Ueberlagerte Methode fuer Klasse tools.io.Help.
		 * Die Methode ueberlagert toString() der Superklasse und wird von der Klasse Help benoetigt.
		 * @see tools.io.Help
		 */
		public String toString()
		{
			String info;
			info="Diese Klasse implementiert einen um Mehrkriterialitaet erweiterten Basis-GA\n"
				+"zur Loesung reellwertiger Probleme.\n"
				+"Es existieren dir folgenden Kommandozeilenparameter:\n"
				+"-t Testfunktion -> zu verwendende Problemklasse\n"
				+"-g Integer -> Anzahl der Generationen\n"
				+"-psize Integer -> Groesse der Populationen (optional, Standard: 200)\n"
				+"-rprob Double -> Rekombinationswahrscheinlichkeit (optional, Standard: 0.6)\n"
				+"-mprob Double -> Mutationswahrscheinlichkeit (optional, Standard: 0.01)\n"
				+"-c Dateiname -> Name einer Config-Datei (optional)\n"
				+"-r Integer -> Random-seed zur Initialisierung des \"Zufalls\" (optional)\n"
				+"-o Prefix -> Namenspraefix der Ausgabedateien (optional)\n"
				+"-d -> Debug-Infos ausgeben (ToDo)\n"
				+"-live -> Abstand, wieviele Evaluationen zwischen 2 liveOutputs liegen soll.";
			return info;
		}
		
		/**
		 * &Uuml;berpr&uuml;ft die &uuml;bergebenen Individuen auf ihre Dominanz bez. des Archivs und aktualisiert dieses ggf.
		 * @see tools.ParetoArchive
		 * @param candidates ArrayList: Liste der Individuen
		 */
		private void freshenArchive(ArrayList candidates) throws Exception
		{
		    int i = 0;
		    //AlgoOutput.writeFitness(candidates,outputFileNamePrefix+"freshenCan",false);
		    for (i=0;i<candidates.size(); i++)
		    {
		        this.archive=org.nobeltje.tools.ParetoArchive.refreshArchive(0,(ArrayList)this.archive.clone(),(Individual)candidates.get(i));
		    }
		}

		/**
		 * Generiert die Ausgangspopulation 
		 * unter Ber&uuml;cksichtigung der durch die Testfunktion angegebenen Schranken.
		 * @throws BinaryStringFormatException
		 */
		private void genFirstPopulation() throws BinaryStringFormatException
		{
		    population = new ArrayList();
		    actualGeneration=1;
			int j,k;
			double temp;
			for ( j=0; j < populationSize; j++)
			{
				StringBuffer sb = new StringBuffer ("");
			    ArrayList newChrom = new ArrayList();
			    Individual newInd = new Individual(actualGeneration);
			    Double min,max;
				for (k=0; k<dimension;k++)
				{
					min=(Double)test.getOptimalBounds().get(2*k);
					max=(Double)test.getOptimalBounds().get(1+(2*k));
					try {
						temp=rand.nextDouble(min.doubleValue(),max.doubleValue());
						sb.append(DoubleIEEE754.getBinaryString(temp));				
					} catch (InvalidBoundException e) {
						e.printStackTrace();
					}
				}
			    newChrom.add(sb.toString());
			    newInd.setObjectVariables(newChrom);
			    population.add(newInd);	
			//logger.log(Level.FINEST,"",sb);
			}
		}
		
		/**
		 * N&auml;chste Population erzeugen. 
		 * Dabei wird die aktuelle Population durch die Neue ersetzt und anschliessend diese auf die Einhaltung
		 * der durch die Testfunktion &uuml;bergebenen Grenzwerte &uuml;berpr&uuml;ft.
		 * @see #checkBounds()
		 * @see #marriage(Individual, Individual)
		 * @param matingPool ArrayList: Die zur Rekombination selektierten Individuen  
		 */
		private void genNextPopulation(ArrayList matingPool) throws BinaryStringFormatException
		{
			ArrayList newPop=new ArrayList();
			ArrayList temp;
		    actualGeneration++;
		    int halfPop=matingPool.size()/2;
		    int i,j;
		    for (i=0;i<halfPop;i++)
		    {
		    	temp=marriage((Individual)matingPool.get(i),(Individual)matingPool.get(i+halfPop));
		    	for (j=0;j<temp.size();j++)
		    		newPop.add(temp.get(j));
		    }
		    this.population=newPop;
		    this.checkBounds();
		}
		
		
	
	   /**
	    * Startmethode des Algorithmus. 
	    * An dieser Stelle befindet sich die zentrale Schleife des Algorithmus
	    */	
		public boolean startAlgo() throws Exception
		{
			
			//logger = Logger.getLogger(this.getClass().getName());
			//FileHandler    fh = new FileHandler( "c:\\log.txt" ); //Das muss noch plattformuebergreifend realisiertw erden!
		    //logger.addHandler( fh );
		    genFirstPopulation();
		    //logger.log(Level.INFO,"1. Population generiert");
		    setNewFitness(); //Fitnesswerte durch Testfkt. ermitteln sowie Archiv pflegen
		    //writeSolutions(String.valueOf(1));
		    //logger.log(Level.INFO,"gefittet");
		    ArrayList matingPool; //Enthaelt die Individuen, die zu Rekombination/Mutation herangezogen werden
		    for (int i=actualGeneration+1; i<=maxGenerations; i++)
		    {
		    	//logger.log(Level.INFO,"Befinde mich in Generation "+i);
		        matingPool = recoSelection(); //Selektiere Individuen aus Population fuer Rekombination
		        //logger.log(Level.INFO,"Mating-Pool selektiert");
		        genNextPopulation(matingPool); //Erzeuge naechste Population durch Rekombination/Mutation
		        //logger.log(Level.INFO,"Naechste population erzeugt");
		        setNewFitness(); //Fitnesswerte durch Testfkt. ermitteln sowie Archiv pflegen
		        //writeSolutions(String.valueOf(i));
		        //logger.log(Level.INFO,"Fitness zugewiesen!");
		    }
		    //logger.log(Level.INFO,"Fertig");

		    /* archiveDouble aus archive erzeugen */
		    Individual temp,temp2;
			for (int i=0; i<this.archive.size(); i++) {
				temp=(Individual)this.archive.get(i);
				temp2=(Individual)temp.clone();
				temp2.setObjectVariables(this.makeFloat(temp.getObjectVariables()));
				archiveDouble.add(temp2);					
			}
			
			archiveDouble = (ArrayList)test.postProcessing(archiveDouble);
		    test.finalize();
		    
			return (true);
		}
		
		
		/**
		 * Methode zur Konvertierung eines Strings in eine Flie&szlig;kommazahl
		 * 
		 * @param input ArrayList: Beinhaltet als einziges Element einen String bestehend aus {'0','1'}
		 * @return ArrayList, die alle im String kodierten Werte als Double enth&auml;lt
		 * @throws BinaryStringFormatException
		 * @see tools.DoubleIEEE754
		 */
		private ArrayList makeFloat (ArrayList input) throws BinaryStringFormatException
		{			 
			ArrayList output = new ArrayList();
			int i;
			String bitArray=(String)input.get(0);
			for (int currentVar = 0; currentVar < dimension; currentVar++)
			{
				output.add(new Double(DoubleIEEE754.getDouble(bitArray.substring(currentVar*DoubleIEEE754.VARSIZE,(currentVar+1)*DoubleIEEE754.VARSIZE))));
			}
			return output;
		}
		
		/**
		 * Wandelt eine ArrayList mit n Double-Zahlen in einen String mit n*64 Boolean-Werten (nach IEEE 754).
		 *   
		 * @param input ArrayList: beinhaltet Dezimalzahlen als Double
		 * @return ArrayList, das als einziges Element einen String enth&auml;lt.
		 * @see DoubleIEEE754 
		 */
		private ArrayList makeBin(ArrayList input) throws BinaryStringFormatException
		{
			ArrayList output = new ArrayList();
			String bitArray="";
			Double temp;
			int i,j;
			for (i=0;i<input.size();i++)
			{
				temp=(Double)input.get(i);
				bitArray+=DoubleIEEE754.getBinaryString(temp.doubleValue());
			}
			output.add(bitArray);
			return output;
		}
		
		/**
		 * Bewertung der einzelnen Fitnesswerte eines Individuums durch eine Gesamtfitness. 
		 * Die ermittelte Gesamtfitness entspricht dem arithmetischen Mittel der einzelnen Fitnesswerte.
		 * @param objectFitness ArrayList: Eine Liste aller Fitnesswerte eines Individuums als Double
		 * @return Ermittelte Gesamtfitness als double
		 */
		private double giveCompleteFitness(ArrayList objectFitness)
		{
			
			double fitness=0; // Fittness initialisieren
			Double helper;

			  /* Alle Elemente der Ergebnisliste summieren (Fitnessberechnung) */ 	 
			for (Iterator iter = objectFitness.iterator(); iter.hasNext();)
			{
				helper = (Double) iter.next();
				fitness += helper.doubleValue();
			}

			//Und jetzt das arithmetische Mittel bilden
			fitness=fitness/(double)objectFitness.size();
			return (fitness);
		}
		
		
		/**
		 * Selektion des Mating-Pools fuer die Rekombination. 
		 * Fitnessproportionale Selektion auf Grundlage der aktuellen Population
		 * @return Mating-Pool als ArrayList mit den Selektierten Individuen
		 */
		private ArrayList recoSelection()
		{
		    double wholeFitness=0.0; //Die Summe aller Fitnesswerte
		    ArrayList matingPool = new ArrayList();
		    ArrayList fortuneWheel = new ArrayList(); //enthaelt Indizes der Individuen, Anzahl ist fitnessproportional
		    double[] completeFitnesses = new double[population.size()]; //Hier werden die Gesamtfitnesswerte der einzelnen Individuen festgehalten.]
		    Integer candidate;
		    int i,j; //Iterator
		    Individual temp;
		    
		    // Je Individuum die Gesamtfitness aller Kriterien ermitteln und die Summe
		    // ueber allen Individuen ermitteln
			//System.out.println("Hole alle Gesamtfitnesswerte und ermittle Summe");		    
		    for (i=0; i<population.size();i++)
		    {
		        temp = (Individual)population.get(i);
		        completeFitnesses[i]=giveCompleteFitness(temp.getFitness());
		        wholeFitness+=completeFitnesses[i];
		        //System.out.println("Ind. "+i+" hat Komplettfitness "+completeFitnesses[i]+"; wholeFitness:"+wholeFitness);
		    }
		    
		    
		    // Je Individuum die zuvor ermittelte Gesamtfitness durch die proportionale Fitness ueberschreiben.
		    //System.out.println("Lege fitnessproportionale Wahrscheinlichkeiten fest");
		    for (i=0; i<completeFitnesses.length;i++)
		    {
		    	//System.out.println("Gesamtfitnessumme: "+wholeFitness+"; Gesamtfitness des akt. Ind.: "+completeFitnesses[i]);
		        completeFitnesses[i]=Math.abs((completeFitnesses[i]-wholeFitness)/wholeFitness); // Absoluter Differenzbetrag wg. Minimierung!
		        //System.out.println("Gesamtfitness nach Normierung: "+completeFitnesses[i]);
		    }
		    
		    // ...und fortuneWheel generieren: Der Index eines Individuums wird gerundet 100*proportionaleFitness
		    // zum Gluecksrad hinzugefuegt.
		    // Somit bekommt das Individuum mit einer kelinen proportionalen Fitness weniger Plaetz auf dem
		    // Gluecksrad als Individuen mit besseren proportionalen Fitnesswerten..
		    //System.out.println("Generiere Gluecksrad ");
		    for (i=0; i<completeFitnesses.length;i++)
		    {
		    	//System.out.println("  Anzahl der Segmente fuer Ind. "+i+" mit Fitness "+completeFitnesses[i]+": "+Math.round(completeFitnesses[i]*100));
				for (j=0;j<Math.round(completeFitnesses[i]*100);j++)
									fortuneWheel.add(new Integer(i));
		    }
		    
		    // Auswahl der Eltern
		    ////System.out.println("Waehle Eltern aus");
		    for (i=0; i<population.size();i++)
		    {
		        candidate=(Integer)fortuneWheel.get(rand.nextInt(fortuneWheel.size()));
		        matingPool.add(population.get(candidate.intValue()));
		    }
		        
			return matingPool;
		}
		
		
		/**
		 * Rekombination von zwei Individuen. 
		 * Die Individuen werden abh&auml;ngig von der jeweilig festgelegten Wahrscheinlichkeit rekombiniert und mutiert.
		 * <ul>
		 * <li> Rekombination durch 1-Punkt-Crossover abh&auml;ngig von Rekombinationswahrscheinlichkeit (Standard: 0.6)</li>
		 * <li> Mutation je Bit abh&auml;ngig von Mutationswahrscheinlichkeit (Standard: 0.01)
		 * </ul>
		 * @param one Individual: Erster Elter
		 * @param two Individual: Zweiter Elter
		 * @return Zwei Nachkommen in einer ArrayList
		 */
		private ArrayList marriage(Individual one, Individual two)
		{
			ArrayList returnIndividuals = new ArrayList(); //Die ArrayList, die die beiden neuen Individuals beinhalten soll
			ArrayList array_one = new ArrayList();
			ArrayList array_two = new ArrayList();
			String orig_one=(String)one.getObjectVariables().get(0); 
			String orig_two=(String)two.getObjectVariables().get(0); 
			StringBuffer temp_one=new StringBuffer(orig_one); // Objektveriablen von Individual 1, mit denen gearbeitet wird
			StringBuffer temp_two=new StringBuffer(orig_two); // Objektveriablen von Individual 2, mit denen gearbeitet wird
			////System.out.println("Vor Rekombination 1: "+getIndividualString(temp_one));
			////System.out.println("Vor Rekombination 2: "+getIndividualString(temp_two));
			int i; //Iterator
			// Findet Crossover statt?
			if (rand.nextDouble()<=recoProbability )
			{
				////System.out.println("Rekombination!");
				int coPoint= rand.nextInt(chromLength);
				////System.out.println("Rekombination mit COP "+coPoint);
				for (i=coPoint; i < chromLength; i++)
				{
					temp_one.setCharAt(i,orig_two.charAt(i));
					temp_two.setCharAt(i,orig_one.charAt(i));
				}
				////System.out.println("Nach Rekombination 1:"+getIndividualString(temp_one));
				////System.out.println("Nach Rekombination 2:"+getIndividualString(temp_two));
			}
			//Durchlaufen und mutieren
			for (i=0; i<chromLength; i++)
			{
				// Individdum 1 an STelle i mutieren wenn Zufall es will?
				if (rand.nextDouble()<=mutaProbability)
					if (temp_one.charAt(i)=='1')
						temp_one.setCharAt(i, '0');
					else
						temp_one.setCharAt(i, '1');
				// Individuum 2 an STelle i mutieren wenn Zufall es will?
				if (rand.nextDouble()<=mutaProbability )
					if (temp_two.charAt(i)=='1')
						temp_two.setCharAt(i, '0');
					else
						temp_two.setCharAt(i, '1');
			}
			//System.out.println("Nach Mutation 1:     "+getIndividualString(temp_one));
			//System.out.println("Nach Mutation 2:     "+getIndividualString(temp_two));
			Individual newOne=new Individual(actualGeneration);
			Individual newTwo=new Individual(actualGeneration);
			array_one.add(temp_one.toString());
			array_two.add(temp_two.toString());
			newOne.setObjectVariables(array_one);
			newTwo.setObjectVariables(array_two);
			returnIndividuals.add(newOne);
			returnIndividuals.add(newTwo);
			return returnIndividuals;
		}
		

		/**
		 * &Uuml;berpr&uuml;fung der Wertebereiche der Individuen der aktuellen Population. 
		 * Es wird &uuml;berpr&uuml;ft, ob die Objektvariablen aller Individuen im von der Testfunktion vorgegebenen
		 * g&uuml;ltigen Bereich liegen. Ist das nicht der Fall, wird ein Wert auf den jeweilige Max (bzw. Min)-Grenzwert
		 * zur&uuml;ckgesetzt.
		 *
		 */
		private void checkBounds() throws BinaryStringFormatException
		{
			Individual individual;
			ArrayList doubleListOrg,doubleListNew;
			ArrayList newPopulation = new ArrayList();
			Double temp, min, max;
			for (int i=0; i<populationSize;i++)
			{
				individual=(Individual)this.population.get(i);
				doubleListOrg=this.makeFloat(individual.getObjectVariables());
				doubleListNew=new ArrayList();
				for (int j=0; j<dimension;j++)
				{
					temp=(Double)doubleListOrg.get(j);
					min=(Double)test.getOptimalBounds().get(2*j);
					max=(Double)test.getOptimalBounds().get(1+(2*j));
					if (temp.floatValue() < min.floatValue())
						doubleListNew.add(min); //Wenn der urspruengliche Wert zu klein war, wird er auf das Minimum gesetzt
					else
						if (temp.floatValue() > max.floatValue())
							doubleListNew.add(max); //Wenn der urspruengliche Wert zu gross war, wird er auf das Maximum gesetzt.
						else
							doubleListNew.add(temp); //Der Wert ist innerhalb der Schranken, daher wird er uebernommen.
				}
				individual.setObjectVariables(this.makeBin(doubleListNew));
				newPopulation.add(individual);
			}
			
		}
		
		/**
		 * Setzt mit Hilfe der Testfunktion fuer jedes Individuum der aktuellen Population die Fitnesswerte 
		 * @throws Exception
		 */
		private void setNewFitness() throws Exception
		{   
			Individual indi;
			ArrayList temp;
			ArrayList temp2;
			ArrayList newPopulation = new ArrayList();
			////System.out.println("Populationsgroesse: "+this.population.size());
			for (int i =0; i < this.population.size(); i++)
			{
				//System.out.println("  Berechne Fitness fuer Ind. "+i);
				indi =(Individual)this.population.get(i);
				temp2=makeFloat(indi.getObjectVariables());
				Double doubletemp;
				for (int k=0; k<temp2.size();k++)
				{
					doubletemp=(Double)temp2.get(k);
					//System.out.println("    Fkt.Wert"+k+": "+doubletemp.doubleValue());
				}
				temp=(ArrayList)test.execute(temp2);
				
				/* Evaluationsbasierter LiveOutput */
				EvaluationBasedLiveOutput(this.archive);
				
				//test.finalize();
				for (int j=0; j<temp.size();j++)
				{
					doubletemp=(Double)temp.get(j);
					//System.out.println("    Fitness"+j+": "+doubletemp.doubleValue());
				}
					
				indi.setFitness(temp);
				newPopulation.add(indi);
			}
			this.population=newPopulation;
			this.freshenArchive(this.population);
		}
		
		/**
		 * Schreibt Archiv und Funktionswerte des Archivs auf die Festplatte mit zus&auml;tzlichem benutzerdefiniertem String. 
		 * Die Methode kann zum Debuggen verwendet werden, damit die Debug-Outputs nicht von den Endergebnissen &uuml;berschreiben werden.<br>
		 * Das "normale" Output wird durch diese Methode mit leerem Suffix geschrieben.
		 * 
		 * @param text String Zus&auml;tzliches Suffix des definierten Dateinamen-Prefixes der Ausgabedateien
		 * @see #writeSolutions()
		 */
		private void writeSolutions(String text)
		{
			/* Objektvariablen der Individuen muessen vorher in Double-Werte umgewandelt
			 * und in archiveDouble gespeichert werden!
			 */
			try
			{
				/* Archiv in Textdatei ausgeben */ 
				AlgoOutput.writeObjectVariables(archiveDouble,outputFileNamePrefix+text,false);
				/* Archiv (Fitnesswerte) in Textdatei ausgeben */ 
				AlgoOutput.writeFitness(archiveDouble,outputFileNamePrefix+text,false);
			}
			catch (Exception e)
			{
				System.err.println("Fehler beim Schreiben der Ergebnisse!");
				System.err.println(e.toString());
			}
		}
		
		/**
		 * Schreibt Archiv und Funktionswerte des Archivs auf die Festplatte. 
		 * Als Dateiname wird entweder ein beim Aufruf des Algos &uuml;bergebener Text oder aktuelles Datum/Uhrzeit verwendet.
		 * @see #writeSolutions(String)
		 */
		private void writeSolutions()
		{
			this.writeSolutions("");
		}

	
	/**
	 * Hauptroutine des Algorithmus
	 * <ul>
	 * <li>Instanziiert ein objekt des Algorithmus</li>
	 * <li>Startet den instanziierten Algo</li>
	 * <li> Veranlasst die Ausgabe der Ergebnisse</li>
	 * </ul>
	 * @param args String[]: Kommandozeilenparameter des Benutzers
	 * @throws Exception
	 */
	public static void main(String[] args) throws Exception
	{
   	/* Algorithmus-Objekt erzeugen */	
		BasisGAPG algo = new BasisGAPG(args);
   	/* Algorithmus starten */
		algo.startAlgo();
	/* Ergebnisse schreiben */
		algo.writeSolutions();
	}
}

