/*
 * Copyright (C) 2005  PG447 - University Dortmund
 * 
 * This file is part of NObELTJE.
 * 
 * NObELTJE is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as published by
 * the Free Software Foundation.
 * 
 * NObELTJE is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with NObELTJE; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

 /*
 * Created on 20.06.2004
 */
package org.nobeltje.algo;

import java.util.ArrayList;

import org.nobeltje.tools.Individual;
import org.nobeltje.tools.ParetoArchive;
import org.nobeltje.tools.ParetoDomination;
import org.nobeltje.tools.Particle;
import org.nobeltje.tools.io.AlgoOutput;
import org.nobeltje.tools.io.ArgsInputBeautifier;

/**
 * Dies ist ein Particle Swarm Optimization Algorithmus, der an mehrkriterielle
 * Probleme angepat ist.<br>
 * Folgende Parameter erwartet der Algorithmus entweder in einer Properties Datei
 * oder ueber Kommandozeile:
 * <ul>
 * <li>archive - Archivgroesse
 * 	<li>cycles - Anzahl der Durchlaeufe (nicht zu verwechseln mit Iterationen)</li>
 * 	<li>populationSize - Groee der Population</li>
 * 	<li>inertia - Traegheitswert; beeinflusst ob lokal oder global gesucht wird</li>
 * 	<li>c1 - Lernfaktor; steuert, wie stark die beste Position eines Partikels
 * 		beruecksichtigt wird</li>
 * 	<li>c2 - Lernfaktor; steuert, wie stark die Position des Schwarmbesten
 * 		beruecksichtigt wird</li>
 * </ul>
 *  
 * 
 * <br><b>Benoetigte Testfunktionsaufrufe:</b>
 * <br>- Initialisierung : populationSize
 * <br>- Pro Generation  : cycles * populationSize * #Generationen
 *  
 * 
 * @author $Author: noot $
 * @version $Revision: 726 $
 * $Date: 2005-02-02 00:50:20 +0100 (Mi, 02 Feb 2005) $
 */

public class BasisPSO extends AbstractAlgo
{
	private Particle	population[];	// Population
	private	Particle	gBest;			// Bestes Partikel
	private	int			cycles;			// Anzahl der Durchlufe
	private int			populationSize;	// Gre der Population
	private double		inertia;		// Trgheitswert
	private double		c1, c2;			// Lernfaktoren
	ArrayList help= new ArrayList(); //Hilfs-Archiv fr EvaluationBasedLiveOutput
	int archiveSize;
	/**
	 * Konstruktor: Ruft den Konstruktor der Superklasse (AbstractAlgo) auf
	 * und initialisiert die Parameter von PSO
	 * 
	 * @param args enthaelt die Kommandozeilen Parameter
	 */
	public BasisPSO(String[] args)
	{
		// Rufe den Konstruktor von AbstractAlgo auf
		super(args);
		test.init(specificTestfktParameters);
		
		// Initialisiere die Variablen fr PSO
		archiveSize =ArgsInputBeautifier.getIntProperty(specificParameters, "archive", 100, false);
		cycles 			= ArgsInputBeautifier.getIntProperty(specificParameters, "cycles", 0, true);
		populationSize	= ArgsInputBeautifier.getIntProperty(specificParameters, "populationSize", 0, true);
		inertia			= ArgsInputBeautifier.getDoubleProperty(specificParameters, "inertia", 0.2f, false);
		c1				= ArgsInputBeautifier.getDoubleProperty(specificParameters, "c1", 0.6f, false);
		c2				= ArgsInputBeautifier.getDoubleProperty(specificParameters, "c2", 0.6f, false);
		//zu kleines Archiv... Archivgroesse wird auf Anzahl der Cyclen gesetzt.
		if (archiveSize < cycles){
			archiveSize=cycles;
			}

		population = new Particle[populationSize];
		
	}

	
	
	/**
	 * Initialisiert die Ausgangspopulation mit Zufallswerten aus dem Intervall,
	 * das die Testfunktion uebergibt
	 */
	private void initPopulation() throws Exception
	{
		for (int i = 0; i < population.length; i++) {
			Particle particle = new Particle();

			// Setze fuer jedes Partikel eine zufaellige Position und Geschwindigkeit  
			for (int d = 0; d < test.getDimension(); d++) {
				double lowerBound = ((Double)test.getOptimalBounds().get(2*d)).doubleValue();
				double upperBound = ((Double)test.getOptimalBounds().get(2*d+1)).doubleValue();
				particle.getPosition().add(new Double(rand.nextDouble(lowerBound, upperBound)));
				particle.getVelocity().add(new Double(rand.nextDouble(0.0, 1.0)));
			}
			// Die beste Position ist zu Beginn die Anfangsposition
			particle.setPBest((ArrayList)particle.getPosition().clone());
			// Setze die Fitness Werte fuer die Position
			particle.setFitness((ArrayList)test.execute(particle.getPosition()));

			population[i] = particle;
		}
	}

	/**
	 * Bewertet die aktuelle Position des Partikels
	 * 
	 * @param particle
	 */
	private void evaluateParticle(Particle particle) throws Exception
	{
		// Hole von der Testfunktion die Fitness-Werte fr die aktuelle Position
		ArrayList results = (ArrayList)test.execute(particle.getPosition());

		// Falls die aktuelle Position besser ist als die bisher beste,
		// speichere in pBest die aktuelle Position und aktualisiere die
		// Fitness Werte
		if (ParetoDomination.checkDomination(results, particle.getFitness()) == ParetoDomination.DOMINATION_IND1) {
			particle.setPBest((ArrayList)particle.getPosition().clone());
			particle.setFitness((ArrayList)results.clone());
		}
	}

	/**
	 * Berechnet fr das uebergebene Partikel den neuen Geschwindigkeitsvektor
	 * nach der Formel:
	 * 	v_1 =	inertia * v_i +
	 * 			c1 * rand[0..1] * (pBest.position - particle.position) +
	 * 			c2 * rand[0..1] * (gBest.position - particle.position)
	 * 
	 * @param particle
	 */
	private void calculateNewVelocity(Particle particle)
	{
		double factor1 = c1 * rand.nextDouble();
		double factor2 = c2 * rand.nextDouble();

		for (int i = 0; i < particle.getVelocity().size(); i++) {
			double v = inertia * ((Double)particle.getVelocity().get(i)).doubleValue() +
				factor1 * (((Double)particle.getPBest().get(i)).doubleValue() - ((Double)particle.getPosition().get(i)).doubleValue()) +
				factor2 * (((Double)gBest.getPosition().get(i)).doubleValue() - ((Double)particle.getPosition().get(i)).doubleValue());
			particle.getVelocity().set(i, new Double(v));
		}
	}

	/**
	 * Startroutine des Algorithmus
	 * 
	 * @see algo.AbstractAlgo#startAlgo()
	 */
	public boolean startAlgo() throws Exception
	{
		// Initialisiere die Ausgangspopulation
		initPopulation();
		// Setze das beste Partikel zu Beginn auf das erste Partikel der Population
		gBest = population[0];

		// Hauptschleife
		for (int i = 0; i < maxGenerations; i++) {
			for (int p = 0; p < population.length; p++) {
				// Bewerte die aktuelle Position des Partikels
				evaluateParticle(population[p]);

				// Wenn die Fitness Werte des aktuellen Partikels besser sind als die
				// des besten Partikels, setze das beste auf das aktuelle Partikel
				if (ParetoDomination.checkDomination(population[p].getFitness(), gBest.getFitness()) == ParetoDomination.DOMINATION_IND1) {
					gBest = population[p];
					help = ParetoArchive.refreshArchive(archiveSize, help, new Individual(gBest.getPosition(), gBest.getFitness()));
					//help.add(new Individual(gBest.getPosition(), gBest.getFitness()));
				}
				//LiveOutput: evaluationsbasiert
				EvaluationBasedLiveOutput(help);
				
				
				// Berechne den neuen Geschwindigkeitsvektor
				calculateNewVelocity(population[p]);

				// Bewege das Partikel
				population[p].updatePosition();
			}
		}
		
		return true;
	}

	/** 
	 * Gibt eine Kurzbeschreibung des Algos zurueck.
	 * 
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		String out;

		out = 	"\n\nKlasse BasisPSO\n";
		out +=	"=================\n";
		out +=	"\n";
		out +=	"Beschreibung:\n";
		out +=	"Implementation eines Mehrkriteriellen Particle Swarm Opitmization Agorithmus\n";
		out +=	"Bei diesem Algorithmus fliegt ein Schwarm von Partikeln ber den Suchraum.\n";
		out +=	"Jedes Partikel kennt seine aktuelle Position, seine bisher beste Position" +
				"und die beste erreichte Position des gesamten Schwarms. Auerdem besitzt es einen" +
				"Geschwindigkeitsvektor\n";
		out +=	"Pro Iteration bewertet jedes Partikel seine aktuelle Position und berechnet daraufhin" +
				"seinen Geschwindigkeitsvektor neu. Schlielich wird die Position aktualisiert.\n";
		out +=	"Bei diesem Ansatz wird pro Durchlauf immer nur eine Loesung erzeugt, deswegen sind" +
				"mehrere \"cycles\" noetig";
		out +=	"\n";
		out +=	"Kommandozeilenparameter:\n";
	    out +=	"-t: Klasse der Testfunktion (z.B. testfkt.FonsecaF1)\n";
	    out +=	"-g: (int): Anzahl der Generationen/Iterationen\n";
	    out +=	"-r: (int): optional, Random-Seed (Initialisierung der Zufallsklasse)\n";
	    out +=	"-o: optional, Praefix fuer die Dateinamen der Ausgabe (An diese wird noch" +
	    		" das Kuerzel fuer die Art der Ausgabedatei angehaengt, z.B. .best.res);" +
	    		" Wird der Wert nicht gesetzt, so wird ein Praefix aus Datum und Uhrzeit gewaehlt\n";
		out +=  "-live -> Abstand, wieviele Evaluationen zwischen 2 liveOutputs liegen soll.\n";
	    out +=	"-cycles: Anzahl der Durchlaeufe (nicht zu verwechseln mit Iterationen)\n";
	    out +=	"-populationSize: Groee der Population";
	    out +=	"-archive: Groesse des Archivs der pareto-optimalen Individuen.";
		out += 	"-inertia: Traegheitswert; beeinflusst ob lokal oder global gesucht wird\n";
		out += 	"-c1: Lernfaktor; steuert, wie stark die beste Position eines Partikels" +
				"beruecksichtigt wird\n";
		out +=	"-c2: Lernfaktor; steuert, wie stark die Position des Schwarmbesten" +
				"beruecksichtigt wird\n";
		out += 	"\n\n";

		return out;
	}

	/**
	 * Die Main Methode erzeugt ein neues Algo Objekt, um den Particle
	 * Swarm Algorithmus aufzurufen. Nach dem Durchlauf schreibt sie die
	 * Fitnesswerte und Objektvariablen der besten Partikel jeweils in eine
	 * Ausgabedatei.
	 * @param args enthaelt die Kommandozeilen Parameter
	 * @throws Exception
	 */
	public static void main(String[] args) throws Exception
	{
		// Algorithmus-Objekt erzeugen	
		BasisPSO algo = new BasisPSO(args);
		ArrayList output = new ArrayList();

		System.out.println("Starte BasisPSO mit " + algo.cycles + " Durchlaeufen...");

		// Algorithmus starten
		// Bei diesem Ansatz von PSO wird pro Durchlauf immer nur eine Loesung erzeugt
		// Deswegen werden mehrere "cycles" benoetigt
		for (int i = 0; i < algo.cycles; i++) {
			algo.startAlgo();
			System.out.print(".");
			// output ist nun ein paretooptimales Archiv.. paretoOptimal bzgl der Inividuen
			Individual next= new Individual(algo.gBest.getPosition(), algo.gBest.getFitness());
			output = ParetoArchive.refreshArchive(algo.archiveSize, output, next);
			//output.add(new Individual(algo.gBest.getPosition(), algo.gBest.getFitness()));
		}
		
		// Testfkt-Nachbearbeitung
		output = (ArrayList)algo.test.postProcessing(output);
		// Testfkt-Finalize
		algo.test.finalize();
		
		System.out.println("Schreibe Ausgabe in \"" + algo.outputFileNamePrefix + "\"...");

		AlgoOutput.writeFitness(output, algo.outputFileNamePrefix, false);
		AlgoOutput.writeObjectVariables(output, algo.outputFileNamePrefix, false);

		System.out.println("Beendet");
	}
}

