/*
 * Copyright (C) 2005  PG447 - University Dortmund
 * 
 * This file is part of NObELTJE.
 * 
 * NObELTJE is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as published by
 * the Free Software Foundation.
 * 
 * NObELTJE is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with NObELTJE; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

 /*
 * Created on June 20, 2004
 *  
 */
package org.nobeltje.algo ;

import java.io.File;
import java.io.FileWriter;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;
import java.util.Properties;

import org.nobeltje.tools.HyperCubeTree;
import org.nobeltje.tools.Individual;
import org.nobeltje.tools.Leaf;
import org.nobeltje.tools.ParetoDomination;
import org.nobeltje.tools.Particle;
import org.nobeltje.tools.io.ArgsInputBeautifier;

/**
 * 
 * <br><b>Benoetigte Testfunktionsaufrufe:</b>
 * <br>- Initialisierung : 2 * quantityParticle
 * <br>- Pro Generation  : quantityParticle * #Generationen
 * 
 * 
 * @author $Author: noot $
 * @version $Revision: 726 $
 *  $Date: 2005-02-02 00:50:20 +0100 (Mi, 02 Feb 2005) $
 */

public class MopsoOne extends AbstractAlgo {
	/**
	 * Exogener Parameter: Anzahl der Partikel(Population)
	 */
	private int quantityParticle = 120 ;	
	/**
	 * Dimension des Definitiionsbereichs; wird von der Testfunktion bestimmt
	 */
	private int dimDef; 
	/**
	 * Dimension des Zielbereichs ; wird von der Testfunktion bestimmt
	 */
	private int dimFit; 
	/**
	 * Exogener Parameter: Traegheitswert
	 */
	private double inertia  = 0.6        ;  
	/**
	 * Exogener Parameter: 1. Parameter fuer die Zufahlsvariable
	 */	
	private double c1	=1.2		    ;
	/**
	 * Exogener Parameter: 2. Parameter fuer die Zufahlsvariable
	 */	
	private double c2	=1.2		    ;
	/**
	 * Exogener Parameter: Maximalwert fuer den Geschwindigkeitsvektor
	 */	
	private double maxVelocity =5.0		    ;
	/**
	 *  Array der Partikel
	 */  	
	private Particle [] particle ;
	/**
	 *   Liste der nicht dominierten Vektoren im Definitionsbereich
	 */  				
	private List repDef = new ArrayList();
	/**
	 *   Paretofront: Liste der nicht dominierten Vektoren im Zielbereich
	 */  
	private List repFit	= new ArrayList();  
    /**
     *  maximale Groesse fuer das Repository/Aufbewahrungsort
     */
	private int maxRepository  =150      ;
	/**
	 *  Anzahl der Hyperkuben, die in den Zielraum gelegt werden
	 */	
	private int quantityHypercube =150 ;
	/**
	 * Anzahl der maximalen Funktionsauswertungen.
	 * Wird dieser parameter per Kommandozeile oder Datei uebergeben, so wird die Anzahl der Generationen in Abhaengigkeit 
	 * von der Anzahl der Partikel bestimmt und ueberschrieben
	 * #Generationen = #fevals / #Partikel
	 * Wird die Anzahl der Partikel nicht angegeben, so wird diese durch die Formel bestimmt: 
	 * #Partikel = #fevals / #Generationen
	 */
	private int fevals =-1;
	
	/**
	 *  scalingInertia gibt an, ob der Traegheitswert(inertia) waehrend des Laufs veraendert wird.(Startwert maxInertia, Endwert minInertia).
	 * bei scylingInertia = 1  wird inertia veraendert, ansonsten bleibt sie waerend des Laufs gleich
	 */
	private int scalingInertia = 0 ;
	
	/**
	 * Startwert fuer inertia, falls scalingIneria auf eins gesetzt ist 
	 */
	private double maxInertia =0.9 ;
	
	/**
	 *  Endwert fuer inertia, falls scalingInertia auf eins gesetzt ist
	 */
	private double minInertia = 0.4 ;
	/**
	 * gibt den Anteil an Generationen an, in denen der Traegheitswert veraendert wird.
	 * der wert sollte zwischen 0 und 1 liegen.  
	 */
	private double dueGeneration =0.7 ;
	
	private long randomSeed =0 ;
	
	private int onlineViz =0;
	private String onlineVizDir ;
	
	/**
	 * Konstruktor: Liest alle Kommandozeilenparameter  und 
	 * beendet bei Fehlern das Programm 
	 * @param args
	 */ 
	public MopsoOne(String args[]){
		super(args);
		onlineVizDir = new String("OnlineViz"+System.currentTimeMillis());
		quantityParticle=ArgsInputBeautifier.getIntProperty(specificParameters, "quantityParticle", quantityParticle, false);
		inertia=ArgsInputBeautifier.getDoubleProperty(specificParameters, "inertia", inertia, false);
		c1=ArgsInputBeautifier.getDoubleProperty(specificParameters, "c1", c1, false);
		c2=ArgsInputBeautifier.getDoubleProperty(specificParameters, "c2", c2, false);
		maxVelocity=ArgsInputBeautifier.getDoubleProperty(specificParameters, "maxVelocity", maxVelocity, false);
		maxRepository=ArgsInputBeautifier.getIntProperty(specificParameters, "maxRepository", maxRepository, false);
		quantityHypercube=ArgsInputBeautifier.getIntProperty(specificParameters, "quantityHypercube", quantityHypercube, false);
		fevals = this.specificParameters.getIntProperty("fevals",-1,false);
		if (fevals != -1){
			
			if(this.specificParameters.containsKey("quantityParticle"))
				this.maxGenerations = this.fevals / this.quantityParticle ;
			else
				quantityParticle = this.fevals / this.maxGenerations ;
			
			fevals = this.maxGenerations * this.quantityParticle ;
			
			int indexOfPathseparator =outputFileNamePrefix.lastIndexOf(File.separator);
			String pathName = outputFileNamePrefix.substring(0,indexOfPathseparator+1);
			String fileName = outputFileNamePrefix.substring(indexOfPathseparator+1,outputFileNamePrefix.length());
		
		
			Properties outputFileProperties = this.extractParameterValues(fileName);
			if(outputFileProperties.containsKey("fevals")){
				StringBuffer nameBuffer = new StringBuffer();
				nameBuffer.append("g="+this.maxGenerations);
				nameBuffer.append("_quantityParticle="+this.quantityParticle);
				nameBuffer.append("_fevals="+this.fevals);
				Enumeration enu = outputFileProperties.keys();
				while( enu.hasMoreElements()){
					String key = (String)enu.nextElement();
					if(!(key.equals("g")||key.equals("quantityParticle")||key.equals("fevals")))
					nameBuffer.append("_"+key+"="+outputFileProperties.getProperty(key));
				}
				this.outputFileNamePrefix = pathName+nameBuffer.toString();
			}
		}
			
		randomSeed = ArgsInputBeautifier.getIntProperty(specificParameters, "randomSeed", 0, false);
		if(randomSeed != 0){
			rand.setSeed(randomSeed);
		}
		scalingInertia=this.specificParameters.getIntProperty("scalingInertia",0,false);
		if (scalingInertia == 1){
			maxInertia = this.specificParameters.getDoubleProperty("maxInertia",0.9,false);
			minInertia = this.specificParameters.getDoubleProperty("minInertia",0.4,false);
			dueGeneration = this.specificParameters.getDoubleProperty("dueGeneration",0.7,false);
			
		}
		if(dueGeneration <0 || dueGeneration >1){
			System.out.println("der Parameter dueGenerations muss zwischen 0 und 1 liegen!!!   uebergebener Wert = "+ dueGeneration);
			dueGeneration = 0.7;
		}
		onlineViz = this.specificParameters.getIntProperty("onlineViz",0,false);
		
		System.out.println("fevals ="+this.fevals+" "+
							"generationen ="+this.maxGenerations +" "+
							"quantityParticle ="+this.quantityParticle+" "+
							"inertia ="+ this.inertia+" "+
							"c1 ="+this.c1+" "+
							"c2 ="+this.c2+" "+
							"scalingInertia ="+this.scalingInertia+" "+
							"maxInertia ="+this.maxInertia+" "+
							"minInertia ="+this.minInertia+" "+
							"dueGeneration ="+this.dueGeneration+" "+
							"maxVelocity ="+this.maxVelocity+" "+
							"quantityHypercubes ="+this.quantityHypercube+
							"maxRepository"+this.maxRepository+
							"RandomSeed ="+this.rand.getSeed());
		
		try{
			test.init(specificTestfktParameters);
			dimDef = test.getDimension();
		
			
			dimFit =test.getNumberOfObjectives();
			
			particle = new Particle[this.quantityParticle] ;
			for(int i = 0 ; i < this.quantityParticle ; i++)
			   particle[i] = new Particle();
			
		}catch(Exception e){
			System.out.println("Testfunktion nicht in Ordnung");
			e.printStackTrace();
		}
		
		
	}
	
	/**
	 * Startroutine des Algorithmus
	 */	
	
	public boolean startAlgo() {
		
		int changeInertia = (int)(this.maxGenerations * dueGeneration )-1;
		if (scalingInertia == 1){
			inertia = maxInertia ;
		}
		  
		initializeParticle();
		if(onlineViz == 1){
			drawParticle(0);
		}
		updateRep();
		
		if(debug == true){
			System.out.println("Anfangsstadium");
			ausgabe();
		}
		
		for(int i = 0 ; i<maxGenerations ; i++){
			if(scalingInertia ==1 && i <= changeInertia){
				inertia = maxInertia - (maxInertia - minInertia)*i /changeInertia ;
			}
			
			computeSpeed();
			computeNewPos();
			updateRep();
			
			
			if(onlineViz == 1){
				 drawParticle(i+1);
			}
			
			if(debug == true){
			   System.out.println("Iteration   "+ i);
			   ausgabe();
			}
			
		}
		if(debug == true){
			System.out.println("Endstadium");
			ausgabe();
			
		}
		
		/* Nachbearbeitung durch die Testfunktion */
		// Ergebnisse in Individuen umwandeln
		List archive = Individual.makeIndividuals(repDef, repFit);
		// PostProcessing starten
		archive = test.postProcessing(archive);
		// Ergebnisse des Postprocessings wieder in die alten Listen umwandeln
		Individual.extractIndividuals(archive, repDef, repFit);
		
		/* Testfunktion-Finalize */
		test.finalize();
			
		printOutput();
		return true;
	}

	
	
	
	/**
	 * Hier werden die Partikel also die Attribute 
	 *  posDef, posFit, bestPosDef, bestPosFit und speed initialisiert 
	 *
	 */
	private void initializeParticle(){
		List paramIntervall = new ArrayList() ;
		try{
			paramIntervall = test.getOptimalBounds();
		}
		catch (Exception e){
			
		}
		/**
		 * Partikel werden initialisiert
		 */
		try{
		   for (int i= 0 ; i < quantityParticle ; i++)
		   {
			  ArrayList pos = new ArrayList();
			  ArrayList pBest = new ArrayList();
			  ArrayList vel   = new ArrayList();
			  for (int j =0 ; j < dimDef ; j++)
			  {
				  double from = ((Double)paramIntervall.get(j*2)).doubleValue() ;
				  double to   = ((Double)paramIntervall.get(j*2+1)).doubleValue() ;
				  double wert = (rand.nextDouble(from,to));
				  pos.add(j,new Double(wert)) ;
				  pBest.add(j,new Double(wert));
				  vel.add(j,new Double(0));
			  }
			  particle[i].setPosition(pos);
			  particle[i].setPBest(pBest);
			  particle[i].setVelocity(vel);
			  particle[i].setFitness((ArrayList)test.execute(pos));
			  particle[i].setPBestFit((ArrayList)test.execute(pBest));
		  }
		}catch(Exception e){
			System.out.println("Probleme beim initialisieren der Partikel");
		}
		
	}
	
	/**
	 * Die neuen Geschwindigkeitsvektoren speed werden berechnet
	 */
	private void computeSpeed(){
		// rep ist die Position des global besten Partikels
		ArrayList rep   = (ArrayList)repDef.get(this.getBestPart());  
		for ( int i = 0 ; i<  this.quantityParticle; i++ ) {
			ArrayList vel   = particle[i].getVelocity();
			ArrayList pBest = particle[i].getPBest();
			ArrayList pos   = particle[i].getPosition();
			
			
			for (int j = 0; j< dimDef ; j++){
				
				double d =  this.inertia * ((Double)vel.get(j)).doubleValue(); 
				
				 d +=  this.c1 * rand.nextDouble()* (((Double)(pBest.get(j))).doubleValue() - ((Double)(pos.get(j))).doubleValue());  
				 d += this.c2 * rand.nextDouble()*(((Double)rep.get(j)).doubleValue()-  ((Double)(pos.get(j))).doubleValue()) ;
				vel.set(j,new Double(d)); 
			} 
			// 
			double VelocityLength =0 ;
			for(int j = 0 ; j <dimDef ; j++)
				   VelocityLength += Math.pow(((Double)vel.get(j)).doubleValue(), 2.0);
			VelocityLength = Math.sqrt(VelocityLength);
			if (VelocityLength > maxVelocity)
				   for(int j =0 ; j< dimDef ; j++){
				   Double d = new Double(((Double)vel.get(j)).doubleValue()* maxVelocity / VelocityLength);
				   vel.set(j, d);
				   }
				
			
		}
		
	}
	
	/**
	 * neue Positionen und beste Positionen werden berechnet
	 */
	private void computeNewPos(){
		List defBereich = test.getBounds();
		for (int i = 0 ; i < this.quantityParticle ; i++){
		
			try{
				particle[i].updatePosition();
				/*   // falls der Definitionsbereich verlassen wird, auf den Rand setzen
				for (int j =0 ; j < this.dimDef ; j++){
					if( ((Double)particle[i].getPosition().get(j)).doubleValue() < ((Double)defBereich.get(j *2)).doubleValue())
				         particle[i].getPosition().set(j, defBereich.get(j *2)) ;
				         
					if( ((Double)particle[i].getPosition().get(j)).doubleValue() > ((Double)defBereich.get(j *2+1)).doubleValue())
						particle[i].getPosition().set(j, defBereich.get(j *2+1)) ;  
					
				}*/
				particle[i].setFitness((ArrayList)test.execute(particle[i].getPosition()));
				
				
				List archiveOutput = Individual.makeIndividuals(repDef, repFit);
				EvaluationBasedLiveOutput(archiveOutput);
							
				int domination = ParetoDomination.checkDomination(particle[i].getFitness(), particle[i].getPBestFit());
				if(domination ==2 ||( domination !=1 && this.rand.nextBoolean()== true)){ 
					particle[i].setPBest((ArrayList)particle[i].getPosition().clone());
					particle[i].setPBestFit((ArrayList)particle[i].getFitness().clone());			
				}
			}catch(Exception e){
				e.printStackTrace();
			}	
		}
		
		
	}
	
	/**
	 * Das repository wird aktualisiert.
	 * Es werden von allen Partikeln die Fitnesswerte betrachtet und mit den Eintraegen in repFit verglichen und 
	 * die dominierten Vektoren aus repFit und repDef entfernt.
	 */
	
	private void updateRep(){
		for (int i = 0; i < this.quantityParticle; i++){
			ArrayList list = particle[i].getFitness();
			int index = 0 ;
			while(index <= this.repFit.size()){
				if (index == this.repFit.size())
				{
					this.repDef.add((ArrayList)particle[i].getPosition().clone());
					this.repFit.add((ArrayList)list.clone());
				}
				else
				{
					int check = ParetoDomination.checkDomination((ArrayList)this.repFit.get(index),list) ;
					switch(check){
						case 1: this.repDef.remove(index); this.repFit.remove(index);break;
						case 2:  index = this.repFit.size()+1;break;
						case 3: index =this.repFit.size() +1;break;
						case 4: index++; break;
					}
				}
				     
			}
			// wenn maximale Kapazitaet des Repository ueberschritten
			while(this.repDef.size() > this.maxRepository){
				int h = getWorstPart();
				repDef.remove(h);
				repFit.remove(h);
			}
			
		}
		
		
	}
	/** 
	 * gibt den Index eines Partikels aus der Repository an, der sich in einem 
	 * wenig bewohntem Hypercube befindet. Benutzt dabei nur repFit
	 */  
	private int getBestPart(){
		HyperCubeTree hCT = new HyperCubeTree(dimFit , this.quantityHypercube,(ArrayList) repFit);
		ArrayList cubes =hCT.getCubes();
		if(debug == true){
		
		System.out.println(" \n Methode getBest  ; Cubes werden ausgegeben \n");
		for(int i =0; i < cubes.size(); i++)
		    System.out.println("\n cube :  "+i+"   Anzahl=  "+  ((Leaf)cubes.get(i)).getQuantity()+"\n"+ ((Leaf)cubes.get(i)).getIndexList()); 
		}
		int minNumberOfParticles = ((Leaf)cubes.get(0)).getQuantity() ;
		int i = 0 ;
		while(i+1 < cubes.size() && minNumberOfParticles ==  ((Leaf)cubes.get(i+1)).getQuantity())
		     i++ ;
		i = rand.nextInt(i+1) ;
		Leaf  foundCube = (Leaf)cubes.get(i);
		int ret =((Integer)foundCube.getIndexList().get(rand.nextInt(foundCube.getIndexList().size()))).intValue();
		if(debug == true)
		   System.out.println(" zurueckzugebender Partikel :  "+ ret);
		return 	ret ;
			
		}
		     
		
		
	/** 
	 * gibt den Index eines Partikels aus dem Repository an, der sich in einem 
	 * stark bewohntem Hypercube befindet. 
	 */	
	private int getWorstPart(){
		HyperCubeTree hCT= new HyperCubeTree(dimFit , this.quantityHypercube,(ArrayList) repFit);
				ArrayList cubes =hCT.getCubes();
		        if(debug == true){
		        
					System.out.println(" \n Methode getWorst ;Cubes werden ausgegeben \n");
					for(int i =0; i < cubes.size(); i++)
						System.out.println("\n cube :  "+i+"   anzahl=  "+  ((Leaf)cubes.get(i)).getQuantity()+"\n"+ ((Leaf)cubes.get(i)).getIndexList()); 
		        }
				Leaf  foundCube = (Leaf)cubes.get(cubes.size() -1);
				int ret =((Integer)foundCube.getIndexList().get(rand.nextInt(foundCube.getIndexList().size()))).intValue();
			    if(debug == true)
					System.out.println(" zurueckzugebender Partikel :  "+ ret);
				return 	ret ;
	}
	
	private Properties extractParameterValues(String fileName){
	 	Properties prop = new Properties();
	 	String [] namesAndValues = fileName.split("_");
	 	for (int i = 0; i< namesAndValues.length ; i++){
	 		
	 		String [] nAndV = namesAndValues[i].split("=");
	 		if(nAndV.length == 2)
	 			prop.setProperty(nAndV[0],nAndV[1]);
	 	}
	 	return prop ;
	 }
	
	/**
	 * Ausgabedatei wird geschrieben
	 */
	private void printOutput(){
		/*
		ArrayList ausgabe = new ArrayList();
		for(int i=0; i<this.repDef.size(); i++)
		{
			Individual particle = new Individual((ArrayList)repDef.get(i), (ArrayList)repFit.get(i));
			ausgabe.add(particle);
		}
		*/
		List ausgabe = Individual.makeIndividuals(repDef, repFit);
		try
		{
			org.nobeltje.tools.io.AlgoOutput.writeFitness(ausgabe, outputFileNamePrefix,false);
			org.nobeltje.tools.io.AlgoOutput.writeObjectVariables(ausgabe, outputFileNamePrefix,false);
		}
		catch(Exception e)
		{
		    System.out.println("Fehler  beim schreiben der Ausgabe in Datei");
		}
		
	}
	
	/**
	 * 
	 */
	private void ausgabe(){
		
		for(int i =0 ; i< this.quantityParticle; i++){
		
		System.out.println("\npartikel  "+i);
		System.out.println("position :    "+particle[i].getPosition());
		System.out.println("fitness   :    "+particle[i].getFitness());
		System.out.println("bestPos:    "+particle[i].getPBest());
		System.out.println("bestFit  :    "+particle[i].getPBestFit());
		System.out.println("speed   :    "+particle[i].getVelocity());
		}
		System.out.println("\nRespository mit "+repDef.size()+" Elementen"+( this.repDef.size() == this.repFit.size()));
		
		for (int i= 0; i< repDef.size();i++){
			System.out.println("Element  " +i );
			System.out.println("Position  :"+repDef.get(i));
			System.out.println("Fitness   :"+repFit.get(i));
			
		}
		  
	}
	
	private void drawParticle(int generation){
		
		new File(onlineVizDir).mkdirs();
		String str = ""+generation ;
		String onlineFile = onlineVizDir+File.separator+"OnlinePng";
		
		String pictureName = "generation" ;
		for (int i =1 ; i<= 6 -str.length() ; i++ ){
			pictureName= pictureName+"0";
		}
		pictureName = pictureName + generation+".png";
		try{
		
		FileWriter fwPng = new FileWriter(onlineFile+".best.in", false );
		String valueInertia = new String(""+inertia);
		valueInertia = valueInertia.substring(0,Math.min(5,valueInertia.length()));
		fwPng.write("set title '"+specificParameters.getProperty("t")+"inertia="+valueInertia+"_generation=" + generation+"_PartikelAnzahl="+quantityParticle+"_c1="+c1+"_c2="+c2+  "'\n");
        fwPng.write("set xlabel \""+"X"+"\"\n");
        fwPng.write("set ylabel \""+"Y"+"\"\n");
        
		fwPng.write("set term png\n");
	    fwPng.write("set output \"" + onlineVizDir+File.separator+ pictureName+ "\"\n" );
		fwPng.write("plot [-1:2][-1:2] '-'\n");
		fwPng.close();
		ArrayList ausgabe = new ArrayList();
				for(int i=0; i<this.quantityParticle; i++)
				{
					Individual particle = new Individual((ArrayList)this.particle[i].getPosition(), this.particle[i].getFitness());
					ausgabe.add(particle);
				}
				try
				{
					
					org.nobeltje.tools.io.AlgoOutput.writeObjectVariables(ausgabe,onlineFile ,true);
				}
				catch(Exception e)
				{
					System.out.println("Fehler in der beim schreiben der Ausgabe in Datei");
				}
			fwPng = new FileWriter(onlineFile+".best.in", true );
			fwPng.write("e\n");
			fwPng.close();		
			Process visualizationProcess = Runtime.getRuntime().exec("gnuplot -persist "+onlineFile+".best.in" );
			visualizationProcess.waitFor();
		    
		}			
		catch (Exception e){
			System.out.println("Fehler in der Methode  drawParticle");
			e.printStackTrace();		 
		}
							 
							 
	}
	
	/** 
		 * Gibt eine Kurzbeschreibung des Algos zurueck.
		 * 
		 * (non-Javadoc)
		 * @see java.lang.Object#toString()
		 */
		public String toString() {
			String out;

			out =  "\n\nKlasse MopsoOne\n";
			out += "=================\n";
			out += "\n";
			out += "Beschreibung:\n";
			out += "Implementation eines Mehrkriteriellen Particle Swarm Opitmization Agorithmus";
			out += " Dieser Algorithmus laesst einen Schwarm von Partikeln auf den Steuerraum fliegen." +
				"Dabei werden die Partickel zufaellig auf den Raum verteilt.\n";
			out += "Jeder Partikel hat ein Archive, in dem die BestePosition, BesteFitness, die aktuelle Fitnesse bzw. Position " +
				" und die Geschwindigkeit gespeichert wird.\n";
			out += "Je Iteration veraendern die Partikel ihre Positionen und berechnen ihre Fitness, aktuallisieren ihr Archive.\n";
			out += "Im Repository werden alle nichtdominierten Partikel gespeichert.\n";
			out += "Wenn das Repository kein Partikel mehr aufnehmen kann, wird nach dem Dominanzkriterium oder mit Hilfe von " +
				"Hypercubes das Repository gefuellt bzw. geleert.\n";
			out += "Es werden dazu Hypercubes auf den Zielraum gelegt, um aus mehrbevoelkerten Regionen Partickel zu entfernen," +
				"Partickel in wenigerbevoelkerten Regionen erhalten hoehere Fitnesswerte. \n";
			out += "Dadurch wird gesichert, dass die Partikel einen groesseren Raum erkunden.\n";
			out += "\n";
			out += "Kommandozeilenparameter:\n";
			out += "-t: Klasse der Testfunktion (z.B. testfkt.FonsecaF1)\n";
			out += "-g: Anzahl der Generationen/Iterationen\n";
			out += "-live -> Abstand, wieviele Evaluationen zwischen 2 liveOutputs liegen soll.\n";
			out += "-quantityParticle: Anzahl der Partikel(Population) \n";
			out += "-fevals : maximale Anzahl der Funktionsauswertungen" ;
			out += "Wird dieser parameter per Kommandozeile oder Datei uebergeben, so wird die Anzahl der Generationen in Abhaengigkeit\n"; 
			out += " von der Anzahl der Partikel bestimmt und ueberschrieben\n";
			out += " #Generationen = #fevals / #Partikel\n";
			out += " Wird die Anzahl der Partikel nicht angegeben, so wird diese durch die Formel bestimmt:\n"; 
			out += " #Partikel = #fevals / #Generationen\n";
			out += "-inertia: Traegheitswert\n";
			out += "-c1, c2: Parameter fuer die Zufahlsvariable\n";
			out += "-quantityHypercube: Anzahl der Hyperkuben, die in den Zielraum gelegt werden\n";
			out += " maxRepository: maximale Groesse fuer das Repository/Aufbewahrungsort\n";
			out += " maxVelocity :Maximalwert fuer den Geschwindigkeitsvektor\n";
			out += " randomSeed : ist dieser parameter ungleich 0, so wird \n" +
				" der Algo mit einem festen RandomSeed laufen";
			out += "\n\n";
	    
			return out;
		}
	public static void main(String args []) {
		
		MopsoOne mopso = new MopsoOne(args);
		mopso.startAlgo();
		
			
	}

}



