/*
 * Copyright (C) 2005  PG447 - University Dortmund
 * 
 * This file is part of NObELTJE.
 * 
 * NObELTJE is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as published by
 * the Free Software Foundation.
 * 
 * NObELTJE is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with NObELTJE; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

 /*
 * Created on May 27, 2004
 *
 */
package org.nobeltje.algo;

import java.util.ArrayList;
import java.util.Iterator;

import org.nobeltje.tools.Individual;
import org.nobeltje.tools.ParetoArchive;
import org.nobeltje.tools.ParetoDomination;
import org.nobeltje.tools.io.AlgoOutput;
import org.nobeltje.exceptions.InvalidBoundException;

/**
 * <p>Algorithmus, der einen 1  plus 1 Algorithmus realisiert.</p>
 * Implementiert von Daniel Saltmann, Nicola Beume, Kay Thielmann und Miriam Padberg
 * 
 * 
 * <br><b>Benoetigte Testfunktionsaufrufe:</b>
 * <br>- Initialisierung : 1
 * <br>- Pro Generation  : #Generationen
 * 
 * 
 * @author $Author: noot $
 * @version $Revision: 726 $
 * $Date: 2005-02-02 00:50:20 +0100 (Mi, 02 Feb 2005) $
 */

public class OnePlusOnePG extends AbstractAlgo {
	
	/** Variablen */

	/**
	 * Standardabweichung fuer Mutation
	 */
 	private double standardDeviation;	
 	/**
 	 * Maximale Groesse des Archivs
 	 */
	private int archiveSize;
	/**
	 * Objekt fuer Archiv
	 */
	private ArrayList archive = new ArrayList();
	/**
	 * Zaehler fuer die Anzahl der komplett bearbeiteten Generationen
	 */
    private int anzahlFertigeIterationen = 0; 


	/**
	 * Leerer Konstruktor, der nur den Konstruktor der Superklasse (AbstractAlgo) aufruft. (Wird fuer Help-Klasse benoetigt.)
	 */
	public OnePlusOnePG() {
	    super();
	}
	
	
   /**
    * <p>Konstruktor</p>
    * <p>Dieser Konstruktor braucht die Paramter der abstrakten Klasse abstractAlgo, sowie 
    * die zusaetzlichen Argumente deviation (Standardabweichung), dessen default Wert bei nicht-Angabe auf 0 gesetzt
    * wird, sowie die Angabe der Archivgrsse (archiveSize). Falls die Archivgrsse nicht angeben wurde, so wird der
    * Algorithmus abgebrochen und eine Exception geworfen, die darauf hinweisst, dass die Eingabe nicht vollstaendig war.</p>
    * 
    * @param args
    */
    public OnePlusOnePG (String[] args)
	{
		/* Konstruktor von AbstractAlgo aufrufen um wichtigste Dinge zu setzen */
		super(args);
	    
	   /* Parameter holen */	
		standardDeviation = specificParameters.getIntProperty("deviation", 1, false);
		archiveSize = specificParameters.getIntProperty("archive", 0, false);

	   /* Archiv initialisieren */	
		archive = new ArrayList();
 	
	}


   /** 
    * <p>Startroutine des Algorithmus</p> 
    * 
    * <p>Die eigentliche Hauptroutine des Algorithmus. Es wird hier zuerst ein Element randomisiert erzeugt, was in den Grenzen 
    * zwischen -5 und 5 liegt. Dazu wird die Vorkomma und die Nachkommastellen unabhaengig voneinander erzeugt. Anschliessend wird 
    * das Elter-Individuum aus den Objektvariablen generiert.</p>
    * 
    * <p>Fuer jede Iteration wird also das Elter- Element reproduziert. Dieses Element (offspring) bekommt die als Generation die Iteration zugwiesen,
    * anschliessend wird es mutiert, danach wird ueberprueft, ob das Element dominierend ist und das Archiv entsprechend upgedatet.<br />
    * Falls das so erzeugte Individuum offspring mindestens genau so gut ist, wie das Elter, so wird es zur Reproduktion weiterverwendet, d.h. fuer
    * die folgende Iteration ist das offspring dann das neue Elter.</p>
    */	
	public boolean startAlgo() throws InvalidBoundException
	{		
		test.init(specificTestfktParameters);
		
	   /* Start-Indiviuum (parent) erzeugen */	
		/* Objektvariablen von parent setzen */ 
		ArrayList parentObj = new ArrayList();  // parentObj Objektvariablen des neuen Individuums
		
		for (int j = 0; j < test.getDimension(); j++)
		{
		    /* Die Objektvariablen werden zufaellig in von der Testfunktion gegebenen Schranken generiert */
			double min = (((Double)test.getOptimalBounds().get(2*j)).doubleValue()); // Min-Grenze fuer ObjVar aus Testfkt
			double max = (((Double)test.getOptimalBounds().get(2*j+1)).doubleValue()); // Max-Grenze fuer ObjVar aus Testfkt
		    Double randomValue = new Double(rand.nextDouble(min, max));
			parentObj.add(randomValue);					
		}
		/* erstes parent-Individuum aus Objektvariablen generieren */
		Individual parent;
		try
		{
			parent = new Individual(parentObj, (ArrayList) test.execute(parentObj));
		}
		catch (Exception e)
		{
			System.err.println("Fehler bei ausfuehren der Tesfkt!");
			e.printStackTrace();
			parent = new Individual(parentObj, new ArrayList());
		}
		archive.add(parent);  // Parent schonmal zum archiv hinzufuegen, sonst wird das verworfen

		/* offspring Element*/
		Individual offspring;
		/* Hauptschleife des Algorithmus ueber die Anzahl der Iterationen */	
		for (int i = 0; i<maxGenerations; i++)
		{	
			/* Reproduktion */
			offspring = (Individual)parent.clone();
			offspring.setGeneration(i+1);
			/* Mutation */
			mutate(offspring);		//Inklusive Aufruf der Testfunktion   
				
			/* Archiv updaten -- dazu dann Pareto-Dominanz fuer das Individuum testen und mit allen Elementen im Archiv
			 * auf Pareto-Dominanz testen */
  			archive = ParetoArchive.refreshArchive(archiveSize, archive, offspring);
			
  			/* Evaluationsbasierter LiveOutput */
  			EvaluationBasedLiveOutput(archive);
			
			/* Live Output, damit auch schoen was zu sehen ist ;-) */
  			anzahlFertigeIterationen++;
  			if (debug){
  				liveOutput(archive);
  			}
			
			/* Selektion: offspring weiter verwenden zur Reproduktion, wenns mind. gleich gut ist */
			if (ParetoDomination.nonDominated(parent.getFitness(), offspring.getFitness()) == true) {
				parent = offspring;
			}
		}
		
	    archive = (ArrayList)test.postProcessing(archive);
		test.finalize();
		
		return true; // erstmal stets true zurueckgeben
	}
	

	/**
	 *<p> Mutationsmethode</p>
	 * 
	 * <p>Die Mutation in diesem Algorithmus basiert darauf, dass jede einzelne Objektvariable dieses Individuums ausgelesen wird. <br />
	 * Weiterhin wird eine gaussche Normalabweichung randomisiert ausgewhlt, mit der Standardabweichung multipliziert und 
	 * zu der urprnglichen Objektvariable des Individuums addiert. Dieses ergibt dann die Objektvariable des mutierten Individuums.</p>
	 * 
	 * @param offspring Zu mutierendes Individuum
	 */
	private void mutate(Individual offspring){
		ArrayList offspringObj = offspring.getObjectVariables();
		//System.out.println("vorher: "+offspringObj.toString());
		for (int i=0; i < test.getDimension(); i++){
			/*offspring bei i wird mit einer Zufallsvariable nach Gauss addiert*/
			Double value = new Double( ((Double)offspringObj.get(i)).doubleValue() + standardDeviation*rand.nextGaussian());
			offspringObj.set(i, value);
			
		}
		//System.out.println("nachher: "+offspringObj.toString());
		try
		{
			offspring.setObjectVariables(offspringObj);
			offspring.setFitness((ArrayList)test.execute(offspringObj));
		}
		catch (Exception e)
		{
			System.err.println("Fehler beim ausfuehren der Testfkt!");
			e.printStackTrace();
			offspring.setFitness(new ArrayList());
		}
	}
  

	/** 
	 * <p>Gibt das aktuelle Individuen auf die Konsole aus.</p>
	 * 
	 * @param archive Auszugebendes Archiv
	 */
	private void liveOutput(ArrayList archive)
	 {
		 try 
		 {
			StringBuffer row=new StringBuffer();
			
			row.delete(0,row.length());
			System.out.println(row.toString());
			System.out.println("#### Archive ####");
			row.append("*** INTERATION "+ anzahlFertigeIterationen);
			System.out.println(row.toString());
			
			for (Iterator iter = archive.iterator(); iter.hasNext();)
			{
				Individual ind = (Individual)iter.next();
				ArrayList obj = ind.getObjectVariables();
				System.out.println("--------------- " + ind.getGeneration() + " ---------------");
				row.delete(0,row.length());
				row.append("Obj: ");
				for (Iterator objIter = obj.iterator(); objIter.hasNext();)
				{
					Double value = (Double) objIter.next();
					row.append(value.toString()+" ");
				}
				System.out.println(row.toString());
				ArrayList fit = ind.getFitness();
				row.delete(0,row.length());
				row.append("Fit: ");
				for (Iterator fitIter = fit.iterator();fitIter.hasNext();)
				{
					Double value = (Double) fitIter.next();
					row.append(value.toString()+" ");
				}
				System.out.println(row.toString());
			}
		 }
		 catch (Exception e) 
		 {
			 e.printStackTrace();
		 }
		 System.out.println("--------------------");
	 }

	 
	/** 
	 * Gibt eine Kurzbeschreibung des Algos zurueck.
	 * 
	 * (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
	    String out;

	    out =  "\n\nKlasse OnePlusOnePG\n";
	    out += "===================\n";
	    out += "\n";
	    out += "Beschreibung:\n";
	    out += "Implementation eines 1+1 Evolutionsstrategie.\n";
	    out += "Sowohl Elternpopulation als auch Offspringgeneration bestehen aus nur einem" +
	    		" Individuum. \n";
	    out += "Das Nachkommen wird durch Reproduktion des Elter-Indvididuums mit anschliessender" +
	    		" Mutation gebildet. Bei der Mutation wird zu jeder Objektvariable ein gauss-verteilter" +
	    		" Wert (Erwartungswert 0), gewichtet mit einer Standardabweichung, addiert." +
	    		" Die Standardabweichung wird fuer den gesamten Lauf fest exogen bestimmt.\n";
	    out += "\n";
	    out += "Die Selektion erfolgt durch einen einfachen Pareto-Vergleich der Eltern- und" +
	    		" der Offspringgeneration. Wird das neue Individuum nicht vom Elter dominiert," +
	    		" so bildet dies die neue Elternpopulation.\n";
	    out += "In einem globalen Archiv werden nicht dominierte Individuen gesammelt," +
	    		" die schliesslich ausgegeben werden.\n";
	    out += "\n";
	    out += "Kommandozeilenparameter:\n";
	    out += "-t: Klasse der Testfunktion (z.B. testfkt.FonsecaF1)\n";
	    out += "-g (int): Anzahl der Generationen/Iterationen\n";
	    out += "-r (int): optional, Random-Seed (Initialisierung der Zufallsklasse)\n";
	    out += "-o: optional, Praefix fuer die Dateinamen der Ausgabe (An diese wird noch" +
	    		" das Kuerzel fuer die Art der Ausgabedatei angehaengt, z.B. .best.res);" +
	    		" Wird der Wert nicht gesetzt, so wird ein Praefix aus Datum und Uhrzeit gewaehlt\n";
	    out += "-d: optional, wenn gesetzt, werden Debug-Ausgaben angezeigt\n";
		out += "-live -> Abstand, wieviele Evaluationen zwischen 2 liveOutputs liegen soll.\n";
	    out += "-archive (int): optional, Maximalgroesse des globalen Archives, welches am Ende ausgegeben wird; 0=unbegrenzt (Standardwert)\n";
	    out += "-deviation (int): optional, Standardabweichung fuer die Mutation (Standardwert ist 1)\n";
	    out += "\n\n";
	    
	    return out;
	}
	
		
	/** HAUPTROUTINE */
		 
	/**
	 * <p>Mainmethode.</p>
	 * 
	 * <p>Hier wird ein neues Objekt erzeugt, mit der Methode startAlgo aufgerufen.<br />
	 * Nach dem Durchlauf werden die Fitnesswerte der Individuum im  Archiv in eine bestimmte Datei geschrieben (output.fitness),
	 *  ebenso die Objektvariablen der Archivindividuen.</p>
	 * 
	 * @param args Angabe der Testfunktion, Standardabweichung, Archivgrosse, Anzahl Iterationen, etc.
	 */	 
	public static void main(String[] args) throws Exception	{
   	/* Algorithmus-Objekt erzeugen */	
		OnePlusOnePG algo = new OnePlusOnePG(args);
   	/* Algorithmus starten */
		algo.startAlgo();
   	/* Archiv (Objektvariablen) Textdatei ausgeben */
		AlgoOutput.writeObjectVariables(algo.archive, algo.outputFileNamePrefix, false);
   	/* Archiv (Fitnesswerte) in Textdatei output.fitness ausgeben */
		AlgoOutput.writeFitness(algo.archive, algo.outputFileNamePrefix, false);
	}
}

