/*
 * Copyright (C) 2005  PG447 - University Dortmund
 * 
 * This file is part of NObELTJE.
 * 
 * NObELTJE is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as published by
 * the Free Software Foundation.
 * 
 * NObELTJE is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with NObELTJE; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

 /*
 * Created on Nov 10, 2004
 *
 */
package org.nobeltje.algo;

import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.nobeltje.metrics.DeltaS;
import org.nobeltje.tools.Individual;
import org.nobeltje.tools.ParetoArchive;
import org.nobeltje.tools.io.AlgoOutput;

/**
 * 
 * <br><b>Benoetigte Testfunktionsaufrufe:</b>
 * <br>- Initialisierung : mu
 * <br>- Pro Generation  : #Generationen ??
 * 
 *  
 * @author $Author: noot $
 * @version $Revision: 726 $
 * $Date: 2005-02-02 00:50:20 +0100 (Mi, 02 Feb 2005) $
 *
*/
public class SMS_EMOA_pg extends AbstractAlgo{
	
	//TODO Probleme mit negativen Fitnesswerten ???
    //TODO Testcase schreiben
    //TODO Auswahl der Selektionen
    //TODO zufaellige Auswahl ueber einen Parameter
    //TODO unsere Metrik ins Metrik Packet
	
	/**
	  * Anzahl der Individuen in der Elterngeneration (exogen)
	  */
	private int mu;
	
	/**
	 * Selektionsmethode 
	 */
	private char selectMethod;
				
	/**
	 * Aktuelle Population, enthaelt mu Individuen
	 */
	private List population = new ArrayList(); 

	/**
	 * Nachkommen
	 */
	private List offsprings = new ArrayList();
	
	/**
	 * ArrayListe mit Individuen der letzten nicht-dominierten Front
	 */
	private List lastFront = new ArrayList();
	
	/**
	 * Anzahl nicht-dominierter Fronten
	 */
	private int frontCounter = 0;
	
	/**
	 * temporaere Population waehrend einer Generation,
	 * enthaelt mu+1 Individuen
	 */
	private List populationTemp = new ArrayList();
	
	/**
	 * Wahrscheinlichkeit, dass eine Rekombination stattfindet 
	 */
	private double recProb = 1.0;
	
	/**
	 * Aktuelle Generation
	 */
	private int currentGenerationCount = 0;

	
	
	//	--- Dokumentation des Algorithmus ---
	
	/**
	 * Anzahl der Anwendungen von selectDeltaS
	 */
	private int deltaSCounter = 0;
	
	/**
	 * Anzahl von Anwendungen von selectDomPoints 
	 */
	private int domPointsCounter = 0;
	
	/**
	 * Anzahl von:
	 * letzte Front enthaelt nur ein Element.
	 * Die Selektion ist dann besonders einfach,
	 * weil dieses Element entfernt wird.
	 */
	private int easySelectCounter = 0;
		
	/**
	 * Schreib die Anzahl Elemente in pro Front und die verwendete Selektionsvariante
	 * pro Generation in die Datei outputFileNamePrefix.fronts
	 */
	private FileWriter fileFronts;

	/**
	 * Schreibt fuer jede Generation fuer jede Front eine Datei mit den 
	 * Fitnesswerte der enthaltenen Individuen.
	 * Dateinamen: outputFileNamePrefix_gen=i_frontj.nondom
	 */
	private FileWriter fileNonDom;

	/**
	 * Zeilen, die fileFronts schreibt 
	 */
	StringBuffer row = new StringBuffer();

	/**
	 * Ausgabe der Dokumentations-Counter auf die Console
	 */
	boolean outputCounter = true;
	
	/**
	 * Ausgabe-Datei fuer Nichtdominierten-Fronten
	 */
	boolean outputNonDom = false;
	
	/**
	 * Ausgabedatei fuer Anzahl der Fronten und Counter
	 */
	boolean outputFronts = true;
	


	/**
	 * Leerer Konstruktor, der nur den Konstruktor der Superklasse (AbstractAlgo) aufruft. (Wird fuer Help-Klasse benoetigt.)
	 */
	public SMS_EMOA_pg() {
		super();
	}

	/**
	 * Konstruktor: ruft den Konstruktor der Superklasse (AbstractAlgo) auf und...
	 * liesst Algo-spezifische Parameter aus specificParameters ein
	 * 
	 * @param args String-Array der Kommandozeilenparameter
	 */
	public SMS_EMOA_pg(String [] args) {
		/* Konstruktor von AbstractAlgo aufrufen um wichtigste Dinge zu setzen */
		super(args);
		
		/* Algo-Spezifische Variablen setzen */
		mu = specificParameters.getIntProperty("mu", 0, true);
		selectMethod = (specificParameters.getProperty("sm")).charAt(0);
	}

	/*
	 *  (non-Javadoc)
	 * @see algo.AbstractAlgo#startAlgo()
	 */
	public boolean startAlgo() throws Exception {
		// SMS-EMOA-pg Algorithmus

		// Dokumentation
		fileFronts = new FileWriter(outputFileNamePrefix + ".fronts");
	
		System.out.println("SMS_EMOA_pg gestartet.");
		
		test.init(specificTestfktParameters);
		
		// Initialisierung
		currentGenerationCount = 0;
		
		// Dokumentation
		row.append("Gen F1 F2 F3 F4 F5 F6 F7 F8 F9 F10 F11 F12 F13 F14 F15 Fronts Select DomP DeltaS Easy");
		fileFronts.write(row.toString()+"\n");
		// Buffer leeren
		row.delete(0,row.length());
		
		// Startgeneration generieren
		population = generateFirstPopulation();
		
		// DEBUG-Ausgabe
		if (debug) {
			System.out.println("Initiale Population:");
			for (int i=0; i<population.size(); i++) {
				System.out.println("ObjVars: " + ((Individual)population.get(i)).getObjectVariables().toString() );
				System.out.println("FitVars: " + ((Individual)population.get(i)).getFitness().toString() );
				System.out.println("StratVars: " + ((Individual)population.get(i)).getStrategyVariables().toString() );
			}
		}
		
		// ----- Evolution ------ //
		
		
		int outputeverytenth = 1000;
		
		
		for(currentGenerationCount = 0; currentGenerationCount<maxGenerations; currentGenerationCount++) {
			// DEBUG-Ausgabe
			if (debug) {
				System.out.println("-------");
				System.out.println("-------");
				System.out.println("Start Generation:"+currentGenerationCount);
				System.out.println("-------");
			}
			
			//DEBUG-Ausgabe
			if (debug) {
				System.out.println("Aktuelle Population:");
				for (int i=0; i<population.size(); i++) {
					System.out.println("ObjVars: " + ((Individual)population.get(i)).getObjectVariables().toString() );
					System.out.println("FitVars: " + ((Individual)population.get(i)).getFitness().toString() );
					System.out.println("StratVars: " + ((Individual)population.get(i)).getStrategyVariables().toString() );
				}
			}	
			
			frontCounter = 0;
			lastFront.clear();
			Individual offspring = new Individual();
			
			/* Rekombination */	
			if ( (rand.nextDouble() <= recProb) && (mu >1) ) {
				try {
					offspring = recombineSBX(population);
				} catch (Exception e) {
					e.printStackTrace();
					int parentIndex = rand.nextInt(mu);
					// TODO reicht clonen hier aus?
					offspring = (Individual)( (Individual)population.get(parentIndex) ).clone();
				}
			} else { 
				// Reproduktion
				// waehle einen Elter zufaellig und kopiere ihn
				int parentIndex = rand.nextInt(mu);
				// TODO reicht clonen hier aus?
				offspring = (Individual)( (Individual)population.get(parentIndex) ).clone();
			}
			
			// DEBUG-Ausgabe			
			if (debug) {
				System.out.println("Offspring Nach Rekombination:");
				System.out.println("FitnessOffspring: "+ ( ( offspring.getFitness()).toString() ) );
				System.out.println("ObjektVarsOffspring: "+ offspring.getObjectVariables().toString() );
				System.out.println("-----------");
			}

		
			/* Mutation */
			mutatePolynomial(offspring);
			
			//DEBUG-Ausgabe
			if (debug) {
				System.out.println("nach Mutate: ");
				System.out.println("ObjektVarsOffspring: "+ offspring.getObjectVariables().get(0).toString() );
			 }
			
			// DEBUG-Ausgabe		
			if (debug) {	
				System.out.println("Offspring Nach Mutation:");
				System.out.println("FitnessOffspring: "+ ( ( offspring.getFitness()).toString() ) );
				System.out.println("ObjektVarsOffspring: "+ offspring.getObjectVariables().toString() );
				System.out.println("-----------");
			}
			
			/* Auswertung */
			List fitOffspring = new ArrayList();
			fitOffspring = test.execute(offspring.getObjectVariables());
			offspring.setFitness(fitOffspring);
			//offspring.setFitness ((ArrayList)test.execute((List)offspring.getObjectVariables()));
			
			// DEBUG-Ausgabe			
			if (debug) {
				System.out.println("Offspring Nach Auswertung:");
				System.out.println("FitnessOffspring: "+ ( ( offspring.getFitness()).toString() ) );
				System.out.println("ObjektVarsOffspring: "+ offspring.getObjectVariables().toString() );
				System.out.println("-----------");
			}

			// neues Individuum temporaer in Population einfuegen
			populationTemp.addAll(population);
			populationTemp.add(offspring);
		

			/* Selektion */
			
			//Dokumentation
			row.append(currentGenerationCount + " ");
			
			// Non-Dominated-Sorting
			lastFront = nonDomSort(populationTemp);
			
			// Dokumentation
			// Spalten auff?ellen auf 10
			for (int f=frontCounter; f<15; f++) {
				row.append(0 + " ");
			}
			row.append(frontCounter + " ");
			
			// DEBUG-Ausgabe			
			if (debug) {
				System.out.println("#Fronten: " + frontCounter);
				System.out.println("lastFront.size()"+ lastFront.size());
				System.out.println("Letzte Front:");
				for(int i = 0; i<lastFront.size(); i++) {
					System.out.println("Fitness: "+ ( ( (Individual)lastFront.get(i)).getFitness()).toString() );
					System.out.println("ObjektVars: "+ ( ( (Individual)lastFront.get(i)).getObjectVariables()).toString() );
				}
				System.out.println("-----------");
			}

			// Dokumentation
			// 0: keine, 1:selectDomPoints, 2:selectDeltaS, 3:zufaellig
			int docFronts = -1;
			
			// waehle Selektionsmethode
			Individual kill = new Individual();
			switch(selectMethod){
				case '0': // die bevorzugte Version mit allen Features
				    if(frontCounter > 1) {
				        kill = selectDomPoints_yesExtrema_rand(lastFront, populationTemp);
				        domPointsCounter++;
				        docFronts = 1;
				    } else {
				        kill = selectDeltaS_yesExtrema_rand(lastFront);
						docFronts = 2;
				    }
				    break;
				case '1': // zufaellige Selektion
					if(frontCounter > 1) {
						//zufaelliges Individuum wird aus der letzten Front geworfen
						int next = rand.nextInt(lastFront.size());
						kill = (Individual) lastFront.get(next);
						docFronts = 3;
					} else {
						//sonst benutze selectDeltaS
						kill = selectDeltaS_yesExtrema_rand(lastFront);
						docFronts = 2;
					}
					break;
				case '2': // der SMS-EMOA-Ls11, mit Features
					kill = selectDeltaS_yesExtrema_rand(lastFront);
					docFronts = 2;
					break;
				case '3': // ohne Features: kein Schutz der Extrempunkte, keine zusaetzliche Randomisierung
					if(frontCounter > 1) {
						kill = selectDomPoints_noExtrema_det(lastFront, populationTemp);
						domPointsCounter++;
						docFronts = 1;
					} else {
						kill = selectDeltaS_noExtrema_det(lastFront);
						docFronts = 2;
					}
					break;
				case '4': // Schutz der Extrempunkte, keine zusaetzliche Randomisierung
					if(frontCounter > 1) {
						kill = selectDomPoints_yesExtrema_det(lastFront, populationTemp);
						domPointsCounter++;
						docFronts = 1;
					} else {
						kill = selectDeltaS_yesExtrema_det(lastFront);
						docFronts = 2;
					}
					break;
				case '5': // kein Schutz der Extrempunkte, aber zusaetzliche Randomisierung
					if(frontCounter > 1) {
						kill = selectDomPoints_noExtrema_rand(lastFront, populationTemp);
						domPointsCounter++;
						docFronts = 1;
					} else {
						kill = selectDeltaS_noExtrema_rand(lastFront);
						docFronts = 2;
					}
					break;
	            default:
					throw new Exception ("wrong selection parameter");
			}
			
			// Dokumentation
			if (lastFront.size() == 1) {
				docFronts = 0;
				easySelectCounter++;
			}
			row.append(docFronts + " ");
			row.append(domPointsCounter + " ");
			row.append(deltaSCounter + " ");
			row.append(easySelectCounter);
			fileFronts.write(row.toString() + "\n");
			// row geleert
			row.delete(0, row.length());
			
			// DEBUG-Ausgabe			
			if (debug) {
				System.out.println("Kill Nach Selektion:");
				System.out.println("FitnessKill: "+ ( ( kill.getFitness()).toString() ) );
				System.out.println("ObjektVarsKill: "+ kill.getObjectVariables().toString() );
				System.out.println("-----------");
			}
			
			// entferne ausgewaehltes Individuum
			int killIndex = populationTemp.indexOf(kill);
			populationTemp.remove(killIndex);
			
			population.clear();
			population.addAll(populationTemp);
			/*if(currentGenerationCount == 99){
			    //System.out.println("hier in der Generation: "+currentGenerationCount+ "mit outputNr: "+outputeverytenth);
			    //outputeverytenth = outputeverytenth + 1000;
			    //liveOutput(population, currentGenerationCount);
			    AlgoOutput.writeFitness(population, "currentGenerationCount", true);
			}*/
			    
			populationTemp.clear();
			/*Evaluationsbasierter LiveOutput*/
			EvaluationBasedLiveOutput(population);
		}
		
		// Nachbearbeitung des Optimierungsproblems
		population = (ArrayList)test.postProcessing(population);
		test.finalize();

		
		// Dokumentation
		if (outputCounter) {
			System.out.println("deltaSCounter= " + deltaSCounter);
			System.out.println("domPointsCounter= " + domPointsCounter);
			System.out.println("easySelectCounter= " + easySelectCounter);
		}
		if (outputFronts){
			// Front-Datei schliessen
			fileFronts.close();
			System.out.println("Datei " + outputFileNamePrefix + ".fronts geschrieben.");
		} else {
			System.out.println("Keine Datei " + outputFileNamePrefix + ".fronts geschrieben.");
		}
		if (outputNonDom) {
			System.out.println("Dateien " + outputFileNamePrefix + "_gen=i_frontj.nondom geschrieben.");
		} else {
			System.out.println("Keine Dateien " + outputFileNamePrefix + "_gen=i_frontj.nondom geschrieben.");
		}
		System.out.println("SMS_EMOA_pg beendet.");
		return true;
	}
	
	
	/**
	 * Generiert eine erste Elternpopulation und gibt diese zurueck 
	 * (gibt ggf. eine Exception von Methoden der Testfunktion weiter)
	 *  
	 * @return List Erste zufaellig generierte Parent-Generation 
	 * @throws Exception bei irgendwelchen, z.B. Cast-Problemen oder Testfunktions-Fehlern
	 */
	private List generateFirstPopulation() throws Exception {
		List initialPop = new ArrayList(); // Liste fuer Start-Generation initialisieren 
		
		// Grenzen fuer initiale Objektvariablen aus Testfunktion holen
		List bounds = test.getOptimalBounds();
		
		// Schleife ueber alle MU benoetigten Individuen der Population 
		for (int i=0; i<mu; i++) {
			List objectValues = new ArrayList(); // Liste fuer Objektvariablen eines Ind. initialisieren
			// Strategievariablen werden zunaechst nicht benutzt
			List strategyValues = new ArrayList(); // Liste fuer Strategievariablen eines Ind. initialisieren
			
			// Schleife ueber die Dimensionen (Anzahl Objektvariablen) des Individuums
			for (int j=0; j<test.getDimension(); j++) {
				double min = ((Double)bounds.get(2*j)).doubleValue(); // Min-Grenze fuer ObjVar aus Testfkt
				double max = ((Double)bounds.get(2*j+1)).doubleValue(); // Max-Grenze fuer ObjVar aus Testfkt
				// Objektvariable setzen: Minimum + ZufallsIntegerZahl (max-min) + Double-Wert zwischen 0 und 1
				objectValues.add(new Double(rand.nextDouble(min, max)));
				// Strategievariable setzen: ZufallsDoubleZahl zwischen 0,0001 und 5 ([0,0001,5[)
				strategyValues.add(new Double(rand.nextDouble(0.0001, 5)));
			}
			
			// Neues Individuum-Objekt erzeugen, dafuer Testfkt aufrufen und Generation 0 waehlen
			Individual ind = new org.nobeltje.tools.Individual ((ArrayList)objectValues, (ArrayList)strategyValues, (ArrayList) test.execute(objectValues), 0);
			// Individuum zur Generation hinzufuegen
			initialPop.add(ind);
		}
		return initialPop;
	}
	
	private Individual recombineSBX(List population) throws Exception {
		//System.out.println("in recombineSBX");

		int i = 0;
		int j = 0;
		int p1 = -1; /* Eltern-Index */
		int p2 = -1; /* Eltern-Index */
		int dim = test.getDimension();
		double eeta = 15.0;
		double rnd = 0.0; // Zufall
		double par1 = 0.0; //Wert_j von p1
		double par2 = 0.0; //Wert_j von p2
		double chld1= 0.0;
		double chld2 = 0.0;
		double betaq = 1.0;
		double beta = 0.0;
		double alpha = 0.0;
    	double y1 = 0.0; // Kopie von par1
    	double y2 = 0.0; // Kopie von par2
    	double yl = 0.0; // lower bound
    	double yu = 0.0; // upper bound
    	double expp = 0.0;
		List objVariables = new ArrayList();
		List straValues = new ArrayList();

		// waehle zwei verschiedene Eltern zufaellig
		p1 = rand.nextInt(population.size());
		p2 = p1;
		while (p1 == p2) {
			p2 = rand.nextInt(population.size());
		}		

		Individual ind = new Individual();
    	Individual mum = new Individual();
       	Individual dad = new Individual();
       	mum = (Individual)population.get(p1);
       	dad = (Individual)population.get(p2);
       	//System.out.println("mum" + mum.getObjectVariables().toString());
       	//System.out.println("dad" + dad.getObjectVariables().toString());
       	// Anzahl Objektvariablen
		int nObjVars = (mum.getObjectVariables()).size();
		if (nObjVars != (dad.getObjectVariables()).size())
			throw new Exception("Eltern haben ungleiche Anzahl von Objektvariablen.");
        
        for (j=0; j<nObjVars; j++) { // fuer alle Objektvariablen
           	//System.out.println("in for Schleife: "+j);
           	
           	// Objekvariable j der Eltern
           	par1 = ((Double)mum.getObjectVariables().get(j)).doubleValue(); 
           	par2 = ((Double)dad.getObjectVariables().get(j)).doubleValue(); 
           	
           	yl = ((Double)(test.getOptimalBounds()).get(2*j)).doubleValue();
        	yu = ((Double)(test.getOptimalBounds()).get(2*j+1)).doubleValue();
			//System.out.println("yl: " + yl);
			//System.out.println("yu: " + yu);
        	
			rnd = rand.nextDouble();
			//System.out.println("rand: "+rnd);
        	
        	if (rnd <= 0.5) {	// true: Variable wird veraendert
				
				//System.out.println("in if Schleife");
    	  		//System.out.println("Beta vor if: "+beta);	
    	 		
    	 		if (Math.abs(par1 - par2) > 0.000001) {	// changed by Deb (31/10/01)
    	 			
    	 			// y1 = kleinerer Eltern-Wert, y2 = groesserer Eltern-Wert
    	     		if (par2 > par1) {
	    		 		y2 = par2;
	    		        y1 = par1;
	     			} else {
				    	y2 = par1;
				    	y1 = par2;
				    }
				    
    	     		/* Bestimme Wert von beta */
					//System.out.println("yu "+yu+"y2 "+y2+"yl "+yl+ "y1 "+y1);
    	     		// bestimme kleineren Abstand zum Grenzwert
    	     		if ((y1 - yl) > (yu - y2)) {
    	     			beta = 1 + (2 * (yu - y2) / (y2 - y1));
						//System.out.println("Beta find beta value if : "+beta);
    	     		} else {
    	     			beta = 1 + (2 * (y1 - yl) / (y2 - y1));
						//System.out.println("Beta find beta value else : "+beta);
    	     		}

    	     		/* Bestimme alpha */
    	     		expp = eeta + 1.0;
    	     		beta = 1.0 / beta;
					//System.out.println("Beta find alpha value: "+beta);
    	     		alpha = 2.0 - Math.pow (beta, expp);
    	     		if (alpha < 0.0) {
    	     			//alpha = 0.0;
						throw new Exception("Alpha=" + alpha + "<0 Fehler");
    	     		}
    	     		
    	     		rnd = rand.nextDouble(); 
    	     		if (rnd <= 1.0 / alpha) {
	    	     		alpha = alpha * rnd;
	    	     		expp = 1.0 / (eeta + 1.0);
	    	     		betaq = Math.pow (alpha, expp);
    	     		} else {
	    	     		alpha = alpha * rnd;
	    	     		alpha = 1.0 / (2.0 - alpha);
	    	     		expp = 1.0 / (eeta + 1.0);
						if (alpha < 0.0) {
							//alpha = 0.0;
							throw new Exception("Alpha=" + alpha + "<0 Fehler");
						}
	    	     		betaq = Math.pow (alpha, expp);
    	     		}
    	    		
    	 			/* Generiere zwei Nachkommen */
    		    	chld1 = 0.5 * ((y1 + y2) - betaq * (y2 - y1));
    		   		chld2 = 0.5 * ((y1 + y2) + betaq * (y2 - y1));
    	    
    	    	}// end: Math.abs( (par1 - par2) > 0.000001) 
    	    	else { // Math.abs( (par1 - par2) <= 0.000001)
    	    	
	    			betaq = 1.0;
					y1 = par1;
					y2 = par2;
					/* Generiere zwei Nachkommen */
					chld1 = 0.5 * ((y1 + y2) - betaq * (y2 - y1));
					chld2 = 0.5 * ((y1 + y2) + betaq * (y2 - y1));
				}
        	
        		// added by deb (31/10/01)
				if (chld1 < yl)
					chld1 = yl;
				if (chld1 > yu)
					chld1 = yu;
				if (chld2 < yl)
					chld2 = yl;
				if (chld2 > yu)
					chld2 = yu;
				
				//System.out.println("if: chld1: " + chld1);
				//System.out.println("if: chld2: " + chld2);
				
			}// end if (rnd<=0.5) // Variable veraendert
			
			else { // Variable j wird unveraendert kopiert
				//System.out.println("in else Schleife");
			   	chld1 = par1;
			   	chld2 = par2;
				//System.out.println("else: chld1: " + chld1);
				//System.out.println("else: chld2: " + chld2);
        	}
	        	
        	Double firstChld1 = new Double(chld1);
        	Double firstChld2 = new Double(chld2);
    	
    		// weise Objektvariablen zu
    		if (rand.nextDouble() < 0.5) {
				//System.out.println("firstChld1: " + firstChld1);
        		objVariables.add(j,firstChld1);
        	} else {
				//System.out.println("firstChld2: " + firstChld2);
				objVariables.add(j,firstChld2);
        	}
        	
        	// weise Strategievariablen zu
        	Double mumStrVar = new Double(((Double)mum.getStrategyVariables().get(j)).doubleValue());
        	Double dadStrVar = new Double(((Double)dad.getStrategyVariables().get(j)).doubleValue());
        	Double childStrVar = new Double((mumStrVar.doubleValue() + dadStrVar.doubleValue())/2.0);
        	straValues.add(j,childStrVar);
  	 
		}// end for
	
		// erstelle Nachkommen
		//System.out.println("Var1: " + objVariables.get(0));
		ind.setObjectVariables(objVariables);
        ind.setStrategyVariables(straValues);
        return ind;
	
	}// recombineSBX
	
	
	private void mutatePolynomial(Individual offspring) {
		// System.out.println("in mutatePolynomial");

		List objVariables = new ArrayList();
		
		int i,j,k;
		double nc;

		// Here starts Deb
		int dim = test.getDimension();
		double n_distribution_m = 20.0;
		double p_mutation_real = 1.0 / (double)dim;
		//System.out.println("Mutate: p_mutation_real: " + p_mutation_real);
		
		int r;
		double rnd, delta, indi, deltaq;
		double y, yl, yu, val, xy;
		double a, b, x;
		for (i = 0; i < dim; i++) {
			rnd = rand.nextDouble();
			
			/* For each variable find whether to do mutation or not */
			
			if (rnd <= p_mutation_real) {
				
				y = ((Double)(offspring.getObjectVariables()).get(i)).doubleValue();
				yl = ((Double)(test.getOptimalBounds()).get(2*i)).doubleValue();
				yu = ((Double)(test.getOptimalBounds()).get(2*i+1)).doubleValue();
				
				//System.out.println("Mutate: yl: " + yl);
				//System.out.println("Mutate: yu: " + yu);
				
				if (y > yl) {
					/* Calculate delta */
					if ( (y - yl) < (yu - y) )
						delta = (y - yl) / (yu - yl);
					else
						delta = (yu - y) / (yu - yl);

					rnd = rand.nextDouble();
					indi = 1.0 / (n_distribution_m + 1.0);

					if (rnd <= 0.5) {
						xy = 1.0 - delta;
						val = 2.0 * rnd + (1.0 - 2.0 * rnd) * 
						      (Math.pow (xy, (n_distribution_m + 1.0)));
						deltaq = Math.pow (val, indi) - 1.0;
					} else {
						xy = 1.0 - delta;
						val = 2.0 * (1.0 - rnd) + 
								2.0 * (rnd - 0.5) * 
								(Math.pow (xy, (n_distribution_m + 1.0)));
						deltaq = 1.0 - (Math.pow (val, indi));
					}

					/* Change the value for the parent */
					//  *ptr  = *ptr + deltaq*(yu-yl);
					
					// Added by Deb (31/10/01)
					y = y + deltaq * (yu - yl);

					if (y < yl)
						y = yl;
					if (y > yu)
						y = yu;

					objVariables.add(i, new Double(y) );
				}//(y > yl)
				else {// y == yl
					xy = rand.nextDouble();
					objVariables.add(i, new Double(xy * (yu - yl) + yl) );
				}
				
				
			}//end: if (rnd <= p_mutation_real)
			
			else { // kopiere Wert unveraendert			
				objVariables.add(i, offspring.getObjectVariables().get(i) );
			}
					
		} // All vector positions		
		
		// setze mutierte Objektvariablen
		offspring.setObjectVariables(objVariables);
		
	}// mutatePolynomial
	
	
	public List nonDomSort(List pop) {
		this.frontCounter = 0;
		List firstFront = new ArrayList();
		List worseFronts = new ArrayList();

		// fuelle lastFront mit Population und neuen Individuum
		worseFronts.addAll(pop);

		while(worseFronts.size() > 0 ) {
			// leere firstFront
			firstFront.clear();

			// bestimme aktuell vorderste Front
			for (int i=0; i<worseFronts.size(); i++) {
				firstFront = ParetoArchive.refreshArchive(0, (ArrayList)firstFront, (Individual)worseFronts.get(i));
			}
			
			// Dokumentation
			if (outputFronts)
				if (frontCounter<15) { // nur Daten ueber die ersten 15 Fronten speichern
				row.append(firstFront.size() + " ");
			}
			
			if (outputNonDom) {
				//if (currentGenerationCount == 500) { // nur eine bestimmte Generation dokumentieren
					try {
						fileNonDom = new FileWriter(outputFileNamePrefix + "_gen=" + currentGenerationCount + "_front" + frontCounter + ".nondom");
						for (int i=0; i<firstFront.size(); i++) {
							for(int j=0; j<test.getNumberOfObjectives(); j++) {
								fileNonDom.write( (((Individual)firstFront.get(i)).getFitness()).get(j).toString() + " ");
							}
							fileNonDom.write("\n");
						}
						fileNonDom.close();
					} catch (IOException e) {
						e.printStackTrace();
					}
				//}
			}
			
			// entferne vorderste Front
			worseFronts.removeAll(firstFront);
			
			// erhoehe Fronten-Zaehler
			this.frontCounter++;
		}
	
		// fuege Elemente der letzten Front in lastFront ein
		this.lastFront.clear();
		this.lastFront.addAll(firstFront);
		
		return this.lastFront;
	}
	
	
	/**
	 * Bestimmt aus den Individuen der letzten Front das mit der groessten Anzahl
	 * dominierender Individuen und gibt dieses zurueck 
	 * (das letzte bei gleich schlechten).
	 * @param lastFront
	 * @param currentPopulation
	 * @return Individuum mit den meisten dominierenden Individuen
	 */
	private Individual selectDomPoints_noExtrema_det(List lastFront, List currentPopulation) {
		
		Individual indReturn = new Individual();

		/* falls nur ein Element in der Liste ist, wird dieses selektiert */
		if(lastFront.size()==1){
			indReturn = (Individual)lastFront.get(0);
			return indReturn;
		}

		/* sonst wird ein Element aus der Front selektiert */

		// Liste der indPlusDomCount-Listen
		List lastFrontData = new ArrayList();
		
		for(Iterator lastFrontIter=lastFront.iterator(); lastFrontIter.hasNext(); ){
			/* nehme jedes Individuum aus der Front */
			Individual currentOfFront = (Individual)lastFrontIter.next();
			int domCount=0;
			// Index 0: Individuum, Index 1: domCount
			List indPlusDomCount = new ArrayList();
			
			for(Iterator popIter = currentPopulation.iterator(); popIter.hasNext(); ){
				/* nehme jedes Individuum aus der Population */
				Individual currentOfPop = (Individual)popIter.next();
				int domObjectives = 0;
				
				// ueberpruefe Dominanz:
				// ist currentOfPop in allen Zielfunktionswerten besser (kleiner)?
				for(int i=0; i<test.getNumberOfObjectives(); i++){
					if(((Double)currentOfPop.getFitness().get(i)).doubleValue()<= ((Double)currentOfFront.getFitness().get(i)).doubleValue()) {
						domObjectives++;
					}
				}
				// Falls currentOfPop das aktuelle Individuum der Front dominiert,
				// erhoehe den domCount-Wert
				if(domObjectives == (test.getNumberOfObjectives())) {
					domCount++;
				}
			}
			
			// fuelle indPlusDomCount
			indPlusDomCount.add(currentOfFront);
			Integer count = new Integer(domCount);
			indPlusDomCount.add(count);
			// fuelle lastFrontData
			//System.out.println("domCount = "+domCount);
			lastFrontData.add(indPlusDomCount);			
		}
		
		if(debug) {
			System.out.println("lastFrontData:");
			for (int z=0; z<lastFrontData.size(); z++) {
				System.out.println("Fitness: " + ((Individual)((ArrayList)lastFrontData.get(z)).get(0)).getFitness().toString() );
				System.out.println("DomCount: " + ((ArrayList)lastFrontData.get(z)).get(1).toString() );
			}
		}
		
		// Variable fuer den groessten domCount
		int max = -1;
		// Individuum mit dem groessten domCount
		indReturn = (Individual)currentPopulation.get(0);
		// suche groessten domCount-Wert
		for(Iterator lastFrontDataIter=lastFrontData.iterator(); lastFrontDataIter.hasNext(); ){
			ArrayList currentList = ((ArrayList)lastFrontDataIter.next());
			if(max <= ((Integer)currentList.get(1)).intValue()){
				max = ((Integer)currentList.get(1)).intValue();
				indReturn = (Individual)currentList.get(0);
			}
		}
		
		return indReturn;
	}


	/**
	 * Bestimmt aus den Individuen der letzten Front das mit der groessten Anzahl
	 * dominierender Individuen und gibt dieses zurueck 
	 * (das letzte bei gleich schlechten, Extrempunkte werden geschuetzt).
	 * @param lastFront
	 * @param currentPopulation
	 * @return Individuum mit den meisten dominierenden Individuen
	 */
	private Individual selectDomPoints_yesExtrema_det(List lastFront, List currentPopulation) {
		
		Individual indReturn = new Individual();

		/* falls nur ein Element in der Liste ist, wird dieses selektiert */
		if(lastFront.size()==1){
			indReturn = (Individual)lastFront.get(0);
			return indReturn;
		}

		/* sonst wird ein Element aus der Front selektiert */

		// Liste der indPlusDomCount-Listen
		List lastFrontData = new ArrayList();
		
		for(Iterator lastFrontIter=lastFront.iterator(); lastFrontIter.hasNext(); ){
			/* nehme jedes Individuum aus der Front */
			Individual currentOfFront = (Individual)lastFrontIter.next();
			int domCount=0;
			// Index 0: Individuum, Index 1: domCount
			List indPlusDomCount = new ArrayList();
			
			for(Iterator popIter = currentPopulation.iterator(); popIter.hasNext(); ){
				/* nehme jedes Individuum aus der Population */
				Individual currentOfPop = (Individual)popIter.next();
				int domObjectives = 0;
				
				// ueberpruefe Dominanz:
				// ist currentOfPop in allen Zielfunktionswerten besser (kleiner)?
				for(int i=0; i<test.getNumberOfObjectives(); i++){
					if(((Double)currentOfPop.getFitness().get(i)).doubleValue()<= ((Double)currentOfFront.getFitness().get(i)).doubleValue()) {
						domObjectives++;
					}
				}
				// Falls currentOfPop das aktuelle Individuum der Front dominiert,
				// erhoehe den domCount-Wert
				if(domObjectives == (test.getNumberOfObjectives())) {
					domCount++;
				}
			}
			
			// fuelle indPlusDomCount
			indPlusDomCount.add(currentOfFront);
			Integer count = new Integer(domCount);
			indPlusDomCount.add(count);
			// fuelle lastFrontData
			//System.out.println("domCount = "+domCount);
			lastFrontData.add(indPlusDomCount);			
		}
		
		if(debug) {
			System.out.println("lastFrontData vor Extrema:");
			for (int z=0; z<lastFrontData.size(); z++) {
				System.out.println("Fitness: " + ((Individual)((ArrayList)lastFrontData.get(z)).get(0)).getFitness().toString() );
				System.out.println("DomCount: " + ((ArrayList)lastFrontData.get(z)).get(1).toString() );
			}
		}	
		
		/* Bestimme Extrema (Randpunkte der letzten Front) */
		List extrema = new ArrayList();
		for (int i=0; i<test.getNumberOfObjectives(); i++) {
			double min = 10000.0;
			int minIndex = -1;
			for (int j=0; j<lastFront.size(); j++) {
				double fitVal = ((Double)((Individual)lastFront.get(j)).getFitness().get(i)).doubleValue();
				if ( fitVal <= min  ) {
					min = fitVal;
					minIndex = j;
				}
			}
			extrema.add(new Integer(minIndex));
			//System.out.println("Extrema "+i+": " + (((Individual)lastFront.get(((Integer)extrema.get(i)).intValue())).getFitness()));
		}

		// setze domCount-Wert der Extrema auf 0
		for (int k=0; k<test.getNumberOfObjectives(); k++) {
			int index = ((Integer)extrema.get(k)).intValue();
			//System.out.println("Extrema " + k + ": " + (((Individual)front1.get(((Integer)extrema.get(k)).intValue())).getFitness()));
			((ArrayList)lastFrontData.get(index)).set(1,new Integer(0));
		}
		
		if(debug) {
			System.out.println("lastFrontData nach Extrema:");
			for (int z=0; z<lastFrontData.size(); z++) {
				System.out.println("Fitness: " + ((Individual)((ArrayList)lastFrontData.get(z)).get(0)).getFitness().toString() );
				System.out.println("DomCount: " + ((ArrayList)lastFrontData.get(z)).get(1).toString() );
			}
		}	

		
		// Variable fuer den groessten domCount
		int max = -1;
		// Individuum mit dem groessten domCount
		indReturn = (Individual)currentPopulation.get(0);
		// suche groessten domCount-Wert
		for(Iterator lastFrontDataIter=lastFrontData.iterator(); lastFrontDataIter.hasNext(); ){
			ArrayList currentList = ((ArrayList)lastFrontDataIter.next());
			if(max <= ((Integer)currentList.get(1)).intValue()){
				max = ((Integer)currentList.get(1)).intValue();
				indReturn = (Individual)currentList.get(0);
			}
		}
		
		return indReturn;
	}	



	/**
	 * Bestimmt aus den Individuen der letzten Front das mit der groessten Anzahl
	 * dominierender Individuen und gibt dieses zurueck
	 * (Randomisierung unter gleich schlechten Individuen).
	 * @param lastFront
	 * @param currentPopulation
	 * @return Individuum mit den meisten dominierenden Individuen
	 */
	private Individual selectDomPoints_noExtrema_rand(List lastFront, List currentPopulation) {
	
		Individual indReturn = new Individual();

		/* falls nur ein Element in der Liste ist, wird dieses selektiert */
		if(lastFront.size()==1){
			indReturn = (Individual)lastFront.get(0);
			return indReturn;
		}

		/* sonst wird ein Element aus der Front selektiert */

		// Liste der indPlusDomCount-Listen
		ArrayList lastFrontData = new ArrayList();
	
		for(Iterator lastFrontIter=lastFront.iterator(); lastFrontIter.hasNext(); ){
			/* nehme jedes Individuum aus der Front */
			Individual currentOfFront = (Individual)lastFrontIter.next();
			int domCount=0;
			// Index 0: Individuum, Index 1: domCount
			ArrayList indPlusDomCount = new ArrayList();
		
			for(Iterator popIter = currentPopulation.iterator(); popIter.hasNext(); ){
				/* nehme jedes Individuum aus der Population */
				Individual currentOfPop = (Individual)popIter.next();
				int domObjectives = 0;
			
				// ueberpruefe Dominanz:
				// ist currentOfPop in allen Zielfunktionswerten besser (kleiner)?
				for(int i=0; i<test.getNumberOfObjectives(); i++){
					if(((Double)currentOfPop.getFitness().get(i)).doubleValue()<= ((Double)currentOfFront.getFitness().get(i)).doubleValue()) {
						domObjectives++;
					}
				}
				// Falls currentOfPop das aktuelle Individuum der Front dominiert,
				// erhoehe den domCount-Wert
				if(domObjectives == (test.getNumberOfObjectives())) {
					domCount++;
				}
			}
		
			// fuelle indPlusDomCount
			indPlusDomCount.add(currentOfFront);
			Integer count = new Integer(domCount);
			indPlusDomCount.add(count);
			// fuelle lastFrontData
			//System.out.println("domCount = "+domCount);
			lastFrontData.add(indPlusDomCount);			
		}
	
		if(debug) {
			System.out.println("lastFrontData:");
			for (int z=0; z<lastFrontData.size(); z++) {
				System.out.println("Fitness: " + ((Individual)((ArrayList)lastFrontData.get(z)).get(0)).getFitness().toString() );
				System.out.println("DomCount: " + ((ArrayList)lastFrontData.get(z)).get(1).toString() );
			}
		}
	
		// Variable fuer den groessten domCount
		int maxNumber = -1;
		// Individuum mit dem groessten domCount
		indReturn = (Individual)currentPopulation.get(0);
		// suche groessten domCount-Wert
		for(Iterator lastFrontDataIter=lastFrontData.iterator(); lastFrontDataIter.hasNext(); ){
			ArrayList currentList = ((ArrayList)lastFrontDataIter.next());
			if(maxNumber <= ((Integer)currentList.get(1)).intValue()){
				maxNumber = ((Integer)currentList.get(1)).intValue();
			}
		}	
	
		// suche alle Individuen heraus, die den Wert maxNumber haben
		ArrayList worstInds = new ArrayList(); //Liste der schlechtesten Individuen
		for (Iterator lastFrontDataIter=lastFrontData.iterator(); lastFrontDataIter.hasNext(); ) {
			ArrayList currentList = ((ArrayList)lastFrontDataIter.next());
			if ( ((Integer)currentList.get(1)).intValue() == maxNumber ) {
				worstInds.add( (Individual)currentList.get(0) );
			}
		}
		
		if(debug) {
			System.out.println("maxNumber: " + maxNumber);
			System.out.println("worstData.size: " + worstInds.size());
			for (int z=0; z<worstInds.size(); z++) {
				System.out.println("Fitness: " + ((Individual)worstInds.get(z)).getFitness() );
			}
		}

		// waehle aus den schlechtesten ein Individuum zufaellig
		int index = rand.nextInt(worstInds.size());
		indReturn = (Individual)worstInds.get(index);
	
		return indReturn;
	}
	
	
	/**
	 * Bestimmt aus den Individuen der letzten Front das mit der groessten Anzahl
	 * dominierender Individuen und gibt dieses zurueck
	 * (Randomisierung unter gleich schlechten Individuen, Randpunkte werden geschuetzt).
	 * @param lastFront
	 * @param currentPopulation
	 * @return Individuum mit den meisten dominierenden Individuen
	 */
	private Individual selectDomPoints_yesExtrema_rand(List lastFront, List currentPopulation) {
	
		Individual indReturn = new Individual();

		/* falls nur ein Element in der Liste ist, wird dieses selektiert */
		if(lastFront.size()==1){
			indReturn = (Individual)lastFront.get(0);
			return indReturn;
		}

		/* sonst wird ein Element aus der Front selektiert */

		// Liste der indPlusDomCount-Listen
		ArrayList lastFrontData = new ArrayList();
	
		for(Iterator lastFrontIter=lastFront.iterator(); lastFrontIter.hasNext(); ){
			/* nehme jedes Individuum aus der Front */
			Individual currentOfFront = (Individual)lastFrontIter.next();
			int domCount=0;
			// Index 0: Individuum, Index 1: domCount
			ArrayList indPlusDomCount = new ArrayList();
		
			for(Iterator popIter = currentPopulation.iterator(); popIter.hasNext(); ){
				/* nehme jedes Individuum aus der Population */
				Individual currentOfPop = (Individual)popIter.next();
				int domObjectives = 0;
			
				// ueberpruefe Dominanz:
				// ist currentOfPop in allen Zielfunktionswerten besser (kleiner)?
				for(int i=0; i<test.getNumberOfObjectives(); i++){
					if(((Double)currentOfPop.getFitness().get(i)).doubleValue()<= ((Double)currentOfFront.getFitness().get(i)).doubleValue()) {
						domObjectives++;
					}
				}
				// Falls currentOfPop das aktuelle Individuum der Front dominiert,
				// erhoehe den domCount-Wert
				if(domObjectives == (test.getNumberOfObjectives())) {
					domCount++;
				}
			}
		
			// fuelle indPlusDomCount
			indPlusDomCount.add(currentOfFront);
			Integer count = new Integer(domCount);
			indPlusDomCount.add(count);
			// fuelle lastFrontData
			//System.out.println("domCount = "+domCount);
			lastFrontData.add(indPlusDomCount);			
		}
		
		if(debug) {
			System.out.println("lastFrontData vor Extrema:");
			for (int z=0; z<lastFrontData.size(); z++) {
				System.out.println("Fitness: " + ((Individual)((ArrayList)lastFrontData.get(z)).get(0)).getFitness().toString() );
				System.out.println("DomCount: " + ((ArrayList)lastFrontData.get(z)).get(1).toString() );
			}
		}	
		
		/* Bestimme Extrema (Randpunkte der letzten Front) */
		List extrema = new ArrayList();
		for (int i=0; i<test.getNumberOfObjectives(); i++) {
			double min = 10000.0;
			int minIndex = -1;
			for (int j=0; j<lastFront.size(); j++) {
				double fitVal = ((Double)((Individual)lastFront.get(j)).getFitness().get(i)).doubleValue();
				if ( fitVal <= min  ) {
					min = fitVal;
					minIndex = j;
				}
			}
			extrema.add(new Integer(minIndex));
			//System.out.println("Extrema "+i+": " + (((Individual)lastFront.get(((Integer)extrema.get(i)).intValue())).getFitness()));
		}

		// setze domCount-Wert der Extrema auf 0
		for (int k=0; k<test.getNumberOfObjectives(); k++) {
			int index = ((Integer)extrema.get(k)).intValue();
			//System.out.println("Extrema " + k + ": " + (((Individual)front1.get(((Integer)extrema.get(k)).intValue())).getFitness()));
			((ArrayList)lastFrontData.get(index)).set(1,new Integer(0));
		}
		
		if(debug) {
			System.out.println("lastFrontData nach Extrema:");
			for (int z=0; z<lastFrontData.size(); z++) {
				System.out.println("Fitness: " + ((Individual)((ArrayList)lastFrontData.get(z)).get(0)).getFitness().toString() );
				System.out.println("DomCount: " + ((ArrayList)lastFrontData.get(z)).get(1).toString() );
			}
		}
	
		// Variable fuer den groessten domCount
		int maxNumber = -1;
		// Individuum mit dem groessten domCount
		indReturn = (Individual)currentPopulation.get(0);
		// suche groessten domCount-Wert
		for(Iterator lastFrontDataIter=lastFrontData.iterator(); lastFrontDataIter.hasNext(); ){
			ArrayList currentList = ((ArrayList)lastFrontDataIter.next());
			if(maxNumber <= ((Integer)currentList.get(1)).intValue()){
				maxNumber = ((Integer)currentList.get(1)).intValue();
			}
		}	
	
		// suche alle Individuen heraus, die den Wert maxNumber haben
		ArrayList worstInds = new ArrayList(); //Liste der schlechtesten Individuen
		for (Iterator lastFrontDataIter=lastFrontData.iterator(); lastFrontDataIter.hasNext(); ) {
			ArrayList currentList = ((ArrayList)lastFrontDataIter.next());
			if ( ((Integer)currentList.get(1)).intValue() == maxNumber ) {
				worstInds.add( (Individual)currentList.get(0) );
			}
		}
		
		if(debug) {
			System.out.println("maxNumber: " + maxNumber);
			System.out.println("worstData.size: " + worstInds.size());
			for (int z=0; z<worstInds.size(); z++) {
				System.out.println("Fitness: " + ((Individual)worstInds.get(z)).getFitness() );
			}
		}

		// waehle aus den schlechtesten ein Individuum zufaellig
		int index = rand.nextInt(worstInds.size());
		indReturn = (Individual)worstInds.get(index);
	
		return indReturn;
	}
	
	
	/**
	 * Es wird aus der letzten Front das Individuum mit dem kleinsten
	 * Beitrag zum S-Metrik-Wert der letzten Front selektiert
	 * (das letzte bei gleich schlechten).
	 * @param lastFront
	 * @return Individuum mit kleistem S-Metrik-Beitrag
	 */
	public Individual selectDeltaS_noExtrema_det(List lastFront) {
		//System.out.println("in selectDeltaS");
		deltaSCounter++;

		/* falls es nur ein Individuum gibt, wird dieses gewaehlt */
		if (lastFront.size()==1)
			return (Individual) lastFront.get(0);

		/* sonst wird das Individuum mit dem kleinsten Beitrag
		* zum S-Metrik-Wert der letzten Front selektiert
		*/
		
		// bestimme kleinstmoeglichen Referenzpunkt (Maxima + 1)
		List refPoint = new ArrayList();
		for (int i=0; i<test.getNumberOfObjectives(); i++) {
			double max = -10000.0;
			for (int j=0; j<lastFront.size(); j++) {
				double fitVal = ((Double)((Individual)lastFront.get(j)).getFitness().get(i)).doubleValue();
				if ( fitVal >= max  )
					max = fitVal;
			}
			refPoint.add(new Double(max + 1.0));
		}
		if (debug) {
			System.out.println("RefPoint: " + refPoint.toString());
		}
		
		DeltaS myDeltaSMetric = new DeltaS(); // S-Metrik-Objekt
		List deltaSValues = new ArrayList();
				
		for (Iterator iterFront=lastFront.iterator(); iterFront.hasNext(); ) {
			double deltaSTemp = 0.0;
			try {
				// berechne DeltaS-Wert fuer jedes Individuum der letzten Front
				deltaSTemp = myDeltaSMetric.computeMetric(lastFront, (Individual)iterFront.next(), refPoint);
				deltaSValues.add(new Double(deltaSTemp));
				//System.out.println("DeltaValue, orig: "+deltaSTemp);
			} catch (Exception e) {
				e.printStackTrace();
			}
		}

		// suche kleinsten DeltaS-Wert
		double deltaSMin = 1000000.0;
		int indexMin = -1;
		for (int i=0; i<lastFront.size(); i++) {
			if ( ((Double)deltaSValues.get(i)).doubleValue() < deltaSMin ) {
				deltaSMin = ((Double)deltaSValues.get(i)).doubleValue();
				//System.out.println("deltaSMin: " + deltaSMin);
				indexMin = i;
			}
		}
		
		if (debug) {
			System.out.println("DeltaS: " + deltaSValues.toString());
			System.out.println("indexMin" + indexMin);
		}
		
		// suche Individuum zu Wert und gebe es zurueck
		return (Individual)lastFront.get(indexMin);
	}


	/**
	 * Es wird aus der letzten Front das Individuum mit dem kleinsten
	 * Beitrag zum S-Metrik-Wert der letzten Front selektiert
	 * (das letzte bei gleich schlechten, Randpunkte werden geschuetzt).
	 * @param lastFront
	 * @return Individuum mit kleistem S-Metrik-Beitrag
	 */
	public Individual selectDeltaS_yesExtrema_det(List lastFront) {
		//System.out.println("in selectDeltaS");
		deltaSCounter++;
		
		// falls es nur ein Individuum gibt, wird dieses gewaehlt
		if (lastFront.size()==1)
			return (Individual) lastFront.get(0);

		/* sonst wird das Individuum mit dem kleinsten Beitrag
		* zum S-Metrik-Wert der letzten Front selektiert
		*/

		// bestimme Referenzpunkt und Extrempunkte
		List refPoint = new ArrayList();
		List extrema = new ArrayList();

		for (int i=0; i<test.getNumberOfObjectives(); i++) {
			double max = -10000.0;
			double min = 10000.0;
			int minIndex = -1;
			for (int j=0; j<lastFront.size(); j++) {
				double fitVal = ((Double)((Individual)lastFront.get(j)).getFitness().get(i)).doubleValue();
				if ( fitVal >= max  )
					max = fitVal;
				if ( fitVal <= min  ) {
					min = fitVal;
					minIndex = j;
				}
			}
			//System.out.println("max="+max+" min="+min+" minIndex="+minIndex);
			refPoint.add(new Double(max +1.0));
			//System.out.println("RefPoint: " + refPoint.get(i));
			extrema.add(new Integer(minIndex));
			//System.out.println("Extrema: " + extrema.get(i));
		}
		if (debug) {
			System.out.println("RefPoint: " + refPoint.toString());
			System.out.println("Extrema: " + extrema.toString());
		}
		
		DeltaS myDeltaSMetric = new DeltaS(); // S-Metrik-Objekt
		List deltaSValues = new ArrayList();
				
		for (Iterator iterFront = lastFront.iterator(); iterFront.hasNext();) {
			double deltaSTemp = 0.0;
			try {
				// berechne DeltaS-Wert fuer jedes Individuum der letzten Front
				deltaSTemp = myDeltaSMetric.computeMetric(lastFront, (Individual)iterFront.next(), refPoint);
				deltaSValues.add(new Double(deltaSTemp));
				//System.out.println("DeltaValue, orig: "+deltaSTemp);
			} catch (Exception e) {
				e.printStackTrace();
			}
		}

		if (debug) {
			System.out.println("DeltaS vor Extrema: " + deltaSValues.toString());
		}
		
		// addiere Konstante zu ExtremPunkten, damit diese einen guten (hohen) Werte haben
		for (int k=0; k<test.getNumberOfObjectives(); k++) {
			int index = ((Integer)extrema.get(k)).intValue();
			deltaSValues.set(index, new Double(((Double)deltaSValues.get(index)).doubleValue() + 1000.0));
		}
		
		if (debug) {
			System.out.println("DeltaS nach Extrema: " + deltaSValues.toString());
		}

		// suche kleinsten DeltaS-Wert
		double deltaSMin = 1000000.0;
		int indexMin = -1;
		for (int i=0; i<lastFront.size(); i++) {
			if ( ((Double)deltaSValues.get(i)).doubleValue() < deltaSMin ) {
				deltaSMin = ((Double)deltaSValues.get(i)).doubleValue();
				//System.out.println("deltaSMin: " + deltaSMin);
				indexMin = i;
			}
		}

		if (debug) {
			System.out.println("indexMin" + indexMin);
		}

		// suche Individuum zu Wert und gebe es zurueck
		return (Individual)lastFront.get(indexMin);
	}


	/**
	 * Es wird aus der letzten Front das Individuum mit dem kleinsten
	 * Beitrag zum S-Metrik-Wert der letzten Front selektiert
	 * (Randomisierung unter gleich schlechten).
	 * @param lastFront
	 * @return Individuum mit kleistem S-Metrik-Beitrag
	 */
	public Individual selectDeltaS_noExtrema_rand(List lastFront) {
		//System.out.println("in selectDeltaS");
		deltaSCounter++;

	    // falls es nur ein Individuum gibt, wird dieses gewaehlt
	    if (lastFront.size()==1)
	        return (Individual) lastFront.get(0);

		/* sonst wird das Individuum mit dem kleinsten Beitrag
		* zum S-Metrik-Wert der letzten Front selektiert
		*/

	    
	    // bestimme Referenzpunkt
		List refPoint = new ArrayList();	
		for (int i=0; i<test.getNumberOfObjectives(); i++) {
			double max = -10000.0;
			double min = 10000.0;
			for (int j=0; j<lastFront.size(); j++) {
				double fitVal = ((Double)((Individual)lastFront.get(j)).getFitness().get(i)).doubleValue();
				if ( fitVal >= max  )
					max = fitVal;
			}
			refPoint.add(new Double(max +1.0));
			//System.out.println("RefPoint: " + refPoint.get(i));
		}
		if (debug) {
			System.out.println("RefPoint: " + refPoint.toString());
		}
		
		DeltaS myDeltaSMetric = new DeltaS(); // S-Metrik-Objekt
		List deltaSValues = new ArrayList();
		for (Iterator iterFront = lastFront.iterator(); iterFront.hasNext();) {
		    double deltaSTemp = 0.0;
		    try {
				// berechne DeltaS-Wert fuer jedes Individuum der letzten Front
				deltaSTemp = myDeltaSMetric.computeMetric(lastFront, (Individual)iterFront.next(), refPoint);
				deltaSValues.add(new Double(deltaSTemp));
				//System.out.println("DeltaValue, orig: "+deltaSTemp);
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		
		if (debug) {
			System.out.println("DeltaS: " + deltaSValues.toString());
		}

		
		// suche kleinsten DeltaS-Wert
		double deltaSMin = 1000000.0;
		//int indexMin = -1;
		for (int i=0; i<lastFront.size(); i++) {
			if ( ((Double)deltaSValues.get(i)).doubleValue() < deltaSMin ) {
				deltaSMin = ((Double)deltaSValues.get(i)).doubleValue();
				//System.out.println("deltaSMin: " + deltaSMin);
				//indexMin = i;
			}
		}

		// speichere schlechteste Individuen 	
		ArrayList worstInds = new ArrayList(); //Indices der schlechtesten Individuen
		for (int i=0; i<lastFront.size(); i++) {
			if ( ((Double)deltaSValues.get(i)).doubleValue() == deltaSMin ) {
				worstInds.add(new Integer(i));
			}
		}
		if (debug) {
			System.out.println("WorstInds: " + worstInds.toString());
		}
		
		// waehle ein schlechtestes Individuum zufaellig
		int indexSelect = rand.nextInt(worstInds.size());
		indexSelect = ((Integer)worstInds.get(indexSelect)).intValue(); 
		
		// suche Individuum zu Wert und gebe es zurueck
		return (Individual)lastFront.get(indexSelect);
	}


	/**
	 * Es wird aus der letzten Front das Individuum mit dem kleinsten
	 * Beitrag zum S-Metrik-Wert der letzten Front selektiert
	 * (Randomisierung unter gleich schlechten, Randpunkte werden geschuetzt).
	 * @param lastFront
	 * @return Individuum mit kleistem S-Metrik-Beitrag
	 */
	public Individual selectDeltaS_yesExtrema_rand(List lastFront) {
		//System.out.println("in selectDeltaS");
		deltaSCounter++;
		
		// falls es nur ein Individuum gibt, wird dieses gewaehlt
		if (lastFront.size()==1)
			return (Individual) lastFront.get(0);
	    
		/* sonst wird das Individuum mit dem kleinsten Beitrag
		* zum S-Metrik-Wert der letzten Front selektiert
		*/

	    
		// bestimme Referenzpunkt und Extrempunkte
		List refPoint = new ArrayList();
		List extrema = new ArrayList();
		
		for (int i=0; i<test.getNumberOfObjectives(); i++) {
			double max = -10000.0;
			double min = 10000.0;
			int minIndex = -1;
			for (int j=0; j<lastFront.size(); j++) {
				double fitVal = ((Double)((Individual)lastFront.get(j)).getFitness().get(i)).doubleValue();
				if ( fitVal > max  )
					max = fitVal;
				if ( fitVal < min  ) {
					min = fitVal;
					minIndex = j;
				}
			}
			//System.out.println("max="+max+" min="+min+" minIndex="+minIndex);
			refPoint.add(new Double(max +1.0));
			//System.out.println("RefPoint: " + refPoint.get(i));

			// speichere Individuen mit minimalem Wert in extremaCandidate
			List extremaCandidate = new ArrayList(); //Indices der extremen Individuen
			for (int j=0; j<lastFront.size(); j++) {
				double fitVal = ((Double)((Individual)lastFront.get(j)).getFitness().get(i)).doubleValue();
				if ( fitVal == min  ) {
					extremaCandidate.add(new Integer(j));
				}
			}
			
			// waehle aus extremaCandidate ein Individuum zufaellig aus
			int extremaIndex = rand.nextInt(extremaCandidate.size());
			extremaIndex = ((Integer)extremaCandidate.get(extremaIndex)).intValue();
			// speichere Index des Extrempunktes in lastFront in extrema
			extrema.add( new Integer(extremaIndex) );
			
			if (debug) {
				System.out.println("extremaCandidate: " + extremaCandidate.toString());
				System.out.println("extremaIndex: " + extremaIndex);
			}
		}	
		if (debug) {
			System.out.println("RefPoint: " + refPoint.toString());
			System.out.println("Extrema: " + extrema.toString());
		}

		
		DeltaS myDeltaSMetric = new DeltaS(); // S-Metrik-Objekt
		List deltaSValues = new ArrayList();			
		for (Iterator iterFront = lastFront.iterator(); iterFront.hasNext();) {
			double deltaSTemp = 0.0;
			try {
				// berechne DeltaS-Wert fuer jedes Individuum der letzten Front
				deltaSTemp = myDeltaSMetric.computeMetric(lastFront, (Individual)iterFront.next(), refPoint);
				deltaSValues.add(new Double(deltaSTemp));
				//System.out.println("DeltaValue, orig: "+deltaSTemp);
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		
		if (debug) {
			System.out.println("DeltaS vor Extrema: " + deltaSValues.toString());
		}

		// addiere Konstante zu ExtremPunkten, damit diese einen 
		// guten (hohen) Werte haben
		for (int k=0; k<test.getNumberOfObjectives(); k++) {
			int index = ((Integer)extrema.get(k)).intValue();
			deltaSValues.set(index, new Double(((Double)deltaSValues.get(index)).doubleValue() + 1000.0));
		}

		if (debug) {
			System.out.println("DeltaS nach Extrema: " + deltaSValues.toString());
		}
		
		// suche kleinsten DeltaS-Wert
		double deltaSMin = 1000000.0;
		//int indexMin = -1;
		for (int i=0; i<lastFront.size(); i++) {
			if ( ((Double)deltaSValues.get(i)).doubleValue() < deltaSMin ) {
				deltaSMin = ((Double)deltaSValues.get(i)).doubleValue();
			}
		}

		// speichere schlechteste Individuen 	
		ArrayList worstInds = new ArrayList(); //Indices der schlechtesten Individuen
		for (int i=0; i<lastFront.size(); i++) {
			if ( ((Double)deltaSValues.get(i)).doubleValue() == deltaSMin ) {
				worstInds.add(new Integer(i));
			}
		}
		if (debug) {
			System.out.println("WorstInds: " + worstInds.toString());
			System.out.println("deltaSMin: " + deltaSMin);
		}
		
		// waehle ein schlechtestes Individuum zufaellig
		int indexSelect = rand.nextInt(worstInds.size());
		indexSelect = ((Integer)worstInds.get(indexSelect)).intValue(); 
		
		// suche Individuum zu Wert und gebe es zurueck
		return (Individual)lastFront.get(indexSelect);
	}
	
	
	/**
	 * Erstellt Objekt des Algorithmus, ruft startAlgo() auf und 
	 * schreibt Daten der letzten Population in Dateien.
	 * @param args
	 * @throws Exception
	 */
	public static void main(String[] args) throws Exception{
		// Algorithmus-Objekt erzeugen	
		SMS_EMOA_pg sms = new SMS_EMOA_pg(args);
		
		// Algorithmus starten
		sms.startAlgo();
			
		/* Archiv (Objektvariablen) Textdatei ausgeben */
		AlgoOutput.writeFitness(sms.population, sms.outputFileNamePrefix, false);
		/* Archiv (Fitnesswerte) in Textdatei output.fitness ausgeben */
		AlgoOutput.writeObjectVariables(sms.population, sms.outputFileNamePrefix, false);
		
	}
}
