/*
 * Copyright (C) 2005  PG447 - University Dortmund
 * 
 * This file is part of NObELTJE.
 * 
 * NObELTJE is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as published by
 * the Free Software Foundation.
 * 
 * NObELTJE is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with NObELTJE; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

 /**
 * Created on May 13, 2004
 *
 */
package org.nobeltje.algo;

import java.util.ArrayList;
import java.util.Iterator;

import org.nobeltje.tools.Individual;
import org.nobeltje.tools.io.AlgoOutput;
import org.nobeltje.exceptions.NoIndividualException;


/**
 * Ein ganz simpler Algorithmus, der einfach ein zufaelliges Indiviuum generiert und die Fitnesswert aufsummiert. 
 * <br>Das Archiv besteht aus den Individuen, deren Fitnesswertsumme am hoechsten ist.
 * Das Archiv beinhaltet also nur Elemente mit dem gleichen Fitnesswert.
 * 
 * <br>Implementiert von Daniel Saltmann, Nicola Beume, Kay Thielmann und Miriam Padberg
 * 
 * @author $Author: noot $
 * @version $Revision: 726 $
 * $Date: 2005-02-02 00:50:20 +0100 (Mi, 02 Feb 2005) $
 */

public class TestAlgo extends AbstractAlgo {
    
    /** Variablen */
	
    /**
     * Maximale Groesse des Archivs
     */
    private int archiveSize;//Archivgroesse
    
    /**
     * Archiv-Objekt
     */
	private ArrayList archive = new ArrayList();
	
	/**
	 * Fitnesswert des aktuellen Individuums (gibt immer nur eins pro Generation)
	 */
	private double fitness=0; //Dummy-Fitnesswert
	
	/**
	 * Aktueller Fitnesswert des Archivs, mit Maximum von Double initialisiert.
	 */
	private double archiveFitness=Double.MAX_VALUE;
	
	
	/**
	 * Leerer Konstruktor, der nur den Konstruktor der Superklasse (AbstractAlgo) aufruft. (Wird fuer Help-Klasse benoetigt.)
	 */
	public TestAlgo() {
	    super();
	}
	
	
    /** 
     * Konstruktor. Ruft Konstruktor von AbstractAlgo auf und initialisiert Archiv.
     * 
     * @param args String-Array der Kommandozeilenparameter
	 */
	public TestAlgo (String[] args)
	{
		/* Konstruktor von AbstractAlgo aufrufen um wichtigste Dinge zu setzen */
	    super(args);
	    
	   /* Algo-spezifische Parameter setzen */	
	    archiveSize = specificParameters.getIntProperty("archive", 0, true);

	   /* Archiv initialisieren */	
	 	archive = new ArrayList();
	 	
	}

	
    /** 
     * Startroutine des Algorithmus.
     * 
     * <br>Generiert ein zufaelliges Individuum mit so vielen Objektvariablen, wie es die Testfunktion
     * erwartet. Dann wir die Testfunktion damit aufgerufen und deren Ergebnisse zum Fitnesswert
     * aufsummiert.
     * <br>Dann wird das Archiv upgedated und der liveOutput gestartet.
     * <br>Ueber all dies wird maxGenerations-mal iteriert und am Ende true zurueckgegeben. 
     * 
	 * (non-Javadoc)
	 * @see algo.AbstractAlgo#startAlgo()
	 */
	public boolean startAlgo()
	{
		test.init(specificTestfktParameters);
		
	   /* Hauptschleife des Algorithmusses ueber die Anzahl der Iterationen */			
		for (int i = 0; i<maxGenerations; i++)
		{
			ArrayList obj = new ArrayList();
			
		   /* Objektvariablen des Indiviuums erzeugen */	
			for (int j = 0; j < test.getDimension(); j++)
			{
				obj.add(new Double(rand.nextDouble()));						
			}
			
		   /* Aufruf der Testfunktion */
			Individual ind;
			try
			{
				ind = new Individual(obj, (ArrayList)test.execute(obj));
			}
			catch (Exception e)
			{
				 ind = new Individual();
				 e.printStackTrace();
			}
			
			fitness=0; // Fitness initialisieren

		   /* Alle Elemente der Ergebnisliste "results" summieren (Fitnessberechnung) */ 	 
			for (Iterator iter = ind.getFitness().iterator(); iter.hasNext();)
			{
				Double helper = (Double) iter.next();
				fitness += helper.doubleValue();
			}

		   /* Archiv updaten */	
			refreshArchive(ind);
		
		   /* Live Output, damit auch schoen was zu sehen ist ;-) */
		    liveOutput(ind.getObjectVariables());	
		}

	    archive = (ArrayList)test.postProcessing(archive);
		test.finalize();
		
		return true; // erstmal stets true zurueckgeben
	}
	
	
    /** 
     * Fuegt wenn noetig das uebergebene Individuum ins Archiv ein und loescht das Archiv,
     * wenn dessen Fitnesswert besser ist als der des Archivs.
     * 
     * @param indi Einzufuegendes Individuum
	 */
	private void refreshArchive(Individual indi) {
		if (fitness < archiveFitness)  // Bessere Fitness
		{
			archive.clear();
			archive.add(indi);
			archiveFitness = fitness;
		} 
		else
		{
			if (fitness == archiveFitness)  // Gleiche Fitness
			{
				if (archive.size()<archiveSize)  // Noch Platz im Archiv?
				{
					archive.add(indi);
				}
			}
		}
	}
	

	/** 
	 * Gibt das aktuelle Individuen auf die Konsole aus
	 * 	
	 * @param indiv Auszugebendes Archiv als ArrayList
	 */
	private void liveOutput(ArrayList indiv) {
		 try 
		 {
 			StringBuffer row=new StringBuffer();
			for (Iterator iter = indiv.iterator(); iter.hasNext();)
			{
				Double value = (Double) iter.next();
				row.append(value.toString()+" ");
			}
			row.append("= " + Double.toString(fitness));
			System.out.println(row.toString());
		 }
		 catch (Exception e) 
		 {
			 e.printStackTrace();
		 }
	 }
	

	/** 
	 * Gibt eine Kurzbeschreibung des Algos zurueck.
	 * 
	 * (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
	    String out;

	    out =  "\n\nKlasse TestAlgo\n";
	    out += "=================\n";
	    out += "\n";
	    out += "Beschreibung:\n";
	    out += "Komplett randomisierter Algorithmus mit Bewertung durch Summierung der Fitnesswerte.\n";
	    out += "Es wird ein zufaelliges Indiviuum generiert und bewertet. Der aufsummierte Fitnesswert" +
	    		" (Ergebnisse der Testfunktion werden hierfuer aufsummiert) bestimmt die Qualitaet" +
	    		" des Individuums. Dies wird so oft wie angegeben wiederholt.\n";
	    out += "Das Archiv wird nicht aufgrund von Pareto-Dominanz erzeugt, sondern beinhaltet nur solche" +
	    		" Individuen, die den hoechsten aufsummierten Fitnesswert haben. In der Regel wird dies folglich" +
	    		" nur ein Individuum sein.\n";
	    out += "\n";
	    out += "Kommandozeilenparameter:\n";
	    out += "-t: Klasse der Testfunktion (z.B. testfkt.FonsecaF1)\n";
	    out += "-g (int): Anzahl der Generationen/Iterationen\n";
	    out += "-archive (int): Maximalgroesse des globalen Archives, welches am Ende ausgegeben wird.\n";
	    out += "-r (int): optional, Random-Seed (Initialisierung der Zufallsklasse)\n";
	    out += "-o: optional, Praefix fuer die Dateinamen der Ausgabe (An diese wird noch" +
	    		" das Kuerzel fuer die Art der Ausgabedatei angehaengt, z.B. .best.res);" +
	    		" Wird der Wert nicht gesetzt, so wird ein Praefix aus Datum und Uhrzeit gewaehlt\n";
	    out += "-d: optional, wenn gesetzt, werden Debug-Ausgaben angezeigt\n";
	    out += "\n\n";
	    
	    return out;
	}
	
	
    /** 
     * main-Methode
     * 
	 * Hier wird ein neues Objekt erzeugt, mit der Methode startAlgo aufgerufen.
	 * <br>Nach dem Durchlauf werden die Fitnesswerte der Individuum im  Archiv in eine
	 * bestimmte Datei geschrieben.
     * 
     * @param args String-Array der Kommandozeilenparameter
	 * @throws NoIndividualException
	 */
	public static void main(String[] args) throws NoIndividualException	{
	   /* Algorithmus-Objekt erzeugen */	
		TestAlgo algo = new TestAlgo(args);
	   /* Algorithmus starten */
		algo.startAlgo();
	   	/* Archiv (Objektvariablen) Textdatei ausgeben */
		AlgoOutput.writeObjectVariables(algo.archive, algo.outputFileNamePrefix, false);
	}
	
}
