/*
 * Copyright (C) 2005  PG447 - University Dortmund
 * 
 * This file is part of NObELTJE.
 * 
 * NObELTJE is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as published by
 * the Free Software Foundation.
 * 
 * NObELTJE is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with NObELTJE; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

 package org.nobeltje.analysis;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import java.util.Properties;

import org.nobeltje.tools.io.ArgsInputBeautifier;
import org.nobeltje.tools.io.FileParser;
import org.nobeltje.tools.io.Number;
import org.nobeltje.tools.io.PropertiesPG;
import org.nobeltje.exceptions.MalInputFormatException;

/**
 * Stellt die Polygonzge des Temperierbohrungsproblems mit Gnuplot 
 * grafisch dar. Erzeugt ein .plt File und startet Gnuplot. Die 
 * Ausgabe erfolgt wahlweise direkt mit Gnuplot, als Png Dateien oder
 * als animiertes Gif File.
 *  
 * @author $Author: noot $
 * @version $Revision: 726 $
 * $Date: 2005-02-02 00:50:20 +0100 (Mi, 02 Feb 2005) $
 */
public class BohrVisualizer {
	private File script;
	private String inputData;
	private String outputPrefix,outputPrefixLong;
	private String mode;
	private List polygons;

	public BohrVisualizer(String[] args) {
		Locale.setDefault(Locale.ENGLISH);
		// Lese Parameter ein
		try {
			Properties input = ArgsInputBeautifier.beautify(args);

			// Daten
			inputData = PropertiesPG.getStringProperty(input, "d", "", true);

			// Prfix fr Ausgabedatei(en)
			outputPrefix = PropertiesPG.getStringProperty(input, "o", "bohrviz", true);

			mode = PropertiesPG.getStringProperty(input, "mode", "gnuplot", true);
		
		}
		catch (MalInputFormatException mife) {
			mife.printStackTrace();
			System.exit(1);
		}

		
		//Lese Polygonzge aus der Datei ein
		polygons = new ArrayList();
		try {
			FileParser.parseFile(inputData, polygons);
		} catch (Exception e) {
			e.printStackTrace();
			System.exit(1);
		}
		
		if (mode.startsWith("gnuplot"))
			gnuplot();
		else
			if (mode.startsWith("povray"))
				System.out.println("more to come");	
		
	}
	
	private void gnuplot () {
		List polygon;
		try {
			script = new File(outputPrefix + ".plt");
			
			FileWriter fw = new FileWriter(script.getAbsolutePath());
						
			fw.write("#Gnuplotaufruf fr die Datei " + inputData +"\n");
			// Lese Daten ein
			fw.write("reset\n");
			
			// setze png ausgabe und erzeuge ordner
			if (mode.indexOf("gif")!=-1||mode.indexOf("png")!=-1) {
				fw.write("set terminal png\n");
				File dir = new File(outputPrefix);
				if (!dir.mkdir()) {
					System.out.println("Directory already exists, please delete!");
					System.exit(1);
				}
				outputPrefixLong = "'" + outputPrefix + File.separator + outputPrefix;
				fw.write("set output " + outputPrefixLong + ".png'\n");
			} else {
				fw.write("set terminal pop\n");
			}
			
			fw.write("set parametric\n");
			fw.write("set xrange [-2:2]\n");
			fw.write("set yrange [-2:2]\n");
			fw.write("set zrange [0:2]\n");
			fw.write("set pm3d explicit\n");
			
			fw.write("set style line 1 lt 3 lw 2\n");
			fw.write("set style arrow 1 ls 1\n");
			
			//box setzen oder nicht
			if (mode.indexOf("boxed")==-1) {
				fw.write("splot [0:pi] [0:pi] cos(u)*cos(v),cos(u)*sin(v),sin(u)\n");
			} else {
				fw.write("splot [0:pi] [0:pi] cos(u)*cos(v),cos(u)*sin(v),sin(u) with pm3d ,  -2,(u/pi-0.5)*4,v*2/pi, 2,(u/pi-0.5)*4,v*2/pi, (u/pi-0.5)*4,-2,v*2/pi, (u/pi-0.5)*4,2,v*2/pi, (u/pi-0.5)*4,(v/pi-0.5)*4,2\n");
			}
			
			Double x,y,z;
			String from,to;
			int counter = 1;
			String countlabel;
			
			for (Iterator iter = polygons.iterator();iter.hasNext(); counter ++) {
				polygon = (ArrayList)iter.next();
				if (polygon.size()%3==0 & polygon.size() != 0) {
					Iterator iter2 = polygon.iterator();
					x = (Double) iter2.next();
					y = (Double) iter2.next();
					z = (Double) iter2.next();
					to = Number.getDecimal(x) + ", " + Number.getDecimal(y) + ", " + Number.getDecimal(z);
					for (int i = 0; i < (polygon.size()/3 -1); i++ ) {
						from = to;
						x = (Double) iter2.next();
						y = (Double) iter2.next();
						z = (Double) iter2.next();
						to = Number.getDecimal(x) + ", " + Number.getDecimal(y) + ", " + Number.getDecimal(z);
						fw.write("set arrow " + (i+1) + " from " + from + " to " + to + "as 1\n");
					}
				} else {
					System.out.println("Falsche Anzahl von Koordinaten");
				}
				
				fw.write("set title \"Bohrung " + counter + " von " + polygons.size() + " \n");
				if (mode.indexOf("gif")!=-1||mode.indexOf("png")!=-1) {
//					0 vor zahlen setzen damit das sortieren klappt
					countlabel = String.valueOf(counter);
					while (countlabel.length() < String.valueOf(polygons.size()).length()) {
						countlabel = "0" + countlabel;
					}
					
					//outputPrefixLong = outputPrefix + File.pathSeparator + outputPrefix + counter + ".png";
					fw.write("set output " + outputPrefixLong + "_" + countlabel + ".png'\n");
					fw.write("replot\n");
				} else {
				fw.write("replot\n");
				fw.write("pause -1 \"Bohrung " + counter + " von " + polygons.size() + " (-> return)\" \n");
				}
			}
			
			fw.write("set output\n");
			fw.write("set terminal pop\n");
			fw.write("");
			
			// Schliee Device
			fw.close();

			
			try {
				System.out.println("gnuplot -persist "+ script);
				Process visualizationProcess = Runtime.getRuntime().exec("gnuplot -persist "+ script);
				System.out.println("Fertig!");
				//PrintStream gnuStream = new PrintStream(visualizationProcess.getOutputStream());
			}
			catch(Exception e){
				e.printStackTrace();
			}
			
			//anigifs bauen
			if (mode.indexOf("anigif")!= -1) {
				AnimatedGifEncoder anigifencoder = new AnimatedGifEncoder();
				String anigifargs[] = { "-d", outputPrefix, "-o", outputPrefix + ".gif", "-delay", "1000"};
				
				System.out.println("Aufruf des AnigifEncoders noch nicht automatisiert, bitte manuel startenmit den Parametern: \n -d " + outputPrefix + " -o " + outputPrefix + ".gif " + " -delay 1000");
				//TODO warten auf das ende von gnuplot einbauen
				
				//anigifencoder.runEncoder(anigifargs);
			}
		}
		catch (IOException ioe) {
			ioe.printStackTrace();
			System.exit(1);
		}
	}
	
	/**
	 * Kommandozeilenparameter
	 * <ul>
	 *   <li><b>-d</b> Name des Files, der die Eingabedaten enthlt</li>
	 *   <li><b>-o</b> Namensprfix fr die Ausgabedatei bzw bei Dateiausgabe das Verzeichnis</li>
	 *   <li><b>-mode</b> Modus(die Klammern sind optional, / bedeutet oder): gnuplot(boxed)(png/anigif)</li>
	 * </ul>
	 * <p>Beispiel: -d 2004-11-18_19-51-32.best.in -mode gnuplotanigif -o testviz
	 * </p>
	 */
	public static void main(String[] args) {
		BohrVisualizer h = new BohrVisualizer(args);
	}
}