/*
 * Copyright (C) 2005  PG447 - University Dortmund
 * 
 * This file is part of NObELTJE.
 * 
 * NObELTJE is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as published by
 * the Free Software Foundation.
 * 
 * NObELTJE is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with NObELTJE; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

 /*
 * Created on 17.11.2004
 */
package org.nobeltje.analysis;

import java.io.File;
import java.io.FilenameFilter;
import java.util.ArrayList;
import java.util.Iterator;

/**
 * Diese Klasse implementiert das Interface FilenameFilter.<br>
 * Alle Dateien, deren Endung "suffix" entspricht, passieren den Filter.
 * Das Suffix wird beispielsweise fr Bitmap Dateien ("datei.bmp") einfach
 * in der Form "bmp" angegeben.<br>
 * Dem Konstruktor kann entweder ein String als Argument bergeben werden oder
 * ein Array von Strings, falls mehrere Dateiendungen zugelassen werden sollen.
 * 
 * @author $Author: noot $
 * @version $Revision: 726 $
 * $Date: 2005-02-02 00:50:20 +0100 (Mi, 02 Feb 2005) $
 */
public class CustomFilter implements FilenameFilter
{
	private ArrayList suffixList = new ArrayList();

	/**
	 * Konstruktor
	 * 
	 * @param suffix - die Endung, auf die die Dateien geprft werden
	 */
	public CustomFilter(String suffix) {
		suffix = "." + suffix;
		suffixList.add(suffix);
	}

	/**
	 * Konstruktor
	 * 
	 * @param suffixArray - mehrere Endungen, auf die die Dateien geprft werden
	 */
	public CustomFilter(String[] suffixArray) {
		for (int i = 0; i < suffixArray.length; i++) {
			suffixArray[i] = "." + suffixArray[i];
			suffixList.add(suffixArray[i]);
		}
	}

	/**
	 * Der erste Parameter ist durch das Interface vorgeschrieben, wird aber
	 * ignoriert.
	 * 
	 * @param name - der zu vergleichende Dateiname
	 * @return true - wenn die Dateiendung dem Suffix entspricht
	 */
	public boolean accept(File dir, String name) {
		Iterator it = suffixList.iterator();

		while (it.hasNext())
			if (name.endsWith((String)it.next()))
				return true;

		return false;
	}
}
