/*
 * Copyright (C) 2005  PG447 - University Dortmund
 * 
 * This file is part of NObELTJE.
 * 
 * NObELTJE is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as published by
 * the Free Software Foundation.
 * 
 * NObELTJE is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with NObELTJE; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

 /*
 * Created on Nov 16, 2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package org.nobeltje.analysis;
import java.io.File;
import java.io.FileWriter;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.List;
import java.util.Properties;

import org.nobeltje.metrics.Metric;
import org.nobeltje.metrics.NewSMetric;
import org.nobeltje.metrics.SMetricFleischer;
import org.nobeltje.testfkt.TestFunctions;
import org.nobeltje.tools.io.ArgsInputBeautifier;
import org.nobeltje.tools.io.FileParser;
import org.nobeltje.tools.io.ListOperation;
import org.nobeltje.tools.io.PropertiesPG;
import org.nobeltje.exceptions.MalInputFormatException;


/**
 * @author $Author: noot $
 * @version $Revision: 726 $
 * $Date: 2005-02-02 00:50:20 +0100 (Mi, 02 Feb 2005) $
 */
public class MeanMetricPlotter {
	
	/**
	 * <p>ist ein String, der den Namen der Ausgabedatei enth?lt, in die die
	 * einzelnen Durchl?ufe des Designs mit dem jeweils zugeh?rigen Wert aus der Metrik geschrieben
	 * werden.</p>
	 */
	private String outputFile;
	
	/**
	 * <p>enth?lt den Namen der zu benutzenden Metrik. Dieser wird vom SurfacePlotter automatisch um den 
	 * passenden Paketnamen .metrics erg?nzt.</p>
	 */
	private String metricName;
	
	/**
	 * Metricobjekt
	 */
	private Metric metric ;
	
	/**
	 * Name der testfunktion
	 */
	private String testfktName;

	/**
	 * Testfunktionobjekt; wird f?r die Euklidmetric benutzt
	 */
	private TestFunctions testfkt ;
	
	/**
	 * Angabe des Referenzpunktes fuer die SMetrik
	 */
	private List refPoint;
	
	/**
	 * <p>in diesem Verzeichnis wird nach Dateien mit der Endung .res gesucht, die dann vom MetricsPlotter
	 * ausgewertet werden.</p>
	 */
	private File dir;

	/**
	 * die Namen der  Parameter, die variiert wurden. Diese  Parameter werden vom MeanmetricPlotter beachtet und  
	 * Metric werte von Dateien mit gleichen Parameterwerten gemittelt
	 */
	private String[] pName ;

	/**
	 * Die Klasse visualisiert die Metrikegebnisse eines Designaufrufs in Abh?ngigkeit von einem Parameter p1 in einem 2-dimensionalen Gnu-Plot.
	 *Zu allen Dateien im angegebenen Verzeichnis werden Metrikwerte berechnet. Die Metrikwerte aus Dateien, die den selben p1-Wert im
	 *Dateinamen haben, werden gemittelt und als Wert auf der Y-Achse dargestellt.
	 * 
	 * @param args Enth?lt die Kommandozeilenparameter
	 * @see tools.io.ArgsInputBeautifier
	 */

	// String, um Dateinamen zu speichern & zu behandeln
	private String filename;
	private PrintWriter fwPng ;
	private PrintWriter fw    ;
	private PrintWriter dataWriter ;
	
	
	public MeanMetricPlotter(String [] args){
		    
			// Kommandozeilen-Parameter extrahieren
			extractParameters(args);
			writeHeader();
			// Zu plottende Daten in Datei schreiben & Gnuplot damit aufrufen 
			plotSurface();
		}




		/*
		 * Ruft Gnuplot auf, setzt die Datei mit den zu plottenden Punkten zusammen, etc.
		 * 
		 */
		private void plotSurface(){

			String [] pValue ;
			// Die Werte der beiden zu plottenden Parameter p1 und p2
			


			// Hole aus dem angegebenen Verzeichnis alle Dateinamen mit der
			// Endung die im Fitness Filter festgelegt ist
			File fitnessFiles[] = dir.listFiles(new FitnessFilter());
			
			System.out.println("Anzahl der Dateien : "+fitnessFiles.length);
					
			

				
			try{


				// Binde die Metrikenklasse anhand des Namens an
			    metric = (Metric)Class.forName("metrics." + metricName).newInstance();
				

				// Automatische Referenzpunktbestimmung fuer die SMetric:  			
				this.computeReferencePoint(metric, fitnessFiles);



				// Rufe die Metrik fuer jede Datei auf
				Hashtable metricValuesHash = new Hashtable();
				for (int i = 0; i < fitnessFiles.length; i++) {
					// Lese die Fitness-Werte aus der jew. Datei ein
					
					 pValue = new String[pName.length];
					
					
					boolean dateiInOrdnung = true ;
					Properties paramNamesAndValues = this.extractParameterValues(fitnessFiles[i].getName());
					for (int j =1 ; j< pValue.length ; j++){
						pValue[j] =paramNamesAndValues.getProperty( pName[j]);
						if (pValue[j] == null){
							System.out.println("Datei"+fitnessFiles[i].getName()+"   enthaelt den Parameter \""+pName[j]+"\" nicht");
							dateiInOrdnung = false ;
						}
					}
					if(dateiInOrdnung){    		
						//Metric aufrufen
						double metricValue = computeMetricValue(fitnessFiles,i);
						
						// die ergebnisse der metriken werden in Listen geschrieben
						// und die Listen werden in einer Hashtable gespeichert
						//in einer Liste befinden sich ergebnisse von Laeufen mit selben Parametern und anderem RandomSeed
						String key ="";
						for (int j= 1; j<pName.length ; j++)
							key += pValue[j]+"-";
						
						
						ArrayList dieListe = (ArrayList)metricValuesHash.get(key);
						
						if(dieListe == null){
							dieListe = new ArrayList();
							dieListe.add(pValue);
							
							metricValuesHash.put(key,dieListe );
							
						}
						
						dieListe.add(new Double(metricValue));
					}
					
				}
				// die durchschnittswerte  der Laeufe mit selben Parameterwerten werden berechnet 
				// und in die dateien geschrieben  
				Enumeration enu =metricValuesHash.keys();
				System.out.println("Anzehl eintraege in hashtabelle :  "+metricValuesHash.size());
				while (enu.hasMoreElements()){
					ArrayList metricValueList = (ArrayList)metricValuesHash.get(enu.nextElement());
					double durchschnitt =0;
					for (int kk =1 ;kk< metricValueList.size(); kk++){
						// Durchschnittswert berechnen
						double aktuellerWert = ((Double)metricValueList.get(kk)).doubleValue();
						durchschnitt += aktuellerWert ;
						
					}

					durchschnitt /=(metricValueList.size()-1);

					// Die Parameterwerte in die Ausgebadatei schreiben
					pValue = (String[])metricValueList.get(0);
					for (int i =1 ; i< pValue.length; i++){
						if (pName.length ==2){
							fw.print(pValue[i] + " ");
							fwPng.print(pValue[i] + " ");
						}
					    dataWriter.print(pValue[i] + " ");
					}
					 // Schreibe das Ergebnis der Metrik in eine Ausgabedatei
					if (pName.length ==2){
						fw.println(durchschnitt );
						fwPng.println(durchschnitt );
					}
					dataWriter.println(durchschnitt );
					//dataWriter.println(durchschnitt+" "+min+" "+max );  
				}
				System.out.println("pname.length  "+pName.length);
				if (pName.length ==2){
					fw.println("e");
					fwPng.println("e");
					
					fw.close();
					System.out.println("fw-stream wurde geschlossen !!!");
					fwPng.close();
					System.out.println("fwPng-stream wurde geschlossen !!!");
				}
				dataWriter.close();
				System.out.println("dataWriter-stream wurde geschlossen !!!");



				// Gnuplot aus Java aufrufen
				//visualizationProcess = Runtime.getRuntime().exec("gnuplot -persist");
				
				try {
					if (pName.length ==2){
					System.out.println("gnuplot -persist "+ "png_"+outputFile);
					Process visualizationProcess = Runtime.getRuntime().exec("gnuplot -persist "+ "png_"+outputFile);
					System.out.println("ende von gnuplot");
					}
					//PrintStream gnuStream = new PrintStream(visualizationProcess.getOutputStream());
				}
				catch(Exception e){
					e.printStackTrace();
				}
				try {
					if (pName.length ==2){
					System.out.println("gnuplot -persist "+ outputFile);
					
					Process visualizationProcess = Runtime.getRuntime().exec("gnuplot -persist "+ outputFile);
					System.out.println("ende von gnuplot");
					}
					//PrintStream gnuStream = new PrintStream(visualizationProcess.getOutputStream());
				}
				catch(Exception e){
					e.printStackTrace();
				}
				
			}

			// Fehler bei nicht gefundener Algo-Klasse ausgeben und abbrechen
			catch (ClassNotFoundException cnfe){
				System.err.println("\nFEHLER: Angegebene Algo-Klasse konnte nicht gefunden werden.\n");
				cnfe.printStackTrace();
				System.exit(1);	// Direkter Abbruch des Programms
			}
			// Fehler bei unzulaessiger Algo-Klasse ausgeben und abbrechen
			catch (ClassCastException cce){
				System.err.println("\nFEHLER: Angegebene Algo-Klasse ist nicht zulaessig (Cast-Fehler).\n");
				cce.printStackTrace();
				System.exit(1);	// Direkter Abbruch des Programms
			} 
			
			/*catch (FileNotFoundException fnfe) {
				System.err.println("\nFEHLER: Metrik wurde falsch angegeben.\n");
				fnfe.printStackTrace();
				System.exit(1);	// Direkter Abbruch des Programms
			}
			catch (IOException ioe) {
				System.err.println("\nFEHLER: Metrik wurde falsch angegeben.\n");
				ioe.printStackTrace();
				System.exit(1);	// Direkter Abbruch des Programms
			}*/
			catch (Exception e) {
				// Entweder lief das Parsen schief, oder
				// die Metrik unterstuetzt den Aufruf nicht
				e.printStackTrace();
				System.exit(1);	// Direkter Abbruch des Programms
			}
				
			
		}
		
		/**
		 * Berechnet fuer gewuenschte Datei mit Fitnesswerten den Metrikwert mit der gewuenschten Metrik
		 * 
		 * @param files File[] Array mit Filenamen
		 * @param index int Array-Index der zu bearbeitenden Datei 
		 * @return double Berechneten Metrikwert
		 */ 
		private double computeMetricValue(File [] files , int index ){
			double metricValue =0;
			List fitnessValues = new ArrayList();
		     try{
		     	FileParser.parseFile(files[index].getAbsolutePath(), fitnessValues);
		     	
		     }
		     catch(Exception e){
		     	System.out.println("Exception in Methode ComputeMetricValue");
		     	e.printStackTrace();
		     }
			if(metric instanceof NewSMetric || metric instanceof SMetricFleischer){
				try{
				metricValue = metric.computeMetric(fitnessValues);
				}
				catch(Exception e){
			     	System.out.println("Exception in Methode ComputeMetricValue");
			     	e.printStackTrace();
			     }
			}
			
			if(metricName.equals("EuklidMetric")){
				
			  File paretoFile = new File("testfkt"+File.separator+testfktName+"-1000.pareto");
			  List paretoList = new ArrayList();
			  
			  
			  
			  if (!paretoFile.exists()) {
				System.err.println("Verzeichnis oder ZDT nicht vorhanden");
				System.exit(1);
			  }
			  try{
			  FileParser.parseFile(paretoFile.getAbsolutePath(), paretoList);
			  metricValue = metric.computeMetric(fitnessValues,paretoList);
			  }
			  catch(Exception e){
		     	System.out.println("Exception in Methode ComputeMetricValue");
		     	e.printStackTrace();
		     	System.exit(1);
		     }
			}
			
			
			return metricValue ;
		}


		/**
		 * Extrahiert die Werte der zu betrachtenden Parameter
		 * uebergeben werden der Name des auszuwertenden Parameters und die FitnessFiles-Liste
		 * 
		 * Zurueckgegeben wird ein String, in dem der Parameterwert steht
		 */
		 private String extractParameterValues(String paramName, File fitnessFiles[], int i){

			// String, um Dateinamen zu speichern & zu behandeln
			String filename;
					
			// Die zugehoerigen StringBuffer
			StringBuffer parambuf = new StringBuffer();
					
			filename = fitnessFiles[i].getName();

			// Position der beiden Parameter im Dateinamen lokalisieren
			
			int j = filename.indexOf(paramName);
			if (j == -1)
				return null;
						
			// um die Laenge des Parameter-Namens und um das "="-Zeichen verschieben	
			j = j + paramName.length()+1;
					
			// Den Parameter-Wert extrahieren (beginnt hinter dem '=' hinter dem Parameternamen und endet mit einem '_' oder einem ".best")
			for (int m = 0; (filename.charAt(j+m)!='_')&&(!filename.substring(j+m ,  j+m+5).equals(".best"));m++){
				parambuf.append(filename.charAt(j+m));
			}

			// Strings kopieren
			return parambuf.toString();

		 }
		 
		 /**
		  * Extraahiert aus dem Dateinamen die Parameternamen und die dazugehoerenden Werte 
		  * und gibt sie in einem Propertiesobjekt zurueck
		  * @param fileName
		  * @return
		  */
		 private Properties extractParameterValues(String fileName){
		 	Properties prop = new Properties();
		 	String [] namesAndValues = fileName.split("_");
		 	int index = namesAndValues[namesAndValues.length-1].indexOf(".best");
		 	namesAndValues[namesAndValues.length-1] = namesAndValues[namesAndValues.length-1].substring(0,index);
		 	for (int i = 0; i< namesAndValues.length ; i++){
		 		String [] nAndV = namesAndValues[i].split("=");
		 		if(nAndV.length == 2)
		 			prop.setProperty(nAndV[0],nAndV[1]);
		 	}
		 	return prop ;
		 }



		/**
		 * Berechnet & setzt den Referenzpunkt fuer die NewSMetric und die SMetricFleischer
		 */
		 private void computeReferencePoint(Metric metric, File fitnessFiles[]){

			try {
				if (metric instanceof NewSMetric || metric instanceof  SMetricFleischer ){
				    if (refPoint == null) {
				        refPoint = new ArrayList();
				        System.out.println("Referenzpunkt fuer NewSmetric wird bestimmt");
					   // die Dimension des Raumes wird bestimmt
					   List front = new ArrayList();
					   FileParser.parseFile(fitnessFiles[0].getAbsolutePath(), front);
					   
					   double [] maximas = new double [((List)front.get(0)).size()];
					   
					   // Die Maximalwerte werden bestimmt
					   for (int ii = 0; ii < fitnessFiles.length; ii++) {
							   front = new ArrayList();
								FileParser.parseFile(fitnessFiles[ii].getAbsolutePath(), front);
	//							System.out.println(fitnessFiles[ii].getAbsolutePath());
								for (int jj = 0; jj < front.size(); jj++) {
								    ArrayList point = (ArrayList) front.get(jj);
								    for (int kk = 0; kk < point.size(); kk++)
								        if (ListOperation.getDouble(point, kk) > maximas[kk])
								            maximas[kk] = ListOperation.getDouble(point,kk) ;
							   }
					   }
					   // Referenzpunkt wird erstellt . Dabei wird zu jedem maximalwert 1 addiert .
					   for (int i = 0; i < maximas.length; i++)
					       refPoint.add(new Double(maximas[i]+1));
				    }
				    System.out.println(" refPoint  :  "+refPoint);
				    if(metric instanceof NewSMetric ){
				    	   NewSMetric newsmetric = (NewSMetric)metric;
				    	   newsmetric.setRefPoint(refPoint);
				    }
				    if( metric instanceof  SMetricFleischer){
				    	    SMetricFleischer smetricFleischer = (SMetricFleischer)metric ;
				    	    smetricFleischer.setReferencePoint(refPoint);
				    }
				}
			} catch (Exception e) {
				System.out.println("Problem mit Datei-I/O");
				e.printStackTrace();
				System.exit(1);	// Direkter Abbruch des Programms

			}
		 	
		 	
		 }


		/**
		 * Extrahiert die Kommandozeilen-Parameter 
		 * 
		 */
		private void extractParameters(String []args){
			try {

				PropertiesPG input = (PropertiesPG)ArgsInputBeautifier.beautify(args);
				outputFile = ArgsInputBeautifier.getStringProperty(input, "o", "SurfaceData.plt", false);
				metricName = ArgsInputBeautifier.getStringProperty(input, "m", "", true);
				if(metricName.equals("EuklidMetric")){
					testfktName =ArgsInputBeautifier.getStringProperty(input, "t", "", true);
					if(testfktName.substring(0,3).compareToIgnoreCase("zdt")==0)
						testfktName = new String("zdt"+testfktName.substring(3));
				}
				int numberOfParameters = getNumberOfParameters(input, "p");
				System.out.println("Anzahl Parameter pi : "+numberOfParameters);
				pName = new String[numberOfParameters+1];
				for (int i = 1 ; i< pName.length ; i++)
				pName[i] = ArgsInputBeautifier.getStringProperty(input, "p"+i, "", true);
				
				numberOfParameters = getNumberOfParameters(input, "r");
				System.out.println("Anzahl Parameter r : " + numberOfParameters);
				if (numberOfParameters == 0) {
				    // Kein Referenzpunkt angegeben
				    refPoint = null;
				} else {
				    // Referenzpunkt einlesen
				    refPoint = new ArrayList();
				    for (int i = 0 ; i<numberOfParameters ; i++){
						refPoint.add(new Double(input.getDoubleProperty("r" + (i + 1), 0, true)));
					}
				}
				
				dir = new File(ArgsInputBeautifier.getStringProperty(input, "d", "", true));
					
				if (!dir.exists()) {
					System.err.println("Verzeichnis nicht vorhanden");
					System.exit(1);
				}
		
			} 
			catch (MalInputFormatException e) {
				System.out.println("Fehler in den Parametern");
				e.printStackTrace();
				System.exit(1);	// Direkter Abbruch des Programms
			}
		
		}
		
		
		
		/**
		 * gibt die Anzahl der mit "key" gewaehlten Kommandozeilenparameter (z.B. p1,p2...) zurueck, 
		 * die beim Aufruf des MeanMetricPlotters uebergeben werden
		 * 
		 * @param input Properties Properties-Objekt mit Kommandozeilenparametern
		 * @param key String Prefix des zu bearbeitenden Kommandozeilenparameters (z.B. p oder r)
		 * @return int Anzahl der gewuenschten-Parameter
		 */
		private int getNumberOfParameters(Properties input, String key) {
		    boolean parameterVorhanden = true;
		    int parameterNumber = 0;
		    while (parameterVorhanden) {
		        if (input.containsKey(key + (parameterNumber + 1)))
		            parameterNumber++;
		        else
		            parameterVorhanden = false;
		    }
		    return parameterNumber;
		}
		
		
		/**
		 * Erstellt die FileWriter-Objekte, schreibt die Gnuplot-Befehle in die Ausgabedateien
		 * und schreibt die Variablennamen in die .dat Datei
		 */
		private void writeHeader(){
			try {
				
				if (pName.length ==2){
					 
					 fw = new PrintWriter(new FileWriter(""+outputFile, false));
					 fwPng = new PrintWriter(new FileWriter("png_"+outputFile, false));
					 
					fw.println("set title '" + metricName + " " + dir + "'");
					fwPng.println("set title '" + metricName + " " + dir + "'");
					
					fw.println("set xlabel \""+pName[1]+"\"");
					fwPng.println("set xlabel \""+pName[1]+"\"");
					fw.println("set ylabel \""+"Metrkiwert"+"\"");
					fwPng.println("set ylabel \""+"Metrikwert"+"\"");
					
					
					
					String nameWithNewSuffix = outputFile.replaceAll(".plt", ".png");
					fwPng.println("set term png");
					fwPng.println("set output \"" + nameWithNewSuffix + "\"" );
					fw.println("plot '-'");
					
					
					//	 Falls man ein explizites Intervall zum Plotten festlegen will:
					//	 Intervall festlegen				fw.println("splot [][][1:10]'-'");
					
					
					
					fwPng.println("plot '-'");
				}
				dataWriter =new PrintWriter(new FileWriter(outputFile.substring(0,outputFile.length()-4)+".dat",false));
				for (int i =1 ; i< pName.length; i++)
					dataWriter.print(pName[i]+" ");
				dataWriter.println("Y");
				
				
			}
			catch(Exception e){
				e.printStackTrace();
			}
			
		}






		/**
		 * Kommandozeilenparameter des Designs:
		 * <ul>
		 * 	<li>-m Metrikenname</li>
		 * 	<li>-d Verzeichnisname</li>
		 *  	<li>-o Dateiname f?r die Ausgabe (optional)</li>
		 * 	<li>-p1 Parameter 1,  also der erste der beiden variierten Parameter
		 * 	<li>-t Name der Testfunktion.Wird f?r die Euklidmetric benutzt.Moeglich sind nur die Zdt-funktionen</li>
		 * </ul>
		 * <p>Beispiel: -m NewSMetric -d MopsoOne_Deb_1099909856036 -p1 quantityParticle </p>
		 */
		public static void main(String[] args) {
			MeanMetricPlotter mmp = new MeanMetricPlotter(args); 		
		}
}
