/*
 * Copyright (C) 2005  PG447 - University Dortmund
 * 
 * This file is part of NObELTJE.
 * 
 * NObELTJE is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as published by
 * the Free Software Foundation.
 * 
 * NObELTJE is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with NObELTJE; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

 /*
 * Created on Jul 8, 2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package org.nobeltje.analysis;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Properties;
import java.util.StringTokenizer;

import org.nobeltje.metrics.Metric;
import org.nobeltje.metrics.NewSMetric;
import org.nobeltje.metrics.SMetricFleischer;
import org.nobeltje.tools.io.ArgsInputBeautifier;
import org.nobeltje.tools.io.FileParser;
import org.nobeltje.tools.io.PropertiesPG;
import org.nobeltje.tools.io.SortFile;
import org.nobeltje.exceptions.MalInputFormatException;

/**
 * @author $Author: noot $
 * @version $Revision: 726 $
 * $Date: 2005-02-02 00:50:20 +0100 (Mi, 02 Feb 2005) $
 */
public class MetricsPlotter
{
	/**
	 * <p>ist ein String, der den Namen der Ausgabedatei enth?lt, in die die
	 * einzelnen Durchl?ufe des Designs mit dem jeweils zugeh?rigen Wert aus der Metrik geschrieben
	 * werden.</p>
	 */
	private String outputFile;
	
	/**
	 * <p>enth?lt den Namen der zu benutzenden Metrik. Dieser wird vom MetricsPlotter automatisch um den 
	 * passenden Paketnamen .metrics erg?nzt.</p>
	 */
	private String metricName;
	
	/**
	 * Angabe des Referenzpunktes fuer die SMetrik
	 */
	private List refPoint;
	
	/**
	 * <p>in diesem Verzeichnis wird nach Dateien mit der Endung .res gesucht, die dann vom MetricsPlotter
	 * ausgewertet werden.</p>
	 */
	private File dir;
	
	/**
	 * Die Dateien im Verzeichnis werden nach diesem Wert sortiert.
	 */
	private String sort ="";
	
	private boolean ggobi = false;
	
	/**
	 * Der MetricsPlotter dient zur Erstellung einfacher 2-D Balkendiagrammen, die dazu dienen sollen
	 * eine Uebersicht ueber die einzelnen Algorithmenaufrufe zu schaffen und deren Effektivitaet
	 * grafisch darzustellen.
	 * 
	 * @param args Enthaelt die Kommandozeilenparameter
	 * @see tools.io.ArgsInputBeautifier
	 */
	
	public MetricsPlotter(String []args)
	{
		String parametervalues;
		try {
			PropertiesPG input = (PropertiesPG)ArgsInputBeautifier.beautify(args);
			outputFile = ArgsInputBeautifier.getStringProperty(input, "o", "MetricPlotData.plt", false);
			metricName = ArgsInputBeautifier.getStringProperty(input, "m", "", true);
			dir = new File(ArgsInputBeautifier.getStringProperty(input, "d", "", true));
			sort = ArgsInputBeautifier.getStringProperty(input, "sort", "", true);
			ggobi = input.containsKey(new String("ggobi"));
			
			int numberOfParameters = getNumberOfParameters(input, "r");
			System.out.println("\nAnzahl Parameter r : " + numberOfParameters);
			if (numberOfParameters == 0) {
			    // Kein Referenzpunkt angegeben
			    refPoint = null;
			} else {
			    // Referenzpunkt einlesen
			    refPoint = new ArrayList();
			    for (int i = 0 ; i<numberOfParameters ; i++){
					refPoint.add(new Double(input.getDoubleProperty("r" + (i + 1), 0, true)));
				}
			}
			
			if (!dir.exists()) {
				System.err.println("Verzeichnis nicht vorhanden");
				System.exit(1);
			}

			// Hole aus dem angegebenen Verzeichnis alle Dateinamen mit der
			// Endung die im Fitness Filter festgelegt ist
			File fitnessFiles[] = dir.listFiles(new FitnessFilter());
			
			if (sort.compareTo("")!=0){
				StringTokenizer st = new StringTokenizer(sort,",");
				while(st.hasMoreElements()){
					fitnessFiles = SortFile.sortFileList(fitnessFiles,st.nextToken());
				}
			} else {
				Arrays.sort(fitnessFiles);
			}
			
			for (int i = 0; i < fitnessFiles.length; i++)
				System.out.println(fitnessFiles[i].getName());

			try {
				// Schreibe den Header fuer die Gnuplot Datei
				FileWriter fwPng = new FileWriter("png_"+outputFile, false);
				FileWriter fw = new FileWriter(""+outputFile, false);
				FileWriter fwAll = new FileWriter("data_"+outputFile, false);
				FileWriter fwGgobiDat = null, fwGgobiCol = null;
				
				if(ggobi) {
					fwGgobiDat = new FileWriter("ggobi_"+outputFile+".dat", false);
					fwGgobiCol = new FileWriter("ggobi_"+outputFile+".col", false);
				}
				fw.write("set title '" + metricName + " " + dir + "'\n");
				fwPng.write("set title '" + metricName + " " + dir + "'\n");
				fw.write("set xlabel \"Sortierung: "+sort+"\"\n");
				fwPng.write("set xlabel \"Sortierung: "+sort+"\"\n");
				fw.write("set ylabel \"Metrikwert\"\n");
				fwPng.write("set ylabel \"Metrikwert\"\n");
				fw.write("set nokey\n");
				fwPng.write("set nokey\n");
				fw.write("set data style boxes\n\n");
				fwPng.write("set data style boxes\n\n");
				String nameWithNewSuffix = outputFile.replaceAll(".plt", ".png");
				fwPng.write("set term png\n");
				fwPng.write("set output \"" + nameWithNewSuffix + "\"\n" );
				fw.write("plot '-'\n");
				fwPng.write("plot '-'\n");

				// Binde die Metrikenklasse anhand des Namens an
				Metric metric = (Metric)Class.forName("metrics." + metricName).newInstance();
				if (metric instanceof NewSMetric || metric instanceof SMetricFleischer) {
				    ArrayList max = new ArrayList();
			        if (refPoint == null) {
				        System.out.println("Referenzpunkt fuer Newsmetric wird bestimmt");
				        List front = new ArrayList();
				        //					 double max1 = 0 ;
				        for (int ii = 0; ii < fitnessFiles.length; ii++) {
				            front = new ArrayList();
				            FileParser.parseFile(fitnessFiles[ii].getAbsolutePath(), front);
				            for (int jj = 0; jj < front.size(); jj++) {
				                ArrayList point = (ArrayList) front.get(jj);
				                for (int k = 0; k < point.size(); k++) {
				                    if ((ii == 0) && (jj == 0)) {
				                        max.add(point.get(k));
				                    } else {
				                        if (((Double) max.get(k)).doubleValue() < ((Double) point.get(k)).doubleValue()) {
				                            max.set(k, point.get(k));
				                        }
				                    }
				                }
	//								 if(((Double)point.get(0)).doubleValue() > max0)
	//									 max0 = ((Double)point.get(0)).doubleValue() ;
							       
	//								 if(((Double)point.get(1)).doubleValue() > max1)
	//										 max1 = ((Double)point.get(1)).doubleValue() ;   
								
				            }
										
				        }
	//					 ArrayList refPoint = new ArrayList();
	//					 refPoint.add(new Double(max0));
	//					 refPoint.add(new Double(max1));
	
						// erhoehe Werte des Refernezpunktes, damit er gr??er ist als alle
						// Punkte
						for (int kk=0; kk<max.size(); kk++) {
							max.set(kk, new Double(((Double)max.get(kk)).doubleValue() +1));
						}
						refPoint = max;
				    }
				    
				    
				    System.out.println(" refPoint  :  " + refPoint);
				    if (metric instanceof NewSMetric) {
				        NewSMetric newsmetric = (NewSMetric) metric;
				        newsmetric.setRefPoint(refPoint);
				    }
				    if (metric instanceof SMetricFleischer) {
				        SMetricFleischer smetricFleischer = (SMetricFleischer) metric;
				        smetricFleischer.setReferencePoint(refPoint);
				    }
				}
				// Rufe die Metrik fuer jede Datei auf
				List fitnessValues = new ArrayList();
				for (int i = 0; i < fitnessFiles.length; i++) {
					// Lese die Fitness Werte ein
					FileParser.parseFile(fitnessFiles[i].getAbsolutePath(), fitnessValues);
					// Schreibe das Ergebnis der Metrik in eine Ausgabedatei
					double metricValue = metric.computeMetric(fitnessValues);
//					if(metricValue > 4 )
					System.out.print(".");
					fw.write(metricValue + "\n");
					fwPng.write(metricValue + "\n");
					fwAll.write(fitnessFiles[i].getName() + "\n" + metricValue + "\n");
					if(ggobi && fwGgobiDat != null) {
						parametervalues = fitnessFiles[i].getName();
						parametervalues = parametervalues.replaceAll(".best.res","");
						parametervalues = parametervalues.replaceAll("_\\w*="," ");
						parametervalues = parametervalues.replaceAll("^\\w*=","");
						fwGgobiDat.write(metricValue  + " " + parametervalues + "\n");
					}
				}
				//Ggobi Spaltenbezeichnungen setzen
				if(ggobi) {
					fwGgobiCol.write("Metric\n");
					
					parametervalues = fitnessFiles[0].getName();
					parametervalues = parametervalues.replaceAll(".best.res","_");
					parametervalues = parametervalues.replaceAll("=[^=_]*_"," ");
					StringTokenizer paramnames = new StringTokenizer(parametervalues," ");
					while(paramnames.hasMoreTokens()) {
						fwGgobiCol.write(paramnames.nextToken() + "\n");
					}
				}
				
				System.out.println();
				fw.write("e\n");
				fwPng.write("e\n");
				fw.close();
				fwPng.close();
				fwAll.close();
				if(fwGgobiCol!=null) fwGgobiCol.close();
				if(fwGgobiDat!=null) fwGgobiDat.close();
				
				
				// Gnuplot aus Java aufrufen
				//visualizationProcess = Runtime.getRuntime().exec("gnuplot -persist");
				try {
					System.out.println("gnuplot -persist "+ "png_"+outputFile);
					Process visualizationProcess = Runtime.getRuntime().exec("gnuplot -persist "+ "png_"+outputFile);
					System.out.println("ende von gnuplot");
					//PrintStream gnuStream = new PrintStream(visualizationProcess.getOutputStream());
				}
				catch(Exception e){
					e.printStackTrace();
				}
				try {
					System.out.println("gnuplot -persist "+ outputFile);
					Process visualizationProcess = Runtime.getRuntime().exec("gnuplot -persist "+ outputFile);
					System.out.println("ende von gnuplot");
					//PrintStream gnuStream = new PrintStream(visualizationProcess.getOutputStream());
				}
				catch(Exception e){
					e.printStackTrace();
				}
			}

			// Fehler bei nicht gefundener Algo-Klasse ausgeben und abbrechen
			catch (ClassNotFoundException cnfe){
				System.err.println("\nFEHLER: Angegebene Algo-Klasse konnte nicht gefunden werden.\n");
				cnfe.printStackTrace();
				System.exit(1);	// Direkter Abbruch des Programms
			}
			// Fehler bei unzulaessiger Algo-Klasse ausgeben und abbrechen
			catch (ClassCastException cce){
				System.err.println("\nFEHLER: Angegebene Algo-Klasse ist nicht zulaessig (Cast-Fehler).\n");
				cce.printStackTrace();
				System.exit(1);	// Direkter Abbruch des Programms
			} 
			catch (NoSuchMethodException nsme) {
				System.err.println("\nFEHLER: Metrik wurde falsch angegeben.\n");
				nsme.printStackTrace();
				System.exit(1);	// Direkter Abbruch des Programms
			}
			catch (FileNotFoundException fnfe) {
				System.err.println("\nFEHLER: Metrik wurde falsch angegeben.\n");
				fnfe.printStackTrace();
				System.exit(1);	// Direkter Abbruch des Programms
			}
			catch (IOException ioe) {
				System.err.println("\nFEHLER: Metrik wurde falsch angegeben.\n");
				ioe.printStackTrace();
				System.exit(1);	// Direkter Abbruch des Programms
			}
			catch (Exception e) {
				// Entweder lief das Parsen schief, oder
				// die Metrik unterst?tzt den Aufruf nicht
				e.printStackTrace();
				System.exit(1);	// Direkter Abbruch des Programms
			}
		} 
		catch (MalInputFormatException e) {
			e.printStackTrace();
		}
	}

	
	
	/**
	 * gibt die Anzahl der mit "key" gewaehlten Kommandozeilenparameter (z.B. p1,p2...) zurueck, 
	 * die beim Aufruf des MeanMetricPlotters uebergeben werden
	 * 
	 * @param input Properties Properties-Objekt mit Kommandozeilenparametern
	 * @param key String Prefix des zu bearbeitenden Kommandozeilenparameters (z.B. p oder r)
	 * @return int Anzahl der gewuenschten-Parameter
	 */
	private int getNumberOfParameters(Properties input, String key) {
	    boolean parameterVorhanden = true;
	    int parameterNumber = 0;
	    while (parameterVorhanden) {
	        if (input.containsKey(key + (parameterNumber + 1)))
	            parameterNumber++;
	        else
	            parameterVorhanden = false;
	    }
	    return parameterNumber;
	}
	
	
	/**
	 * Kommandozeilenparameter des Designs:
	 * <ul>
	 * 	<li>-m Metrikenname</li>
	 * 	<li>-d Verzeichnisname</li>
	 *  <li>-o Dateiname f?r die Ausgabe (optional)</li>
	 *  <li>-ggobi Dateiname f?r die Ausgabe (optional)</li> 
	 * </ul>
	 * <p>Beispiel: -m SMetric -d OnePlusOnePG_FonsecaF1_1089582482343</p>
	 */
	public static void main(String[] args) {
		MetricsPlotter mp = new MetricsPlotter(args); 		
	}
}
