/*
 * Copyright (C) 2005  PG447 - University Dortmund
 * 
 * This file is part of NObELTJE.
 * 
 * NObELTJE is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as published by
 * the Free Software Foundation.
 * 
 * NObELTJE is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with NObELTJE; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

 /*
 * Created on Nov 16, 2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package org.nobeltje.analysis;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.List;
import java.util.Properties;

import org.nobeltje.metrics.Metric;
import org.nobeltje.metrics.NewSMetric;
import org.nobeltje.metrics.SMetricFleischer;
import org.nobeltje.tools.io.ArgsInputBeautifier;
import org.nobeltje.tools.io.FileParser;
import org.nobeltje.tools.io.ListOperation;
import org.nobeltje.tools.io.PropertiesPG;
import org.nobeltje.exceptions.MalInputFormatException;


/**
 * @author $Author: noot $
 * @version $Revision: 726 $<br>
 * $Date: 2005-02-02 00:50:20 +0100 (Mi, 02 Feb 2005) $
 */
public class PercentageMetricPlotter {
	
	/**
	 * <p>enthaelt den Namen der zu benutzenden Metrik. Dieser wird automatisch um den 
	 * passenden Paketnamen metrics. ergaenzt.</p>
	 */
	private String metricName;
	
	/**
	 * Metrikobjekt
	 */
	private Metric metric ;
	
	/**
	 * <p>in diesem Verzeichnis wird nach Dateien mit der Endung .res gesucht, die dann 
	 * vom PercentageMetricPlotter ausgewertet werden.</p>
	 */
	//private String testfktName;
	
	/**
	 * <p>Testfunktionsobjekt</p>
	 */
	//private TestFunctions testfkt;
	
	/**
	 * Der Name des Parameters , der variiert wurde. Dieser Parameter bildet 
	 * die x-Koordinaten-Achsen im Plot. 
	 */
	private String[] pName;
	
	/**
	 * Angabe des Referenzpunktes fuer die SMetrik
	 */
	private List refPoint;
	
	/**
	 * <p>Bester Metrikwert aller Algo-Laeufe (kann manuell gewaehlt werden).</p>
	 */
	private double bestMetricValue = 0;
	
	/**
	 * <p>Angabe, wieviel Prozent vom groessten Metrikwert als Gueteschranke gewaehlt werden sollen.</p>
	 */
	private int percentageOfBestMetricValue;
	
	/**
	 * <p>Metrikwert, den die Algo-Laeufe erzielen muessen (manuell Gueteschranke).</p>
	 */
	private double metricBoundary = -1;
	
	/**
	 * <p>Fileobjet zum Verzeichnis, in dem sich die zu verarbeitenden Quelldaten beinfinden</p>
	 */
	private File dir;

	
	/**
	 * <p>ist ein String, der den Namen der Ausgabedatei enthaelt, in die die
	 * einzelnen Durchlaeufe des Designs mit dem jeweils zugehoerigen Wert aus der Metrik geschrieben
	 * werden. Dies ist nur das Prefix!</p>
	 */
	private String outputFile;
	
	
	/**
	 * FileWriter fuer gnuplot-Steuer-Datei, die das Plot in ein PNG-Bild speichert
	 */
	private FileWriter fwPng;

	/**
	 * Dateiname fuer die fwPng-Datei
	 */
	private String fileNameFwPng = "";
	
	/**
	 * Dateiname der zu erstellenden png-Datei
	 */
	private String fileNamePng = "";
	
	
	/**
	 * FileWriter fuer gnuplot-Stuer-Datei, die das Plot darstellt
	 */
	private FileWriter fw;
	
	/**
	 * Dateiname fuer die fw-Datei
	 */
	private String fileNameFw = "";
	
	
	/**
	 * FileWriter fuer zu plotttende Daten-Datei
	 */
	private FileWriter dataWriter;
	
	/**
	 * Dateiname fuer die data-Datei
	 */
	private String fileNameData = "";
	
	
	/**
	 * FileWriter fuer Dump-Datei (in die alle Konsolenausgaben geschrieben werden)
	 */
	private FileWriter dumpWriter;
	
	/**
	 * Dateiname fuer die dump-Datei
	 */
	private String fileNameDump = "";
	
	
	/**
	 * Konstruktor
	 * <br>Ruft die verschiedenen Arbeitsschritte auf.
	 * 
	 * @param args Enthaelt die Kommandozeilenparameter
	 * @see tools.io.ArgsInputBeautifier
	 */
	public PercentageMetricPlotter(String[] args) {
	    // Kommandozeilen-Parameter extrahieren
	    extractParameters(args);
	    // Gnuplot-Steuer-Dateien schreiben (gnuplot-Befehle)
	    writeHeader();
	    // Zu plottende Daten in Datei schreiben & Gnuplot damit aufrufen
	    computeAndPlot();
	}

	
	/**
	 * toString-Methode, wird hier benutzt, um eine Algorithmen-Beschreibung zu uebergeben.
	 * 
	 * @see tools.io.Help
	 */
	public String toString() {
		String out = "";
		out =  "\n\nKlasse PercenatageMetricPlotter\n";
	    out += "================================\n";
	    out += "\n";
	    out += "Beschreibung:\n";
	    out += "Die Klasse visualisiert, wie viele Laeufe eines Algorithmus einen gewissen Metrikwert erreicht haben. Die geschieht in Abhaengigkeit von einem Parameter p1 in einem 2-dimensionalen Gnu-Plot.\n";
	    out += "Zu allen Dateien im angegebenen Verzeichnis werden Metrikwerte berechnet. Dann wird unter allen berechnet Metrikwerten der Groesste bestimmt. Der zu erreichende Metrikwert betrgt dann \"-percentage\" Prozent von diesem Groessten.\n";
	    out += "Ausgabe: Es werden 2 Steuerdateien mit Befehlen fuer gnuplot erstellt, sowie eine mit den gewonnen Daten.\n";
	    out += "Hinweis zur Eingabe: Alle Dateien, die den mit \"-p1\" angegebenen Algo-Parameter nicht im Dateinamen beinhalten, werden ignoriert!\n";
	    out += "\n";
	    out += "Auf der Y-Achse wird in Prozent dargestellt, wie viele Metrikwerte aus Dateien, die den selben p1-Wert im Dateinamen haben (also Algo-Laeufe, bei denen sich nur der Random-Seed geaendert hat), den gewnschten Metrikwert erreichen.\n";
	    out += "Auf der X-Achse werden die Werte des Parameters \"-p1\" aufgetragen.\n";
	    out += "\n";
	    out += "Kommandozeilenparameter:\n";
	    out += "-d: Verzeichnis in dem sich die zu bearbeitenden Dateien (Design-Ausgaben!) befinden (notwendig)\n";
	    out += "-m: Zu verwendende Metrik (Bsp: \"SMetricFleischer\"; notwendig)\n";
	    out += "-r1, r2,..,rn: Diese Parameter geben einen manuellen Referenzpunkt an, wenn eine SMetrik verwendet werden soll. Jeder Parameter steht fuer eine Dimension (Bsp: \"-r1 2.5 -r2 3\"; optional). Warnung: Bei zu kleinen Werten kann es zu sehr vielen Fehler bei der Metrik kommen!\n";
	    //out += "-t: Name der Testfunktion oder des Simulators (Bsp. FonsecaF1). Muss angegeben werden, wenn die Euklidmetrik werwendet werden soll.\n";
	    out += "-p1: Parameter, der im Design variiert worden ist (Bsp: \"evaluations\"; notwendig)\n";
	    out += "-boundary: Metrikwert, den die Algo-Laeufe erzielen muessen (manuelle Gueteschranke). (Bsp: \"-boundary 1.5\"; optional; hat Vorrang vor \"-percentage\")\n";
	    out += "-percentage: Angabe, wieviel Prozent des besten Metrikwertes die Algo-Laeufe erreicht haben muessen. (Bsp: \"-percentage 70\"; Standard ist 90)\n";
	    out += "-bestMetricValue: Manuelle Wahl des besten Metrikwertes, auf dessen Basis die Berechnung der Gueteschrank mit \"-percentage\" durchgefuert wird. (Bsp: \"-maxMetricValue 5\"; optional)\n";
	    out += "-o: Name der Output-Datei\n";
	    return out;
		
	}
		

	/**
	 * Ruft Gnuplot auf, setzt die Datei mit den zu plottenden Punkten zusammen, etc.
	 * 
	 */
	private void computeAndPlot() {
		
		// Die Werte der beiden zu plottenden Parameter p1 und p2
		String [] pValue;
		
		// Hole aus dem angegebenen Verzeichnis alle Dateinamen mit der
		// Endung die im Fitness Filter festgelegt ist
		File fitnessFiles[] = dir.listFiles(new FitnessFilter());
		print("Anzahl der Dateien  : " + fitnessFiles.length + "\n");
		
		try {
			
			// Binde die Metrikenklasse anhand des Namens an
			metric = (Metric) Class.forName("metrics." + metricName).newInstance();
			
			// Automatische Referenzpunktbestimmung fuer die SMetric:
			
			this.computeReferencePoint(metric, fitnessFiles);
			
			/* Rufe die Metrik fuer jede Datei auf */
			Hashtable metricValuesHash = new Hashtable();
			
			print("\nLese Dateien ein..");
			for (int i = 0; i < fitnessFiles.length; i++) {
			    //print("Verarbeite Datei " + fitnessFiles[i].getName());
			    // Nur Datein verabeiten, die auch den zu betrachtenden Parameter im Dateinamen enthalten
			    if ( (pName.length>1) && (fitnessFiles[i].getName().indexOf(pName[1]) == -1) ) {
			        print("Skip File " + fitnessFiles[i].getName());
			    } else {
			    
			        // Lese die Werte der manipulierten Parameter aus den Namen der
			        // jeweiligen Dateien ein
			        pValue = new String[pName.length];
			        for (int j = 1; j < pValue.length; j++)
			            pValue[j] = this.extractParameterValues(pName[j], fitnessFiles, i);
			        
			        // Metrik aufrufen
			        double metricValue = computeMetricValue(fitnessFiles, i);
			        
			        /* Die Ergebnisse der Metriken werden in Listen geschrieben
			         * und die Listen werden in einer Hashtable gespeichert.
			         * In einer Liste befinden sich Ergebnisse von Laeufen mit
			         * selben Parametern und lediglich unterschiedlichem RandomSeed
			         */
			        String key = "";
			        for (int j = 1; j < pName.length; j++) {
			            key += pValue[j] + "-";
			        }
			        // "dieListe" beihaltet relevante Parameter und Metrikwerte aller Algo-Laeufe
			        // metricValuesHash sammelt diese
			        ArrayList dieListe = (ArrayList) metricValuesHash.get(key);
			        // Wenn Algo-Lauf mit diesen Parametern neu ist: Liste dafuer anlegen 
			        if (dieListe == null) {
			            dieListe = new ArrayList();
			            dieListe.add(pValue);
			            metricValuesHash.put(key, dieListe);
			        }
			        // Metrikwert einfach hinzufuegen (auch wenn Algo-Lauf mit neuen RandomSeed wiederholt wurde) 
			        dieListe.add(new Double(metricValue));
			        
			    }
			}
			print("");
			
			
		   
			Enumeration enu = metricValuesHash.keys();
			print("Anzahl Eintraege in Hashtabelle : " + metricValuesHash.size());
            
		   /* Guetegrenze bestimmen, nach welchem die Metrikenwerte als gut oder schlecht klassifiziert werden */
			
			// Maximalen Metrikwert finden
			double bestMetricValue = 0;
			if (this.bestMetricValue == 0) {
			    while (enu.hasMoreElements()) {
	                ArrayList metricValueList = (ArrayList) metricValuesHash.get(enu.nextElement());
					for (int kk = 1; kk < metricValueList.size(); kk++) {
						bestMetricValue = java.lang.Math.max(bestMetricValue, ((Double) metricValueList.get(kk)).doubleValue());
					}
				}
			    print("\nBester Metrik-Wert (auto)    : " + bestMetricValue);
			} else {
			    bestMetricValue = this.bestMetricValue;
			    print("\nBester Metrik-Wert (manuell) : " + bestMetricValue);
			}
			
			// Guetegrenze bestimmen
			double metricBoundary = -1;
			if (this.metricBoundary == -1) {
				metricBoundary = percentageOfBestMetricValue * bestMetricValue / 100;
				print("Guetegrenze ("+percentageOfBestMetricValue+"% vom Besten) : " + metricBoundary+"\n");
			} else {
			    metricBoundary = this.metricBoundary;
			    print("Manuelle Guetegrenze         : " + metricBoundary+"\n");
			}
			
			
		   /* Ausgabe */
			enu = metricValuesHash.keys();
			while (enu.hasMoreElements()) {
                ArrayList metricValueList = (ArrayList) metricValuesHash.get(enu.nextElement());			
                
                /* Anzahl der Algo-Laeufe bestimmen, die Guetegrenze erreicht haben */
                int goodMetricCount = 0; // Algo-Laeufe, die mind. den geforderten Metrikwert haben
                // Hinweis: metricValueList[0] ist der Parameterwert, aber Index 1 kommen die Metrikenwerte
                for (int kk = 1; kk < metricValueList.size(); kk++) {
                    // Metrikwerte einzeln testen 
					if (((Double) metricValueList.get(kk)).doubleValue() >= metricBoundary) {
					    goodMetricCount++;
					}
				}
                /* Prozentualen Anteil der Algo-Laeufe bestimmen, die Guetegrenze erreicht haben */
                double goodMetricPercentage = ((new Double(goodMetricCount).doubleValue() / (new Double(metricValueList.size()-1).doubleValue())) * 100); 
                
                /* Schreibe Parameterwerte in Datei */
                pValue = (String[]) metricValueList.get(0);
                for (int i = 1; i < pValue.length; i++) {
                    dataWriter.write(pValue[i] + " ");
                    /*if (pName.length == 2) {
                        fw.write(pValue[i] + " ");
                        fwPng.write(pValue[i] + " ");
                    } else {
                        dataWriter.write(pValue[i] + " ");
                    }
                    */
                }
                /* Schreibe den prozentualen Anteil der guten Laeufe in eine Ausgabedatei */
                dataWriter.write(goodMetricPercentage + "\n");
                /*
                if (pName.length == 2) {
                    fw.write(goodMetricPercentage + "\n");
                    fwPng.write(goodMetricPercentage + "\n");
                } else {
                    dataWriter.write(goodMetricPercentage + "\n");
                }
                */
			}
			/*
			if (pName.length == 2) {
				fw.write("e\n");
				fwPng.write("e\n");
				fw.close();
				fwPng.close();
			} else
				dataWriter.close();
			*/
			
			
			/* FileWriter schliessen */
			
			fw.close();
			fwPng.close();
			dataWriter.close();
			

			/* Gnuplot aus Java aufrufen */
			
			//visualizationProcess = Runtime.getRuntime().exec("gnuplot -persist");
			
			if (pName.length == 2) {
			    print("Starte gnuplot-Aufrufe:");
			} else {
			    print("Verarbeitung beendet.");
			}
			
			/* Nur PNG erstellen */
			try {
				if (pName.length == 2) {
				    //print("gnuplot -persist " + "png_" + outputFile);
				    print("gnuplot -persist " + fileNameFwPng);
                    Process visualizationProcess = Runtime.getRuntime().exec("gnuplot -persist " + fileNameFwPng);
                    //Process visualizationProcess = Runtime.getRuntime().exec("C:\\-Tools-\\gnuplot\\bin\\wgnuplot.exe " + "png_" + outputFile);
                    print("Ende von gnuplot.");
				}
				//PrintStream gnuStream = new
				// PrintStream(visualizationProcess.getOutputStream());
			} catch (Exception e) {
				e.printStackTrace();
			}
			
			/* Gnuplot so zum Betrachten aufrufen ohne Dateiausgabe */
			try {
				if (pName.length == 2) {
					print("gnuplot -persist " + fileNameFw);
					Process visualizationProcess = Runtime.getRuntime().exec("gnuplot -persist " + fileNameFw);
					//Process visualizationProcess = Runtime.getRuntime().exec("C:\\-Tools-\\gnuplot\\bin\\wgnuplot.exe -persist " + outputFile);
					print("Ende von gnuplot.");
				}
				//PrintStream gnuStream = new
				// PrintStream(visualizationProcess.getOutputStream());
			} catch (Exception e) {
				e.printStackTrace();
			}

			
			/* FileWriter schliessen */
			
			dumpWriter.close();
			
		}
		
		
		
		// Fehler bei nicht gefundener Algo-Klasse ausgeben und abbrechen
		catch (ClassNotFoundException cnfe) {
			System.err.println("\nFEHLER: Angegebene Algo-Klasse konnte nicht gefunden werden.\n");
			cnfe.printStackTrace();
			System.exit(1); // Direkter Abbruch des Programms
		}
		// Fehler bei unzulaessiger Algo-Klasse ausgeben und abbrechen
		catch (ClassCastException cce) {
			System.err.println("\nFEHLER: Angegebene Algo-Klasse ist nicht zulaessig (Cast-Fehler).\n");
			cce.printStackTrace();
			System.exit(1); // Direkter Abbruch des Programms
		}
		
		catch (FileNotFoundException fnfe) {
			System.err.println("\nFEHLER: Metrik wurde falsch angegeben.\n");
			fnfe.printStackTrace();
			System.exit(1); // Direkter Abbruch des Programms
		} catch (IOException ioe) {
			System.err.println("\nFEHLER: Metrik wurde falsch angegeben.\n");
			ioe.printStackTrace();
			System.exit(1); // Direkter Abbruch des Programms
		} catch (Exception e) {
			// Entweder lief das Parsen schief, oder
			// die Metrik unterstuetzt den Aufruf nicht
			e.printStackTrace();
			System.exit(1); // Direkter Abbruch des Programms
		}
		
	}
	
	
	/**
	 * Berechnet fuer gewuenschte Datei mit Fitnesswerten den Metrikwert mit der gewuenschten Metrik
	 * 
	 * @param files File[] Array mit Filenamen
	 * @param index int Array-Index der zu bearbeitenden Datei 
	 * @return double Berechneten Metrikwert
	 */
	private double computeMetricValue(File[] files, int index) {
		double metricValue =0;
		List fitnessValues = new ArrayList();
		/* Fitnesswerte aus Datei einlesen */
		try {
			FileParser.parseFile(files[index].getAbsolutePath(), fitnessValues);
		}
		catch(Exception e) {
			print("Exception in Methode ComputeMetricValue");
			e.printStackTrace();
		}
		
		/* Metrik aufrufen - Unterscheiden zwischen SMetriken und EuklidMetrik */
		// SMetriken..
		if (metric instanceof NewSMetric || metric instanceof SMetricFleischer) {
			try {
				metricValue = metric.computeMetric(fitnessValues);
			}
			catch(Exception e) {
				print("Exception in Methode ComputeMetricValue");
				e.printStackTrace();
			}
		}
		// EuklidMetrik..
		/*
		if (metricName.equals("EuklidMetric")) {
			print("bin in EuklidMetrik");
			File paretoFile = new File("testfkt" + File.separator + testfktName + "-1000.pareto");
			print(paretoFile + " = ZDT");
			
			print("mit der ZDT-Datei: " + paretoFile.getName());
			List paretoList = new ArrayList();
			
			print(paretoFile.getAbsolutePath());
			if (!paretoFile.exists()) {
				System.err.println("Verzeichnis oder ZDT nicht vorhanden");
				System.exit(1);
			}
			try {
				FileParser.parseFile(paretoFile.getAbsolutePath(), paretoList);
				print(files[index].getAbsolutePath());
				print(fitnessValues.size() + " anzahl punkte der fronten  " + paretoList.size());
                metricValue = metric.computeMetric(fitnessValues, paretoList);
			} catch (Exception e) {
				print("Exception in Methode ComputeMetricValue");
				e.printStackTrace();
				System.exit(1);
			}
		}
		*/
		/* Rueckgabe */
		return metricValue;
	}
	

	/**
     * Extrahiert aus dem gewuenschten Dateinamen den Wert eines zu
     * betrachtenden Parameters
     * 
     * <br>
     * Uebergeben werden der Name des auszuwertenden Parameters und die
     * FitnessFiles-Liste Zurueckgegeben wird ein String, in dem der
     * Parameterwert steht
     * 
     * @param paramName String Name des auszuwertenden Parameters
     * @param fitnessFiles File[] File-Objekte der Dateien mit Fitnesswerten
     * @param i int Zu verwendende Datei aus fitnessFiles
     * @return String Parameterwert
     */
    private String extractParameterValues(String paramName, File fitnessFiles[], int i) {

        // String, um Dateinamen zu speichern & zu behandeln
        String filename;
		
		// Die zugehoerigen StringBuffer
		StringBuffer parambuf = new StringBuffer();
		
		filename = fitnessFiles[i].getName();
		
		// Position der beiden Parameter im Dateinamen lokalisieren
		
		int j = filename.indexOf(paramName);						
		
		// um die Laenge des Parameter-Namens und um das "="-Zeichen verschieben	
		j = j + paramName.length()+1;
		
		/* Den Parameter-Wert extrahieren (beginnt hinter dem '=' hinter dem Parameternamen
		 * und endet mit einem '_' oder einem ".best")
		 */	
		for (int m=0; (filename.charAt(j + m) != '_') && (!filename.substring(j+m, j+m+5).equals(".best")); m++) {
            parambuf.append(filename.charAt(j+m));
		}
		
		// Strings kopieren
		return parambuf.toString();
		
	}
	
	
	/**
	 * 
	 * @param fileName
	 * @return
	 */
/*
     private Properties extractParameterValues(String fileName) {
		Properties prop = new Properties();
		String [] namesAndValues = fileName.split("_");
		int index = namesAndValues[namesAndValues.length-1].indexOf(".best");
		namesAndValues[namesAndValues.length-1] = namesAndValues[namesAndValues.length-1].substring(0,index);
		for (int i = 0; i< namesAndValues.length ; i++) {
			String [] nAndV = namesAndValues[i].split("=");
			prop.setProperty(nAndV[0],nAndV[1]);
		}
		return prop ;
	}
*/


	/**
	 * Berechnet & setzt den Referenzpunkt fuer die NewSMetric 
	 */
	private void computeReferencePoint(Metric metric, File fitnessFiles[]) {
		
		try {
			if (metric instanceof NewSMetric || metric instanceof SMetricFleischer) {
			    
			    if (refPoint == null) {
			        refPoint = new ArrayList();
			        print("Referenzpunkt fuer NewSmetric wird bestimmt");
			        
			        // die Dimension des Raumes wird bestimmt
			        List front = new ArrayList();
			        //FileParser.parseFile(fitnessFiles[0].getAbsolutePath(), front);
			        
			        // Leere Dateien ueberspringen
			        for (int i=0; (front.size() == 0) && (i<fitnessFiles.length); i++ ) {
			            FileParser.parseFile(fitnessFiles[i].getAbsolutePath(), front);
			        }
			        // Sicherheitsabfrage, falls alle Dateien leer sind -> Abbruch
			        if (front.size() == 0) {
			            System.err.println("\nFehler: Alle Dateien sind leer. Verarbeitung wird abgebrochen !!!\n");
						System.exit(1);	// Direkter Abbruch des Programms
			        }
			        
			        // Anzahl der Kriterien holen
			        double[] maximas = new double[((List)front.get(0)).size()];
			        
			        // Die Maximalwerte werden bestimmt
			        for (int ii = 0; ii < fitnessFiles.length; ii++) {
			            front = new ArrayList();
			            FileParser.parseFile(fitnessFiles[ii].getAbsolutePath(), front);
			            // print(fitnessFiles[ii].getAbsolutePath());
			            for (int jj = 0; jj < front.size(); jj++) {
			                ArrayList point = (ArrayList) front.get(jj);
			                for (int kk = 0; kk < point.size(); kk++)
			                    if (ListOperation.getDouble(point, kk) > maximas[kk])
			                        maximas[kk] = ListOperation.getDouble(point, kk);
			            }
			        }
			        // Referenzpunkt wird erstellt . Dabei wird zu jedem maximalwert
			        // 1 addiert .
			        for (int i = 0; i < maximas.length; i++) {
			            refPoint.add(new Double(maximas[i] + 1));
			        }
			    }
			    
				print("Referenzpunkt : " + refPoint);
				if (metric instanceof NewSMetric) {
					NewSMetric newsmetric = (NewSMetric) metric;
					newsmetric.setRefPoint(refPoint);
				}
				if (metric instanceof SMetricFleischer) {
					SMetricFleischer smetricFleischer = (SMetricFleischer) metric;
					smetricFleischer.setReferencePoint(refPoint);
				}
			}
		} catch (Exception e) {
			print("Problem mit Datei-I/O");
			e.printStackTrace();
			System.exit(1); // Direkter Abbruch des Programms
			
		}
		
	}


	/**
	 * Extrahiert die Kommandozeilen-Parameter
	 *  
	 */
	private void extractParameters(String[] args){
		try {
			
			/* Infomeldung geben, wenn keine Kommandozeilenparameter angegeben */ 
		    
		    if (args.length ==0) {
			    print(toString());
			    System.exit(1);	// Direkter Abbruch des Programms
			}
		    
		    
		    /* Plotter initialisieren */
		    
		    PropertiesPG input = (PropertiesPG)ArgsInputBeautifier.beautify(args);
			
		    // erste Parameter einlesen
			outputFile = input.getStringProperty("o", "PercentageData", false);
			fileNameFw = outputFile +".plt";
			fileNameFwPng = outputFile +"_png.plt";
			fileNameData = outputFile +".dat";
			fileNameDump = outputFile +".dump";
			fileNamePng = outputFile + ".png";
			
			// Dump-Datei initialisieren (notwendigerweise hier, weil Output-Name notwendig)
		    try {
	            dumpWriter = new FileWriter(fileNameDump, false);
	            dumpWriter.write("Startparameter: \n");
	            StringBuffer dumpBuffer = new StringBuffer(args.length);
	            for (int i=0; i<args.length; i++) {
	                dumpBuffer.append(args[i] + " ");
	            }
	            dumpWriter.write(dumpBuffer.toString()+"\n\n");
	        } catch (IOException e) {
	            e.printStackTrace();
	        }
	        
	        // weiter Parameter einlesen
			metricName = input.getStringProperty("m", "", true);
			/*
			if(metricName.equals("EuklidMetric")){
				testfktName = input.getStringProperty("t", "", true);
				if(testfktName.substring(0,3).compareToIgnoreCase("zdt")==0)
					testfktName = new String("zdt"+testfktName.substring(3));
			}
			*/
			
			int numberOfParameters = getNumberOfParameters(input, "r");
			print("\nAnzahl Parameter r  : " + numberOfParameters);
			if (numberOfParameters == 0) {
			    // Kein Referenzpunkt angegeben
			    refPoint = null;
			} else {
			    // Referenzpunkt einlesen
			    refPoint = new ArrayList();
			    for (int i = 0 ; i<numberOfParameters ; i++){
					refPoint.add(new Double(input.getDoubleProperty("r" + (i + 1), 0, true)));
				}
			}			
			
			//if (refPoint != -1) print("Referenzpunkt fr SMetriken : " + refPoint);
			/*
			print("\nAnzahl Parameter p : "+numberOfParameters);
			if (numberOfParameters == 0) {
			    print("!!! Keine zu untersuchenden Parameter angegeben! Bestimme NUR Referenzpunkt und maximale Metrikwerte !!!");
			    for (int i=1; i<=6; i++) {
			        try {
			            System.out.print(".");
			            Thread.sleep(500);
			        } catch(InterruptedException e) {}
			    }				
			    print("\n");
			}			
			pName = new String[numberOfParameters+1];
			for (int i = 1 ; i< pName.length ; i++){
				pName[i] = input.getStringProperty("p"+i, "", true);
			}
			*/
			numberOfParameters = getNumberOfParameters(input, "p");
			pName = new String[numberOfParameters+1];
			pName[1] = input.getStringProperty("p1", "", true);
			
			
			/* Wenn Parameter "boundary" gesetzt wurde, so wird dieser verwendet.
			 * Andernfalls wird nach Parametern "percentage" und "bestMetricValue" gesucht.
			 */
			if (input.containsKey("boundary")) {
				metricBoundary = input.getDoubleProperty("boundary", -1, true);
			} else {
			    metricBoundary = -1;
			    if (input.containsKey("percentage")) {
			        percentageOfBestMetricValue = input.getIntProperty("percentage", 90, true);
			    } else {
			        percentageOfBestMetricValue = 90;
			        print("Hinweis: Kein Parameter \"-percentage\" oder \"-boundary\" angegeben. Verwende percentage="+percentageOfBestMetricValue+".");
			    }
			    if (input.containsKey("bestMetricValue")) {
			        bestMetricValue = input.getDoubleProperty("bestMetricValue", 0, true);
			    } else {
			        bestMetricValue = 0;
			        print("Hinweis: Kein Parameter \"-bestMetricValue\" angegeben. Groesster Metrikwert wird automatisch berechnet.");
			    }
			}
			
			dir = new File(input.getStringProperty("d", "", true));
			
			if (!dir.exists()) {
				System.err.println("\nVerzeichnis nicht vorhanden!\n\n");
				System.exit(1);
			}
			
		}
		catch (MalInputFormatException e) {
			print("Fehler in den Parametern");
			e.printStackTrace();
			System.exit(1);	// Direkter Abbruch des Programms
		}
		
	}
	
		
	/**
	 * gibt die Anzahl der mit "key" gewaehlten Kommandozeilenparameter (z.B. p1,p2...) zurueck, 
	 * die beim Aufruf des MeanMetricPlotters uebergeben werden
	 * 
	 * @param input Properties Properties-Objekt mit Kommandozeilenparametern
	 * @param key String Prefix des zu bearbeitenden Kommandozeilenparameters (z.B. p oder r)
	 * @return int Anzahl der gewuenschten-Parameter
	 */
	private int getNumberOfParameters(Properties input, String key) {
	    boolean parameterVorhanden = true;
	    int parameterNumber = 0;
	    while (parameterVorhanden) {
	        if (input.containsKey(key + (parameterNumber + 1)))
	            parameterNumber++;
	        else
	            parameterVorhanden = false;
	    }
	    return parameterNumber;
	}
	
	
	/**
	 * Erstellt die FileWriter-Objekte und schreibt die Gnuplot-Befehle in die Ausgabedateien
	 */
	private void writeHeader() {
		try {
			// Schreibe den Header fuer die Gnuplot Datei
		    
		    	// String dataWriterName = outputFile.substring(0, outputFile.length() - 4) + ".dat";
			
				fwPng = new FileWriter(fileNameFwPng, false);
				fw = new FileWriter(fileNameFw, false);
				dataWriter = new FileWriter(fileNameData, false);
				
				fw.write("set title '" + metricName + " " + dir + "'\n");
				fw.write("set title '" + metricName + " " + dir + "'\n");
				fwPng.write("set title '" + metricName + " " + dir + "'\n");
				
				fw.write("set xlabel \"" + pName[1] + "\"\n");
				fwPng.write("set xlabel \"" + pName[1] + "\"\n");
				if (this.metricBoundary == -1) {
				    fw.write("set ylabel \"" + "Anzahl Laeufe mit mind. " + percentageOfBestMetricValue + "% des besten Metrkiwerts in Prozent" + "\"\n");
				    fwPng.write("set ylabel \"" + "Anzahl Laeufe mit mind. " + percentageOfBestMetricValue + "% des besten Metrkiwerts in Prozent" + "\"\n");
				} else {
					fw.write("set ylabel \"" + "Anzahl Laeufe mit mind. " + Double.toString(metricBoundary) + " als Metrkiwert" + "\"\n");
					fwPng.write("set ylabel \"" + "Anzahl Laeufe mit mind. " + Double.toString(metricBoundary) + " als Metrkiwert" + "\"\n");
				}
				
				//String nameWithNewSuffix = outputFile.replaceAll(".plt", ".png");
				fwPng.write("set term png\n");
				fwPng.write("set output \"" + fileNamePng + "\"\n");
				fw.write("plot [][0:110] '" + fileNameData + "' \n");
				
				//	 Falls man ein explizites Intervall zum Plotten festlegen
				// will:
				//	 Intervall festlegen fw.write("splot [][][1:10]'-'\n");
				
				fwPng.write("plot [][0:110] '" + fileNameData + "' \n");

				/*
                for (int i = 1; i < pName.length; i++) {
                    dataWriter.write(pName[i] + " ");
                }
				dataWriter.write("Y");
				dataWriter.write("\n");
				*/	
				
					
			
		} catch (Exception e) {
			e.printStackTrace();
		}
		
	}

	
	private void print(String output) {
	    System.out.println(output);
	    try {
	        dumpWriter.write(output + "\n");
	    } catch (Exception e) {
	        System.out.println("\n\n!!! Fehler beim Schreiben in die dump-Datei !!!:\n");
	        e.printStackTrace();
	    }
	}


	/**
	 * Kommandozeilenparameter des Designs:
	 * <ul>
	 * 	<li>-m Metrikenname</li>
	 * 	<li>-d Verzeichnisname</li>
	 *  	<li>-o Dateiname fuer die Ausgabe (optional)</li>
	 * 	<li>-p1 Parameter 1,  also der erste der beiden variierten Parameter
	 * </ul>
	 * <p>Beispiel: -m NewSMetric -d MopsoOne_Deb_1099909856036 -p1 quantityParticle </p>
	 */
	public static void main(String[] args) {
		PercentageMetricPlotter mmp = new PercentageMetricPlotter(args);
		
	}
	
}
