/*
 * Copyright (C) 2005  PG447 - University Dortmund
 * 
 * This file is part of NObELTJE.
 * 
 * NObELTJE is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as published by
 * the Free Software Foundation.
 * 
 * NObELTJE is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with NObELTJE; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

 package org.nobeltje.analysis;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Properties;

import org.nobeltje.tools.io.ArgsInputBeautifier;
import org.nobeltje.tools.io.PropertiesPG;
import org.nobeltje.exceptions.MalInputFormatException;

/**
 * @author $Author: noot $
 * @version $Revision: 726 $
 * $Date: 2005-02-02 00:50:20 +0100 (Mi, 02 Feb 2005) $
 */
public class RScatterplot {

	private File script = new File("script.r");
	private String inputData;
	private String outputFilePrefix;

	public RScatterplot(String[] args) {
		// Lese Parameter ein
		try {
			Properties input = ArgsInputBeautifier.beautify(args);

			// Daten
			inputData = PropertiesPG.getStringProperty(input, "d", "", true);

			// Prfix fr Ausgabedatei(en)
			outputFilePrefix = PropertiesPG.getStringProperty(input, "o", "", true);
		}
		catch (MalInputFormatException mife) {
			mife.printStackTrace();
			System.exit(1);
		}

		// Schreibe Script fr R
		try {
			FileWriter fw = new FileWriter(script.getAbsolutePath());

			fw.write("options(digits=3)\n");
			// Lese Daten ein
			fw.write("data <- read.table(\""+inputData+"\")\n");
			// ffne Device fr die Grafikausgabe
			fw.write("png(\""+outputFilePrefix+".png\")\n");
			// Zeichne Scatterplot
			fw.write("plot(data, main=\"Scatterplot\", col=\"blue\", pch=20)\n");
			// Schliee Device
			fw.write("dev.off()\n");
			// Schreibe Daten der Korrelations-Analyse in eine Textdatei
			fw.write("write.table(cor(data), \""+ outputFilePrefix +".txt\")\n");

			fw.write("readline(\"Drcke <Enter> um fortzufahren\")");

			fw.close();

			System.out.println("Aufruf von R:");
			System.out.println("\tR <"+script.getAbsolutePath()+" --vanilla --slave\n");
//			Process analysisProcess = Runtime.getRuntime().exec("R <"+script.getAbsolutePath()+" --vanilla --slave");
			System.out.println("\tGrafikausgabe --> "+outputFilePrefix+".png");
			System.out.println("\tTextausgabe --> "+outputFilePrefix+".txt");
			System.out.println("Beenden von R");
		}
		catch (IOException ioe) {
			ioe.printStackTrace();
			System.exit(1);
		}
	}

	/**
	 * Kommandozeilenparameter
	 * <ul>
	 *   <li><b>-d</b> Name des Files, der die Eingabedaten enthlt</li>
	 *   <li><b>-o</b> Namensprfix fr die Ausgabedatei(en)</li>
	 * </ul>
	 * <p>Beispiel: -d pg447/tabelle.dat -o meineAnalyse
	 * </p>
	 */
	public static void main(String[] args) {
		RScatterplot sp = new RScatterplot(args);
	}
}
