/*
 * Copyright (C) 2005  PG447 - University Dortmund
 * 
 * This file is part of NObELTJE.
 * 
 * NObELTJE is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as published by
 * the Free Software Foundation.
 * 
 * NObELTJE is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with NObELTJE; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

 /*
 * Created on Nov 8, 2004
 * Ralf Kosse
 *
 */
package org.nobeltje.analysis;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.List;
import java.util.Properties;

import org.nobeltje.metrics.Metric;
import org.nobeltje.metrics.NewSMetric;
import org.nobeltje.metrics.SMetricFleischer;
import org.nobeltje.tools.io.ArgsInputBeautifier;
import org.nobeltje.tools.io.FileParser;
import org.nobeltje.tools.io.ListOperation;
import org.nobeltje.tools.io.PropertiesPG;
import org.nobeltje.exceptions.MalInputFormatException;

/**
 * @author $Author: noot $
 * @version $Revision: 726 $
 * $Date: 2005-02-02 00:50:20 +0100 (Mi, 02 Feb 2005) $
 */
public class SurfacePlotter
{
	/**
	 * <p>ist ein String, der den Namen der Ausgabedatei enth?lt, in die die
	 * einzelnen Durchl?ufe des Designs mit dem jeweils zugeh?rigen Wert aus der Metrik geschrieben
	 * werden.</p>
	 */
	private String outputFile;
	/**
	 * <p>enth?lt den Namen der zu benutzenden Metrik. Dieser wird vom SurfacePlotter automatisch um den 
	 * passenden Paketnamen .metrics erg?nzt.</p>
	 */
	private String metricName;
	/**
	 * <p>in diesem Verzeichnis wird nach Dateien mit der Endung .res gesucht, die dann vom MetricsPlotter
	 * ausgewertet werden.</p>
	 */
	private File dir;
	
	/**
	 * die Namen der beiden Parameter, die variiert wurden. Diese beiden Parameter bilden 2 der Koordinaten-Achsen im 
	 * SurfacePlot. Die 3. Achse ist der Metrik-Wert. 
	 */
	private String p1, p2;
	
	/**
	 * Angabe des Referenzpunktes fuer die SMetrik
	 */
	private List refPoint;
	
	/**
	 * Der SurfacePlotter dient zur Erstellung von Surface-Plots mit GnuPlot, die eine Oberflaeche (=Metrik-Werte)
	 * in Abhaengigkeit von 2 Parametern darstellen. Ein Algo wird also laufengelassen, dabei werden 2 seiner
	 * Parameter in bestimmten Bereichen variiert. Der jeweilige Metrik-Wert, der beim Algo-Lauf mit jeweils diesen 
	 * beiden Parametern entsteht, bildet einen Punkt auf der Oberflaeche des SurfacePlots.
	 * 
	 * @param args Enthaelt die Kommandozeilenparameter
	 * @see tools.io.ArgsInputBeautifier
	 */

	// String, um Dateinamen zu speichern & zu behandeln
	private String filename;
	
	/**
	 * Die Dateien im Verzeichnis werden nach diesem Wert sortiert.
	 */
//	private String sort ="";
	
	
	public SurfacePlotter(String []args)
	{
		// Kommandozeilen-Parameter extrahieren
		extractParameters(args);
		
		// Zu plottende Daten in Datei schreiben & Gnuplot damit aufrufen 
		plotSurface();
	}




	/*
	 * Ruft Gnuplot auf, setzt die Datei mit den zu plottenden Punkten zusammen, etc.
	 * 
	 */
	private void plotSurface(){

		// Zaehlervariable
		int j, k;
		// Die Werte der beiden zu plottenden Parameter p1 und p2
		String p1Value, p2Value;


		// Hole aus dem angegebenen Verzeichnis alle Dateinamen mit der
		// Endung die im Fitness Filter festgelegt ist
		File fitnessFiles[] = dir.listFiles(new FitnessFilter());
		
		/*if (sort.compareTo("")!=0){
			StringTokenizer st = new StringTokenizer(sort,",");
			while(st.hasMoreElements()){
				fitnessFiles = SortFile.sortFileList(fitnessFiles,st.nextToken());
			}
		} else {
			Arrays.sort(fitnessFiles);
		}*/
		
		System.out.println("Anzahl der Dateien : "+fitnessFiles.length);
				
		try {
			// Schreibe den Header fuer die Gnuplot Datei
			FileWriter fwPng = new FileWriter("png_"+outputFile, false);
			FileWriter fw = new FileWriter(""+outputFile, false);
			fw.write("set title '" + metricName + " " + dir + "'\n");
			fwPng.write("set title '" + metricName + " " + dir + "'\n");

			fw.write("set xlabel \""+p1+"\"\n");
			fwPng.write("set xlabel \""+p1+"\"\n");
			fw.write("set ylabel \""+p2+"\"\n");
			fwPng.write("set ylabel \""+p2+"\"\n");

			fw.write("set data style lines\n");
			fwPng.write("set data style lines\n");

			fw.write("set contour both\n");
			fwPng.write("set contour both\n");

			fw.write("set surface\n");
			fwPng.write("set surface\n");

			fw.write("set dgrid3d 40,51,3\n");
			fwPng.write("set dgrid3d 40,51,3\n");

			fw.write("show contour\n");
			fwPng.write("show contour\n");


			String nameWithNewSuffix = outputFile.replaceAll(".plt", ".png");
			fwPng.write("set term png\n");
			fwPng.write("set output \"" + nameWithNewSuffix + "\"\n" );
			fw.write("splot '-'\n");


// Falls man ein explizites Intervall zum Plotten festlegen will:
// Intervall festlegen				fw.write("splot [][][1:10]'-'\n");



			fwPng.write("splot '-'\n");

			/**
			 * Um einen Splot mit Gnuplot zu erstellen, muss die Eingabedatei folgendermassen aussehen:
			 *  
			 *           # The valley of the Gnu.
						  0 0 10
						  0 1 10
						  0 2 10
			    
						  1 0 10
						  1 1 5
						  1 2 10
			    
						  2 0 10
						  2 1 1
						  2 2 10

			 *	 In unserem Falle hier waeren die ersten beiden Werte einer Zeile die p1 und p2-Werte; der dritte Wert ist der der Metrik. 
			 * 
			 */


			// Binde die Metrikenklasse anhand des Namens an
			Metric metric = (Metric)Class.forName("metrics." + metricName).newInstance();
			

			// Automatische Referenzpunktbestimmung fuer die SMetric:  			
			this.computeReferencePoint(metric, fitnessFiles);



			// Rufe die Metrik fuer jede Datei auf
			Hashtable metricValuesHash = new Hashtable();
			for (int i = 0; i < fitnessFiles.length; i++) {
				// Lese die Fitness-Werte aus der jew. Datei ein
				List fitnessValues = new ArrayList();
				FileParser.parseFile(fitnessFiles[i].getAbsolutePath(), fitnessValues);
				
	
	
				p1Value = this.extractParameterValues(p1, fitnessFiles, i);
				p2Value = this.extractParameterValues(p2, fitnessFiles, i);
	
	
				
				//Metric aufrufen
				double metricValue = metric.computeMetric(fitnessValues);
				
				// die ergebnisse der metriken werden in Listen geschrieben
				// und die Listen werden in einer Hashtable gespeichert
				//in einer Liste befinden sich ergebnisse von Laeufen mit selben Parametern und anderem RandomSeed
				ArrayList dieListe = (ArrayList)metricValuesHash.get(p1Value+p2Value);

				if(dieListe == null){
					dieListe = new ArrayList();
					dieListe.add(p1Value);
					dieListe.add(p2Value);
					metricValuesHash.put(p1Value+p2Value,dieListe );

				}
								
				dieListe.add(new Double(metricValue));
				
				
			}
			// die durchschnittswerte  der Laeufe mit selben Parameterwerten werden berechnet 
			// und in die dateien geschrieben  
			Enumeration enu =metricValuesHash.keys();
			System.out.println("Anzehl eintraege in hashtabelle :  "+metricValuesHash.size());
			while (enu.hasMoreElements()){
				ArrayList metricValueList = (ArrayList)metricValuesHash.get(enu.nextElement());
				double durchschnitt =0;
				for (int kk =2 ;kk< metricValueList.size(); kk++){
					// Durchschnittswert berechnen
					durchschnitt +=((Double)metricValueList.get(kk)).doubleValue();
				}

				durchschnitt /=(metricValueList.size()-2);

				// Die Parameterwerte in die Ausgebadatei schreiben
				fw.write(metricValueList.get(0) + " ");
				 fwPng.write(metricValueList.get(0) + " ");
				 fw.write(metricValueList.get(1) + " ");
				 fwPng.write(metricValueList.get(1) + " ");

				 // Schreibe das Ergebnis der Metrik in eine Ausgabedatei
				 
				 fw.write(durchschnitt + "\n");
				 fwPng.write(durchschnitt + "\n");    
				  
			}
			
			fw.write("e\n");
			fwPng.write("e\n");
			fw.close();
			fwPng.close();



			// Gnuplot aus Java aufrufen
			//visualizationProcess = Runtime.getRuntime().exec("gnuplot -persist");
			try {
				System.out.println("gnuplot -persist "+ "png_"+outputFile);
				Process visualizationProcess = Runtime.getRuntime().exec("gnuplot -persist "+ "png_"+outputFile);
				System.out.println("ende von gnuplot");
				//PrintStream gnuStream = new PrintStream(visualizationProcess.getOutputStream());
			}
			catch(Exception e){
				e.printStackTrace();
			}
			try {
				System.out.println("gnuplot -persist "+ outputFile);
				
				Process visualizationProcess = Runtime.getRuntime().exec("gnuplot -persist "+ outputFile);
				System.out.println("ende von gnuplot");
				//PrintStream gnuStream = new PrintStream(visualizationProcess.getOutputStream());
			}
			catch(Exception e){
				e.printStackTrace();
			}
		}

		// Fehler bei nicht gefundener Algo-Klasse ausgeben und abbrechen
		catch (ClassNotFoundException cnfe){
			System.err.println("\nFEHLER: Angegebene Algo-Klasse konnte nicht gefunden werden.\n");
			cnfe.printStackTrace();
			System.exit(1);	// Direkter Abbruch des Programms
		}
		// Fehler bei unzulaessiger Algo-Klasse ausgeben und abbrechen
		catch (ClassCastException cce){
			System.err.println("\nFEHLER: Angegebene Algo-Klasse ist nicht zulaessig (Cast-Fehler).\n");
			cce.printStackTrace();
			System.exit(1);	// Direkter Abbruch des Programms
		} 
		catch (NoSuchMethodException nsme) {
			System.err.println("\nFEHLER: Metrik wurde falsch angegeben.\n");
			nsme.printStackTrace();
			System.exit(1);	// Direkter Abbruch des Programms
		}
		catch (FileNotFoundException fnfe) {
			System.err.println("\nFEHLER: Metrik wurde falsch angegeben.\n");
			fnfe.printStackTrace();
			System.exit(1);	// Direkter Abbruch des Programms
		}
		catch (IOException ioe) {
			System.err.println("\nFEHLER: Metrik wurde falsch angegeben.\n");
			ioe.printStackTrace();
			System.exit(1);	// Direkter Abbruch des Programms
		}
		catch (Exception e) {
			// Entweder lief das Parsen schief, oder
			// die Metrik unterstuetzt den Aufruf nicht
			e.printStackTrace();
			System.exit(1);	// Direkter Abbruch des Programms
		}
		
		
	}


	/*
	 * Extrahiert die Werte der beiden zu betrachtenden Parameter
	 * uebergeben werden der Name des auszuwertenden Parameters und die FitnessFiles-Liste
	 * 
	 * Zurueckgegeben wird ein String, in dem der Parameterwert steht
	 */
	 private String extractParameterValues(String paramName, File fitnessFiles[], int i){

		// String, um Dateinamen zu speichern & zu behandeln
		String filename;
				
		// Die zugehoerigen StringBuffer
		StringBuffer parambuf = new StringBuffer();
				
		filename = fitnessFiles[i].getName();

		// Position der beiden Parameter im Dateinamen lokalisieren
		int j = filename.indexOf(paramName);						
					
		// um die Laenge des Parameter-Namens und um das "="-Zeichen verschieben	
		j = j + paramName.length()+1;
				
		// Den Parameter-Wert extrahieren (beginnt hinter dem '=' hinter dem Parameternamen und endet mit einem '_' oder einem ".best")
		for (int m = 0; (filename.charAt(j+m)!='_')&&(!filename.substring(j+m ,  j+m+5).equals(".best"));m++){
			parambuf.append(filename.charAt(j+m));
		}

		// Strings kopieren
		return parambuf.toString();

	 }



	/*
	 * Berechnet & setzt den Referenzpunkt fuer die NewSMetric 
	 */
	 private void computeReferencePoint(Metric metric, File fitnessFiles[]){

		try {
			if (metric instanceof NewSMetric || metric instanceof  SMetricFleischer ){
			    if (refPoint == null) {
			        refPoint = new ArrayList();System.out.println("Referenzpunkt fuer S-Metrik wird bestimmt");
				   // die Dimension des Raumes wird bestimmt
				   List front = new ArrayList();
				   FileParser.parseFile(fitnessFiles[0].getAbsolutePath(), front);
				   
				   double [] maximas = new double [((List)front.get(0)).size()];
				   
				   // Die Maximalwerte werden bestimmt
				   for (int ii = 0; ii < fitnessFiles.length; ii++) {
						   front = new ArrayList();
							FileParser.parseFile(fitnessFiles[ii].getAbsolutePath(), front);
	//						System.out.println(fitnessFiles[ii].getAbsolutePath());
						   for (int jj = 0 ;jj < front.size() ; jj++){
							   ArrayList point = (ArrayList)front.get(jj);
							   for (int kk = 0 ; kk < point.size() ; kk++)
							   if(ListOperation.getDouble(point,kk) > maximas[kk] )
								   maximas[kk] = ListOperation.getDouble(point,kk) ;
								       
							   
									
						   }
											
				   }
				   // Referenzpunkt wird erstellt . Dabei wird zu jedem maximalwert 1 addiert .
				   for (int i =0 ; i< maximas.length ; i ++)
				           refPoint.add(new Double(maximas[i]+1));
			    }
			    
			    System.out.println(" refPoint  :  "+refPoint);
			    if(metric instanceof NewSMetric ){
			    	   NewSMetric newsmetric = (NewSMetric)metric;
			    	   newsmetric.setRefPoint(refPoint);
			    }
			    if( metric instanceof  SMetricFleischer){
			    	    SMetricFleischer smetricFleischer = (SMetricFleischer)metric ;
			    	    smetricFleischer.setReferencePoint(refPoint);
			    }
			}
		} catch (Exception e) {
			System.out.println("Problem mit Datei-I/O");
			e.printStackTrace();
			System.exit(1);	// Direkter Abbruch des Programms

		}
	 	
	 	
	 }


	/*
	 * Extrahiert die Kommandozeilen-Parameter 
	 * 
	 */
	private void extractParameters(String []args){
		try {

			PropertiesPG input = (PropertiesPG)ArgsInputBeautifier.beautify(args);
			outputFile = ArgsInputBeautifier.getStringProperty(input, "o", "SurfaceData.plt", false);
			metricName = ArgsInputBeautifier.getStringProperty(input, "m", "", true);
			p1 = ArgsInputBeautifier.getStringProperty(input, "p1", "", true);
			p2 = ArgsInputBeautifier.getStringProperty(input, "p2", "", true);
			dir = new File(ArgsInputBeautifier.getStringProperty(input, "d", "", true));
//			sort = ArgsInputBeautifier.getStringProperty(input, "sort", "", true);
			int numberOfParameters = getNumberOfParameters(input, "r");
			System.out.println("\nAnzahl Parameter r : " + numberOfParameters);
			if (numberOfParameters == 0) {
			    // Kein Referenzpunkt angegeben
			    refPoint = null;
			} else {
			    // Referenzpunkt einlesen
			    refPoint = new ArrayList();
			    for (int i = 0 ; i<numberOfParameters ; i++){
					refPoint.add(new Double(input.getDoubleProperty("r" + (i + 1), 0, true)));
				}
			}				
			if (!dir.exists()) {
				System.err.println("Verzeichnis nicht vorhanden");
				System.exit(1);
			}
	
		} 
		catch (MalInputFormatException e) {
			System.out.println("Fehler in den Parametern");
			e.printStackTrace();
			System.exit(1);	// Direkter Abbruch des Programms
		}
	
	}


	/**
	 * gibt die Anzahl der mit "key" gewaehlten Kommandozeilenparameter (z.B. p1,p2...) zurueck, 
	 * die beim Aufruf des MeanMetricPlotters uebergeben werden
	 * 
	 * @param input Properties Properties-Objekt mit Kommandozeilenparametern
	 * @param key String Prefix des zu bearbeitenden Kommandozeilenparameters (z.B. p oder r)
	 * @return int Anzahl der gewuenschten-Parameter
	 */
	private int getNumberOfParameters(Properties input, String key) {
	    boolean parameterVorhanden = true;
	    int parameterNumber = 0;
	    while (parameterVorhanden) {
	        if (input.containsKey(key + (parameterNumber + 1)))
	            parameterNumber++;
	        else
	            parameterVorhanden = false;
	    }
	    return parameterNumber;
	}
	



	/**
	 * Kommandozeilenparameter des Designs:
	 * <ul>
	 * 	<li>-m Metrikenname</li>
	 * 	<li>-d Verzeichnisname</li>
	 *  	<li>-o Dateiname f?r die Ausgabe (optional)</li>
	 * 	<li>-p1 Parameter 1,  also der erste der beiden variierten Parameter
	 * 	<li>-p21 Parameter 2,  also der erste der beiden variierten Parameter
	 * </ul>
	 * <p>Beispiel: -m SMetric -d MopsoOne_Deb_1099909856036 -p1 quantityParticle -p2 g</p>
	 */
	public static void main(String[] args) {
		SurfacePlotter sp = new SurfacePlotter(args); 		
	}
}
