/*
 * Copyright (C) 2005  PG447 - University Dortmund
 * 
 * This file is part of NObELTJE.
 * 
 * NObELTJE is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as published by
 * the Free Software Foundation.
 * 
 * NObELTJE is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with NObELTJE; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

 package org.nobeltje.design;

import java.io.File;
import java.io.FileWriter;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Properties;

import org.nobeltje.testfkt.TestFunctions;

/**
 * Designklasse f?r den mehrfachen Aufruf der Algorithmen.
 * <p>
 * Die Designklasse kann einen Algorithmus mit einer bestimmte (Test-)Funktion aufrufen 
 * und generiert die dazu die benoetigten Parameter. Die Auswahl des Algorithmuses und 
 * der (Test-)funktion geschieht ueber Kommandozeilenparameter. F?r die Parameter des 
 * Algorithmenaufrufs m?ssen auf der Kommandozeile Parameterbereiche angegeben werden,
 * aus denen dann die konkreten Parameter der Aufrufe erzeugt werden. Die Syntax der 
 * Kommandozeilenparameter wird bei der main-Methode{@link #main(String[] args)}
 * beschrieben.
 * </p><p>
 * Die Ausgabe der Algorithmen erfolgt in ein Verzeichnis mit dem Namen 
 * "AlgoName_TestName_Zeitstempel". Die Dateien werden dann nach dem Schema 
 * "Parameter1=Wert1_Parameter2=Wert3_etc.best.res/in" gespeichert. Wobei "Parameter" fue?r
 * die Kommandozeilenparameter des Algorithmenaufrufs steht. 
 * </p>
 * @author $Author: noot $
 * @version $Revision: 727 $
 * $Date: 2005-02-08 09:14:18 +0100 (Di, 08 Feb 2005) $
 */
public class Design {
	java.util.Properties prop = new Properties();
	private String algoName = "";
	private String testName = "";	 // Exogener Parameter: Name der Testfkt-Klasse
	private boolean timeFlag = false;	// Exogener Parameter: Zeitinfos zu Algo schreiben?
	protected TestFunctions test;  // Objekt der Testfunktion
	private HashMap parameterMap = new HashMap();
	private ArrayList algorithmCallParameterSets = new ArrayList(); //algorithm = Algorithmen, calls = aufrufe, Sets = Mengen/Listen
	private File outputDirectory;
	private StringBuffer timeValues = new StringBuffer(); // Zum Speichern der bentigten Zeit pro Algo

	
	/**
	 * Die Methode wertet die Kommandozeilenparameter aus und f?hrt die
	 * die generierten Algo-Aufrufe aus.
	 * @param args @see #main(String[])
	 */
	public Design(String[] args) {
		
		try {
			//auswerten von args
			parse(args);
			//Intervall- & Listendefinitionen durch Zahlen ersetzen
			extractValues();
			
			//Ausgabeverzeichnis erzeugen
			Date now = new Date();

			// ALT (Unix):			outputDirectory = new File("./"+algoName+"_"+testName+"_"+now.toString());
			// Neu (Windows):
			outputDirectory = new File(algoName+"_"+testName+System.currentTimeMillis());


			outputDirectory.mkdir();
			
			ArrayList starter = new ArrayList();
			
			starter.add("-t");
			starter.add(testName);
			
			//Kommandozeilenparameter Arrays fuer die Aufrufe generieren
			getCombinations(parameterMap,starter);			
			//output();
			try {
				// Algoklasse anhand des Namens anbinden
				Class algoClass = Class.forName("org.nobeltje.algo."+algoName);
				Class[] mainArguments = new Class[1];
				mainArguments[0]=args.getClass(); 
				Method start = algoClass.getMethod("main",mainArguments);    						
				
				//Algo aufrufen
				for (Iterator callIter = algorithmCallParameterSets.iterator();callIter.hasNext(); ) {
					/* Kommandozeilenparameter fuer Algo holen */
					String [] call = (String[]) callIter.next();
					Object [] input = new Object [1];
					input[0]=call;
					
					/* Zeitmessung Start */
					long startTime = 0;
					if (timeFlag) {
						startTime = System.currentTimeMillis();	// Zeitmessung
					}
					
					/* Algo starten */
					start.invoke(null,input);
					
					/* Zeitmessung Ende*/
					if (timeFlag) {
						long endTime = System.currentTimeMillis();	// Zeitmessung
						StringBuffer callPrint = new StringBuffer(call.length);
						for (int i = 0; i< call.length; i++) {
							callPrint.append(call[i] + " ");
						}
						timeValues.append( call[call.length-1] + " : " + Long.toString(endTime-startTime) + "\n");
					}
				}
				/* Zeitmessung Ausgabe */
				if (timeFlag) {
					FileWriter fw = new FileWriter(outputDirectory.getAbsolutePath()+File.separator+"timecounter.millisec", false);
					fw.write(timeValues.toString());
					fw.close();
				}
			}
			
			/* Fehler bei nicht gefundener Algoklasse ausgeben und Abbrechen */
			catch (ClassNotFoundException e){
				System.out.println("\nFEHLER: Angegebene Algoklasse konnte nicht gefunden werden.\n\n");
				e.printStackTrace();
				System.exit(1);	// Direkter Abbruch des Programms
			}
			/* Fehler bei unzulaessiger Algoklasse ausgeben und Abbrechen */
			catch (ClassCastException e){
				System.out.println("\n FEHLER: Angegebene Algoklasse ist nicht zulaessig (Cast-Fehler). \n\n");
				e.printStackTrace();
				System.exit(1);	// Direkter Abbruch des Programms
			} 
			/* Noch nen Fehler.. */
			catch (IllegalAccessException e2) {
				e2.printStackTrace();
				System.exit(1);	// Direkter Abbruch des Programms
			}
			
		} catch (Exception e) {
			
			e.printStackTrace();
			System.exit(1);
		}
	}




	/**
	 * <p>Die Methode gibt die Kommandozeilenparameter aus, die in die
	 * dafuer vorgesehe Datenstruktur gespeichert wurden. Desweiteren
	 * werden die Kommandozeilenparamter aller vorgesehener Algo
	 * Aufrufe ausgegeben.</p>
	 * <p>Dient der Debug Ausgabe.</p>
	 */
	private void output() {
		for (Iterator keys = parameterMap.keySet().iterator();keys.hasNext(); ) {
			String key = (String)keys.next();
			System.out.print("Key: "+key+" Values: ");
		
			for (Iterator iter = ((ArrayList)parameterMap.get(key)).iterator();iter.hasNext();)
			{
				System.out.print(iter.next()+" ");
			}
			System.out.println("");
		}
		System.out.println("-------------------------------------------------------------------------------------------------");
		for (Iterator callIter = algorithmCallParameterSets.iterator();callIter.hasNext();) {
			String [] call = (String[])callIter.next();
			for (int i = 0; i<call.length; i++) {
				System.out.print(call[i]+" ");
			}
			System.out.println();
		}
	}


	/**
	 * Die Methode extrahiert Algo und (Test-)Funktionsname aus dem Stringarray
	 * und schreibt die anderen Kommandozeilenparameter in eine HashMap.
	 * @param args @see #main(String[])
	 * @throws Exception
	 */
	private void parse(String[] args) throws Exception{
		for (int i=0; i<args.length;i++) {
			ArrayList possibleValues = new ArrayList();
			String currentArg = args[i];
			if  ( currentArg.startsWith ("-") ) {
				if ( currentArg.startsWith ("--") ) {
					
					if ( currentArg.endsWith ( "a" ) ) {
						algoName = args[i+1];
						i++;
					}
					if (currentArg.endsWith("t")) {
						testName = args[i+1];
						// Pfandangaben zur Testfkt-Klasse zu Packetangaben aendern
						testName = testName.replace('/', '.');
						testName = testName.replace('\\', '.');
						i++;
					}
					if (currentArg.endsWith("time")) {
						timeFlag = true;
					} else {
						timeFlag = false;
					}
					
				}
				String paramName = currentArg;
				while ((i<args.length-1)&&(!args[i+1].startsWith("-"))){
					
					possibleValues.add(args[i+1]);
					i++;
					
				}
				parameterMap.put(currentArg,possibleValues);
			}
		}
		parameterMap.remove("--a");
		parameterMap.remove("--t");
		if (parameterMap.containsKey("--time")) {
			parameterMap.remove("--time");
		}
	}
	
	/** 
	 * <p>Ersetzt in der HashMap die Eingaben fuer Listen und Intervalle durch eine
	 * ArrayList welche alle durch diese Anweisung gegeben Werte enthaelt.</p>
	 * <p>Beispiel: "i 10 25 step 5" wird durch "10 15 20 25" ersetzt</p> 
	 * @throws Exception
	 */
	private void extractValues() throws Exception{
		for (Iterator keys = parameterMap.keySet().iterator();keys.hasNext(); ) {
			String key = (String) keys.next();
			ArrayList values = (ArrayList) parameterMap.get(key);
			ArrayList generatedValues = new ArrayList();
			
			if (!values.isEmpty()) {
				if ( ((String)values.get(0)).equals("i")) {
					
					if (values.size()==3) {
						for (double i = Double.parseDouble((String)values.get(1));i<= Double.parseDouble((String)values.get(2));i++) {
							generatedValues.add(i+"");
						}
					}
					if (values.size()==5) {
						if (values.get(3).equals("step")) {
							for (	double i = Double.parseDouble((String)values.get(1));
									i<= Double.parseDouble((String)values.get(2));
									i+=Double.parseDouble((String)values.get(4))) {
								generatedValues.add(i+"");
							}
						}
						if (values.get(3).equals("count")) {
							double step = (Double.parseDouble((String)values.get(2)) - Double.parseDouble((String)values.get(1)))/(Double.parseDouble((String)values.get(4))-1);
							for (double i = Double.parseDouble((String)values.get(1));i<= Double.parseDouble((String)values.get(2));i+=step) {
								generatedValues.add(i+"");
							}
						}
					}
				}
				if ( ((String)values.get(0)).equals("s")) {
					
					values.remove(0);
					generatedValues.addAll(values);
					
				}
			}
			parameterMap.put(key,generatedValues);
			//output();
		}
	}
	
	/**
	 * <p>Die Methode ist rekursiv und erzeugt aus den Parametern der HashMap die
	 * Parameter fuer die Algo Aufrufe.</p>
	 * <p>So lange es in der HashMap noch Werte gibt, nimmt die Methode das erste
	 * Schluessel/Wert Paar und entfernt diese aus der Liste. Der Wert ist eine 
	 * Arrayliste mit ggf. mehreren Eintraegen. Jeder dieser Eintraege wird mit dem
	 * Schluessel in ein neues (Kopie des bisherigen) String Array gespeichert und
	 * die Methode fuer die restlichen Werte neu aufgerufen.</p>
	 * <p>Wenn keine Werte mehr in der HashMap sind wird der Parameter fuer den Dateinamen
	 * gesetzt und die Kommandozeilenparameterkombination in der Liste der Aufrufe
	 * gespeichert.</p> 
	 *     
	 * @param possibleValues Die noch nicht bearbeiteten Parameter.
	 * @param combination Kommandozeilenparameter fuer einen Algo-Aufruf, der hier aufgebaut wird.
	 */
	private void getCombinations (HashMap possibleValues, ArrayList combination) {
		
		if (!possibleValues.isEmpty())
		{
			Iterator keys = possibleValues.keySet().iterator();
			String key = (String)keys.next();
			HashMap smallerPossibleValuesMap = (HashMap) possibleValues.clone(); //Funktioniert clone hier??? scheint so!
			smallerPossibleValuesMap.remove(key);
			for (Iterator valueIter= ((ArrayList)possibleValues.get(key)).iterator();valueIter.hasNext();) {
				String value = (String)valueIter.next();
				ArrayList largerCombination =(ArrayList) combination.clone(); //Funktioniert clone hier??? scheint so!
				largerCombination.add(key);
				largerCombination.add(value);
				getCombinations(smallerPossibleValuesMap,largerCombination);
			}
		}
		else {
			StringBuffer praefix = new StringBuffer();
			boolean first = true;
			for (Iterator combIter= combination.iterator();combIter.hasNext();) {
				if (first) {
					combIter.next();
					combIter.next();
				}
				String value = (String)combIter.next();
				if (value.startsWith("-")) {
					//value=value.substring(1);
					value=value.replace('-','_');
					value+="=";
				}
				if (first) {
					value=value.substring(1);
					first=false;
				}
				// Moegliche "/" und "\", die in Parametern vorkommen, duerfen nicht in den Dateinamen gelangen.
				praefix=new StringBuffer(praefix.toString().replace('/','_').replace('\\','_'));
				praefix.append(value);
			}
			//System.out.println("###########Praefix: "+praefix);
			combination.add("-o");
			//System.out.println("Path: "+outputDirectory.toString());
			
			combination.add(outputDirectory.getAbsolutePath()+File.separator+praefix.toString());
			String[] call = new String[combination.size()];
			combination.toArray(call);
			algorithmCallParameterSets.add(call);
		}
			
	}
	
	/**
	 * <p>Startet die Algoritmen mit Parametern, welche aus Parameterbereichen generiert werden.
	 * Die Parameter des Algorithmuses k?nnen entweder als Einzelwerte, als List/Menge oder 
	 * Intervall mit Schrittweite/Schrittanzahl angeben werden.</p>
	 * <ul>
	 * 	<li>Liste: Syntax: -Parametername s Wert1 Wert2 ...</li>
	 * 	<li>Intervall mit Schrittweite:  Syntax: -Parametername i Intervallgrenze1 Intervallgrenze2 step Schrittweite</li>
	 * 	<li>Intervall mit Schrittanzahl:  Syntax: -Parametername i Intervallgrenze1 Intervallgrenze2 count Anzahl</li>
	 * </ul>
	 * Kommandozeilenparameter des Designs:
	 * <ul>
	 * 	<li>--a Algorithmenname</li>
	 * 	<li>--t Testfunktionsname</li>
	 * 	<li>Kommandozeilenparameter des Algos wie beim direkten Aufruf des Algos. Zahlen k?nnen wie
	 * oben beschrieben als Liste oder Intervall angegeben werden.</li>
	 * </ul>
	 * Beispiel: --t org.nobeltje.algo.FonsecaF1 --a OnePlusOnePG -g s 100 -archive i 10 100 step 10 -deviation i 0.1 1 count 5
	 * 
	 * @param args siehe Beschreibungstext
	 */
	public static void main (String[] args) {
		Design d = new Design(args);
	}

}
