/*
 * Copyright (C) 2005  PG447 - University Dortmund
 * 
 * This file is part of NObELTJE.
 * 
 * NObELTJE is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as published by
 * the Free Software Foundation.
 * 
 * NObELTJE is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with NObELTJE; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

 /*
 * Created on 28.05.2004
 */
package org.nobeltje.metrics;

import java.util.ArrayList;
import java.util.List;

import org.nobeltje.tools.io.FileParser;
import org.nobeltje.exceptions.MalInputFormatException;

/**
 * Die Klasse EuklidMetric berechnet den euklidschen Abstand der Elemente einer Loesungsmenge eines <br>
 * Algorithmus zu einer Menge von Pareto-optimalen Elementen<br> 
 * 
 * @author $Author: noot $
 * @version $Revision: 726 $
 * $Date: 2005-02-02 00:50:20 +0100 (Mi, 02 Feb 2005) $
 */

public class EuklidMetric implements Metric
{
	private final int parameterCount = 2;	// Gibt an, wieviele Parameter die
											// Metrik als Eingabe erwartet

	/**
	 * Berechnet den euklidschen Abstand zwischen zwei n-dimensionalen Vektoren<br>
	 * 
	 * @param vector1
	 * @param vector2
	 * @return Distanz der Vektoren
	 * @throws Exception, falls die Dimension der Vektoren nicht uebereinstimmt
	 */
	public double euklidDistance(List vector1, List vector2) throws Exception
	{
		if (vector1.size() != vector2.size())
			throw new Exception("Die Dimension der Vektoren stimmt nicht ueberein!");

		double distance = 0.0;

		for (int i = 0; i < vector1.size(); i++)
			distance += java.lang.Math.pow(((Double)vector1.get(i)).doubleValue() - ((Double)vector2.get(i)).doubleValue(), 2.0);

		return java.lang.Math.sqrt(distance);
	}


	/**
	 * @throws Exception, da die Eukliedmetrik zwei Vektorlisten benoetigt 
	 */
	public double computeMetric(List list) throws Exception
	{
		throw new Exception("Diese Methode wird von der Euklid Metrik nicht unterstuetzt");
	}


	/**
	 * Berechnet den mittleren Abstand aller Zielvektoren zur Paretomenge
	 * 
	 * @param vectorList Die Menge der Zielvektoren
	 * @param paretoList Die Paretomenge
	 * @return Gemittelte Distanz der beiden Mengen
	 * @throws Exception
	 */
	public double computeMetric(List vectorList, List paretoList) throws Exception
	{
		double average = 0.0;
		double minDist, temp;

		for (int i = 0; i < vectorList.size(); i++) {
			minDist = Double.POSITIVE_INFINITY;
			// Vergleiche jeden Zielvektor mit jedem Individuum aus der Paretomenge
			// und merke fuer jeden Vektor den kuerzesten Abstand
			for (int j = 0; j < paretoList.size(); j++) {
				temp = euklidDistance((List)vectorList.get(i), (List)paretoList.get(j));
				if (minDist > temp)
					minDist = temp;
			}
			average += minDist;
		}
		// Berechne das Mittel aller kuerzesten Abstaende
		average /= vectorList.size();

		return average;
	}


	/**
	 * @return eine Beschreibung der EuklidMetrik
	 */

	public String toString()
	{
		String str;
		str =  "\n\nKlasse EuklidMetric\n";
		str += "=================\n";
		str += "\n";
		str += "Beschreibung:\n";
		str += "Implementation der Euklid - Metrik im mehrdimensionalen Raum," +
			"nach Ideen der PG447\n";
		str +="Eine Liste von Vektoren aus dem Zielbereich werden mit der Liste"+ 
					  " der zufaellig gewaehlten Vektoren aus der wahren Pareto-Front verglichen.\n";
		str +="Dazu werden die Abstaende zwischen den Zielvektoren und den jeweiligen"+ 
					  " naechsten Pareto-Loesungen berechnet.\n";
		str +="Diese kuerzesten Entfernungen werden dann gemittelt.\n";
		str +="Erhaltener Wert kann zum Vergleich zwischen den Algorithmen benutzt werden.\n";
		str +="Kommandozeilenparameter:\n";
		str +="-list1(ArrayList): Liste der Zielvektoren\n";
		str +="-list2(ArrayList): Liste der wahren Pareto-Loesungen\n";
		return str;
	}


	/**
	 * Gibt an, wieviele Parameter (Dateien) die Metrik erwartet
	 * @return Anzahl der Dateien
	 */
	public int getParameterCount()
	{
		return parameterCount;
	}

	/**
	 * Die Main Methode legt ein neues Objekt der Metrik an. Sie liest die Daten
	 * aus den Files, die als Parameter uebergen wurden, in ArrayLists ein.
	 * Schlielich werden die Daten durch den Aufruf von computeMetric() ausgewertet.<br>
	 * 
	 * @param args enthaelt die Dateinamen fuer die Daten
	 * @throws Exception
	 */
	public static void Main(String[] args) throws Exception
	{
		SMetric metric = new SMetric();

		if (args.length != metric.getParameterCount())
			throw new MalInputFormatException("Die Metrik erwartet " +
					metric.getParameterCount() + " Dateinamen als Parameter");

		List list1 = new ArrayList();
		List list2 = new ArrayList();
		FileParser.parseFile(args[0], list1);
		FileParser.parseFile(args[1], list2);

		double output = metric.computeMetric(list1, list2);

		System.out.println(output);
	}
}
