/*
 * Copyright (C) 2005  PG447 - University Dortmund
 * 
 * This file is part of NObELTJE.
 * 
 * NObELTJE is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as published by
 * the Free Software Foundation.
 * 
 * NObELTJE is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with NObELTJE; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

 /*
 * Created on 28.05.2004
 */
package org.nobeltje.metrics;

import java.util.List;

/**
 * Jede neue Metrik sollte dieses Interface implementieren.<br>
 * Die konkreten Metriken unterstuetzen eine der beiden Methoden computeMetric().
 * Die andere Methode wirft eine Exception, dass sie nicht von dieser Metrik
 * unterstuetzt wird.<br>
 * Mit toString() erhaelt man eine kurze Beschreibung, wie die Metrik funktioniert
 * und was sie berechnet.<br>
 * getParameterCount() gibt an, ob die Metrik 1 oder 2 Listen erwartet.<br>
 * 
 * @author $Author: noot $
 * @version $Revision: 726 $
 * $Date: 2005-02-02 00:50:20 +0100 (Mi, 02 Feb 2005) $
 */
public interface Metric
{	/**
	 * Wendet die Metrik auf die Daten aus der uebergebenen Liste an
	 * 
	 * @param list , enthaelt die zu bewertenden Daten
	 * @return Gibt das Ergebnis als "double" zurueck
	 * @throws Exception, falls die Methode von dieser Metrik nicht unterstuetzt
	 * wird
	 */
	public abstract double computeMetric(List list) throws Exception;

	/**
	 * Wendet die Metrik auf die Daten aus den uebergebenen Listen an
	 * 
	 * @param list1 , enthaelt die zu bewertenden Daten
	 * @param list2 , enthaelt die zu bewertenden Daten
	 * @return Gibt das Ergebnis als "double" zurueck
	 * @throws Exception, falls die Methode von dieser Metrik nicht unterstuetzt
	 * wird
	 */
	public abstract double computeMetric(List list1, List list2) throws Exception;

	/**
	 * @return Liefert eine kurze Beschreibung der Metrik
	 */
	public abstract String toString();

	/**
	 * Gibt die Anzahl der zu uebergebenden Listen zurueck.
	 * 
	 * @return Anzahl der zu uebergebenden Listen
	 */
	public int getParameterCount();
}
