/*
 * Copyright (C) 2005  PG447 - University Dortmund
 * 
 * This file is part of NObELTJE.
 * 
 * NObELTJE is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as published by
 * the Free Software Foundation.
 * 
 * NObELTJE is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with NObELTJE; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

 /*
 * Created on 04.06.2004
 */
package org.nobeltje.metrics;

import java.util.ArrayList;
import java.util.List;

import org.nobeltje.tools.io.FileParser;
import org.nobeltje.exceptions.MalInputFormatException;

/**
 * Die Klasse NewSMetric berechnet eine Flaeche, die durch einen Referenzpunkt und einer Menge von<br>
 * nichtdominierten Elementen einer Loesungsmenge eines Algorithmus eingegrenzt wird.<br> 
 * 
 * 
 * @author $Author: noot $
 * @version $Revision: 726 $
 * $Date: 2005-02-02 00:50:20 +0100 (Mi, 02 Feb 2005) $
 */
public class NewSMetric implements Metric
{
	private List refPoint = new ArrayList();
	private final int parameterCount = 1;	// Gibt an, wieviele Parameter die
											// Metrik als Eingabe erwartet
											
	public NewSMetric(){
		refPoint.add(new Double(10));
		refPoint.add(new Double(10));									
	}

 
	/**
	 * Methode berechnet die Flaeche eines Rechtecks, dass durch zwei Elemente eingegrenzt wird.<br>
	 * 
	 * @param vector1 ein Vektor
	 * @param vector2 ein zweiter Vektor
	 * @return double der Flaeche, die durch die beiden Vektoren umschlossen wird
	 */
	public static double surface(List vector1, List vector2)
	{
		double temp = java.lang.Math.abs(((Double)vector1.get(0)).doubleValue() - ((Double)vector2.get(0)).doubleValue()); 
		for(int i=1; i <vector1.size();i++)
			temp *= java.lang.Math.abs(((Double)vector1.get(i)).doubleValue() - ((Double)vector2.get(i)).doubleValue());

		return 	temp;
	}


	//TODO dynamischen Refernezpunkt realisieren
	/**
	 * Methode die eine Flaeche berechnet, die durch die Vektoren einer Vektorenliste 
	 * und einen noch festen Referenzpunkt eingeschlossen ist.
	 * @param vectorList Liste der Individuen, des Algos der bewertet werden soll
	 * @return double der Groesse der Flaeche
	 */
	public double computeMetric(List vectorList) throws Exception
	{
		if (((List)vectorList.get(0)).size() >2 )
		   throw new Exception("Diese Metrik funktioniert nur bei Dimension kleiner gleich zwei ");
		ArrayList tempRefPoint =(ArrayList) ((ArrayList)refPoint).clone();
		double volume = 0.0f;
		sort(vectorList);
		for (int i = 0; i < vectorList.size(); i++) {
			volume += surface(tempRefPoint, (List)vectorList.get(i));
			tempRefPoint.set(1, ((List)vectorList.get(i)).get(1));
		}
		return volume;
	}


	/**
	 * @throws Exception, da die NewSMetric nur eine Vektorliste bewertet
	 */
	public double computeMetric(List vectorlist1, List vectorlist2) throws Exception
	{
		throw new Exception("Diese Methode wird von der newSMetrik nicht unterstuetzt");
	}

	/**
	 * Gibt eine Kurzbeschreibung des Algos zurueck.
	 * @return eine Beschreibung der NewSMetric
	 */
	public String toString()
	{
		String str;
		str =  "\n\nKlasse NewSMetric\n";
		str += "=================\n";
		str += "\n";
		str += "Beschreibung:\n";
		str += "Implementation der S - Metrik im zweidimensionalen Raum," +
			"nach Ideen von E. Zitzler, K. Deb, L. Thiele\n";
		str +="Eine Liste von nichtdominierten Vektoren aus dem Zielbereich werden"+ 
			  "zusammen mit einem festen Referenzpunkt, zu einer Flaeche geformt.\n";
		str +="Dazu werden Rechtecke aus jedem Vektor und dem Referenzpunkt gebildet,"+ 
			  "indem die waagrerechte Seite durch die x - Koordinate des Refernzpunktes"+ 
			  "- die x - Koordinate des Vektors berechnet wird.\n";
		str +="Die senkrechte Seite des Rechtecks wird durch die y - Koordinate des Referenzpunktes"+
			  "- die y - Koordinate des Vektors bestimmt.\n";
		str +="Fuer die Berechnung der weiteren Rechtecke muss der Referenzpunkt angepasst werden.\n";
		str +="Nachdem diese berechnet wurden, kann die Summe dieser Rechtecke als Volumen des durch"+ 
			  "die nichtdominierten Vektoren des Algorithmus dominierten Raum, als Bewertung desAlgorithmus"+ 
			  "genutzt werden.\n";
		str +="\n";
		str +="Kommandozeilenparameter:\n";
		str +="-liste(ArrayList): nichtdominierte Elemente";
		return str;
	}
	
	/**
	 * Die methode sortiert die liste absteigend bezueglich des zweiten Parameters der liste
	 * @param list
	 */
	private void sort(List list){
		for (int i = list.size()-1 ; i>= 1; i--)
			     for(int j = 0; j< i ; j++)
					if ( (((ArrayList)list.get(j)).size() > 1) && ( ((ArrayList)list.get(j+1)).size() > 1) ){
						if(   ((Double)((ArrayList)list.get(j)).get(1)).doubleValue() <((Double)((ArrayList)list.get(j+1)).get(1)).doubleValue()){
							swap(list,j , j+1);
						}
					}
					
	}



	private void swap(List list ,int i, int j){
		List temp;

				temp = (List)list.get(i);
				list.set(i, list.get(j));
				list.set(j, temp);
	}
	
	/**
	 * Gibt an, wieviele Parameter (Dateien) die Metrik erwartet
	 * @return Anzahl der Dateien
	 */
	public int getParameterCount()
	{
		return parameterCount;
	}

	/**
	 * Die Main Methode legt ein neues Objekt der Metrik an. Sie liest die Daten
	 * aus den Files, die als Parameter uebergen wurden, in ArrayLists ein.
	 * Schlie?lich werden die Daten durch den Aufruf von computeMetric() ausgewertet.<br>
	 * 
	 * @param args enthaelt die Dateinamen fuer die Daten
	 * @throws Exception
	 */
	public static void Main(String[] args) throws Exception
	{
		SMetric metric = new SMetric();

		if (args.length != metric.getParameterCount())
			throw new MalInputFormatException("Die Metrik erwartet " +
					metric.getParameterCount() + " Dateinamen als Parameter");

		List list = new ArrayList();
		FileParser.parseFile(args[0], list);

		double output = metric.computeMetric(list);

		System.out.println(output);
	}
	/**
	 * @return
	 */
	public List getRefPoint() {
		return refPoint;
	}

	/**
	 * @param list
	 */
	public void setRefPoint(List list) {
		refPoint = list;
	}

}