/*
 * Copyright (C) 2005  PG447 - University Dortmund
 * 
 * This file is part of NObELTJE.
 * 
 * NObELTJE is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as published by
 * the Free Software Foundation.
 * 
 * NObELTJE is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with NObELTJE; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

 /*
 * Created on Nov 15, 2004
 *
 */
package org.nobeltje.metrics;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.nobeltje.tools.Individual;
import org.nobeltje.tools.ParetoArchive;

/**
 * @author $Author: noot $
 * @version $Revision: 726 $
 * $Date: 2005-02-02 00:50:20 +0100 (Mi, 02 Feb 2005) $
 *
 */
public class SMetricFleischer implements Metric {

	private List paretoSet = new ArrayList();
	
	private List referencePoint = new ArrayList();
	
	private List spawnData = new ArrayList();
	
	private int objectives = 0;
	
	private boolean delta = false;
		

	/**
	 * 
	 */
	public SMetricFleischer() {
		super();
	}
	
	
	public void setDelta(boolean contribution){
		this.delta = contribution;
	}
	
	public void setReferencePoint(List userRefPoint) {
		this.referencePoint = userRefPoint;
	}

	public List getReferencePoint() {
		return this.referencePoint; 
	}

	/* (non-Javadoc)
	 * @see metrics.Metric#computeMetric(java.util.List)
	 */
	public double computeMetric(List solutions) throws Exception {
		
		if (this.referencePoint.size() == 0) {
			// Referenzpunkt fest setzen: 100^dim
			List refPointDefault = new ArrayList();	
			this.objectives = ((ArrayList)(solutions.get(0))).size();
			for (int i=0; i<objectives; i++) {
				refPointDefault.add(new Double(100));
			}
			this.setReferencePoint(refPointDefault);
		}
		
		return computeMetric(solutions, this.referencePoint);
	}

	/**
	 * 
	 * @param population
	 * @param refPoint
	 * @return
	 * @throws Exception
	 * 
	 * Nimmt als Parameter eine Population und filtert die Fitnesswerte heraus.
	 * Mit der Liste der Fitnesswerte wird computeMetrik aufgerufen
	 */
	public double computeMetricPopulation(List population, List refPoint) throws Exception {
		List fitnessList = new ArrayList();
		for (int i=0; i<population.size(); i++) {
			fitnessList.add(((Individual)population.get(i)).getFitness());
		}
		return this.computeMetric(fitnessList, refPoint);
	}

	/* (non-Javadoc)
	 * @see metrics.Metric#computeMetric(java.util.List, java.util.List)
	 */
	public double computeMetric(List solutions, List refPoint) throws Exception {
	   
	    
	    List solutionsValid = new ArrayList();
		
		/*
		solutionsValid.addAll(solutions);	
		// filtere Loesungen raus, die groesser sind als der ReferenzPunkt
		List solutionVector;
		double solValue = 0.0;
		for (int i=0; i<solutionsValid.size(); i++) { // alle Loesungen
			solutionVector = ((ArrayList)solutionsValid.get(i));
			for (int j=0; j<refPoint.size(); j++) { // alle Kriterien
				solValue = ((Double)solutionVector.get(j)).doubleValue();
				if ( solValue > ((Double)refPoint.get(j)).doubleValue()) {
					solutionsValid.remove(i);
					i--; // erniedrige Index
					break;
				}
			}
		}
		*/
		
		// uebernehme nur Loesungen, die kleiner sind als der Referenzpunkt
		List solutionVector;
		double solValue = 0.0;
		boolean valid = true;
		for (int i=0; i<solutions.size(); i++) { // alle Loesungen
			solutionVector = ((ArrayList)solutions.get(i));
			valid = true;
			for (int j=0; j<refPoint.size(); j++) { // alle Kriterien
				solValue = ((Double)solutionVector.get(j)).doubleValue();
				if ( solValue > ((Double)refPoint.get(j)).doubleValue()) {
					valid = false; // Wert groesser als Referenzpunkt
					break;
				}
			}
			if (valid) { // fuege gueltige Loesung hinzu
				solutionsValid.add(solutions.get(i));
			}
		}
		
		
		// Abbruch, falls keine gueltigen Loesungen vorhanden sind
		if (solutionsValid.size() == 0) {
			return 0.0;
		}
		
		this.referencePoint = refPoint;
		paretoSet.clear();
		// filtere pareto-optimale Loesungen,
		for (Iterator iterSolutions = solutionsValid.iterator(); iterSolutions.hasNext();) {
			this.paretoSet = ParetoArchive.refreshArchive(0, (ArrayList)paretoSet, (ArrayList)iterSolutions.next());
		}
		
		
		this.objectives = ((ArrayList)(paretoSet.get(0))).size();

		
		// Hypervolumen nach M. Fleischer
		double hypervolume = 0.0;
		double lopOffVol = 1.0;
		double upperBound;
		int current;
		this.objectives = ((ArrayList)(paretoSet.get(0))).size();
		
		while(paretoSet.size() > 0) {
			// work with the last entry of paretoSet
			current = paretoSet.size() - 1;
			
			lopOffVol = 1.0;
			for (int i=0; i<objectives; i++) {
				upperBound = getBoundValue(current, i);
				
				spawnVector(current, i, upperBound);
				
				
				lopOffVol = lopOffVol * (upperBound - ( (Double)( (ArrayList)paretoSet.get(current) ).get(i) ).doubleValue() );
				
			}
			
			hypervolume += lopOffVol;
			
			paretoSet.remove(paretoSet.size()-1);
			
			
			//Nun sollte f?r ein anderes Element aus der ParetoMenge der MetrikWert berechnet werden...
			//Hier werden Daten aus dem spawnData geloescht, oder 
			nonDomFilter();
			
			this.spawnData.clear();
			
			//System.out.println("curerent for if: "+current+"paretoSize: "+paretoSet.size());
			// Abbruch nach Berechnung des Beitrags der ersten Loesung? 
			if (delta){
				if (current == paretoSet.size())
					break; // verlasse while-Schleife
			}
			
		}//end while
		
		return hypervolume;
	}
	
	double getBoundValue(int entry, int obj) {
		// Searches in pareto[*][obj] the next bigger element (smallest upperBound)
		// of pareto[entry][obj].
		// (upperBound - pareto[entry][obj]) is one dimension of the hypercube
		// *only* dominated by pareto[entry].

		double upperBound =((Double)referencePoint.get(obj)).doubleValue();
		double mySolutionValue = ((Double)((ArrayList)paretoSet.get(entry)).get(obj)).doubleValue() ;
		double nextSolutionValue = 0.0;
		
		for (Iterator iterPareto = paretoSet.iterator(); iterPareto.hasNext();) {
			nextSolutionValue = ( (Double) ( (ArrayList)iterPareto.next() ).get(obj) ).doubleValue();
			
			if (      ( mySolutionValue < nextSolutionValue) 		// nextSolutionValue bigger
			    && (nextSolutionValue < upperBound) )			// smaller upperBound found
			{
				upperBound = nextSolutionValue;
			}
		}
		return upperBound;
	}
	
	
	void spawnVector(int entry, int obj, double ub) {		
		// creates the new entry
		// pareto[entry][0],..,pareto[entry][obj-1],ub,..,pareto[entry][objective]
		// and adds it to spawn data;
	    
		// copy pareto entry
		ArrayList spawnPoint = new ArrayList();
		spawnPoint.addAll( (ArrayList)paretoSet.get(entry) );
		
		// replace desired element by upperBound
		spawnPoint.set(obj, new Double(ub));
		
		// add spawnPoint to spawnData
		spawnData.add(spawnPoint);
		
		// increase spawnSize
	}
	
	void nonDomFilter() {
		// adds nondominated entries of spawnData to pareto
		//
		// sorts out those entries in spawnData,
		// that are either dominated by entries of pareto 
		// or contain elements equal to those in the reference point
	   
		// Entries that contain elements equal to the reference point
		// can be discarded.
		for (Iterator iterSpawn = spawnData.iterator(); iterSpawn.hasNext();) {			
			ArrayList spawnPoint = new ArrayList();
			spawnPoint = (ArrayList)iterSpawn.next();
			
			for(int k=0; k<objectives; k++) {
			   
				if( ((Double)spawnPoint.get(k)).doubleValue() == 
				    ((Double)referencePoint.get(k)).doubleValue() ) 
				{
					// remove entry from spawnData
					
				    
					iterSpawn.remove();
					
					break;
					
				}
				
			}
		}
		
		// Sort out dominated entries.
		for (Iterator iterSpawn = spawnData.iterator(); iterSpawn.hasNext();) {
			this.paretoSet = ParetoArchive.refreshArchive(0, (ArrayList)paretoSet, (ArrayList)iterSpawn.next());				
		}
		
	}
	

	/* (non-Javadoc)
	 * @see metrics.Metric#getParameterCount()
	 */
	public int getParameterCount() {
		return 2;
	}

	public static void main(String[] args) {
	}
}
