/*
 * Copyright (C) 2005  PG447 - University Dortmund
 * 
 * This file is part of NObELTJE.
 * 
 * NObELTJE is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as published by
 * the Free Software Foundation.
 * 
 * NObELTJE is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with NObELTJE; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

 /*
 * Created on 07.11.2004
 *
 * Window - Preferences - Java - Code Style - Code Templates
 */
package org.nobeltje.simulator.temperaturbohrungen;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import kea.Parameter;
import kea.Parameters;
import kea.Type;
import kea.Value;
import kea.problems.external.External;
import org.nobeltje.testfkt.AbstractTestfkt;
import org.nobeltje.tools.io.PropertiesPG;
;

/**
 * @author $Author: noot $
 * @version $Revision: 727 $
 * $Date: 2005-02-08 09:14:18 +0100 (Di, 08 Feb 2005) $
 */
public class Temp extends AbstractTestfkt {
	
	private int boringCount = 3;//Anzahl der Bohrungen
	public static final int XMAX = 2;
	public static final int XMIN = -2;
	public static final int YMAX = 2;
	public static final int YMIN = -2;
	public static final int ZMAX = 2;
	public static final int ZMIN = 0;
	
	/**
	 * @return Gibt die Anzahl der Bohrungen zur?ck.
	 */
	public int getBoringCount() {
		return boringCount;
	}
	
	public void init(PropertiesPG params) {
		// Super-Konstruktor
		super.init(params);
		// Anzahl Bohrungen
		this.boringCount = specificTestfktParameters.getIntProperty("boringCount", this.boringCount, false);
		System.out.println();
		System.out.println("Anzahl der Bohrungen: " + boringCount);
	}
		
	/**
	 * @param boringCount Setzt die Anzahl der Bohrungen.
	 */
	public void setBoringCount(int boringCount) {
		this.boringCount = boringCount;
	}
	private External ex;
	private Logger log;
	private static final Level level = Level.WARNING;
//	leerer Konstruktor, falls sp?ter die Anzahl der Bohrungen auch als Minimierungskriterium genommen werden soll
	public Temp (){
		log =Logger.getLogger(this.getClass().getName());
		log.setLevel(level);
		PropertiesPG args = new PropertiesPG();
		  
		try
		{
			args.load(new FileInputStream("org"+File.separator+"nobeltje"+File.separator+"simulator"+File.separator+"temperaturbohrungen"+File.separator+"Temp.param"));
		}
		catch (FileNotFoundException e)
		{
			e.printStackTrace();
		}
		catch (IOException e)
		{
			e.printStackTrace();
		}
		setBoringCount(args.getIntProperty("boringCount",3,false));
		ex = new External(args.getStringProperty("ServerIP","127.0.0.1",false),args.getIntProperty("ServerPort",6201,false));
		//ex.ObjectiveName();
	}

//	Konstruktor, dem die Anzahl der Bohrungen ?bergeben wird.
	public Temp(int counter){
		this();
		boringCount = counter;		
	}
	
	
	/* (non-Javadoc)
	 * @see testfkt.TestFunctions#execute(java.util.List)
	 */
	public List execute(List input) throws Exception {
		//Ausgabe des Individuums:
		log.log(Level.INFO,"#################################### Ausgabe des zu testenden Individuums:");
		for (Iterator i=input.iterator();i.hasNext();){
			log.log(Level.INFO,i.next().toString());
		}
		
		//Parameter werden auf die Anzahl der Bohrungen gesetzt.
		Parameters param = new Parameters();
		param.setMembership(ex.getClass().getName());
		
		param.addParameter(
				new Parameter(
						"dimension",
						getDimension(),
				"Dimension des Entscheidungsraums."));
		param.addParameter(
				"minRange",
				-2147483640,
		"Untere Schranke im Entscheidungsraum aller Dimensionen.");
		param.addParameter(
				"maxRange",
				2147483640,
		"Obere Schranke im Entscheidungsraum aller Dimensionen.");
		ex.setParameter(param);
		//?berpr?fen ob die Liste die L?nge hat, die laut Dimension da sein m?sste.
		if (input.size()!=getDimension()){
			throw new Exception("Anzahl der Double-Werte entspricht nicht der Dimension");
		}
		
		
		
		//Testen, ob keine 2 Punkte aufeinander liegen
		
		boolean correct = true;
		boolean outOfDefinition = false;
		for (int i=0;i<(getDimension()-3)&&correct;i+=3){
			//Konvertierung dahingehend, dass ein E nicht in der Zahl vorkommt,
			// dazu halt konvertierung nach String der L?nge 26 konvertieren, zur?ck nach double konvertieren
			//und erst dann ?berpr?fen.
			Double x1=new Double(org.nobeltje.tools.io.Number.getDecimal((Double)input.get(i),3));
			Double y1=new Double(org.nobeltje.tools.io.Number.getDecimal((Double)input.get(i+1),3));
			Double z1=new Double(org.nobeltje.tools.io.Number.getDecimal((Double)input.get(i+2),3));
			for (int j=i+3;j<getDimension()&&correct;j+=3){
				Double x2=new Double(org.nobeltje.tools.io.Number.getDecimal((Double)input.get(j),3));
				Double y2=new Double(org.nobeltje.tools.io.Number.getDecimal((Double)input.get(j+1),3));
				Double z2=new Double(org.nobeltje.tools.io.Number.getDecimal((Double)input.get(j+2),3));
				if ((x1.doubleValue()==x2.doubleValue())&&(y1.doubleValue()==y2.doubleValue())&&(z1.doubleValue()==z2.doubleValue())){
					correct =false;//2 Pkt liegen ?bereinander
					break;
				}
			}
		}
		
		//falls die Distanz zum g?ltigen Wertebereich gr?sser ist als default Wert 1
		if (distanz(input)>1) {
			outOfDefinition=true;
		}
		//Test fertig
		
		//Ein Array der L?nge getDimension() + Anzahl der Bohrungen von Values erzeugen.
		//Dieses Array kann, wenn es mit Daten gef?llt ist, an den Simulator ?bergeben werden.
		Value  [] individual = new Value[getDimension()+boringCount];
		
		if (correct && !outOfDefinition) {
			for (int i=0; i<getDimension();i++){
				individual[i]=new Value(((Double)input.get(i)).doubleValue());
				}
				//Bohrungsrichtung wird erstmal ?berall auf false gesetzt.
				Value boringDirection= new Value(Type.BOOLEAN);
				boringDirection.setData(false);
				//Bohrungsrichtungen werden hinten an die Double-Werte angef?gt.
				for (int j = 0;j<boringCount;j++){
					individual[getDimension()+j]= boringDirection;
				}
				log.log(Level.INFO,"######## Ausgabe des Individuums:");
				for (int i=0; i<individual.length;i++){
					log.log(Level.INFO,individual[i].getTypeName()+" "+individual[i].getData());
				}
				//Repair durchf?hren, um die Bohrrichtung so auszurichten, dass es ggf. zu eine g?ltigen L?sung kommt.
				Value[] repaired = ex.repair(individual);
				if (repaired[0]==null) {
					log.log(Level.WARNING, "Warnung: Beim Repair wurde Null zur?ckgegeben, 2 Bohrungspunkte liegen uebereinander.");
				} else {
				
					// Testen, ob keine 2 Punkte aufeinander liegen - erneut fur reparierte Werte
					correct = true;
					for (int i=0;i<(getDimension()-3)&&correct;i+=3){
						//Konvertierung dahingehend, dass ein E nicht in der Zahl vorkommt,
						// dazu halt konvertierung nach String der L?nge 26 konvertieren, zur?ck nach double konvertieren
						//und erst dann ?berpr?fen.
						//Problem: der Evolver hat eine Ungenauigkeit in den Nachkommastellenbereich, d.h. 0.0002=0.00
						// daher wird hier geprueft, ob die Werte auf 3 Nachkommastellen gleich sind.
						// Die 3 hat sich bisher nur durch Testlaufe als sinnvoll erwiesen, bei 4 Nachkomma-
						//stellen verweigert der Evolver die Arbeit bei einigen Testlaeufen.
						Double x1=new Double(org.nobeltje.tools.io.Number.getDecimal((Double)repaired[i].getData(),3));
						Double y1=new Double(org.nobeltje.tools.io.Number.getDecimal((Double)repaired[i+1].getData(),3));
						Double z1=new Double(org.nobeltje.tools.io.Number.getDecimal((Double)repaired[i+2].getData(),3));
						for (int j=i+3;j<getDimension()&&correct;j+=3){
							Double x2=new Double(org.nobeltje.tools.io.Number.getDecimal((Double)repaired[j].getData(),3));
							Double y2=new Double(org.nobeltje.tools .io.Number.getDecimal((Double)repaired[j+1].getData(),3));
							Double z2=new Double(org.nobeltje.tools.io.Number.getDecimal((Double)repaired[j+2].getData(),3));
							if ( (x1.doubleValue()==x2.doubleValue()) && (y1.doubleValue()==y2.doubleValue()) && (z1.doubleValue()==z2.doubleValue()) ) {
								correct =false;//2 Pkt liegen ?bereinander
								break;
							}
						}
					}
				}
				
				if (correct) {
					Value[] result = ex.score(repaired);
					//Erwartet Value[]
					// R?ckgabe der Methode execute muss eine Liste sein. Daher den Fitnesswert einzeln auslesen, zu Double
					// casten und an die Liste result anf?gen
					List back = new ArrayList();
				
					//////////////GEWICHTUNG//////////////////
					double restriction_all = ((Double)result[2].getData()).doubleValue();//erreichbares optimum ist 1
					///////////GEWICHTUNG-ENDE/////////////////
			
					log.log(Level.INFO,"Summe der Restriktionen: "+restriction_all);
					for (int k=0;k<result.length;k++){
						if (k<result.length-5){
							back.add(new Double(((Double)result[k].getData()).doubleValue()*restriction_all));
						}
						else
						{
							//Multiplikation der Restriktionen, die gegen 0 gehen sollen mit 10
							back.add(new Double((((Double)result[k].getData()).doubleValue()*10)));
						}
					}
					return back;
				}
		}
			//Falls die die zul?ssigen Werte nicht im Bereich -2<=x<=2; -2<=y<=2;0<=z<=2 liegen: R?ckgabe eines hohen Wertes an die Testfunktion (1000)
		
		
		//} else {
		if (!correct || outOfDefinition) {
									
			ArrayList bad = new ArrayList();			
			if (!correct){//falls 2 Punkte ?bereinanderliegen
				for (int i=0;i<ex.getDimObjectiveSpace();i++){
					bad.add(new Double(10000));
				}
			} else {
				if (outOfDefinition){
					//ansonsten: Definitionsbereich nicht eingehalten
					ArrayList distanzes= distanzDetail(input);
					//System.out.print("-");
					for (int i=0;i<ex.getDimObjectiveSpace();i+=3){
						//R?ckgabe ist dann: Quadrat von den Distanzen in den einzelnen Koordinaten. Dieses ist dann ein recht hoher R?ckgabewert, da die 
						// Distanz in einer der Koordinaten mindestens 1 ist. Und dieses f?r alle Werte, d.h. auch f?r die Restriktionen
						bad.add(new Double(2*Math.pow(2,((Double)distanzes.get(i%3)).doubleValue())));
						bad.add(new Double(2*Math.pow(2,((Double)distanzes.get((i+1)%3)).doubleValue())));
						bad.add(new Double(2*Math.pow(2,((Double)distanzes.get((i+2)%3)).doubleValue())));
					}
				}
			}
			return bad;
		}
		
		return null; // klappt das so? steht nur da, weil die methode irgendwas zurueckgeben muss..denke, dass mit return die methode abgebrochen wird
	}
	
	
	
	//Methode, die berechnet, wie weit die Individuen von dem zul?ssigen Wertebereich entfernt sind.
	public double distanz (List ind){
		double result =1;
		for (int i=0;i<getDimension();i+=3){
			double x1 = ((Double)ind.get(i)).doubleValue();
			double y1 = ((Double)ind.get(i+1)).doubleValue();
			double z1 = ((Double)ind.get(i+2)).doubleValue();
			if (x1<Temp.XMIN){
				result+=(Temp.XMIN-x1);
			}
			if (y1<Temp.YMIN){
				result+=(Temp.YMIN-y1);
			}
			if (z1<Temp.ZMIN){
				result+=(Temp.ZMIN-z1);
			}
			if (x1>Temp.XMAX){
				result+=(x1-Temp.XMAX);
			}
			if (y1>Temp.YMAX){
				result+=(y1-Temp.YMAX);
			}
			if (z1>Temp.ZMAX){
				result+=(z1-Temp.ZMAX);
			}
			
		}
		return result;
	}
	//Methode, welche die Summe ?ber die Distanz aufsummiert, in Abh?ngigkeiten ob es x, y oder z Koordinate ist
	public ArrayList distanzDetail (List ind){
		ArrayList result = new ArrayList();
		double xdist=1;
		double ydist=1;
		double zdist=1;
		for (int i=0;i<getDimension();i+=3){
			double x1 = ((Double)ind.get(i)).doubleValue();
			double y1 = ((Double)ind.get(i+1)).doubleValue();
			double z1 = ((Double)ind.get(i+2)).doubleValue();
			if (x1<Temp.XMIN){
				xdist+=(Temp.XMIN-x1);
			}
			if (y1<Temp.YMIN){
				xdist+=(Temp.YMIN-y1);
			}
			if (z1<Temp.ZMIN){
				ydist+=(Temp.ZMIN-z1);
			}
			if (x1>Temp.XMAX){
				ydist+=(x1-Temp.XMAX);
			}
			if (y1>Temp.YMAX){
				zdist+=(y1-Temp.YMAX);
			}
			if (z1>Temp.ZMAX){
				zdist+=(z1-Temp.ZMAX);
			}			
		}
		result.add(new Double(xdist));
		result.add(new Double(ydist));
		result.add(new Double(zdist));
		return result;
	}

	/* (non-Javadoc)
	 * @see testfkt.TestFunctions#writeParetoPoints(int, java.lang.String)
	 */
	public void writeParetoPoints(int n, String filename) throws Exception {//entf?llt!

	}

	/* (non-Javadoc)
	 * @see testfkt.TestFunctions#getOptimalBounds()
	 */
	public List getOptimalBounds() {
		List bounds = new ArrayList();
		
		// Der Wertebereich, in dem sich das Standardwerkst?ck (Kugel) liegt, ist in  
		// der x-Achse durch -2 und 2 begrenzt, in der y-Achse im Bereich -2 und 2 und
		// in der z-Achse zwischen 0 und 2.
		// Diese Werte m?ssen f?r alle einzelnen Eckpunkte des Polygonzuges gelten, d.h.
		// f?r Anzahl der Bohrungen +1 Eckpunkte in allen 3 Dimensionen
		// ?bergeben x_1,y_1,z_1....x_n,y_n,z_n.
		
		for (int counter =1; counter <=boringCount+1; counter ++){
			//x-Achse
			bounds.add(new Double(Temp.XMIN));	
			bounds.add(new Double(Temp.XMAX));
			//y-Achse
			bounds.add(new Double(Temp.YMIN));
			bounds.add(new Double(Temp.YMAX));
			//z-Achse
			bounds.add(new Double(Temp.ZMIN));
			bounds.add(new Double(Temp.ZMAX));
		}
		return bounds;
	}

	/* (non-Javadoc)
	 * @see testfkt.TestFunctions#getDimension()
	 */
	public int getDimension() {//Eckpunkte des Polygonzugs x 3 (+ evtl sp?ter Booleans) Anzahl der Bohrungen x3+3
		int help = 0;
		help = boringCount*3 +3; // ggf. ?ber den Server erfragen --> sp?ter
		//log.log(Level.INFO,"Dimension: "+help);
		return help;
	}

	/* (non-Javadoc)
	 * @see testfkt.TestFunctions#getNumberOfObjectives()
	 */
	public int getNumberOfObjectives() {//4 + der Fehlende: Anzahl der kriterien
		int result = 12;/*
		ex.ObjectiveName();
		LinkedList names = ex.Objnamelist;
		result = names.size();
		for (ListIterator iter = names.listIterator();iter.hasNext();)
		{
			log.log(Level.INFO,"Zu optimieren: "+iter.next());
		}*/
		return result;
	}
	
	public boolean ping(){
		return ex.ping();
	}
	
	public void closeConnection(){
		ex.cleanUp();
	}
	
	public List getStartPoint(){
		List result = new ArrayList();
		Parameters param = new Parameters();
		param.setMembership(ex.getClass().getName());
		param.addParameter(
				new Parameter(
						"dimension",
						getDimension(),
				"Dimension des Entscheidungsraums."));
		param.addParameter(
				"minRange",
				-2147483640,
		"Untere Schranke im Entscheidungsraum aller Dimensionen.");
		param.addParameter(
				"maxRange",
				2147483640,
		"Obere Schranke im Entscheidungsraum aller Dimensionen.");
		ex.setParameter(param);
		Value[] recieved = ex.StartIndividuals(new Value[getDimension()+boringCount]);
		for (int i=0;i<recieved.length-boringCount;i++){
			result.add((Double)recieved[i].getData());
		}
		return result;
	}
		
	public static void main (String[]args){
		Temp tmp = new Temp();
		tmp.setBoringCount(3);
		if (tmp.ping()){
			System.out.println("Verbindung vorhanden!");
			//tmp.closeConnection();
		}
		//int i=tmp.getNumberOfObjectives();
		//tmp.getOptimalBounds();
		tmp.setBoringCount(3);
		List values = new ArrayList();values = tmp.getStartPoint();
		try {
			List result = tmp.execute(values);
			for (Iterator iter = result.iterator();iter.hasNext();){
				Double value=(Double)iter.next();
				System.out.println("Wert: "+value);
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		tmp.closeConnection();
		
		//System.out.println("Number of Objectives: "+i);
		
	}
	/**
	 * Methode zum Beenden. Wichtig ist diese Methode gerade fuer das Temperierbohrungsproblem, 
	 * da dieses die einzige Moeglichkeit ist, die Verbindung zum Server zu disconnecten.
	 *
	 */
	public void finalize(){
		this.closeConnection();
		//warten bis der Simulator wieder da ist -- probeweise mit 2 sec
		try{
			Thread.sleep(2000);
			//System.out.println();
			}
		catch(InterruptedException e){}
		//System.out.println("neue Bewertungsrunde");
	}

}
