/*
 * Copyright (C) 2005  PG447 - University Dortmund
 * 
 * This file is part of NObELTJE.
 * 
 * NObELTJE is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as published by
 * the Free Software Foundation.
 * 
 * NObELTJE is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with NObELTJE; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

 package org.nobeltje.testfkt;

import java.util.ArrayList;
import java.util.List;

import org.nobeltje.tools.io.PropertiesPG;

/**
 * Diese Klasse dient bei Erweiterung des Interfaces TestFunctions dazu, dass
 * nicht alle Testfunktionen geaendert werden muessen. Notwendige Aenderungen um
 * das Interface zu erfuellen muessen dann lediglich hier eingepflegt werden, wie
 * z.B. bei den Methoden finalize oder postProcessing.
 * Testfunktionen, die diese Methoden wirklich benoetigen, koennen sie problemlos
 * nach dem Erben ueberschreiben.
 * 
 * @author $Author: noot $
 * @version $Revision: 726 $
 * $Date: 2005-02-02 00:50:20 +0100 (Mi, 02 Feb 2005) $
 */
public abstract class AbstractTestfkt implements TestFunctions {

	/**
	* Fuer Sammlung aller Kommandozeilenparameter fuer die Testfunktion; 
	* Speicherung von Strings, wo jeweils 2 miteinander assoziiert sind (Schluessel + Wert)
    * specificTestfktParameters = Testfunktions-spezifische Parameter
	*/ 
	protected PropertiesPG specificTestfktParameters;
	

	/* (non-Javadoc)
	 * @see testfkt.TestFunctions#init(tools.io.PropertiesPG)
	 */
	public void init(PropertiesPG params) {
		specificTestfktParameters = params;
	}
	
	/**
	 * Falls die Testfunktion fuer alle reelen Zahlen definiert ist,
	 * kann diese Methode benutzt werden und muss nicht 
	 * ueberschrieben werden.
	 * 
	 * Als Grenzen werden grosse negative und positive Konstanten
	 * zurueck gegeben.
	 */
	public List getBounds() {
		List bounds = new ArrayList();
		
			for (int i=0; i<getDimension(); i++) {
				bounds.add(new Double(-100000)); // untere Schranke 
				bounds.add(new Double(100000)); // obere Schranke
			}
			return (bounds);
	}
	

	/**
	 * Diese Methode sollte benutzt werden, wenn Individuen au?erhalb
	 * des Def. Bereichs sind. Je weiter weg, desto gr??er der Strafwert, der ?ber
	 * die Var. k gesetzt wird. Man sollte diesen Wert abh?ngig von der geg. Testfkt
	 * geeignet w?hlen  
	 */
	
	public List setPenalty(List input) throws Exception {
		List check = new ArrayList();
		List mittelpunkt= new ArrayList();
		List bounds = getBounds();
		for (int i = 0 ; i < (bounds.size()/2) ; i++){
			double x = (((Double)bounds.get(2*i+1)).doubleValue() + ((Double)bounds.get(2*i)).doubleValue())/2; 
			mittelpunkt.add(new Double(x));
			
		}
		double abstand =0.0 ;
		if (mittelpunkt.size() > 1)   //Gedacht f?r Fkt Dim > 1
			for (int i =0 ; i< input.size(); i++)
				// abstand ist hier eigentlich der wahre Mittelpunkt und noch nicht der Abstand
				abstand += Math.pow(((Double)input.get(i)).doubleValue() - ((Double)mittelpunkt.get(i)).doubleValue(),2);
	
		else 
			for (int i =0 ; i< input.size(); i++)
				abstand += Math.pow(((Double)input.get(i)).doubleValue() - ((Double)mittelpunkt.get(0)).doubleValue(),2);
		
			
		abstand = Math.sqrt(abstand);
		
		//System.out.println(abstand);
		for (int i=0; i<getNumberOfObjectives();i++)
			{   
	            double x = ((Double) input.get(i)).doubleValue();
		       
		        double k = abstand* Math.pow(10,4);
	           
	            check.add(new Double(x+k));
		        
  		    }
			//System.out.println(check);
		
		
	return (check);
		
	}	
	public int  setZDTPenalty(List input) {
		double x;
		int k=0;
		
		for ( int i=0 ;i< getDimension(); i++){
		 x = ((Double)input.get(i)).doubleValue();
		
		 if (x < 0 || x > 1)
			
		 	k=1;
	
		}
		return (k);
		
	}
	
	/* (non-Javadoc)
	 * @see java.lang.Object#finalize()
	 */
	public void finalize() {
	}

	
	/* (non-Javadoc)
	 * @see testfkt.TestFunctions#postProcessing(java.util.List)
	 */
	public List postProcessing(List individuals) {
		return individuals; // Einfach nur das Eingegangene unveraendert zurueckgeben
	}

}
