/*
 * Copyright (C) 2005  PG447 - University Dortmund
 * 
 * This file is part of NObELTJE.
 * 
 * NObELTJE is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as published by
 * the Free Software Foundation.
 * 
 * NObELTJE is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with NObELTJE; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

 /*
 * Created on 05.11.2004
 *
 * @author kosse
 */
//package org.nobeltje.org.pg447.functions;
package org.nobeltje.testfkt;

import java.util.ArrayList;
import java.util.List;

/**
 * @author $Author: noot $
 * @version $Revision: 726 $
 * $Date: 2005-02-02 00:50:20 +0100 (Mi, 02 Feb 2005) $
 */

public class Deb extends AbstractTestfkt {
    
	public static final int INPUT_PARAMETERS = 2;
	public static final int RETURN_PARAMETERS = 2;
	
	double alpha = 2;	// Parameter fuer die Testfunktion
	double q = 4;			// Parameter fuer die Testfunktion
	
	public Deb ()
	{
	}


	public Deb (double a, double u)
	{
		alpha = a;
		q = u;
	}

	
	/**
	 * Eine erste Testfunktion, liefert eine ArrayList mit den Funktionsweten von f1 und f2 zurueck.
	 * @return ArrayList Werte f1(x,y) und f2(x,y)
	 * @param List input Liste mit den beiden Funktionswerten x und y
	 *
	 */
	public List execute (List input) throws Exception {
		if (input.get(0).getClass().equals(Double.class)) {
			double x = ((Double) input.get(0)).doubleValue();
			double y = ((Double) input.get(1)).doubleValue();
			List results = new ArrayList();
			
			// Def.-Bereich checken
			if (x<0 || x>1 || y<0 || y>1){
				
				
				// Falls Def.-Bereich fuer x und y ueberschritten, wird eine sehr grosse Zahl ausgegeben
				results.add(new Double(x));
				results.add(new Double(y));
			
				results = setPenalty(results);
				
			}
			else{
				results.add(new Double(x)); 
				results.add(new Double((1+10*y)  *  (1- Math.pow((x/(1+10*y)), alpha) - (x/(1+10*y)*Math.sin(2*Math.PI*q*x)) )));
			}
			//System.out.println(results);
			return results;
			
		} else {
			throw new Exception ("Kein Double Objekt als Parameter erhalten!");
		}
	}

	
	public void writeParetoPoints(int n, String filename) throws Exception {
		// Da wir nicht wissen, wie die "wahre" Pareto-Front analytisch angegeben 
		// werden kann, wird diese Methode (zunaechst?) nicht implementiert. 
	}	

	
	public List getOptimalBounds() {
		List bounds = new ArrayList();
		
		// ein Vernuenftiger Wertebereich fuer "gute" Individuen dieser 
		// Funktion liegt z.B. zwischen (0|0) und (1|1), da x und y nur dort definiert sind
		
		bounds.add(new Double(0));	
		bounds.add(new Double(1));
		bounds.add(new Double(0));	
		bounds.add(new Double(1));
		
		return (bounds);
		
	}
	
	public List getBounds() {
		return getOptimalBounds();
		}

	
	public List getStartPoint() {
		List point = new ArrayList();
		
		// Hier wird ein Punkt ausgegeben, der nicht sehr weit der Pareto-Optimalen Loesungen liegt.
		/* ALT:
		point.add(new Double(0.5));
		point.add(new Double(0.5));
		*/
		// am 19.11.2004 ge?ndert (R.Kosse)
		point.add(new Double(0));
		point.add(new Double(0));
		
		return(point);
	}

	
	public int getDimension() {
		return(INPUT_PARAMETERS);		// Dimension ist 2
	}

	
	public int getNumberOfObjectives() {
		return (RETURN_PARAMETERS);
	}

	/**
	 * 
	 * toString-Methode, wird hier benutzt, um eine Testfunktions-Beschreibung zu uebergeben
	 * 
	 * @see tools.io.Help
	 */
	public String toString() {
		String description;

		description = "Klasse Deb, Paket testfkt \n\n";
		description += "Beschreibung :\n\n";
		description += "Zweikriterielle Testfunktion mit zwei Variablen\n";
		description += "Eingabe: Variablenwerte, als Double-Objekte in einer ArrayList\n";
		description += "Ausgabe: eine ArrayList mit zwei Double-Objekten, naemlich den Funktionswerten f1 und f2\n";
		description += "\n";
		description += "Funktion:\n";
		description += "minimiere f1=x und minimiere \n f2=(1+10*y)  *  (1- Math.pow((x/(1+10*y)), alpha) - (x/(1+10*y)*Math.sin(2*Math.PI*q*x)) )\n";
		description += "Die Pareto-Optimalen Punkte liegen zwischen 0<=x, y<=1, der Definitionsbereich ist analog.\n";
	
		return (description);
	}

	
}
