/*
 * Copyright (C) 2005  PG447 - University Dortmund
 * 
 * This file is part of NObELTJE.
 * 
 * NObELTJE is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as published by
 * the Free Software Foundation.
 * 
 * NObELTJE is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with NObELTJE; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

 /*
 * Created on 11.05.2004
 *
 * @author Hendrik
 */
//package org.nobeltje.org.pg447.functions;
package org.nobeltje.testfkt;

import java.io.FileOutputStream;
import java.io.PrintStream;
import java.util.ArrayList;
import java.util.List;

/**
 * @author $Author: noot $
 * @version $Revision: 726 $
 * $Date: 2005-02-02 00:50:20 +0100 (Mi, 02 Feb 2005) $
 */

public class DoubleParabola extends AbstractTestfkt {
    
	public static final int INPUT_PARAMETERS = 1;
	public static final int RETURN_PARAMETERS = 2;
	
	
	/**
     * Eine erste Testfunktion, liefert eine ArrayList mit x**2 und (x-2)**2 zurueck.
     * @return ArrayList Werte x**2 und (x-2)**2
     * @param input x Der einzige Parameter der Parabel
     *
     */
	public List execute (List input) throws Exception {
		if (input.get(0).getClass().equals(Double.class)) {
			double x = ((Double) input.get(0)).doubleValue();
			List results = new ArrayList();
			results.add(new Double(x * x));
			results.add(new Double((x-2) * (x-2)));
			return results;
		} else {
		throw new Exception ("Kein Double Objekt als Parameter erhalten!");
		}
	}

	
	public void writeParetoPoints(int n, String filename) throws Exception {
		// Gibt n Punkte von der "wahren" Pareto-Front aus in die Datei mit dem Namen "filename". 
		// Die Punkte werden gleichverteilt (also aequi-distant) aus der Menge der Pareto-Optimalen 
		// Punkte ausgewaehlt. 

		double min = 0;
		  double max = 2;
		  double step = (max - min) / (n -1);									//Schrittweite der Punkte
		  List result;
		  List parameter = new ArrayList(); 
		  PrintStream out = new PrintStream(new FileOutputStream(filename));	// In Datei schreiben
		  parameter.add(new Double(min));
		  for (int i = 1; i  < (n); i++) {
		  	result = this.execute(parameter);
		  	out.println(result.get(0) + " " + result.get(1));
		  	parameter.add(0, new Double(min + step * i));		  	
		  }
		parameter.add(0, new Double(max));
		result = this.execute(parameter);
		out.println(result.get(0) + " " + result.get(1));
		
		out.close();
	}	

	
	public List getOptimalBounds() {
		List bounds = new ArrayList();
		
		// ein Vernuenftiger Wertebereich fuer "gute" Individuen dieser 
		// Funktion liegt z.B. zwischen -10 und 10. (der Pareto-Optimale Bereich
		// liegt zwischen 0 und 2).
		bounds.add(new Double(-100000));	
		bounds.add(new Double(100000));
		
		return (bounds);
		
	}

	
	public List getStartPoint() {
		List point = new ArrayList();
		
		// Hier wird ein Punkt ausgegeben, der nicht sehr weit der Pareto-Optimalen Loesungen liegt.
		point.add(new Double(10000));
		
		return(point);
	}

	
	public int getDimension() {
		return(INPUT_PARAMETERS);		// Dimension ist 1
	}

	
	public int getNumberOfObjectives() {
		return (RETURN_PARAMETERS);
	}

	/**
	 * 
	 * toString-Methode, wird hier benutzt, um eine Testfunktions-Beschreibung zu uebergeben
	 * 
	 * @see tools.io.Help
	 */
	public String toString() {
		String description;

		description = "Klasse DoubleParabola, Paket testfkt \n\n";
		description += "Beschreibung :\n\n";
		description += "Zweikriterielle Testfunktion mit einer Variable\n";
		description += "Eingabe: Variablenwert, als Double-Objekt in einer ArrayList\n";
		description += "Ausgabe: eine ArrayList mit zwei Double-Objekten, naemlich den Funktionswerten f1 und f2\n";
		description += "\n";
		description += "Funktion:\n";
		description += "minimiere f1=x^2 und minimiere f2=(x-2)^2\n";
		description += "Die Pareto-Optimalen Punkte liegen zwischen 0<=x<=2\n";
	
		return (description);
	}

}
