/*
 * Copyright (C) 2005  PG447 - University Dortmund
 * 
 * This file is part of NObELTJE.
 * 
 * NObELTJE is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as published by
 * the Free Software Foundation.
 * 
 * NObELTJE is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with NObELTJE; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

 
package org.nobeltje.testfkt;

import java.io.FileOutputStream;
import java.io.PrintStream;
import java.util.ArrayList;
import java.util.List;


/**
 * @author $Author: noot $
 * @version $Revision: 726 $
 * $Date: 2005-02-02 00:50:20 +0100 (Mi, 02 Feb 2005) $
 *
 * Fonsecas F1 Funktion:
 * f1(x1,x2)=1-exp(-(x1-1)**2-(x2+1)**2) 
 * f2(x1,x2)=1-exp(-(x1+1)**2-(x2-1)**2) 
 */

public class FonsecaF1 extends AbstractTestfkt {

	public static final int INPUT_PARAMETERS = 2;
	public static final int RETURN_PARAMETERS = 2;

	/**
	 * Berechnet Fonsecas F1 Funktion
	 * @return ArrayList Werte x1 und x2
	 * @param input : List mit 2 Parametern
	 *
	 */
	public List execute(List input) throws Exception {
		if (input.get(0).getClass().equals(Double.class) && input.get(1).getClass().equals(Double.class) ) {
			double x1= ((Double) input.get(0)).doubleValue();
			double x2= ((Double) input.get(1)).doubleValue();
			List results = new ArrayList();
			
			double d;
			d = 1 - Math.exp(- Math.pow(x1-1, 2) - Math.pow(x2+1, 2)); // f1 berechnen
			results.add(new Double(d));
			d = 1 - Math.exp(- Math.pow(x1+1, 2) - Math.pow(x2-1, 2)); //f2 berechnen
			results.add(new Double(d));
			return results;
		} else {
		throw new Exception ("wrong parameters!");
		}
	}

	
	/* (non-Javadoc)
	 * @see testfkt.TestFunctions#writeparetopoints(int, java.lang.String)
	 */
	public void writeParetoPoints(int n, String filename) throws Exception {
		// Gibt n Punkte von der "wahren" Pareto-Front aus in die Datei mit dem Namen "filename". 
		// Die Punkte werden gleichverteilt (also aequi-distant) aus der Menge der Pareto-Optimalen 
		// Punkte ausgewaehlt. 

		double minx = -1;			//Minimale X-Koordinate
		double miny = 1;			//Minimale Y-Koordinate
		double maxx = 1;			//Maximale X-Koordinate
		double maxy = -1;			//Maximale Y-Koordinate
		
		// Hier wird nun eine Geradengleichung aufgestellt. Im Werte-Raum liegen die Pareto-Optimalen 
		// Punkte auf der Geraden x=-y, die die Optima der beiden Funktionen verbindet.
		// Der Anfangspunkt der Werte auf dieser Geraden ist (minx, miny), der Endpunkt ist (maxx, maxy).
		// Das intervall -1<=x<=1 wird in n-1 gleichlange Intervalle unterteilt, um n Punkte zu erhalten.
		// Der zug. x-Wert ergibt sich durch die Geradengleichung.
		
		double step = (maxx - minx) / (n -1);								//Schrittweite der Punkte
		List result;
		List parameter = new ArrayList(); 
		PrintStream out = new PrintStream(new FileOutputStream(filename));	// In Datei schreiben
		

		// Den ersten Punkt des Intervalls uebergeben
		parameter.add(new Double(minx));
		parameter.add(new Double(miny));
		
		for (int i = 1; i  < (n); i++) {
			result = this.execute(parameter);
		  	out.println(result.get(0) + " " + result.get(1));
		  	parameter.add(0, new Double(minx + step * i));		// naechster X-Wert auf der Geraden	  	
		  	parameter.add(1, new Double(-(minx + step * i)));	// zugehoeriger Y-Wert	  	
		}
		// Der Endpunkt
		parameter.add(0, new Double(maxx));	
		parameter.add(0, new Double(maxy));
		result = this.execute(parameter);
		out.println(result.get(0) + " " + result.get(1));
		
		out.close();

	}

	
	public List getOptimalBounds() {
			List bounds = new ArrayList();
		
			// ein Vernuenftiger Wertebereich fuer "gute" Individuen dieser 
			// Funktion liegt z.B. zwischen (-3|3) und (-3|3). (der Pareto-Optimale Bereich
			// liegt zwischen (-1|1) und (1|-1)).
			bounds.add(new Double(-3));	
			bounds.add(new Double(3));
			bounds.add(new Double(-3));	
			bounds.add(new Double(3));
		
			return (bounds);
		
		}


		public List getStartPoint() {
			List point = new ArrayList();
		
			// Hier wird ein Punkt ausgegeben, der nicht sehr weit der Pareto-Optimalen Loesungen liegt.
			point.add(new Double(-1));
			point.add(new Double(2));
		
			return(point);
		}

		
		public int getDimension() {
			return(INPUT_PARAMETERS);		// Dimension ist 2 !!!!
		}
		

		public int getNumberOfObjectives() {
			return (RETURN_PARAMETERS);
		}
	
		/**
		 * 
		 * toString-Methode, wird hier benutzt, um eine Testfunktions-Beschreibung zu uebergeben
		 * 
		 * @see tools.io.Help
		 */
		public String toString() {
			String description;

			description = "Klasse FonsecaF1, Paket testfkt \n\n";
			description += "Beschreibung :\n\n";
			description += "Zweikriterielle Testfunktion mit zwei Variablen\n";
			description += "Eingabe: zwei Variablenwerte, als Double-Objekte in einer ArrayList\n";
			description += "Ausgabe: eine ArrayList mit zwei Double-Objekten, naemlich den Funktionswerten f1 und f2\n";
			description += "\n";
			description += "Funktion:\n";
			description += "minimiere f1(x1,x2)=1-exp(-(x1-1)-(x2+1)) und minimiere\n";
			description += "f2(x1,x2)=1-exp(-(x1+1)-(x2-1))\n";
			description += "Die Minima der beiden Funktionen liegen bei (1,-1) fr f1 und bei (-1,1) fr f2. Die Funktionswerte sind in diesen Fllen =0.\n";
			description += "Die (wahre) Paretofront im Funktionsraum liegt nun zwischen den Werten (0, 1-exp(-8)) und (1-exp(-8), 0) und hat in etwa eine Bogenform.\n";
			
			return (description);
		}

	
}
