/*
 * Copyright (C) 2005  PG447 - University Dortmund
 * 
 * This file is part of NObELTJE.
 * 
 * NObELTJE is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as published by
 * the Free Software Foundation.
 * 
 * NObELTJE is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with NObELTJE; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

 /*
 * Created on 21.06.2004
 * by Hendrik
 */
package org.nobeltje.testfkt;

import java.io.FileOutputStream;
import java.io.PrintStream;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Random;

import org.nobeltje.tools.Individual;
import org.nobeltje.tools.ParetoArchive;

/**
 * <p>Klasse fuer das 0/1 Rucksackproblem</p>
 * <p>Im Gegensatz zum klassischen Problem soll diese Klasse eine
 * variable Testfunktion bieten, die das Problem nicht statisch 
 * festlegt. Die Anzahl der Gegenstaenden und Nutzwerte ist variabel.<br />
 * Nutzwerte und Gewicht werden mit einen Random Seed "reproduzierbar 
 * zufaellig" bestimmt.</p>
 * 
 * @author $Author: noot $
 * @version $Revision: 726 $
 * $Date: 2005-02-02 00:50:20 +0100 (Mi, 02 Feb 2005) $
 */


public class KnapsackProblem extends AbstractTestfkt {
    /**
     * Anzahl der Gegenstaende
     */
	private int objects; //Anzahl der Gegenstaende
	/**
	 * Anzahl der Nutzenwerte
	 */
	private int objectives; //Anzahl der Nutzenwerte
	/**
	 * Rucksackgroesse
	 */
	private int knapsacksize; //Rucksackgroesse
	/**
	 * maximaler Nutzenwert pro einzelnem Nutzenwert
	 */
	private int maxobjectvalue; //maximaler Nutzwert (pro einzelnem Nutzwert)
	/**
	 * maximale Groesse eines Gegenstandes
	 */
	private int maxobjectsize; //maximale Groesse eines Gegenstandes
	/**
	 * "Random Seed" um die Gegenstaende zu reproduzieren
	 */
	private long randomseed;  //"Random Seed" um die Gegenstaende zu reproduzieren
	/**
	 * Tabelle mit den Groessen & Nutzwerten der Gegenstaende
	 */
	private int objectproperties[][]; //Tabelle mit den Groessen & Nutzwerten der Gegenstaende
	
	/**
	 * <p>Konstruktor, der einem ein individuelles Rucksackszenario baut:</p>
	 * <p>Es wird ein Rucksack randomisiert erstellt, der eine bestimmte Anzahl von Gegenstnden, eine bestimmte Anzahl
	 * von Nutzenwerten sowie eine klar definierte Gewichtsschranke des Rucksacks hat. Randomisiert erzeugt werden die Gegenstaende
	 * des Rucksackproblems; diese haben alle Nutzenwerte, die im Intervall [0,maxobjectvalue] liegt, sowie ein Gewicht haben, die im Intervall 
	 * [0, maxobjectsize] liegen. Da hier die Nutzenwerte und die Gewichte randomisiert gewaehlt werden, wird ebenfalls ein Random-Seed angegeben, um 
	 * eine Rucksackeingabe reproduzierbar zu machen.</p>
	 * 
	 * @param objects : Anzahl der Gegenstaende
	 * @param objectives : Anzahl der Nutzenwerte 
	 * @param knapsacksize : Gewichtsschranke des Rucksacks 
	 * @param maxobjectvalue : maximaler Nutzenwert eines Gegenstandes
	 * @param maxobjectsize : maximale Gewichtsgrenze der Gegenstaende  
	 * @param randomseed : long
	 */
	public KnapsackProblem(int objects, int objectives, int knapsacksize,
			int maxobjectvalue, int maxobjectsize, long randomseed) {
		super();
		this.objects = objects;
		this.objectives = objectives;
		this.knapsacksize = knapsacksize;
		this.maxobjectvalue = maxobjectvalue;
		this.maxobjectsize = maxobjectsize;
		this.randomseed = randomseed;
		
		//TODO parameter auf wertebereich pruefen, da kann ja jeder kommen ... du SACK
		
		//Zufallszahlengenerator bauen
		Random lotterybox = new Random (randomseed);
		//tabelle fuer die gegenstaende erzeugen
		//erster index ist das object, der zweite das gewicht(0); die weiteren nutzwerte
		objectproperties = new int[objects][objectives+1];
		//und fuellen
		for (int i = 0; i < objects; i++){
			for (int j = 0; j <= objectives; j++) {
				if (j==0) {
					objectproperties[i][j] = lotterybox.nextInt(maxobjectsize) + 1;
				} else {
					objectproperties[i][j] = -(lotterybox.nextInt(maxobjectvalue) + 1);
				}
			}
		}
	}
	
	
	/**
	 * <p>Standardkonstruktor</p>
	 * <p>Ruft den anderen Konstruktor mit "sinnvollen" default Werten auf,
	 * hier sind die Defaultwerte wie folgt:
	 * <ul><li>Anzahl Objekte: 15</li>
	 * <li> Anzahl der Nutzenwerte: 2</li>
	 * <li>Gewichtsgrenze des Rucksacks: 50</li>
	 * <li>obere Grenze fr die Nutzen- und die Gewichtswerte: je 20</li>
	 * <li>RandomSeed (als Long): 45</li></ul<</p>
	 */
	public KnapsackProblem() {
		this(15, 2, 50, 20, 20, 45L);		// 15,2,50,20,20
	}
	
	
/**
 * <p>Methoede execute, der auf die Nutenwerte und das Gewicht einer gegebenen Bepackung berechnet</p>
 * 
 * <p>Dazu wird eine Liste mit boolschen Strings uebergeben, wobei der boolsche Wert von 1 bei einem Index i dafuer 
 * steht, dass das Objekt i in der aktuellen Bepackung drin ist.<br />
 * D.h. hier wird die Summe ueber die einzenen Nutzenwerte gebildet, sowie die Summe ueber die Gewichte der Objekte, die
 * in der uebergebenen Stringliste den Wert true haben.<br />
 * Anschliessend wird noch geprueft, ob die Summe der Gewichte die Gewichtsschranke des Rucksacks nicht ueberschreitet. Falls doch, so wird
 * eine Exception geworfen.</p>
 * 
 * @param input : List mit boolschen Werten input
 * @return List mit der Summe der Gewichte, Summe NW1, Summe NW2,...
 */
	public List execute(List input) throws Exception {
		/**
		 * Array der Laenge Nutzenwerte +1
		 */
		int values[] = new int[objectives + 1]; //Array der Laenge Nutzenwerte +1
		
		for (int i = 0; i < objects; i++){ //Zaehler ber die Anzahl der Objekte
			for (int j = 0; j <= objectives; j++) { //j=1, da ja die Nutzenwerte hier eingetragen werden sollen ??
				if (((Boolean)input.get(i)).booleanValue()) { //booleanwert des Objekts in der input an Position i
					values[j] += objectproperties[i][j];
				}
			}
		}
		if (values[0]>knapsacksize) {
			throw new Exception ("Keine Loesung!");
		} else {
			List result = new ArrayList();
			for (int j = 0; j < objectives; j++) {
				result.add(new Double(values[j+1])); 			
			}
			return result;
		}
	}
	
	
	/**
	 * <p>Methode die "n" Punkte auf der Paretofront berechnet und diese in eine Datei namens "filename" schreibt</p>
	 * 
	 * <p>Da wir hier keine durchgehende Paretofront haben und zudem eine variable (also keine feste Eingabe, sondern durch die 
	 * Randomisierung keine klar definierte) Testfunktion realisieren, muessen alle Loesungen 
	 * durch Ausprobieren aller moeglichen Bepackungskombinationen ermittelt werden.<br /> 
	 * Problem ist dabei, dass dieses schnell zu sehr langer Rechenzeit fuehren kann, so dass es ratsam ist nur kleine Probleme
	 * als Eingabe zu definieren.<br />
	 * Um zu testen, ob das Individuum pareto-optimal ist, wird die Methode refreshArchive aus der Klasse ParetoArchive genutzt,
	 * hier arbeitet diese Methode allerdings auf boolschen Listen.</p>
	 * 
	 * @param n : int: maximale Anzahl der pareto-optimalen Loesungen
	 *@param filename : String, der die Bepackung kodiert
	 */
	public void writeParetoPoints(int n, String filename) throws Exception {
		ArrayList individualvariables;
		Individual individual;
		ArrayList paretooptimalindividuals = new ArrayList();
		
		//alle kombinationen durchgehen
		for (int i = 0; i < Math.pow(2, objects); i++) {
			individualvariables = new ArrayList();
			//zahl in einen binaerstring wandeln
			String binaryindividualvariables = Integer.toBinaryString(i);
			//fuehrende nullen (nicht im string) in die liste einf?gen
			for (int j = 0; j < (objects - binaryindividualvariables.length()); j++) {
				individualvariables.add(new Boolean(false));
			}
			//Boolean nach binaerdarstellung anhaengen
			for (int j = 0; j < binaryindividualvariables.length(); j++) {
				if (binaryindividualvariables.charAt(j)=='1') {
					individualvariables.add(new Boolean(true));
				} else {
					individualvariables.add(new Boolean(false));
				}
			}
			try {
				individual = new Individual(individualvariables, null, (ArrayList)execute(individualvariables), i);
			
				//paretoarchiv (nicht dominierte werte) anpassen
				paretooptimalindividuals = ParetoArchive.refreshArchive(0, paretooptimalindividuals, individual);
			} catch (Exception e) {
				//nicht loesung ist nicht definiert, d. h. das gewicht zu hoch
			}
		}
		
		PrintStream out = new PrintStream(new FileOutputStream(filename));	// In Datei schreiben
		String outputline;
		Iterator outputiter = paretooptimalindividuals.iterator();
		//n viele po werte ausgeben, falls es so viele gibt
		for (int i=0; i < n && outputiter.hasNext(); i++) {
			for (Iterator criteria = ((Individual)outputiter.next()).getFitness().iterator(); criteria.hasNext();) {
				out.print(criteria.next() + " ");
			}
			out.println();
		}
		out.close();
	}
	
	
	/**
	 * <p>Methode des Interface Testfkt, die einen Wertebereich fuer die Variablen der Funktion ausgeben soll,
	 *  in denen sich die Pareto-Optimalen Werte befinden. Dadurch kann der Algo leichter seine Start-Individuen 
	 * erzeugen, ohne im komplett falschen Bereich zu suchen.</p>
	 * <p>Da hier keine klar strukturierte Paretofront vorliegt, sondern nur Bepackungen gibt diese Methode "null" zurueck:
	 * sie hat keine optimalBounds</p>
	 *
	 *@return null
	 */
	public List getOptimalBounds() {
		return null;
	}
	
	
	/**
	 * <p>Methode des Interface Testfkt, die einen Punkt in der Naehe der Pareto-Optimalen Punkte ausgeben soll, den  der Algo 
	 * als Start-Individuum benutzen kann.</p> 
	 * <p>Da der leere Rucksack immer eine zulaessige Bepackung darstellt, wird hier ein leerer Rucksack als 
	 * Startbepackung zurueckgegeben, also:</p>
	 * 
	 * @return Liste von #objects von boolean Werte mit dem Wert false
	 */
	public List getStartPoint() {
		List point = new ArrayList();
		for (int i = 0; i < objects; i++) {
			point.add(new Boolean(false));
		}
		return point;
	}
	
	
	/**
	 * <p>Methode, die ausgibt, wieviele Dimensionen der Variablen-Raum dieser Testfunktion hat, hier also
	 * wieviele verschiedene Nutzenwerte die Funktion hat.</p>
	 * 
	 * @return #Nutzenwerte: int objects
	 */
	public int getDimension() {
		return objects;
	}

	/**
	 * <p>Methode, die ausgibt, wieviele Dimensionen der Ziel-Raum dieser Testfunktion hat (Anzahl Kriterien).</p>
	 * <p>Da die Loesungen Rucksackbepackungen sind, die durch einen boolschen String repraesentiert wird, der 
	 * die Laenge der Anzahl der Objekte hat, wird die Anzahl der Gegenstaende in dem Problem zurueckgegeben.</p>
	 * 
	 * @return Anzahl der Gegenstaende: int objectives
	 */
	public int getNumberOfObjectives() {
		return (objectives);
	}

/**
 * <p>Methode, die auf der Konsole, die wichtigsten Daten des Problems ausgibt:</p>
 * <ul><li> Das es sich um ein Rucksackproblem handelt</li>
 * <li>Welcher Randomseed gewaehlt wurde</li>
 * <li>Die Gewichtsschranke des Rucksacks, sowie die Anzahl der Gegenstaende</li>
 * <li> Die Anzahl der daraus moeglichen Kombinationen fuer die Bepackung (Groesse des Zielraums)</li>
 * <li>Anzahl der Nutzenwerte (Dimension des Zielraums)</li>
 * <li>Auflistung der randomisiert erzeugten Gewichte und Nutzenwerten in einer Tabelle der Gegenstaende</li></ul>
 */
	public String toString() {
		StringBuffer textoutput = new StringBuffer();
		textoutput.append("Dies ist ein Rucksackproblem!\n");
		textoutput.append("Und zwar mit einem Random Seed von " + randomseed + "\n");
		textoutput.append("Groesse des Rucksacks: " + knapsacksize);
		textoutput.append("\nAnzahl der Gegenstaende: " + objects);
		textoutput.append("\nAnzahl der Kombinationen: " + Math.pow(2, objects));
		textoutput.append("\nAnzahl der Nutzwert: " + objectives);
		textoutput.append("\nGroesse der Gegenstaende von 1 bis " + maxobjectsize);
		textoutput.append("\nNutzwert der Gegenstaende von 1 bis " + maxobjectvalue);
		textoutput.append("\nTabelle der Gegenstaende, Syntax ist Gewicht Nutzwert1 NW2 NW3 ...");
		for (int i = 0; i < objects; i++){
			textoutput.append("\n" + (i+1) + " ");
			for (int j = 0; j <= objectives; j++) {
				textoutput.append(objectproperties[i][j] + " ");
			}
		}		
		return textoutput.toString();
	}

}