/*
 * Copyright (C) 2005  PG447 - University Dortmund
 * 
 * This file is part of NObELTJE.
 * 
 * NObELTJE is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as published by
 * the Free Software Foundation.
 * 
 * NObELTJE is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with NObELTJE; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

 /*
 * Created on 12.05.2004
 */

package org.nobeltje.testfkt;

import java.util.List;

import org.nobeltje.tools.io.PropertiesPG;

/**
 * @author $Author: noot $
 * @version $Revision: 726 $
 * $Date: 2005-02-02 00:50:20 +0100 (Mi, 02 Feb 2005) $
 */


public interface TestFunctions {
    
	
	/**
	 * Initialisiert das Testfunktionsobjekt
	 */
	public void init(PropertiesPG params);
	
	
	/**
	 * berechnet fuer ein Eingabeindividuum (List aus Doubles) die Funktionswerte (List aus Doubles)
	 *   
	 * @param input
	 * @return
	 * @throws Exception
	 */
	public List execute (List input) throws Exception;

	
	/**
	 * speichert n Punkte welche auf der Pareto Front liegen in eine Datei filename ab
	 */
	public void writeParetoPoints(int n, String filename) throws Exception;

	
	/**
	 * gibt einen Wertebereich fuer die Variablen der Funktion aus, in denen sich die
	 * Pareto-Optimalen Werte befinden. Dadurch kann der Algo leichter seine Start-Individuen 
	 * erzeugen, ohne im komplett falschen Bereich zu suchen.
	 * 
	 * Ausgabe: Liste, in der ausgegeben wird: der unterste Wert der 1. Variable, dann der oberste 
	 * Wert der 1. Variable. Dann der unterste Wert der 2. Variable, danach der Oberste 
	 * Wert der 2. Variable. Das gleiche dann fuer 3., 4., ... Variable.
	 * 
	 * @throws Exception
	 */
	public List getOptimalBounds();
	
	/**
	 * Gibt eine Liste mit den Grenzen des Definitionsbereichs zurueck.
	 * Ist in AbstractTestfkt implementiert, sollte von jeder Testfunktion ueberschrieben werden.
	 * 
	 * @return Liste mit Grenzen des Wertebereichs
	 */
	public List getBounds();
	
	/**
	 * gibt einen Punkt in der Naehe der Pareto-Optimalen Punkte aus, damit der Algo diesen 
	 * als Start-Individuum benutzen kann. Analog zu getoptimalbounds.
	 * 
	 * Ausgabe: Liste mit x1, x2, x3, ... des Punktes
	 */
	public List getStartPoint();

	
	/**
	 * gibt aus, wieviele Dimensionen der Variablen-Raum dieser Testfunktion hat
	 * 
	 * Ausgabe: Integer-Wert der Dimension  / Eingabeanzahl
	 */
	public int getDimension();


	/**
	 * gibt aus, wieviele Dimensionen der Ziel-Raum dieser Testfunktion hat (Anzahl Kriterien)
	 * 
	 * Ausgabe: Integer-Wert der Kriterienanzahl
	 */
	public int getNumberOfObjectives();
	
	
	/**
	 * Methode zum Beenden. Wichtig ist diese Methode gerade fuer das Temperierbohrungsproblem, 
	 * da dieses die einzige Moeglichkeit ist, die Verbindung zum Server zu disconnecten.
	 *
	 */
	public void finalize();
	
	
	/**
	 * Da die Fahrstuhlsimulation eine Nachbearbeitung der Ergebnisse benoetigt, sollen dieser
	 * Methode alle Individuen uebergeben werden, die der Algorithmus als Endergebnis liefert.
	 * Diese koennen von den jeweiligen Problemen dann ueberarbeitet, z.B. erneut simuliert, werden
	 * oder einfach unueberarbeitet zurueckgegeben werden.
	 * Wichtig: Die Liste von Individuen, die hier zurueckgegeben werden, stellen in der Regel
	 * das neue Ergebnis des Algos dar. Also bitte kein null zurueckgeben oder aehnliches.
	 * Partikel-Objekte sind wohl besser erstmal nicht vorgesehen, weswegen die PSO diese Methode
	 * nicht ausfuehren sollten.
	 */
	public List postProcessing(List individuals);
	
	
	/**
	 * berechnet f?r die Individuen, die ausserhalb des Sichtbaren Raums sind, einen Strafwert
	 * Je weiter das Individuum vom Def. Bereich entfernt ist, desto gr??er ist der Strafwert
	 * @param input
	 * @return
	 * @throws Exception
	 */
	public List setPenalty(List input) throws Exception;
	
	public int  setZDTPenalty(List input);
	
}
