/*
 * Copyright (C) 2005  PG447 - University Dortmund
 * 
 * This file is part of NObELTJE.
 * 
 * NObELTJE is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as published by
 * the Free Software Foundation.
 * 
 * NObELTJE is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with NObELTJE; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

 /*
 * Created on 18.11.2004
 *
 */
package org.nobeltje.testfkt;

import java.util.ArrayList;
import java.util.List;

import org.nobeltje.exceptions.MalInputFormatException;

/**
 * @author $Author: noot $
 * @version $Revision: 726 $
 * $Date: 2005-02-02 00:50:20 +0100 (Mi, 02 Feb 2005) $
 */
public class ZDT3 extends AbstractTestfkt {
	
	public static final int INPUT_PARAMETERS = 30;
	public static final int RETURN_PARAMETERS = 2;


	/* (non-Javadoc)
	 * @see testfkt.TestFunctions#execute(java.util.List)
	 */
	public List execute(List input) throws Exception {
		if (input.get(0).getClass().equals(Double.class)) {
			if (input.size() == this.getDimension()) {
				List results = new ArrayList();
				int k;
				k=setZDTPenalty(input);
				if (k==1){
					results = setPenalty(input);
				}
				
				else
				{
					double f1, f2;
					double sumX = 0.0;
					double g, h, x_i;
					
					f1 = ((Double)input.get(0)).doubleValue();
					
					for (int i=1; i<getDimension(); i++){
						x_i = ((Double)input.get(i)).doubleValue();
						sumX += x_i;
					}
					g = 1 + ( 9 * sumX / (getDimension()-1) );
					h = 1 - Math.sqrt(f1/g) - (f1/g) * Math.sin(10 * Math.PI * f1);
					f2 = g * h;			
					
					results.add(new Double(f1));
					results.add(new Double(f2));
				}
				return results;
			} else
				throw new MalInputFormatException("Anzahl Variablen ungleich 30.");
		} else
			throw new Exception ("Kein Double Objekt als Parameter erhalten!");
	}

	/* (non-Javadoc)
	 * @see testfkt.TestFunctions#writeParetoPoints(int, java.lang.String)
	 */
	public void writeParetoPoints(int n, String filename) throws Exception {
		// TODO  
	}


	/* (non-Javadoc)
	 * @see testfkt.TestFunctions#getOptimalBounds()
	 * 
	 * Funktion ist definiert fuer Variablen zwischen 0 und 1
	 */
	public List getOptimalBounds() {
		List bounds = new ArrayList();
		
		for (int i=0; i<getDimension(); i++) {
			bounds.add(new Double(0));
			bounds.add(new Double(1));
		}
		return (bounds);
	}
	public List getBounds() 
	{
		return getOptimalBounds();
	}

	/* (non-Javadoc)
	 * @see testfkt.TestFunctions#getStartPoint()
	 */
	public List getStartPoint() {
		return null;
	}

	/* (non-Javadoc)
	 * @see testfkt.TestFunctions#getDimension()
	 */
	public int getDimension() {
		return INPUT_PARAMETERS;
	}

	/* (non-Javadoc)
	 * @see testfkt.TestFunctions#getNumberOfObjectives()
	 */
	public int getNumberOfObjectives() {
		return RETURN_PARAMETERS;
	}

	/**
	 * 
	 * toString-Methode, gibt eine Testfunktions-Beschreibung zurueck
	 * 
	 * @see tools.io.Help
	 */
	public String toString() {
		String description;

		description = "Klasse ZDT3, Paket testfkt \n\n";
		description += "Testfunktion von Zitzler, Deb, Thiele\n";
		description += "Zweikriterielle Testfunktion mit 30 Variablen\n";
		description += "Eingabe: Variablenwerte, als Double-Objekte in einer ArrayList\n";
		description += "Ausgabe: eine ArrayList mit zwei Double-Objekten, naemlich den Funktionswerten f1 und f2\n";
		description += "\n";
		description += "Funktion:\n";
		description += "minimiere f1 und f2 \n";
		description += "Pareto-Optimale Punkte: 0 <= x_1 <= 1 und x_2 = ... = x_30 = 0 \n";
		description += "Die Pareto-Front ist konvex und unzusammenh?ngend.\n";
		description += "\n";
		description += "Dokumentation: siehe\n";
		description += "K. Deb et al. \n";
		description += "A Fast Elitist Non-Dominated Sorting Genetic Algorithm ";
		description += "for Multi-Objective Optimization: NSGA-II \n";
		description += "2000, KanGAL report 200001. \n";
		description += "oder \n";
		description += "\n";
		description += "K. Deb \n";
		description += "Multi-Objective Optimization using Evolutionary Algorithms \n";
		description += "John Wiley & Sons, Chichester, UK, 2001, ISBN 0-471-87339-X \n";
	
		return (description);
	}

}
