/*
 * Copyright (C) 2005  PG447 - University Dortmund
 * 
 * This file is part of NObELTJE.
 * 
 * NObELTJE is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as published by
 * the Free Software Foundation.
 * 
 * NObELTJE is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with NObELTJE; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

 /*
 * Created on Jul 1, 2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package org.nobeltje.tools;

import java.util.ArrayList;
import java.util.Arrays;




/**
 * Die Klasse implementiert einen Baum, der zur Berechnung von Hypercubes gebraucht wird.
 * Die Blaetter des Baums representieren die Hypercubes, in denen gespeichert wird welche und wieviele Partikel sich in dem 
 * entsprechenden Hypercube befinden.  
 * @author janas
 *@version $Revision: 726 $
 * $Date: 2005-02-02 00:50:20 +0100 (Mi, 02 Feb 2005) $
 * To change the template for this generated type comment go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
public class HyperCubeTree {
	/**
	 *  Dimension des Raums , in dem die Cubes erstellt werden
	 */
	private int dim = 0;	
	/**
	 * //  gibt die Intervalle  des Zielraums an, in dem sich die Partickel befinden, ein
	 */											
	private double [][] intervall ;               
	/**
	 * gibt die Zahl an, in die jedes Intervall geteilt wird
	 */                  
	private int divisions    ; 
	/**
	 * Wurzel des Baumes
	 */                                        
    private Vertex root;                    
    /**
     * enthaelt die Vektoren(als ArrayListen implementiert ) ,die in den Baum eingefuegt werden  
     */                         
    private ArrayList repository ;		
    /**
     *  Liste der Blaetter, die die Hypercubes representieren.
     *  Die Liste enthaelt ausschliesslich Objekte des Typs Leaf.
     */						   
    private ArrayList leafs = new ArrayList();           
    
    /**
     * Kostruktor : Es wird ein Baum mit Wurzel root erzeugt, in den mit der Methode addParticle Vektoren eingefuegt werden
     * @param dimension   -Dimension des Raumes
     * @param quantityHypercubes  - Anzahl der Hypercubes ,in die der Raum geteilt werden soll 
     * @param repFit  - Vektoren des Raums, die in die Hypercubes eingefuegt werden sollen
     */
    public HyperCubeTree(int dimension,int quantityHypercubes, ArrayList repFit){
    	this.dim = dimension ;
    	intervall = new double[dim][2] ;
    	divisions =(int) Math.round(Math.pow(quantityHypercubes, 1.0/dim));
    	repository = repFit ;
    	root	= new Vertex( divisions);
    }
    /**
     * Die Methode fuegt alle dem Konstuktor uebergebenen Vektoren in den Baum ein und 
     * gibt die Blaetter(Leafs) des Baums, die die Hypercubes representieren, in einer aufsteigend sortierten ArrayListe zurueck 
     * 
     * @return -aufsteigend sortierte Liste der Hypercubes
     */
    public ArrayList getCubes(){
		for( int j = 0 ; j< dim ; j++){
			double d = ((Double)((ArrayList)repository.get(0)).get(j)).doubleValue();
			intervall[j][0] =  intervall[j][1]  = d;
		}
		    
    	for( int i =1 ; i < repository.size() ; i++)
    		for( int j = 0 ; j< dim ; j++)
    		{
    			double d = ((Double)((ArrayList)repository.get(i)).get(j)).doubleValue();
    			if (d < intervall[j][0]  )
    			   intervall[j][0] = d ;
    			   else if(d > intervall[j][1])
    			            intervall[j][1] = d;
    		}
    		   
    		
    	for (int i =0 ; i < repository.size() ; i++)
    	    this.addParticle((ArrayList)repository.get(i),new Integer(i));
    	    
    	Object [] obj = leafs.toArray();
    	
    	Arrays.sort(obj);
    	leafs = new ArrayList();
    	for(int i =0 ; i < obj.length ; i++)
    	   leafs.add(obj[i]);       
    	return leafs ;
    }
    /**
     * Die Methode fuegt den Vektor part in den Baum unter dem Schluessel i ein
     * @param part -Vector, der in den Baum eingefuegt werden soll
     * @param i - Schluessel , unter dem der Vektor in den Baum eingefuegt wird
     */
    private void addParticle(ArrayList part,Integer i){
    	addParticle(root, dim-1, part,i);
    }
    /**
     * rekursive Methode um das Einfuegen zu ermoeglichen 
     * @param ver -aktueller Knoten
     * @param aktDim - Hohe des aktuellen Knotens im Baum
     * @param part -Vector, der in den Baum eingefuegt werden soll
     * @param i - Schluessel , unter dem der Vektor in den Baum eingefuegt wird
     */
	private void addParticle(Vertex ver , int aktDim ,ArrayList part,Integer i){
		if(aktDim == -1)
		{
			Leaf leaf = (Leaf)ver ;
			leaf.addInteger(i);
			leaf.setQuantity(leaf.getQuantity() +1)  ;
        }else{
           int in =(int)Math.floor((((Double)part.get(aktDim)).doubleValue()-intervall[aktDim][0]) * divisions / (intervall[aktDim][1] - intervall[aktDim][0]) );
           if(in == divisions)
              in -- ;
           if(aktDim > 0){
           	 if(ver.child[in] ==null)
           	     ver.child[in] = new Vertex( divisions);
           	 addParticle(ver.child[in], aktDim -1 , part, i);
           }else{
           	 
           	  if(ver.child[in] == null)
           	  {
           	     ver.child[in] = new Leaf( i);
           	     leafs.add(ver.child[in]);
             }else addParticle(ver.child[in], aktDim-1, part , i);
        }
        
	}
	}
}