/*
 * Copyright (C) 2005  PG447 - University Dortmund
 * 
 * This file is part of NObELTJE.
 * 
 * NObELTJE is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as published by
 * the Free Software Foundation.
 * 
 * NObELTJE is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with NObELTJE; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

 package org.nobeltje.tools;
import java.util.ArrayList;

/**
 * <p>Klasse, die die Pareto Dominanz testet und das Archiv daraufhin durchlauft und veraendert.</p>
 * <p>Die Klasse hat 2 Methoden: eine, die das Archiv dahingehend durchlauft, dass die Ausgabe aus einen Archiv besteht, 
 * das nur dominierte Elemente enthaellt und eine Methode, die ein Archiv zurueck gibt, das nur nicht-dominierte Elemente 
 * enthaelt.</p>
 * 
 * @author $Author: noot $
 * @version $Revision: 726 $
 * $Date: 2005-02-02 00:50:20 +0100 (Mi, 02 Feb 2005) $
 */

public class ParetoArchive {
	
	/**
	 * Arbeitet auf Archiven von Individuen
	 * <p>Erstellt auf Basis des alten Archivs ein neues mit Maximalgroesse archiveSize.</p>
	 * <p>Zusaetzlich wird das neue Individuum ind hinzugefuegt, falls dieses nicht-dominiert ist. <br />
	 * Alle dominierten Elemente werden aus dem  Archiv geloescht.<br />
	 * Individuen mit gleichem Fitnesswert werden nicht zugelassen.<br />
	 * Wird fuer archiveSize der Wert '0' uebergeben ist die Archivgroesse unbeschraenkt.</p>
	 * Die Dominanz der Elemente untereinander wird nicht ueberprueft, sondern nur, bzgl
	 * des neuen einzufuegenden Individuums. Wenn also oldArchive bereits dominierte 
	 * Elemente enthaehlt, bleiben diese enthalten.
	 * 
	 * 
	 * @param archiveSize int maximale Archivgrosse
	 * @param oldArchiv ArrayList urpruengliches Archiv von pareto-optimalen(!) Individuen
	 * @param ind Individual Individuum, was getestet und ggf. eingefuegt werden soll
	 * @return neues Archiv von pareto-optimalen Individuen
	 * */
	public static ArrayList refreshArchive(int archiveSize, ArrayList oldArchive, Individual ind) {
		
		ArrayList indFit = ind.getFitness(); // Fitnesswerte von ind 
		ArrayList archiveIndFit; // Fitnesswerte der aktuellen ArchivLoesung
		ArrayList archiveNew = new ArrayList(); // neues Archiv wird zurueckgegeben
		int domIndex; // Rueckgabewert von ParetoDomination.checkDomination
		boolean indNonDom = true; // ind wird nicht dominiert
		
		for (int i=0; i<oldArchive.size(); i++) {
			archiveIndFit = ((Individual) oldArchive.get(i)).getFitness();
			domIndex = ParetoDomination.checkDomination(archiveIndFit, indFit);
			
			// fuege Elemente aus oldArchive in newArchive ein, falls sie 
			// nicht von ind dominiert werden
			if (domIndex != 1) {
				// dom=1 ist derFall, das ind die ArchivLoesung dominiert
				archiveNew.add((Individual)oldArchive.get(i));
			}
			
			if ( (domIndex == 2) || (domIndex == 3) ) {
				// ind wird dominiert  oder
				// eine Kopie von ind ist bereits imArchiv vorhanden
				// ind soll daher nicht eingefuegt werden
				indNonDom = false;
			}
		}
		
		// fuege ind ins neue Archiv ein
		if (indNonDom)
			archiveNew.add(ind);
		
		// Archivgroesse ueberpruefen
		if (archiveSize != 0) {
			// 0 ist der Code fuer ein Archive mit unbeschraenkter Groesse
			while(archiveNew.size() > archiveSize) {
				// entferne erstes (vermutlich aeltestes) Individuum
				archiveNew.remove(0);
			}
		}
		
		return archiveNew;
	}
	
	
	/**
	 * Arbeitet auf Archiven von Fitnesswerten, NICHT Individuen!
	 * <p>Erstellt auf Basis des alten Archivs ein neues mit Maximalgroesse archiveSize.</p>
	 * <p>Zusaetzlich wird der neue Fitnesswert indFit hinzugefuegt, falls dieser nicht-dominiert ist. <br />
	 * Alle dominierten Elemente werden aus dem  Archiv geloescht.<br />
	 * Doppelte Fitnesswerte werden nicht zugelassen.<br />
	 * Wird fuer archiveSize der Wert '0' uebergeben ist die Archivgroesse unbeschraenkt.</p>
	 * Die Dominanz der Elemente untereinander wird nicht ueberprueft, sondern nur, bzgl
	 * des neuen einzufuegenden Individuums. Wenn also oldArchive bereits dominierte 
	 * Elemente enthaehlt, bleiben diese enthalten.
	 * 
	 * 
	 * @param archiveSize int maximale Archivgrosse
	 * @param oldArchiv ArrayList urpruengliches Archiv von pareto-optimalen(!) Loesungen (Fitnesswerten)
	 * @param indFit ArrayList Einzufuegender Fitnesswert
	 * @return neues Archiv von pareto-optimalen Loesungen (Fitnessewerten)
	 * */
	public static ArrayList refreshArchive(int archiveSize, ArrayList oldArchive, ArrayList indFit) {
		
		ArrayList archiveIndFit; // Weret der aktuellen ArchivLoesung
		ArrayList archiveNew = new ArrayList(); // neues Archiv wird zurueckgegeben
		int domIndex; // Rueckgabewert von ParetoDomination.checkDomination
		boolean indNonDom = true; // indFit wird nicht dominiert
		
		for (int i=0; i<oldArchive.size(); i++) {
			archiveIndFit = (ArrayList)oldArchive.get(i);
			domIndex = ParetoDomination.checkDomination(archiveIndFit, indFit);
			
			// fuege Elemente aus oldArchive in newArchive ein, falls sie 
			// nicht von ind dominiert werden
			if (domIndex != 1) {
				// dom=1 ist derFall, das indFit die aktuelle ArchivLoesung dominiert
				archiveNew.add((ArrayList)oldArchive.get(i));
			}
			
			if ( (domIndex == 2) || (domIndex == 3) ) {
				// ind wird dominiert  oder
				// eine Kopie von ind ist bereits imArchiv vorhanden
				// ind soll daher nicht eingefuegt werden
				indNonDom = false;
			}
		}
		
		// fuege indFit ins neue Archiv ein
		if (indNonDom)
			archiveNew.add(indFit);
		
		// Archivgroesse ueberpruefen
		if (archiveSize != 0) {
			// 0 ist der Code fuer ein Archive mit unbeschraenkter Groesse
			while(archiveNew.size() > archiveSize) {
				// entferne erstes (vermutlich aeltestes) Individuum
				archiveNew.remove(0);
			}
		}
		
		return archiveNew;
	}
	
	
	/**
	 * <p>Erstellt auf Basis des alten Archivs ein neues mit Maximalgroesse archiveSize.</p>
	 * <p>Zusaetzlich wird das neue Individuum ind hinzugefuegt, falls dieses dominierend ist. <br />
	 * Alle dominierten Elemente werden aus dem  Archiv geloescht.<br />
	 * Doppelte Individuen werden nicht zugelassen.<br />
	 * Wird fuer archiveSize der Wert '0' uebergeben ist die Archivgroesse unbeschraenkt.</p>
	 * 
	 * @param archiveSize int: maximale Archivgrosse
	 * @param oldArchiv ArrayList urpruengliches Archiv von pareto-optimalen(!) Individuen
	 * @param ind Individual: Individuum, was getestet und ggf. eingefuegt werden soll
	 * @return neues Archiv von pareto-optimalen Individuen
	 * */
	public static ArrayList refreshArchiveOld(int archiveSize, ArrayList oldArchiv, Individual ind){
		//System.out.println("in refreshArchiveOld");

		/* Init */
		ArrayList newArchiv= new ArrayList();  // neues Archiv
		boolean isin=false; // Variable, die angibt, ob das neue Individuum bereits im Archiv ist
		boolean better=true; // Gibt an, dass ind nicht dominiert wird (false -> nicht zum Archiv hinzufuegen)  
		//	ParetoDomination Pareto= new ParetoDomination();  // jetzt static
		
		/* Durchgang durch alle Elemente im Archiv und alle 
		 *  pareto-dominanten  Elemente uebernehmen */
		for(int i=0; i<oldArchiv.size(); i ++) {
			/* Element i des Archives wird betrachtet */
			Individual Element=(Individual)oldArchiv.get(i);
			/* falls das i-te Element im Archiv das neue Individuum dominiert, wird das neue Individuum am Ende nicht hinzugefuegt  */
			better = (better && ParetoDomination.nonDominated(Element.getFitness(), ind.getFitness()));
			/* falls Element im Archiv nicht vom Individuum dominiert wird, wird Element in das Archiv aufgenommen. 
			 * Die MaximalGroesse des Archivs wird hier nicht ueberschritten, weil alle Elemente
			 * ja bereits im Archiv waren. Das neue Individuum wird erst am Ende der gesammten Ueberpruefung hinzugefuegt. */
			if (ParetoDomination.nonDominated(ind.getFitness(), Element.getFitness())) {
				newArchiv.add(Element);
				/* Vergleich, ob Individuum bereits im Archiv vorhanden ist, damit dieses spaeter nicht mehr hinzufuegt wird */
				if (Individual.compareObjectVariables(Element, ind) == true)
				{
					isin = true;
				}
			}
		}
			
		/* Falls das neue Element nicht dominiert wird und noch nicht im Archiv ist,
		 * wird es ins neue Archiv gepackt */
		boolean addIsOK = (archiveSize==0) || (newArchiv.size() < archiveSize); //Ist das Archiv voll oder soll es unbegrenzt wachsen?
		if (better && !isin && addIsOK) {
			newArchiv.add(ind);
			isin = true;
		}
		
		/* Ausgabe des neuen Archivs */
		return newArchiv;
	}
	
	/**
	 * <p>Erstellt auf Basis des alten Archivs ein neues mit Maximalgroesse archiveSize.</p>
	 * <p>Zusaetzlich wird das neue Individuum ind hinzugefuegt, falls dieses dominierend ist. <br />
	 * Alle dominierten Elemente werden aus dem  Archiv geloescht.<br />
	 * Doppelte Individuen werden nicht zugelassen.<br />
	 * Wird fuer archiveSize der Wert '0' uebergeben ist die Archivgroesse unbeschraenkt.</p>
	 * 
	 * @param archiveSize int maximale Archivgrosse
	 * @param oldArchiv ArrayList urpruengliches Archiv von pareto-optimalen(!) Loesungen
	 * @param solution ArrayList neue Loesung, die ggf. eingefuegt werden soll
	 * @return neues Archiv von pareto-optimalen(!) Loesungen
	 * */
	public static ArrayList refreshArchiveOld(int archiveSize, ArrayList oldArchive, ArrayList solution){

		/* Init */
		ArrayList newArchiv = new ArrayList();  // neues Archiv
		boolean isin = false; // Variable, die angibt, ob das neue Individuum bereits im Archiv ist
		boolean better = true; // Gibt an, dass ind nicht dominiert wird (false -> nicht zum Archiv hinzufuegen)  
		boolean equal = true;
		
		/* Durchgang durch alle Elemente im Archiv und alle 
		 *  pareto-dominanten  Elemente uebernehmen */
		for(int i=0; i<oldArchive.size(); i++) {
			// Element_i des Archives wird betrachtet
			
			ArrayList oldArchive_i = (ArrayList)oldArchive.get(i); 
			
			// falls das i-te Element im Archiv die neue Loesung dominiert,
			// wird die neue Loesung nicht hinzugefuegt
			better = (better && ParetoDomination.nonDominated(oldArchive_i, solution));
			
			/* falls Element_i im Archiv nicht von solution dominiert wird,
			 * wird Element_i in das Archiv aufgenommen. 
			 * Die MaximalGroesse des Archivs wird hier nicht ueberschritten,
			 * weil alle Elemente ja bereits im Archiv waren.
			 * Die neue Loesung wird erst am Ende der gesamten Ueberpruefung hinzugefuegt.
			 */
			if (ParetoDomination.nonDominated(solution, oldArchive_i)) {
				newArchiv.add(oldArchive_i);
			}
			
			/* Vergleich, ob Individuum bereits im Archiv vorhanden ist,
			 *  damit dieses spaeter nicht mehr hinzufuegt wird 
			 */
			equal = true;
			for (int j=0; j<solution.size(); j++) {
				if (oldArchive_i.get(j)!= solution.get(j)) {
					equal = false;
				}
			}
			if (equal)
				isin = true;
		}
			
		/* Falls das neue Element nicht dominiert wird und noch nicht im Archiv ist,
		 * wird es ins neue Archiv gepackt */
		boolean addIsOK = (archiveSize==0) || (newArchiv.size() < archiveSize); //Ist das Archiv voll oder soll es unbegrenzt wachsen?
		if (better && !isin && addIsOK) {
			newArchiv.add(solution);
			isin = true;
		}
		
		/* Ausgabe des neuen Archivs */
		return newArchiv;
	}

	
	/**
	 * <p>Erstellt auf Basis des alten Archivs ein neues Archiv.</p> 
	 * <p>Dazu werden die nicht dominierten Individuen aus 
	 * dem Archiv geworfen. Am Ende enthaelt das Archiv also nur noch Individuuen, die von einem 
	 * anderen Individuum dominiert werden.</p>
	 *
	 * @param oldArchive ArrayList: urspruengliches Archiv
	 *  @return ArrayList: neues Archiv mit dominierten Elementen
	 */
public static ArrayList refreshDominatedArchive(ArrayList oldArchive){
		// wirft die nicht dominierten Individuen aus dem Archiv, das Archiv enthaelt nachher also nur noch 
		// Individuen, die von irgendwem dominiert werden
		// Archiv von nicht-dominanten Individuuen, um ggf. die Diversitaet bei einigen Algortihmen zu gewaehrleisten.

		ArrayList newArchive= new ArrayList();  // neues Archiv
		boolean add; // Gibt an, dass ind  dominiert wird (false -> nicht zum Archiv hinzufuegen)  

	for (int i = 0; i<oldArchive.size(); i++){
		Individual indI =(Individual)oldArchive.get(i);

		add=false; // Gibt an, dass ind  dominiert wird (false -> nicht zum Archiv hinzufuegen)  


		for (int k=0; k<oldArchive.size(); k++){
			if (i != k){ 										// damit das Individuum nicht mit sich selbst verglichen wird 

				
				// falls das i-te-Individuum von irgendeinem k-ten Individuum dominiert wird, soll es (das i-te) ins neue Archiv hinzugefuegt werden
				Individual indK = (Individual)oldArchive.get(k); 
				
				int dom = ParetoDomination.checkDomination(indK.getFitness(), indI.getFitness());
				if (dom == 2){
					add = true;
				}// Ende: if dom...
				 
			}// Ende: if (i!=k...
						
		}// Ende: for (k...
		
		if (add) newArchive.add(indI);	

	}// Ende for (i...

		/* Ausgabe des neuen Archivs */
		return newArchive;
	} // Ende: refreshDominatedArchive

}// Ende der Klasse
