/*
 * Copyright (C) 2005  PG447 - University Dortmund
 * 
 * This file is part of NObELTJE.
 * 
 * NObELTJE is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as published by
 * the Free Software Foundation.
 * 
 * NObELTJE is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with NObELTJE; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

 package org.nobeltje.tools;

import java.util.ArrayList;

/**
 * <p>Klase, die testet, ob ein Individuum besser ist als ein anderes. Kriterium ist hierbei der Vergleich nach Pareto.<br />
 * Die Klasse besitzt 2 Methoden, eine, die nur boolean als Ruckgabewert angibt, ob eine Dominanz vorliegt, und eine
 * Methode, die die Art der Dominanz als Integerwert zurueck gibt.</p>
 * 
 * @author $Author: noot $
 * @version $Revision: 726 $
 * $Date: 2005-02-02 00:50:20 +0100 (Mi, 02 Feb 2005) $
 */
public class ParetoDomination {
	
	/** ind2 dominiert ind1 */
	public final static int DOMINATION_IND2 =  1;
	
	/** ind1 dominiert ind2 */
	public final static int DOMINATION_IND1 =  2;

	/**
	 *  Variable, die ausgegeben wird, wenn beide Individuuen gleiche f(x) werte haben
	 */
	public final static int DOMINATION_EQUAL = 3;
	
	/** Variable, die ausgebeben wird, wenn die Werte nicht vergleichbar sind miteinander*/
	public final static int DOMINATION_NONE = 4;


/**
 * <p>Ueberprueft, ob das zweite uebergebene Individuum von dem ersten dominiert wird</p>
 * @param ind1 Die 2 zu vergleichenden Individuuen: Individuum 1: ArrayList
 * @param ind2 das mit Individum 1 zu vergleichende Indivuduum: ArrayList
 * @return  boolschen Wert false, wenn ind2 wirklich von ind1 dominiert wird, sonst true
 */
	public static boolean nonDominated(ArrayList ind1, ArrayList ind2) {
		if(checkDomination(ind1, ind2) == DOMINATION_IND1){
			return false;
		}else{
		return true;
		}
	}
	/**
	 * <p>Methode, die die genaue Art der Paretodominanz als Integer zurueckgibt.<br />
	 * Als Rueckgabe gibt es folgende Moeglichkeiten:<br />
	 * <ul>
	 * <li>Int 4: Wert, der ausgebeben wird, wenn die Individuen nicht miteinander vergleichbar sind</li>
	 * <li>Int 3 : Wert, der ausgegeben wird, wenn beide Individuen gleiche f(x)- Werte haben</li>
	 * <li>Int 2 : Wert,  ausgegeben wird, wenn Ind1 Ind2 dominiert, d.h. an mindestens einem Funktionswert einen besseren Wert hat</li>
	 * <li>Int 1: Wert, der ausgegeben wird, wenn Ind2 Ind1 dominiert.</li></ul></p>
	 * 
	 * @param ind1 : ArrayList: Die zu vergleichenden Individuen: Individuum1
	 * @param ind2 ArrayListe: Individuum 2 
	 * @return Integerwert, der die Art der Dominanz angibt
	 * 
	 */
	public static int checkDomination(ArrayList ind1, ArrayList ind2) {
		
			/* Init */
			boolean allEqual= true; //alle x-Werte der zu vergleichenden Elemente sind gleich
			boolean oneWorse = false; //mindestens ein Wert von ind2 ist schlechter als bei dem ind1-individuum
			boolean oneBetter = false; // mindestens ein Wert von ind2 ist besser als von ind1
		
			/* Durchlaufe Fitnesswerte- Vektor*/
			for(int i=0; i<ind2.size(); i++){
				/*an der Postion i ist ind2 besser als ind1*/
				if( ((Double)ind2.get(i)).doubleValue() < ((Double)ind1.get(i)).doubleValue()){
					oneBetter= true;
					allEqual= false;
				}
				else
				/*an der Position i ist ind1 besser als ind2*/
				if(((Double)ind2.get(i)).doubleValue() > ((Double)ind1.get(i)).doubleValue()){
					oneWorse= true;
					allEqual= false;
				}
			}
			/*falls im Archiv Kopien erwuenscht sind*/
			if (allEqual) return DOMINATION_EQUAL;
		
			/*ind2 dominiert Archivloesung ind1*/
			if (oneBetter && !oneWorse) return DOMINATION_IND2;
		
			/*ind2 von ind1 dominiert*/
			if (oneWorse && !oneBetter) return DOMINATION_IND1;
		
			/*ansonsten: ind1 dominert ind2 nicht, ind2 dominiert ind1 nicht*/
			return DOMINATION_NONE;
		}


}
