/*
 * Copyright (C) 2005  PG447 - University Dortmund
 * 
 * This file is part of NObELTJE.
 * 
 * NObELTJE is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as published by
 * the Free Software Foundation.
 * 
 * NObELTJE is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with NObELTJE; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

 /*
 * Created on 24.06.2004
 */
package org.nobeltje.tools;

import java.util.ArrayList;

/**
 * Die Klasse Particle kapselt alle noetigen Daten fuer Partikel, wie
 * sie in den Particle Swarm Optimization Algorithmen verwendet werden.<br>
 * 
 * @author $Author: noot $
 * @version $Revision: 726 $
 * $Date: 2005-02-02 00:50:20 +0100 (Mi, 02 Feb 2005) $
 */
public class Particle implements Cloneable
{
	private ArrayList position = new ArrayList();
	private ArrayList velocity = new ArrayList();
	private ArrayList pBest = new ArrayList();
	private ArrayList fitness = new ArrayList();
	private ArrayList pBestFit = new ArrayList();

	/**
	 * Berechnet die neue Position des Partikels, indem auf die aktuelle
	 * Position der Geschwindigkeitsvektor addiert wird
	 */
	public void updatePosition()
	{
		for (int i = 0; i < position.size(); i++)
			position.set(i, new Double(((Double)position.get(i)).doubleValue() + ((Double)velocity.get(i)).doubleValue()));
	}

	/**
	 * Erstellt eine Kopie des Partikels
	 *   
	 * @return Gibt den geklonten Partikel zurueck
	 * @see java.lang.Object#clone()
	 */
	public Object clone()
	{
		Particle p = new Particle();

		p.position	= (ArrayList)this.position.clone();
		p.velocity	= (ArrayList)this.velocity.clone();
		p.pBest		= (ArrayList)this.pBest.clone();
		p.fitness	= (ArrayList)this.fitness.clone();
		p.pBestFit = (ArrayList)this.pBestFit.clone();

		return p;
	}

	
	/**
	 * @return Gibt die Position des Partikels in einer ArrayList zurueck
	 */
	public ArrayList getPosition()
	{
		return position;
	}

	/**
	 * Setzt die Position des Partikels
	 * 
	 * @param list , enthaelt die Daten der Position
	 */
	public void setPosition(ArrayList list)
	{
		position = list;
	}

	/**
	 * @return Gibt den Geschwindigkeitsvektor des Partikels in einer ArrayList
	 * zurueck
	 */
	public ArrayList getVelocity()
	{
		return velocity;
	}

	/**
	 * Setzt den Geschwindigkeitsvektor des Partikels
	 *  
	 * @param list , enthaelt die Daten des Geschwindigkeitsvektors
	 */
	public void setVelocity(ArrayList list)
	{
		velocity = list;
	}

	/**
	 * @return Gibt die bisher beste, erreichte Position des Partikels in einer
	 * ArrayList zurueck
	 */
	public ArrayList getPBest()
	{
		return pBest;
	}

	/**
	 * Setzt die bisher beste, erreichte Position des Partikels
	 * 
	 * 
	 * @param list , enthaelt die Daten der besten Position
	 */
	public void setPBest(ArrayList list)
	{
		pBest = list;
	}

	/**
	 * @return Gibt die Fitness Werte fuer die aktuelle Position des Partikels
	 * in einer ArrayList zurueck
	 */
	public ArrayList getFitness()
	{
		return fitness;
	}
	
	/**
	 * Setzt die Fitness Werte fuer die aktuelle Position des Partikels
	 * 
	 * @param list , enthaelt die Daten fuer die Fitness Werte
	 */
	public void setFitness(ArrayList list)
	{
		fitness = list;
	}
	
	/**
	 * @return Gibt die Fitness Werte fuer die bisher beste, erreichte Position
	 * des Partikels in einer ArrayList zurueck
	 */
	public ArrayList getPBestFit(){
		return pBestFit ;
	}
	
	/**
	 * Setzt die Fitness Werte fuer die bisher beste, erreichte Position des
	 * Partikels
	 * 
	 * @param list , enthaelt die Daten fuer die Fitness Werte
	 */
	public void setPBestFit(ArrayList list)
	{
		pBestFit = list ;
	}
}
