/*
 * Copyright (C) 2005  PG447 - University Dortmund
 * 
 * This file is part of NObELTJE.
 * 
 * NObELTJE is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as published by
 * the Free Software Foundation.
 * 
 * NObELTJE is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with NObELTJE; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

 /*
 * Created on 24.06.2004
 * 
 */
package org.nobeltje.tools;

import java.util.Random;

import org.nobeltje.exceptions.InvalidBoundException;


/**
 * Generator von Pseudo-Zufallszahlen, der von der Klasse Random erbt.
 * Es gibt eine zusaetzliche Methode nextDouble(lowerBound, upperBound) 
 * und Zugriffsmethoden auf den Seed.
 * 
 * @author $Author: noot $
 * @version $Revision: 726 $
 * $Date: 2005-02-02 00:50:20 +0100 (Mi, 02 Feb 2005) $
 * 
 * @see <a href ="http://java.sun.com/j2se/1.4.2/docs/api/java/util/Random.html">
 * Super-Klasse: Random</a>
 */
public class RandomPG extends Random {
	
	private long seed;		// Startwert der Folge von Pseudozufallszahlen

	/**
	 * Erzeugt ein Objekt von RandomPG.
	 * Setzt den seed per klasseneigener Methode setSeed,
	 * abhaengig von der Systemzeit. 
	 *
	 */
	public RandomPG() {
		// Aufruf der klasseneigenen Methode setSeed()
		setSeed(System.currentTimeMillis());
	}
	
	/**
	 * Konstruktor, der einen Zufallsgenerator erzeugt, mit dem uebergebenen
	 * Parameter als seed.
	 * 
	 * @param seed : Ursprung der Zufallsfolge
	 */
	public RandomPG(long seed) {
		// Aufruf der klasseneigenen Methode setSeed()
		setSeed(seed);
	}
	
	/**
	 * Gibt den random seed zurueck.
	 * 
	 * @return seed: Ursprung der Zufallsfolge
	 */
	public long getSeed() {
		return seed;
	}
	
	/**
	 * Setzt den random seed auf den uebergebenen Wert.
	 *
	 * @param seed Ursprung der Zufallsfolge
	 */
	public void setSeed(long seed) {
		this.seed = seed;
		// Aufruf der Methode setSeed() von Random
		super.setSeed(seed);
	}

	/**
	 * Liefert eine (double) Zufallszahl zwischen lowerBound (inkl.)
	 * und upperBound (exkl.).
	 * 
	 * @param lowerBound untere Grenze
	 * @param upperBound obere Grenze
	 * @return randomNumber: Ein Zufallswert zwischen 'lowerBound' und 'upperBound'
	 * @throws InvalidBoundException falls lowerBound > upperBound
	 */
	public double nextDouble(double lowerBound, double upperBound)  throws InvalidBoundException{
		double randomNumber;
		if (upperBound < lowerBound)	// ungueltige Parameter?
			throw new InvalidBoundException("lowerBound:" +lowerBound+" > upperBound"+upperBound+"."
				+" Es kann keine Zufallszahl in diesem Intervall erzeugt werden.");

		randomNumber = lowerBound + nextDouble() * (upperBound - lowerBound);
		return randomNumber;
	}
}
