/*
 * Copyright (C) 2005  PG447 - University Dortmund
 * 
 * This file is part of NObELTJE.
 * 
 * NObELTJE is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as published by
 * the Free Software Foundation.
 * 
 * NObELTJE is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with NObELTJE; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

 /*
 * Created on 27.06.2004
 */
package org.nobeltje.tools.io;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Calendar;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import java.util.StringTokenizer;

import org.nobeltje.tools.Individual;
import org.nobeltje.exceptions.NoIndividualException;

/**
 * Schreibt die Ergebnisse eines Algorithmus in eine Datei.
 * 
 * @author $Author: noot $
 * @version $Revision: 726 $
 * $Date: 2005-02-02 00:50:20 +0100 (Mi, 02 Feb 2005) $
 */

public class AlgoOutput {

	
	/**
	 * Gibt die Fitnesswerte (Paretofront) des Archivs aus.
	 * 
	 * @param data Auszugebende Daten als List von Individuen 
	 * @param namePrefix Dateiname, angehaengt wird.best.res
	 * @param append anhaengen oder Datei neu erstellen?
	 * @throws NoIndividualException
	 */
	public static void writeFitness(List data,
			                        String namePrefix,
									boolean append) throws NoIndividualException {
		Locale.setDefault(Locale.ENGLISH);
		
		try {
		
			/* Ausgabedatei-Handle erstellen */	
			String name="";
			StringTokenizer outToken = new StringTokenizer(namePrefix,"_");
			while (outToken.hasMoreElements()){
				String out = outToken.nextToken();
				if (out.startsWith("ttc=")){
					StringBuffer outBuffer = new StringBuffer(out);
					while (outBuffer.indexOf(File.separator)>=0){
						outBuffer.replace(outBuffer.indexOf(File.separator),outBuffer.indexOf(File.separator)+1,"_");
					}
					out = outBuffer.toString();
				}
				name+=(out+"_");
			}
			name = name.substring(0,name.length()-1);
			FileWriter fw = new FileWriter(name + ".best.res", append);
		   
			/* Durch alle Archiv-Elemente iterieren */
			for (Iterator iter = data.iterator(); iter.hasNext(); ) {
				
				/* Naechstes Objekt in data nach Pruefung als Individuum casten */
				Individual outputIndividual;
				Object nextObject = iter.next();
				if (nextObject instanceof Individual) {
					outputIndividual = (Individual)nextObject;
				} else  {
					throw new NoIndividualException();
				}
						
				StringBuffer row = new StringBuffer();  // Neuen Stringbuffer fuer Ausgabe erstellen
				Double value;
				String decimal;	
				
				/* Durch alle Fitnesswerte eines Individuums iterieren */
				for (Iterator iter2 = outputIndividual.getFitness().iterator(); iter2.hasNext(); ) {
					value = (Double) iter2.next();
					decimal = Number.getDecimal(value);
					/* Alle Werte eines Ind. in eine Zeile und erst spaeter ausgeben schreiben */ 
					row.append(decimal+" ");
				}
			    /* Ausgabe einer Zeile / eines Indiviuums */	
				fw.write(row.toString()+"\n");
			
			}// for
			
		   /* Ausgabedatei schliessen */	
			fw.close();
			
		} // try
	
		catch (IOException e)  {
			e.printStackTrace();
		}
		
	}// writeFitness

	/**
	 * Schreibt die uebergebene Liste <b>data</b> von Individuen in eine Ausgabedatei
	 *  <b>namePrefix</b>.best.in.
	 * 
	 * @param data Liste von Individuen
	 * @param namePrefix Dateiname
	 * @param append anhaengen?
	 * @throws NoIndividualException
	 */
	public static void writeObjectVariables(List data,
			                                String namePrefix,
											boolean append) throws NoIndividualException {
		Locale.setDefault(Locale.ENGLISH);
	
		try {
			
			/* Ausgabedatei-Handle erstellen */	
			String name="";
			StringTokenizer outToken = new StringTokenizer(namePrefix,"_");
			while (outToken.hasMoreElements()){
				String out = outToken.nextToken();
				if (out.startsWith("ttc=")){
					StringBuffer outBuffer = new StringBuffer(out);
					while (outBuffer.indexOf(File.separator)>=0){
						outBuffer.replace(outBuffer.indexOf(File.separator),outBuffer.indexOf(File.separator)+1,"_");
					}
					out = outBuffer.toString();
				}
				name+=(out+"_");
			}
			name = name.substring(0,name.length()-1);
			FileWriter fw = new FileWriter(name + ".best.in", append);

			/* Durch alle Archiv-Elemente iterieren */
			for (Iterator iter = data.iterator(); iter.hasNext(); ) {
				
				/* Naechstes Objekt in data nach Pruefung als Individuum casten */
				Individual outputIndividual;
				Object nextObject = iter.next();
				if (nextObject instanceof Individual) {
					outputIndividual = (Individual)nextObject;
				} else  {
					throw new NoIndividualException();
				}
						
				StringBuffer row = new StringBuffer();  // Neuen Stringbuffer fuer Ausgabe erstellen
				Object value;
				String decimal;	
				
				/* Durch alle Fitnesswerte eines Individuums iterieren */
				for (Iterator iter2 = outputIndividual.getObjectVariables().iterator(); iter2.hasNext(); ) {
					value = iter2.next();
					if (value instanceof Double) {
						decimal = Number.getDecimal((Double)value);
						/* Alle Werte eines Ind. in eine Zeile und erst spaeter ausgeben schreiben */ 
						row.append(decimal+" ");
					} else {
						/* Alle Werte eines Ind. in eine Zeile und erst spaeter ausgeben schreiben */ 
						row.append(value.toString()+" ");
					}
				}
				
			    /* Ausgabe einer Zeile / eines Indiviuums */	
				fw.write(row.toString()+"\n");
			
			}// for
			
			/* Ausgabedatei schliessen */	
			fw.close();
		
		} // try
	
		catch (IOException e)  {
			e.printStackTrace();
		}
	
	}// writeObjectVariables

	
	/**
	 * Gibt das aktuelle Datum und die Zeit als String zurueck in der Form
	 * YYYY-MM-DD_HH-MM-SS. <br>
	 * Das wird als Standard fuer die Namen der Ausgabedateien verwendet.
	 * 
	 * @return String des aktuellen Datums
	 */
	public static String getFileNameByDate() {
		String res = "";
		int help;
	    Calendar calendar = Calendar.getInstance();
	    
		// Jahr
	    res = String.valueOf(calendar.get(Calendar.YEAR)) + "-";
		// Monat
	    help = calendar.get(Calendar.MONTH) + 1;
	    if (help < 10) res += "0";
	    res += help + "-";
		// Tag
	    help = calendar.get(Calendar.DAY_OF_MONTH);
	    if (help < 10) res += "0";
	    res += help + "_";
		// Stunde
	    help = calendar.get(Calendar.HOUR_OF_DAY);
	    if (help < 10) res += "0";
	    res += help + "-";
		// Minute
	    help = calendar.get(Calendar.MINUTE);
	    if (help < 10) res += "0";
	    res += help + "-";
		// Sekunde
	    help = calendar.get(Calendar.SECOND);
	    if (help < 10) res += "0";
	    res += help;

	    return res;
	
	}// getFileNameByDate
	
	
	
	/** 
	 * nur zum spielen... 
	 *
	 */
	public static void main (String[] a) throws Exception {
/*
 		FileWriter fw = new FileWriter("best.res");
		fw.write("test1\n");
		fw.write("test1\n");
		StringBuffer row = new StringBuffer();  // Neuen Stringbuffer fuer Ausgabe erstellen
		row.append(0.000000000000000000000000000000000000000001d);
		fw.write(row.toString());
		fw.close();
		
		FileWriter fw2 = new FileWriter("best.res", true);
		fw2.write("test2");
		fw2.close();
		
		Calendar calendar = Calendar.getInstance();
		String outputName = calendar.get(Calendar.YEAR) + "-" + calendar.get(Calendar.MONTH) + "-" + calendar.get(Calendar.DAY_OF_MONTH) + "_" + calendar.get(Calendar.HOUR_OF_DAY) + "-" + calendar.get(Calendar.MINUTE);
		System.out.println(outputName);
*/
	    System.out.println(getFileNameByDate());
	    
	}
	
	
}
