/*
 * Copyright (C) 2005  PG447 - University Dortmund
 * 
 * This file is part of NObELTJE.
 * 
 * NObELTJE is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as published by
 * the Free Software Foundation.
 * 
 * NObELTJE is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with NObELTJE; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

 /*
 * Created on 15.06.2004
 *
 */
package org.nobeltje.tools.io;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Properties;

import org.nobeltje.exceptions.MalInputFormatException;

/**
 * @author $Author: noot $
 * @version $Revision: 726 $
 * $Date: 2005-02-02 00:50:20 +0100 (Mi, 02 Feb 2005) $
 * 
 * Diese Klasse soll die Eingabe der Parameter f?r einen Algorithmus wahlweise ueber die Kommandozeile
 * oder ueber eine Eingabedatei vereinfachen.
 * Dazu werden verschiedene Hilfsmethoden bereitgestellt. 
 */

public class ArgsInputBeautifier
{

	 /**
	 * Die Methode beautify bekommt als Eingabe den String-Array, der aus der main Methode stammt.
	 * Die Methode macht fuer jeden Eintrag des Arrays einen Aufruf der Methode decide(). In dieser
	 * wird entschieden, ob es sich um einen Eintrag der Form -mue 200 (also 2 Werte) oder der Form
	 * -d (ohne zusaetzlichen Wert) handelt.
	 * Im ersten Fall wird der String hinter dem - (hier also mue) als Schluessel in eine Hashmap eingetragen
	 * Der zugehoerige Wert (also der folgende Wert im Array( hier 200)) wird als Wert zu diesem Schluessel
	 * in der Hashmap notiert. In diesem Fall muss die Variable i um 2 erhoeht werden, um an der richtigen
	 * Stelle weiter zu machen.
	 * Wenn, wie im zweiten Fall, kein Wert folgt, wird auch ein Eintrag in der hashmap abgelegt. Statt einem
	 * Wert wird einfach der String "true" eingetragen.
	  */
	public static Properties beautify (String [] args) throws MalInputFormatException {
		return beautify(null, args);
	}
	

	/**
	 * Die Methode beautify bekommt als Eingabe den String-Array, der aus der main Methode stammt.
	 * Die Methode macht fuer jeden Eintrag des Arrays einen Aufruf der Methode decide(). In dieser
	 * wird entschieden, ob es sich um einen Eintrag der Form -mue 200 (also 2 Werte) oder der Form
	 * -d (ohne zusaetzlichen Wert) handelt.
	 * Im ersten Fall wird der String hinter dem - (hier also mue) als Schluessel in eine Hashmap eingetragen
	 * Der zugehoerige Wert (also der folgende Wert im Array( hier 200)) wird als Wert zu diesem Schluessel
	 * in der Hashmap notiert. In diesem Fall muss die Variable i um 2 erhoeht werden, um an der richtigen
	 * Stelle weiter zu machen.
	 * Wenn, wie im zweiten Fall, kein Wert folgt, wird auch ein Eintrag in der hashmap abgelegt. Statt einem
	 * Wert wird einfach der String "true" eingetragen.
	 * Die Daten werden an die uebergebene Properties angehaengt.
	 */
   public static Properties beautify (Properties result, String [] args) throws MalInputFormatException {
   	
   	   // Init
	   if (result == null)
	   		result = new PropertiesPG();
	   
	   // Schleife ueber alle Array-Elemente
	   for ( int i = 0; i < args.length; i++) {
		   if (decide (args[i], (i+1 < args.length) ? args[i+1] : null, result)) {
			   i++;		// nachfolgendes Array-Element gehoert auch zu aktuellem -> i doppelt erhoehen (Schleife + manuell)
		   }
	   }

	   // Rueckgabe
	   return result;
   }
	

	/*
	 * Diese Methode entscheidet, ob ein oder zwei Eintraege des Arrays in einen Properties-Eintrag gehoeren
	 */
	private static boolean decide (String args1, String args2, Properties prop)throws MalInputFormatException {
		/* Check, ob Arrayelement ueberhaupt gueltig (mit "-") anfaengt */
		if (args1.startsWith("-")) {
			if ((args2==null)||(args2.startsWith("-"))) {		// Check, ob das naechste Array-Element noch zum Parameter gehoert
				prop.setProperty(args1.substring(1),"true");	// True zuweisen, weil Parameter alleine steht (z.B. "-d")
				return false;	// false zurueckgeben, damit aufrufende Methode weiss, dass nur ein Array-Element genutzt wurde	
			}
			else {
				prop.setProperty(args1.substring(1), args2);
				return true;	// true zurueckgeben, damit aufrufende Methode weiss, dass zwei Array-Elemente genutzt wurden
			}
		}
		
		/* Arrayelement beginnt fehlerhaft -> Exception werfen */
		else {
			throw new MalInputFormatException(args1, args2);
		}		
	}
	
	
	/**
	 * Die Methode beautify bekommt als Eingabe den String-Array, der aus der main Methode stammt.
	 * Die Methode macht fuer jeden Eintrag des Arrays einen Aufruf der Methode decide(). In dieser
	 * wird entschieden, ob es sich um einen Eintrag der Form -mue 200 (also 2 Werte) oder der Form
	 * -d (ohne zusaetzlichen Wert) handelt. 
	 * Ausserdem wird bestimmt, ob der Eintrag fuer den Algorithmus oder die Testfkt bestimmt ist.
	 * <br>Im ersten Fall (Form bestimmen) wird der String hinter dem - (hier also mue) als Schluessel in eine Hashmap eingetragen
	 * Der zugehoerige Wert (also der folgende Wert im Array( hier 200)) wird als Wert zu diesem Schluessel
	 * in der Hashmap notiert. In diesem Fall muss die Variable i um 2 erhoeht werden, um an der richtigen
	 * Stelle weiter zu machen.
	 * <br>Wenn, wie im zweiten Fall, kein Wert folgt, wird auch ein Eintrag in der hashmap abgelegt. Statt einem
	 * Wert wird einfach der String "true" eingetragen.
	 * <br>Die Daten werden an die uebergebene Properties angehaengt.
	 * <br>Im Unterschied zur methode beautify(Properties, String[]) gibts diese Methode nichts zureuck, da 2 Rueckgaben noetig waeren.
	 * Die Daten muessen also per call-by-reference (CBR) zurueckgegeben werden. 
	 */
   public static void beautify (Properties resultAlgo, Properties resultTestfkt, String [] args) throws MalInputFormatException {
   	
   	   // Init
	   if (resultAlgo == null)
	   	resultAlgo = new PropertiesPG();
	   if (resultTestfkt== null)
   		resultTestfkt = new PropertiesPG();
	   
	   // Schleife ueber alle Array-Elemente
	   for ( int i = 0; i < args.length; i++) {
		   if (decide (args[i], (i+1 < args.length) ? args[i+1] : null, resultAlgo, resultTestfkt)) {
			   i++;		// nachfolgendes Array-Element gehoert auch zu aktuellem -> i doppelt erhoehen (Schleife + manuell)
		   }
	   }

   }
	

	/*
	 * Diese Methode entscheidet, ob ein oder zwei Eintraege des Arrays in einen Properties-Eintrag gehoeren
	 * und ob Parameter zu Algo oder Testfunktion gehoeren
	 */
	private static boolean decide (String args1, String args2, Properties algoProp, Properties testfktProp)throws MalInputFormatException {
		
		/* Check, ob Arrayelement ueberhaupt gueltig (mit "-") anfaengt*/
		if (args1.startsWith("-")) {
			
			/* Check, ob das naechste Array-Element noch zum Parameter gehoert */
			if ((args2==null)||(args2.startsWith("-"))) {
				/* Nur aktuelles Array-Element verwerten */
				if ((args1.length()>2) && (args1.startsWith("-tt"))) {		// Check, wofuer Parameter ist (testfkt/algo)
					testfktProp.setProperty(args1.substring(3), "true");	// Parameter ist fuer Testfunktion
				} 
				else {
					algoProp.setProperty(args1.substring(1), "true");		// Parameter ist fuer Algo
				}
				return false;	// false zuweisen, weil Parameter alleine steht (z.B. "-d")
			}
			
			else {
				/* Auch nachfolgendes aktuelles Array-Element verwerten */
				if ((args1.length()>2) && (args1.startsWith("-tt"))) {		// Check, wofuer Parameter ist (testfkt/algo)
					testfktProp.setProperty(args1.substring(3), args2);		// Parameter ist fuer Testfunktion
				} 
				else {
					algoProp.setProperty(args1.substring(1), args2);		// Parameter ist fuer Algo
				}
			return true;	// true zurueckgeben, damit aufrufende Methode weiss, dass zwei Array-Elemente genutzt wurden	
			}
			
		}
		
		/* Arrayelement beginnt fehlerhaft -> Exception werfen */
		else {
			throw new MalInputFormatException(args1,args2);
		}		
	}
	
	
	/**
	 * Laden einer Properties-Datei einfach ueber den Dateinamen.	 
	 * Mit Hilfe dieser Methode laesst sich eine Hashmap aus einer Datei laden, indem einfach
	 * der Dateiname inkl Pfad als String uebergeben wird.	  * 
	 */
	public static Properties getArgsFromFile (String filename){
		Properties args = new PropertiesPG();
		try
		{
			args.load(new FileInputStream(filename));
		}
		catch (FileNotFoundException e)
		{
			e.printStackTrace();
		}
		catch (IOException e)
		{
			e.printStackTrace();
		}
		return args;
	}
	

	/**
	 * 
	 * @param args Ein java.util.Properties Object, das gespeichert werden soll
	 * @param filename Der Dateiname der Datei
	 * @return true, wenn das Speichern ohne Probleme geklappt hat
	 */
	public static boolean saveArgsToFile (Properties args, String filename)
	{
		try
		{
			args.store(new FileOutputStream(filename),filename);
			return true;
		}
		catch (FileNotFoundException e)
		{
			e.printStackTrace();
			return false;
		}
		catch (IOException e)
		{
			e.printStackTrace();
			return false;
		}
	}
	
	
	/**
	 * @param args Ein String-Array (wie aus der main-Methode), der gespeichert werden soll
	 * @param filename Der dazu passende Dateiname
	 * @return true, wenn alles geklappt hat
	 */
	
	public static boolean saveArgsToFile (String[] args, String filename)
	{
		try
		{
			return saveArgsToFile(beautify(args),filename);
		}
		catch (MalInputFormatException e)
		{
			System.err.println("Die Eingaben entsprechen nicht dem Kommandozeilenformat!");
			e.printStackTrace();
			return false;
		}
	}
	
	/**
	 * Gibt aus dem uebergebenen Objekt vom Typ Properties den Wert zum Schluessel "key" als double zurueck.
	 * Falls ein Fehler auftritt (also der Schluessel z.B. nicht existiert oder sich nicht nach double parsen laesst), wird 
	 * - bei "necessary"=true eine Fehlermeldung ausgegeben und das Programm komplett beendet und
	 * - bei "necessary"=false "defaultvalue" zurueckgegeben.
	 * (Vokabeln: key=Schluessel, defaulvalue=Standardwert, necessary=notwendig)    
	 * 
	 * ! Achtung: Methode ist noch hier, damit alter Code auch noch funktioniert.
	 * ! Es sollte stattdessen stehts die Klasse tools.io.PropertiesPG verwendet werden.    
	 */
	public static double getDoubleProperty (Properties prop, String key, double defaultvalue, boolean necessary) {
	    return PropertiesPG.getDoubleProperty(prop, key, defaultvalue, necessary);
/*	    
	    double result = defaultvalue;
		try {
			result = Double.parseDouble(prop.getProperty(key));
		}
		catch (Exception e1) {
			if (necessary) {
				System.err.println("Eingabe ist nicht komplett oder fehlerhaft");
				System.err.println("Argument \"" + key + "\" erwartet");
				System.exit(1);	// Direkter Abbruch des Programms
			}
		}
		return result;
*/		
	}


	/**
	 * Gibt aus dem uebergebenen Objekt vom Typ Properties den Wert zum Schluessel "key" als int zurueck.
	 * Falls ein Fehler auftritt (also der Schluessel z.B. nicht existiert oder sich nicht nach int parsen laesst), wird 
	 * - bei "necessary"=true eine Fehlermeldung ausgegeben und das Programm komplett beendet und
	 * - bei "necessary"=false "defaultvalue" zurueckgegeben.
	 * (Vokabeln: key=Schluessel, defaulvalue=Standardwert, necessary=notwendig)
	 * 
	 * ! Achtung: Methode ist noch hier, damit alter Code auch noch funktioniert.
	 * ! Es sollte stattdessen stehts die Klasse tools.io.PropertiesPG verwendet werden.    
	 */
	public static int getIntProperty (Properties prop, String key, int defaultvalue, boolean necessary) {
	    return PropertiesPG.getIntProperty(prop, key, defaultvalue, necessary);
	}


	/**
	 * Gibt aus dem uebergebenen Objekt vom Typ Properties den Wert zum Schluessel "key" als String zurueck.
	 * Falls ein Fehler auftritt (also der Schluessel z.B. nicht existiert), wird 
	 * - bei "necessary"=true eine Fehlermeldung ausgegeben und das Programm komplett beendet und
	 * - bei "necessary"=false "defaultvalue" zurueckgegeben.
	 * (Vokabeln: key=Schluessel, defaulvalue=Standardwert, necessary=notwendig)    
	 * 
	 * ! Achtung: Methode ist noch hier, damit alter Code auch noch funktioniert.
	 * ! Es sollte stattdessen stehts die Klasse tools.io.PropertiesPG verwendet werden.    
	 */
	public static String getStringProperty (Properties prop, String key, String defaultvalue, boolean necessary) {
	    return PropertiesPG.getStringProperty(prop, key, defaultvalue, necessary);
	}


}
