/*
 * Copyright (C) 2005  PG447 - University Dortmund
 * 
 * This file is part of NObELTJE.
 * 
 * NObELTJE is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as published by
 * the Free Software Foundation.
 * 
 * NObELTJE is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with NObELTJE; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

 /*
 * Created on 28.05.2004
 */
package org.nobeltje.tools.io;

import java.io.FileReader;
import java.io.StreamTokenizer;
import java.util.ArrayList;
import java.util.List;

/**
 * @author $Author: noot $
 * @version $Revision: 726 $
 * $Date: 2005-02-02 00:50:20 +0100 (Mi, 02 Feb 2005) $
 */

public class FileParser
{
	/**
	 * Liest Vektordaten aus einer Datei und speichert die einzelnen Vektoren
	 * in einer ArrayList ab.<br>
	 * Dateiformat:
	 * <ul>
	 * <li>Kommentare werden mit // oder /*... eingeleitet</li>
	 * <li>Ansonsten steht pro Zeile ein Vektor, die Objektvariablen
	 * sind durch Leerzeichen getrennt</li>
	 * <li>Das Dezimaltrennzeichen ist der Punkt '.'</li>
	 * <li>Es ist auch m?glich die Werte in wissenschaftlicher Schreibweise
	 * anzugeben, Beispiel: 1.2345E-4</li>
	 * </ul>
	 * @param filename	Die Datei mit den Vektordaten
	 * @param list		Die Liste, in die die Vektordaten eingelesen werden
	 * @throws Exception
	 */
	public static void parseFile(String filename, List list) throws Exception
	{
		if (list == null)
			throw new Exception("Die ArrayList ist ungueltig!");

		list.clear();

		StreamTokenizer tokenizer = new StreamTokenizer(new FileReader(filename));
		tokenizer.parseNumbers();
		// Behandle End Of Lines als Token
		tokenizer.eolIsSignificant(true);
		// Ueberlese Kommentare
		tokenizer.slashSlashComments(true);
		tokenizer.slashStarComments(true);

		while (tokenizer.nextToken() != StreamTokenizer.TT_EOF) {
			ArrayList vector = new ArrayList();
			// Wenn eine Zahl gefunden wurde, speichere alle Werte bis zum Ende
			// der Zeile in einem Vektor ab
			
			// TODO: Hier muss noch abgefangen werden, dass bei Zahlen >1000 ein Komma (',') als Dezimal-Trennung benutzt wird!!!
			while (tokenizer.ttype == StreamTokenizer.TT_NUMBER) {
				vector.add(new Double(tokenizer.nval));
				tokenizer.nextToken();

				// Behandle wissenschaftliche Schreibweise gesondert
				if (tokenizer.ttype == StreamTokenizer.TT_WORD && tokenizer.sval.startsWith("E-")) {
					Double value = new Double(vector.get(vector.size()-1) + tokenizer.sval);
					vector.set(vector.size()-1, value);
					tokenizer.nextToken();
				}
			}
			// Fuege den Vektor der ArrayListe hinzu
			if (!vector.isEmpty())
				list.add(vector);
		}
	}
}
