/*
 * Copyright (C) 2005  PG447 - University Dortmund
 * 
 * This file is part of NObELTJE.
 * 
 * NObELTJE is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as published by
 * the Free Software Foundation.
 * 
 * NObELTJE is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with NObELTJE; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

 /*
 * Created on Jul 8, 2004
 *
 */
package org.nobeltje.tools.io;

import java.text.DecimalFormat;
import java.util.Locale;


/**
 * Stellt einen double- oder Double-Wert als einen String im Dezimalformat dar.
 * Die Anzahl der Nachkommastellen kann optional gewaehlt werden.
 * 
 * @author $Author: noot $
 * @version $Revision: 726 $
 * $Date: 2005-02-02 00:50:20 +0100 (Mi, 02 Feb 2005) $
 */
public class Number {

/**
 * Gibt einen String von <b>number</b> im Dezimalformat zurueck, 
 * mit <b>fraction</b> Nachkommastellen.
 * 
 * @param number darzustellende Zahl
 * @param fraction Anzahl der Nachkommastellen
 * @return String decimal: String im Dezimalformat
 */
public static String getDecimal(double number, int fraction) {
	Locale.setDefault(Locale.ENGLISH);
	String decimal;
	DecimalFormat df = new DecimalFormat();
	// Komma als Tausender-Trennzeichen abschalten
	df.setGroupingUsed(false);
	df.setMinimumFractionDigits(fraction);
	df.setMaximumFractionDigits(fraction);
	decimal = df.format(number);
	return decimal;
}
	
/**
 * Gibt einen String von <b>number</b> im Dezimalformat zurueck, 
 * mit <b>fraction</b> Nachkommastellen.
 * 
 * @param number darzustellende Zahl
 * @param fraction Anzahl der Nachkommastellen
 * @return String decimal: String im Dezimalformat
 */
public static String getDecimal(Double number, int fraction) {
	Locale.setDefault(Locale.ENGLISH);
	String decimal;
	DecimalFormat df = new DecimalFormat();
	// Komma als Tausender-Trennzeichen abschalten
	df.setGroupingUsed(false);
	df.setMinimumFractionDigits(fraction);
	df.setMaximumFractionDigits(fraction);
	decimal = df.format(number);
	return decimal;
}
	
/**
 * Gibt einen String von <b>number</b> im Dezimalformat zurueck, 
 * mit 6 Nachkommastellen.
 * 
 * @param number darzustellende Zahl
 * @return String decimal: String im Dezimalformat
 */
public static String getDecimal(double number) {
	Locale.setDefault(Locale.ENGLISH);
	String decimal;
	DecimalFormat df = new DecimalFormat();
	// Komma als Tausender-Trennzeichen abschalten
	df.setGroupingUsed(false);
	df.setMinimumFractionDigits(6);
	df.setMaximumFractionDigits(6);
	decimal = df.format(number);
	return decimal;
}

/**
 * Gibt einen String von <b>number</b> im Dezimalformat zurueck, 
 * mit 6 Nachkommastellen.
 * 
 * @param number darzustellende Zahl
 * @return String decimal: String im Dezimalformat
 */
public static String getDecimal(Double number) {
	Locale.setDefault(Locale.ENGLISH);
	String decimal;
	DecimalFormat df = new DecimalFormat();
	// Komma als Tausender-Trennzeichen abschalten
	df.setGroupingUsed(false);
	df.setMinimumFractionDigits(6);
	df.setMaximumFractionDigits(6);
	decimal = df.format(number);
	return decimal;
}






}
