/*
 * Copyright (C) 2005  PG447 - University Dortmund
 * 
 * This file is part of NObELTJE.
 * 
 * NObELTJE is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as published by
 * the Free Software Foundation.
 * 
 * NObELTJE is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with NObELTJE; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

 /*
 * Created on Jul 5, 2004
 */
 
package org.nobeltje.tools.io;

import java.util.Properties;

/**
 * Diese Klasse ergaenzt die Klasse java.util.Properties um einige nuetzliche get-Methoden.
 * Es werden 3 objektgebundene und 3 statische Methoden zur Verfuegung gestellt, mit
 * denen Elemente aus einem Properties-Objekt direkt zu einem der Typen int, double oder String
 * gecastet werden. 
 * Ausserdem ist die Angabe eines Standard-Wertes moeglich, fuer den Fall, dass beim laden
 * oder casten Fehler auftreten:
 * Ein weiteres Boolean-Argument bestimmt dabei, ob im Falle eines Fehlers 
 * eine Fehlermeldung ausgegeben und alles abgebrochen (bei true),
 * oder einfach der Standardwert zurueckgegeben (bei false) werden soll.
 * 
 * Die statischen Methoden koennen diese Funktionalitaet auch auf normale Properties-Objekte
 * anwenden.
 * 
 * @author $Author: noot $
 * @version $Revision: 726 $
 * $Date: 2005-02-02 00:50:20 +0100 (Mi, 02 Feb 2005) $
 */
public class PropertiesPG extends Properties {

	/**
	 * Konsturktor der Super-Klasse. Erstellt nur neues Objekt mit leerer Liste.
	 */
	public PropertiesPG() {
		super();
	}

	/**
	 * Konstruktor der Superklasse. Erstellt neues Objekt mit uebergebener Tabelle.
	 * 
	 * @param defaults Properties-Objekt, dessen Tabelle kopiert werden soll.
	 */
	public PropertiesPG(Properties defaults) {
		super(defaults);
	}
	
	/**
	 * Gibt aus dem aktuellen Objekt den Wert zum Schluessel "key" als double zurueck.
	 * Falls ein Fehler auftritt (also der Schluessel z.B. nicht existiert oder sich nicht nach double parsen laesst), wird 
	 * - bei "necessary"=true eine Fehlermeldung ausgegeben und das Programm komplett beendet und
	 * - bei "necessary"=false "defaultvalue" zurueckgegeben.
	 * (Vokabeln: key=Schluessel, defaulvalue=Standardwert, necessary=notwendig)
	 * Bsp-Aufruf: PropertiesPG-Objekt.getDoubleProperty("size", 0, false);    
	 * 
	 * @param key Name des Schluessels, zu dem der Wert gesucht ist
	 * @param defaultvalue Standardwert, wird bei Fehler zurueckgegeben, wenn necessary=false
	 * @param necessary Bestimmt, ob bei bei einem Fehler das Programm abgebrochen (true) oder der Standardwert zurueckgegeben werden soll (false). 
	 * @return Gibt den entsprechenden Wert als double zurueck 
	 */
	public double getDoubleProperty (String key, double defaultvalue, boolean necessary) {
		return getDoubleProperty(this, key, defaultvalue, necessary);
	}
		 
	
	/**
	 * Gibt aus dem aktuellen Objekt den Wert zum Schluessel "key" als int zurueck.
	 * Falls ein Fehler auftritt (also der Schluessel z.B. nicht existiert oder sich nicht nach int parsen laesst), wird 
	 * - bei "necessary"=true eine Fehlermeldung ausgegeben und das Programm komplett beendet und
	 * - bei "necessary"=false "defaultvalue" zurueckgegeben.
	 * (Vokabeln: key=Schluessel, defaulvalue=Standardwert, necessary=notwendig)    
	 * Bsp-Aufruf: PropertiesPG-Objekt.getIntProperty("size", 0, false);    
	 * 
	 * @param key Name des Schluessels, zu dem der Wert gesucht ist
	 * @param defaultvalue Standardwert, wird bei Fehler zurueckgegeben, wenn necessary=false
	 * @param necessary Bestimmt, ob bei bei einem Fehler das Programm abgebrochen (true) oder der Standardwert zurueckgegeben werden soll (false). 
	 * @return Gibt den entsprechenden Wert als int zurueck 
	 */
	public int getIntProperty (String key, int defaultvalue, boolean necessary) {
		return getIntProperty(this, key, defaultvalue, necessary);
	}
	
		 
	/**
	 * Gibt aus dem aktuellen Objekt den Wert zum Schluessel "key" als String zurueck.
	 * Falls ein Fehler auftritt (also der Schluessel z.B. nicht existiert), wird 
	 * - bei "necessary"=true eine Fehlermeldung ausgegeben und das Programm komplett beendet und
	 * - bei "necessary"=false "defaultvalue" zurueckgegeben.
	 * (Vokabeln: key=Schluessel, defaulvalue=Standardwert, necessary=notwendig)    
	 * Bsp-Aufruf: PropertiesPG-Objekt.getStringProperty("beschreibung", "keine vorhanden", false);    
	 * 
	 * @param key Name des Schluessels, zu dem der Wert gesucht ist
	 * @param defaultvalue Standardwert, wird bei Fehler zurueckgegeben, wenn necessary=false
	 * @param necessary Bestimmt, ob bei bei einem Fehler das Programm abgebrochen (true) oder der Standardwert zurueckgegeben werden soll (false). 
	 * @return Gibt den entsprechenden Wert als String zurueck 
	 */
	public String getStringProperty (String key, String defaultvalue, boolean necessary) {
		return getStringProperty(this, key, defaultvalue, necessary);
	}
		 
	
	/**
	 * Gibt aus dem uebergebenen Objekt vom Typ Properties den Wert zum Schluessel "key" als double zurueck.
	 * Falls ein Fehler auftritt (also der Schluessel z.B. nicht existiert oder sich nicht nach double parsen laesst), wird 
	 * - bei "necessary"=true eine Fehlermeldung ausgegeben und das Programm komplett beendet und
	 * - bei "necessary"=false "defaultvalue" zurueckgegeben.
	 * (Vokabeln: key=Schluessel, defaulvalue=Standardwert, necessary=notwendig)    
	 * Bsp-Aufruf: PropertiesPG.getStringProperty(Properties-Objekt, "size", 0, false);    
	 * 
	 * @param prop Properties-Objekt, von dem der Wert geholt werden, also mit dem gearbeitet werden soll
	 * @param key Name des Schluessels, zu dem der Wert gesucht ist
	 * @param defaultvalue Standardwert, wird bei Fehler zurueckgegeben, wenn necessary=false
	 * @param necessary Bestimmt, ob bei bei einem Fehler das Programm abgebrochen (true) oder der Standardwert zurueckgegeben werden soll (false). 
	 * @return Gibt den entsprechenden Wert als double zurueck 
	 */
	public static double getDoubleProperty (Properties prop, String key, double defaultvalue, boolean necessary) {
		boolean error = false;
		double result = defaultvalue;
		if (prop.containsKey(key)) {
			try {
//				result = Double.parseDouble(prop.getProperty(key, String.valueOf(defaultvalue)));
				result = Double.parseDouble(prop.getProperty(key));
			} catch (Exception e1) {
			}
		} else {
			error = true;
		}
		
		if (necessary && error) {
			System.err.println("Eingabe ist nicht komplett oder fehlerhaft");
			System.err.println("Argument \"" + key + "\" erwartet");
			System.exit(1);	// Direkter Abbruch des Programms
		}
		
		return result;
	}

	/**
	 * Gibt aus dem uebergebenen Objekt vom Typ Properties den Wert zum Schluessel "key" als gerundeten int zurueck.
	 * Falls ein Fehler auftritt (also der Schluessel z.B. nicht existiert oder sich nicht nach int parsen laesst), wird 
	 * - bei "necessary"=true eine Fehlermeldung ausgegeben und das Programm komplett beendet und
	 * - bei "necessary"=false "defaultvalue" zurueckgegeben.
	 * (Vokabeln: key=Schluessel, defaulvalue=Standardwert, necessary=notwendig)    
	 * Bsp-Aufruf: PropertiesPG.getStringProperty(Properties-Objekt, "size", 0, false);    
	 * 
	 * @param prop Properties-Objekt, von dem der Wert geholt werden, also mit dem gearbeitet werden soll
	 * @param key Name des Schluessels, zu dem der Wert gesucht ist
	 * @param defaultvalue Standardwert, wird bei Fehler zurueckgegeben, wenn necessary=false
	 * @param necessary Bestimmt, ob bei bei einem Fehler das Programm abgebrochen (true) oder der Standardwert zurueckgegeben werden soll (false). 
	 * @return Gibt den entsprechenden Wert als int zurueck 
	 */
	public static int getIntProperty (Properties prop, String key, int defaultvalue, boolean necessary) {
		boolean error = false;
		int result = defaultvalue;
		if (prop.containsKey(key)) {
			try {
				//double ungerundet = Double.parseDouble(prop.getProperty(key, String.valueOf(defaultvalue)));
				double ungerundet = Double.parseDouble(prop.getProperty(key));
				result = (new Long(Math.round(ungerundet))).intValue();
			} catch (Exception e1) {
			}
		} else {
			error = true;
		}
		if (necessary && error) {
				System.err.println("Eingabe ist nicht komplett oder fehlerhaft");
				System.err.println("Argument \"" + key + "\" erwartet");
				System.exit(1);	// Direkter Abbruch des Programms
		}
		return result;
	}


	/**
	 * Gibt aus dem uebergebenen Objekt vom Typ Properties den Wert zum Schluessel "key" als String zurueck.
	 * Falls ein Fehler auftritt (also der Schluessel z.B. nicht existiert), wird 
	 * - bei "necessary"=true eine Fehlermeldung ausgegeben und das Programm komplett beendet und
	 * - bei "necessary"=false "defaultvalue" zurueckgegeben.
	 * (Vokabeln: key=Schluessel, defaulvalue=Standardwert, necessary=notwendig)    
	 * Bsp-Aufruf: PropertiesPG.getStringProperty(Properties-Objekt, "beschreibung", "keine vorhanden", false);    
	 * 
	 * @param prop Properties-Objekt, von dem der Wert geholt werden, also mit dem gearbeitet werden soll
	 * @param key Name des Schluessels, zu dem der Wert gesucht ist
	 * @param defaultvalue Standardwert, wird bei Fehler zurueckgegeben, wenn necessary=false
	 * @param necessary Bestimmt, ob bei bei einem Fehler das Programm abgebrochen (true) oder der Standardwert zurueckgegeben werden soll (false). 
	 * @return Gibt den entsprechenden Wert als String zurueck 
	 */
	public static String getStringProperty (Properties prop, String key, String defaultvalue, boolean necessary) {
		boolean error = false;
		String result = defaultvalue;
		if (prop.containsKey(key)) {
			try {
//				result = prop.getProperty(key, defaultvalue);
				result = prop.getProperty(key);
			} catch (Exception e1) {
			}
		} else {
			error = true;
		}
		if (necessary && error) {
				System.err.println("Eingabe ist nicht komplett oder fehlerhaft");
				System.err.println("Argument \"" + key + "\" erwartet");
				System.exit(1);	// Direkter Abbruch des Programms
		}
		return result;
	}
	


}
