/*
 * Copyright (C) 2005  PG447 - University Dortmund
 * 
 * This file is part of NObELTJE.
 * 
 * NObELTJE is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as published by
 * the Free Software Foundation.
 * 
 * NObELTJE is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with NObELTJE; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

 /*
 * Created on Jul 5, 2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */

package org.nobeltje.tools.test;

import java.util.ArrayList;
import java.util.List;

import junit.framework.TestCase;
import org.nobeltje.tools.Individual;
import org.nobeltje.tools.ParetoArchive;



/**
 * @author $Author: noot $
 * @version $Revision: 726 $
 * $Date: 2005-02-02 00:50:20 +0100 (Mi, 02 Feb 2005) $
 */
public class ParetoArchiveTest extends TestCase 
{
	
	
	public void testRefreshArray()
	{
		// Individuen generieren
		Individual ind1 = new Individual();
		Individual ind2 = new Individual();
		Individual ind3 = new Individual();
		
		// Die Individuen muessen auch Objektvariablen bekommen, 
		// da sie sonst vom Pareto-Archiv als alle dieselben Individuen angesehen werden 
		ArrayList obj1 = new ArrayList();
		ArrayList obj2 = new ArrayList();
		ArrayList obj3 = new ArrayList();
		
		obj1.add(new Double(1.0));
		obj1.add(new Double(2.0));
		obj2.add(new Double(3.0));
		obj2.add(new Double(4.0));
		obj3.add(new Double(5.0));
		obj3.add(new Double(6.0));
		
		// den Individuen eine Fitness zuweisen
		ArrayList fitness1 = new ArrayList();
		ArrayList fitness2 = new ArrayList();
		ArrayList fitness3 = new ArrayList();
		
		fitness1.add(new Double(1.0));
		fitness1.add(new Double(2.0));

		fitness2.add(new Double(2.0));	// nicht dominiert von ind1, beide sind nicht vergleichbar, da gleich gut
		fitness2.add(new Double(1.0));
		
		fitness3.add(new Double(3.0));	// schlechter als ind1 und ind2
		fitness3.add(new Double(4.0));
	
		ind1.setFitness(fitness1);
		ind2.setFitness(fitness2);
		ind3.setFitness(fitness3);
		
		// Die Objekt-Variablen zuweisen
		ind1.setObjectVariables(obj1);
		ind2.setObjectVariables(obj2);
		ind3.setObjectVariables(obj3);
		
		
		// Archiv-Listen generieren und updaten
		ArrayList array = new ArrayList();
		ArrayList newArray = new ArrayList();
		
		// Individuum ind1 ind noch leere Archiv
		newArray = ParetoArchive.refreshArchive(2, array, ind1);
		assertEquals("1. Test: ein Ind. im Archiv?", newArray.get(0), ind1);	
		
		
		array = newArray;
		newArray = ParetoArchive.refreshArchive(2, array, ind2);
		assertEquals("2. Test: erstes Ind. noch im Archiv?", newArray.get(0), ind1);	
		assertEquals("3. Test: zweites Ind. auch im Archiv?", newArray.get(1), ind2);	
		
		// ind3 duerfte jetzt nicht ins Archiv aufgenommen werden...
		array = newArray;
		newArray = ParetoArchive.refreshArchive(2, array, ind3);
		assertEquals("4. Test: Ind1 noch im Archiv?", newArray.get(0), ind1);	
		assertEquals("5. Test: Ind2 auch noch im Archiv?", newArray.get(1), ind2);	
		assertEquals("Archiv nur 2 Individuen gross?", 2, newArray.size());
		
	}
	
	
	
	public void testRefreshNeu() {
		System.out.println("testRefreshNeu()");
		
		List pop = new ArrayList();
		List popNew = new ArrayList();
		List popEmpty = new ArrayList();
        
		Individual ind1 = new Individual();
        List fitVal1 = new ArrayList();
        fitVal1.add(new Double(1.0));
		fitVal1.add(new Double(1.0));
		fitVal1.add(new Double(0.0));
        List oVal1 = new ArrayList();
        oVal1.add(new Double(1.0));
		oVal1.add(new Double(1.0));
		oVal1.add(new Double(0.0));
		ind1.setObjectVariables(oVal1);
		ind1.setFitness(fitVal1);
        
		Individual ind2 = new Individual();
        List fitVal2 = new ArrayList();
        fitVal2.add(new Double(3.5));
		fitVal2.add(new Double(2.0));
		fitVal2.add(new Double(1.0));
		ind2.setFitness(fitVal2); 
        
		Individual ind3 = new Individual();
		List fitVal3 = new ArrayList();
		fitVal3.add(new Double(3.0));
		fitVal3.add(new Double(1.0));
		fitVal3.add(new Double(0.0));
		ind3.setFitness(fitVal3);
        
		//pop.add(ind1);
		//pop.add(ind2);
		pop.add(ind3);
		
		System.out.println("Population: ");
		for(int i = 0; i<pop.size();i++)
			System.out.println(((Individual)pop.get(i)).getFitness().toString());
        
		//	Neuer Test:  1 Ind. im Archiv, ein besseres solle eingefuegt und das alste geloescht werden
		
		
		popNew = ParetoArchive.refreshArchiveOld(0, (ArrayList)pop, ind1);
		assertEquals("ind1 im Archiv?", popNew.get(0), ind1);
		//assertEquals("ind2 im Archiv?", popNew.get(1), ind2);
		//assertEquals("ind3 im Archiv?", popNew.get(2), ind3);
		
		System.out.println("PopNew: ");
		for(int i = 0; i<popNew.size();i++)
			System.out.println(((Individual)popNew.get(i)).getFitness().toString());
		
		
		popNew = ParetoArchive.refreshArchive(0, (ArrayList)popNew, ind2);
		assertEquals("ind2 im Archiv?", popNew.get(0), ind2);
		
		System.out.println("PopNew: ");
		for(int i = 0; i<popNew.size();i++)
			System.out.println(((Individual)popNew.get(i)).getFitness().toString());
		
		popNew = ParetoArchive.refreshArchive(0, (ArrayList)popNew, ind3);
		//assertEquals("besseres Ind. im Archiv?", popNew.get(0), ind3);
		
		System.out.println("PopNew: ");
		for(int i = 0; i<popNew.size();i++)
			System.out.println(((Individual)popNew.get(i)).getFitness().toString());
	}
	
	

	public void testRefreshDominatedArchive()
	{
		/*
		 * 
		 * Stimmt noch nicht so ganz...
		 * 
		 */
		
		// Individuen generieren
		Individual ind1 = new Individual();
		Individual ind2 = new Individual();
		Individual ind3 = new Individual();
		
		// Die Individuen muessen auch Objektvariablen bekommen, 
		// da sie sonst vom Pareto-Archiv als alle dieselben Individuen angesehen werden 
		ArrayList obj1 = new ArrayList();
		ArrayList obj2 = new ArrayList();
		ArrayList obj3 = new ArrayList();
		
		obj1.add(new Double(1.0));
		obj1.add(new Double(2.0));
		obj2.add(new Double(3.0));
		obj2.add(new Double(4.0));
		obj3.add(new Double(5.0));
		obj3.add(new Double(6.0));
		
		// den Individuen eine Fitness zuweisen
		ArrayList fitness1 = new ArrayList();
		ArrayList fitness2 = new ArrayList();
		ArrayList fitness3 = new ArrayList();
		
		fitness1.add(new Double(1.0));
		fitness1.add(new Double(2.0));

		fitness2.add(new Double(2.0));	// nicht dominiert von ind1, beide sind nicht vergleichbar, da gleich gut
		fitness2.add(new Double(1.0));
		
		fitness3.add(new Double(0.5));	// besser als ind1 und ind2
		fitness3.add(new Double(0.4));
	
		ind1.setFitness(fitness1);
		ind2.setFitness(fitness2);
		ind3.setFitness(fitness3);
		
		// Die Objekt-Variablen zuweisen
		ind1.setObjectVariables(obj1);
		ind2.setObjectVariables(obj2);
		ind3.setObjectVariables(obj3);
		
		
		// Archiv-Listen generieren und updaten
		ArrayList array = new ArrayList();
		ArrayList newArray = new ArrayList();
		
		array.add(ind1);
		// Individuum ind1 ind noch leere Archiv
		newArray = ParetoArchive.refreshDominatedArchive(array);
		assertEquals("1. Test: Archiv bleibt leer?", newArray.size(), 0);	
		
		
		array.add(ind2);
		System.out.println("1. Test: "+newArray.size());
		newArray = ParetoArchive.refreshDominatedArchive(array);
		System.out.println("2. Test: "+newArray.size());
		assertEquals("2. Test: Archiv noch immer leer?", newArray.size(), 0);	
		
		array.add(ind3);
		System.out.println("2,5. Test: "+array.size());
		newArray = ParetoArchive.refreshDominatedArchive(array);
		System.out.println("3. Test: "+newArray.size());
		assertEquals("3. Test: Ind1 im Archiv?", newArray.get(0), ind1);	
		assertEquals("4. Test: Ind2 im Archiv?", newArray.get(1), ind2);	
		
		
	}

	
	
	
	
}
