/*************************************************************/
/**
 * Simple Elevatormodel based on [BPM05]
 * 
 * IMPORTANT: This Source is NOT well-tested
 *            and will not be supported !!!
 * 
 * For further information about the original,
 * well tested and supported version,
 * please send an e-mail to: 
 * <thomas.bartz-beielstein@udo.edu>
 *
 * ------------------------------------------------------
 *
 * Vereinfachtes Fahrstuhlmodell basierend auf [BPM05]
 * 
 * WICHTIG: Der Quelltext wurde NICHT ausgiebig getestet
 *          und wird nicht mehr gepflegt !!!
 *
 * Fuer weitere Informationen ueber die
 * ausgiebig getestete und gepflegte Original-Version,
 * bitte eine e-mail senden an:
 * <thomas.bartz-beielstein@udo.edu>
 *
 * ------------------------------------------------------
 *
 * @incollection{BPM05,
 * AUTHOR = {Thomas Bartz--Beielstein and Mike Preu\ss{}
 * and Sandor Markon},
 * TITLE = {Validation and optimization of an elevator
 * simulation model with modern search heuristics},
 * EDITOR = {T.~Ibaraki and K.~Nonobe and M.~Yagiura},
 * BOOKTITLE = {Metaheuristics: Progress as Real Problem
 * Solvers},
 * CHAPTER = {5},
 * PAGES = {109--128},
 * PUBLISHER = {Kluwer},
 * ADDRESS = {Boston MA},
 * YEAR = {2005},
 * ISBN = {},
 * NOTE = {(in print)}
 * } 
 */
/*************************************************************/




/**
 * @author $Author: saltmann $
 * @version $Revision: 720 $<br>
 * $Date: 2005-02-01 12:05:35 +0100 (Di, 01 Feb 2005) $
 *
 */

/* Aufruf-Parameter:
	1. n: Anzahl der Stockwerke
	2. m: Anzahl der Wagen
	3. n_iteration: Anzahl der Interationen (eine It. ein Stockwerk?)
	4. l: Wahrscheinlichkeit, fuer Hall-Call auf einem Stockwerk
	5. sd: Random-Seed
	6. ts: Debug-Ausgaben-Flag (1=ja, 0=nein)
Bsp: ./ring 6 2 1000000 0.3 13 0
*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <unistd.h>

#include <sys/select.h>
#include <sys/types.h>
#include <sys/time.h>
#include <errno.h>

#define N 64
static int s[N], c[N], p, pa, r, n, m;
static float w[N+N]; // Gewichtungen - Eigentliche Eingabe
static float l; // Wahrscheinlichkeit, fuer Hall-Call auf einem Stockwerk

int n_weights, n_iterations;

int sd; // Random Seed
int ts; // Debug-Flag


/* Utility: return TRUE with probability p */
static int prob(float p) {
  return ((float)(rand()) / (float)RAND_MAX) < p;
}



/* Initialize parameters and work variables */
static void init_ring(int nx, int mx, float lx, int sd) {
   int i;

   n = nx;
   m = mx;
   /*   printf("\nFloors: %d",n);
	printf("\nCars: %d\n",m);*/
   l = lx;
   r = n;
   for (i=0;i<n;i++) {
     s[i] = c[i] = 0;
     if (prob(l)) {
       c[i] = 1;
       r--;
     }
   }
   for (i=0;i<m;i++) {
     s[i] = 1;
   }
   p = 0;
   pa = 1;
}



/* Step the ring from s to s1; a: decision  d: customer arrival */
static int step_ring(int a, int d) {
  if (d && !c[p]) {
c[p] = 1; r--;
}
  if (s[p]) {
    if (c[p]) {
      if (a || s[pa]) {
	c[p] = 0; r++;
      } else {
	s[pa] = s[p]; s[p] = 0;
      }
    } else if (!s[pa]) {
      s[pa] = s[p]; s[p] = 0;
    }
  }
  pa = p; p = (p + n - 1)%n;
  return r;
}



static int perceptron(int n, double *w) {
  int i,j,k;
  float x=0.0;

  for (i=k=0,j=p;i<n;i++,k++,j=(j+1)%n)
    x += (float)s[j]*w[k];
  for (i=0,j=p;i<n;i++,k++,j=(j+1)%n){
    x += (float)c[j]*w[k];
    /*    printf("\nk: %d\t%f",k,w[k]);*/
  }
  return x>0.0;
}



#define B 256
static void ring_start(void)
{
/*
  int n=6,m=2,sd=1,ts=0;
  float l=0.3;
  FILE *f;
  char buf[B];

  if ((f = fopen("ring.para","r"))==NULL) {
    fprintf(stderr,"Cannot find ring.para file.\n");
    exit(-1);
  }
  while (!feof(f)) {
    if ( fgets(buf,B,f)==NULL ) break;
    switch (*buf) {
    case 'n': sscanf(buf,"n %d",&n); break;
    case 'm': sscanf(buf,"m %d",&m); break;
    case 'i': sscanf(buf,"i %d",&n_iterations); break;
    case 'p': sscanf(buf,"p %f",&l); break;
    case 'r': sscanf(buf,"r %d",&sd); break;
    case 't': sscanf(buf,"t %d",&ts); break;
    default: printf("[%s]\n",buf); break;
    }
  }
  fclose(f);
*/

  if (n>N) n=N;
  if (m>=n) m=n-1;

  if (ts) {
    printf("n=%d\nm=%d\ni=%d\np=%f\nr=%d\n",n,m,n_iterations,l,sd);
  }
  init_ring(n,m,l,sd);
  n_weights = 2*n;
}



double ring(int nn, double *w)
{
  int i;
  int a,c,x=0;
  float h;
  static int first=1;

  /* Ring Init */
  if (first) {
    ring_start();
    first=0;
  }
  
  /* Eigentliche Schleife fuer Ring */
  for (i=0;i<n_iterations;i++) {
    c = prob(l);
    a = perceptron(n, w);
    x += step_ring(a,c);
  }
  
  /* Ergebnis berechnen */
  h = (float)x/(float)n_iterations;
  /* Use this line to make into minimization problem */
  return (double)n-(double)h;
  // return (double)h;
}



#ifdef TEST_RING
int main(int argc, char *argv[]) {

  /* Init/Deklaration */
  int i;
  double w[N+N+1],x;

  /* Fehlerausgabe bei zu wenigen Parametern */
  if (argc<=6) {
  printf ("\n   !!! Fehler in Eingabe aufgetreten !!!\n Aufruf-Parameter:\n\
	1. n: Anzahl der Stockwerke\n\
	2. m: Anzahl der Wagen\n\
	3. n_iteration: Anzahl der Iterationen (eine It. ein Stockwerk?)\n\
	4. l: Wahrscheinlichkeit, fuer Hall-Call auf einem Stockwerk\n\
	5. sd: Random-Seed\n\
	6. ts: Debug-Ausgaben-Flag (1=ja, 0=nein)\n\
	Bsp: ./ring 6 2 1000000 0.3 13 0\n\n");
	exit(-1);
  }

  /* Werte aus Kommandozeilen-Parametern lesen */
  n=atoi(argv[1]); // Stockwerke
  m=atoi(argv[2]); // Wagen
  n_iterations=atoi(argv[3]); // Iterationen

  l=atof(argv[4]); // Wahrscheinlichkeit Hall-Call
  if (l<0.0) l=0.0;
  if (l>1.0) l=1.0;

  sd=atoi(argv[5]); // Random-Seed
  srand(sd);

  ts=atoi(argv[6]); // Debug-Flag

  /* printf("%d\n", sd); */

  /* Werte-Vektor aus Kommandozeilen-Parametern lesen */
  for (i=7;i<argc;i++) {
    w[i-6]=atof(argv[i]);
    /*        printf("%f\t",w[i-6]);  */
  }

  /* Simulation starten */
  x=ring(i,w);

  /* Ergebnis ausgeben */
  printf("%f\n",x);

  return 0;
}

#endif
